//
//  enemy.cpp
//  3ds_metroid
//
//  Created by jbr373 on 16/01/16.
//

#include <string.h>

#include "enemy.h"
#include "actor_facing.h"
#include "filepaths.h"

#define DEFAULT_INVINCIBILITY_FRAMES        30
#define DEFAULT_DEATH_FRAMES                16

#define MIN_ENEMY_FACING_FRAMES_LEFT        0
#define MAX_ENEMY_FACING_FRAMES_LEFT        3

#define MIN_ENEMY_FACING_FRAMES_RIGHT       4
#define MAX_ENEMY_FACING_FRAMES_RIGHT       7

Enemy::Enemy()
{
    enemy_x = -400;
    enemy_y = -240;
    
    enemy_dead = false;
    enemy_shadow = false;
    enemy_metroid_flag = false;
    enemy_loot_rolled = false;
    enemy_moving = false;
    
    enemy_health = 0;
    
    enemy_sprite_path = LINE_DO_NOT_USE;
    enemy_loot = LINE_DO_NOT_USE;
    
    enemy_ai = 0;
    enemy_physics = 0;
    enemy_anim = 0;
    enemy_state = 0;
    enemy_facing = FACING_LEFT;
    enemy_frame = 0;
    enemy_frame_count = 0;
    
    enemy_death_frames = 0;
    enemy_invincibility_frames = 0;
    
    enemy_hitbox_width = 0;
    enemy_hitbox_height = 0;
    enemy_size = 1;
    
    enemy_damage_hitbox_x = 0;
    enemy_damage_hitbox_y = 0;
    enemy_damage_hitbox_width = 0;
    enemy_damage_hitbox_height = 0;
    
    enemy_loot_chance = 0;
    
    enemy_touch_damage = 0;
    enemy_projectile_damage = 0;
    
    enemy_sound_id = 0;
    
    pEnemyProjectile = NULL;
    
    pAI = NULL;
}

Enemy::~Enemy()
{
    // Delete Enemy Projectiles
    enemy_delete_projectile();
    
    // Delete Enemy AI
    if (pAI != NULL)
    {
        delete pAI;
    }
}

void Enemy::enemy_load(EnemyFileStruct* pEnemyFileStruct, s32 new_enemy_x, s32 new_enemy_y)
{
    if (pEnemyFileStruct != NULL)
    {
        // Load Data
        enemy_hitbox_width = pEnemyFileStruct->enemy_hitbox_width;
        enemy_hitbox_height = pEnemyFileStruct->enemy_hitbox_height;
        enemy_size = pEnemyFileStruct->enemy_size;
        
        enemy_health = pEnemyFileStruct->enemy_health;
        
        enemy_ai = pEnemyFileStruct->enemy_ai;
        enemy_physics = pEnemyFileStruct->enemy_physics;
        enemy_speed = pEnemyFileStruct->enemy_speed;
        
        enemy_metroid_flag = pEnemyFileStruct->enemy_is_metroid;
        enemy_phase_metroid_flag = pEnemyFileStruct->enemy_metroid_flag;
        
        enemy_shadow = pEnemyFileStruct->enemy_shadow;
        
        enemy_touch_damage = pEnemyFileStruct->enemy_touch_damage;
        enemy_projectile_damage = pEnemyFileStruct->enemy_projectile_damage;
        
        enemy_damage_hitbox_x = pEnemyFileStruct->enemy_damage_hitbox_x;
        enemy_damage_hitbox_y = pEnemyFileStruct->enemy_damage_hitbox_y;
        enemy_damage_hitbox_width = pEnemyFileStruct->enemy_damage_hitbox_width;
        enemy_damage_hitbox_height = pEnemyFileStruct->enemy_damage_hitbox_height;
        
        enemy_loot_chance = pEnemyFileStruct->enemy_loot_chance;
        enemy_loot = pEnemyFileStruct->enemy_loot;
        enemy_loot_spawn_x = pEnemyFileStruct->enemy_loot_spawn_x;
        enemy_loot_spawn_y = pEnemyFileStruct->enemy_loot_spawn_y;
        
        enemy_sprite_path = pEnemyFileStruct->enemy_spritesheet;
        enemy_sprite_width = pEnemyFileStruct->enemy_spritesheet_width;
        enemy_sprite_height = pEnemyFileStruct->enemy_spritesheet_height;
        enemy_sprite_frame_speed = pEnemyFileStruct->enemy_max_frame_speed;
        enemy_max_spriteline = pEnemyFileStruct->enemy_max_spritelines;
        
        enemy_sound_id = pEnemyFileStruct->enemy_sound_id;
        
        enemy_x = new_enemy_x;
        enemy_y = new_enemy_y;
        
        // Load AI
        pAI = enemy_ai_load(enemy_ai);
    }
    else
    {
        // Move Enemy out of Room
        enemy_x = -400;
        enemy_y = -240;
        enemy_dead = true;
    }
}

void Enemy::enemy_action(Sound* pSound, Player* pPlayer, u8 collision)
{
    // Update Hitbox
    enemy_update_hitbox();
    
    // Update Frame
    if (enemy_facing < FACING_LEFT)
    {
        if (enemy_frame_count == enemy_sprite_frame_speed)
        {
            if (enemy_frame >= MAX_ENEMY_FACING_FRAMES_LEFT)
            {
                enemy_frame = MIN_ENEMY_FACING_FRAMES_LEFT;
            }
            else
            {
                ++enemy_frame;
            }
            
            enemy_frame_count = 0;
        }
        else
        {
            ++enemy_frame_count;
        }
    }
    else
    {
        if (enemy_frame_count == enemy_sprite_frame_speed)
        {
            if (enemy_frame >= MAX_ENEMY_FACING_FRAMES_RIGHT)
            {
                enemy_frame = MIN_ENEMY_FACING_FRAMES_RIGHT;
            }
            else
            {
                ++enemy_frame;
            }
            
            enemy_frame_count = 0;
        }
        else
        {
            ++enemy_frame_count;
        }
    }
    
    // Reset Enemy Anim
    enemy_anim = ENEMYANIM_WALK;
    
    // Remove Invincibility Frames Counter
    if (enemy_invincibility_frames > 0)
    {
        // Set Damage Animation
        if (enemy_max_spriteline > ENEMYANIM_WALK)
        {
            enemy_anim = ENEMYANIM_DAMAGE;
        }
        
        // Reduce Invincibility Frames
        --enemy_invincibility_frames;
    }
    
    // Update Death Animation
    if (enemy_death_frames > 0)
    {
        // Set Damage Animation
        if (enemy_max_spriteline > ENEMYANIM_DAMAGE)
        {
            enemy_anim = ENEMYANIM_DEATH;
        }
        
        // Update AI
        if (pAI != NULL)
        {
            // Set AI to paused
            pAI->enemy_ai_set_paused(true);
        
            // Set AI Movement
            pAI->enemy_ai_set_movement(false);
        }
        
        // Set Enemy Movement Flag
        enemy_moving = false;
        
        
        // Reduce Invincibility Frames
        --enemy_death_frames;
        
        // Check if we've hit 0 Frames!
        if (enemy_death_frames == 0)
        {
            // Flag Enemy as Dead
            enemy_dead = true;
        }
    }
    
    // Update AI if we're alive!
    if (enemy_health > 0 && enemy_dead == false && pAI != NULL)
    {
        // Update AI
        pAI->enemy_ai_update(pPlayer, pEnemyProjectile, enemy_x, enemy_y, enemy_health, enemy_speed, enemy_facing, collision, enemy_anim);
            
        // Set Movement
        enemy_moving = pAI->enemy_ai_get_movement();
    }
    
    // TEMP: TSUMURI
    if (enemy_ai == AI_TSUMURI)
    {
        enemy_anim = ENEMYANIM_STANDING;
    }
}

bool Enemy::enemy_check_hit_collision(s32 attacker_x, s32 attacker_y)
{
    // Get Hitbox Collision
    if (attacker_x >= (enemy_x + enemy_damage_hitbox_x) && attacker_x <= ((enemy_x + enemy_damage_hitbox_x) + enemy_damage_hitbox_width))
    {
        if (attacker_y >= (enemy_y + enemy_damage_hitbox_y) && attacker_y <= ((enemy_y + enemy_damage_hitbox_y) + enemy_damage_hitbox_height))
            return true;
    }
    
    return false;
}

void Enemy::enemy_update_hitbox()
{
    // Check for special Hitbox sizes
    if (enemy_size == 0 || enemy_size == 1 || enemy_size == 100)
        return;
    
    // General Hitbox
    enemy_hitbox_width = enemy_hitbox_width * (enemy_size / 100);
    enemy_hitbox_height = enemy_hitbox_height * (enemy_size / 100);
    
    // Damage Hitbox
    enemy_damage_hitbox_x = enemy_damage_hitbox_x * (enemy_size / 100);
    enemy_damage_hitbox_y = enemy_damage_hitbox_y * (enemy_size / 100);
    enemy_damage_hitbox_width = enemy_damage_hitbox_width * (enemy_size / 100);
    enemy_damage_hitbox_height = enemy_damage_hitbox_height * (enemy_size / 100);
    
    // Enemy Loot Addition
    // TODO: Is this correct?
    enemy_loot_spawn_x = enemy_loot_spawn_x * (enemy_size / 100);
    enemy_loot_spawn_y = enemy_loot_spawn_y * (enemy_size / 100);
}

bool Enemy::enemy_drop_loot()
{
    // Check if we got a valid Item!
    if (strcmp(enemy_loot.c_str(), LINE_DO_NOT_USE) == 0)
    {
        // Return on Loot unsued
        return false;
    }
    
    // Check if we can get Loot
    if (enemy_loot_chance == 0 || enemy_loot_rolled == true)
        return false;

    // Roll Chances
    u8 roll = rand() % 100;
    
    // Set Loot Dropped Flag
    enemy_loot_rolled = true;
    
    // Always return true on 100 % Chance
    if (enemy_loot_chance >= 100)
    {
        return true;
    }

    // Return Result
    if (roll <= enemy_loot_chance)
    {
        return true;
    }
    else
    {
        return false;
    }
}

void Enemy::enemy_damage(u8 damage, u8 damage_type)
{
    // For Metroid Flag
    bool do_damage = true;
    
    // Check Metroid specific
    if (enemy_metroid_flag == true)
    {
        if (damage_type == PROJECTILE_ICEBEAM_CHARGED || damage_type == PROJECTILE_ICEBEAM || damage_type == PROJECTILE_MISSLE)
            do_damage = true;
        else
            do_damage = false;
    }
    
    // Damage only if no invincibility Frames are up
    if (enemy_invincibility_frames == 0 && do_damage == true)
    {
        // Get Full Player Health
        enemy_health -= damage;
        
        // Calculation Result Actions
        if (enemy_health <= 0)
        {
            // Set Death Frames
            enemy_death_frames = DEFAULT_DEATH_FRAMES; // 8 * 2 (Frames * FrameSpeed)
            
            // Set Frame Speed
            enemy_sprite_frame_speed = 2;
            
            // Set Animation
            if (enemy_max_spriteline > ENEMYANIM_DAMAGE)
            {
                if (enemy_anim == ENEMYANIM_STANDING)
                {
                    enemy_anim = ENEMYANIM_STANDING_DAMAGE;
                }
                else
                {
                    enemy_anim = ENEMYANIM_DEATH;
                }
            }
            
            // Reset Frame
            enemy_frame = 0;
        }
        
        // Set Invincibility Frames
        enemy_invincibility_frames = DEFAULT_INVINCIBILITY_FRAMES;
    }
}

void Enemy::enemy_delete_projectile()
{
    delete pEnemyProjectile;
}