//
//  enemy.h
//  3ds_metroid
//
//  Created by jbr373 on 16/01/16.
//

#ifndef enemy_h
#define enemy_h

#include <stdio.h>

#include "enemy_ai.h"
#include "image.h"
#include "projectile.h"
#include "sound.h"
#include "player.h"
#include "files.h"

enum EnemyPhysics
{
    ENEMYPHYSICS_NONE = 0,
    
    ENEMYPHYSICS_WALKING = 1,
    ENEMYPHYSICS_FLYING = 2,
    ENEMYPHYSICS_SWIMMING = 3,
    ENEMYPHYSICS_WALLWALKING = 4,
    ENEMYPHYSICS_ALL = 5,
};

enum EnemyAnim
{
    // Default Animations
    ENEMYANIM_WALK = 0,
    ENEMYANIM_DAMAGE = 1,
    ENEMYANIM_DEATH = 2,
    ENEMYANIM_FROZEN = 3,
    
    // Special Animations
    ENEMYANIM_WALLWALKING_RIGHT = 5, // Ground
    ENEMYANIM_WALLWALKING_DOWN = 6, // Left Wall
    ENEMYANIM_WALLWALKING_LEFT = 7, // Right Wall
    ENEMYANIM_WALLWALKING_UP = 8, // Ceiling
    
    // Standing Animation
    ENEMYANIM_STANDING = 9,
    ENEMYANIM_STANDING_DAMAGE = 10,
    
    // Jumping Animation
    ENEMYANIM_JUMP_START = 11,
    ENEMYANIM_JUMP_FLY = 12,
    ENEMYANIM_JUMP_LAND = 13,
};

class Enemy
{
public:
    Enemy();
    virtual ~Enemy();
    
    void enemy_load(EnemyFileStruct* pEnemyFileStruct, s32 new_enemy_x, s32 new_enemy_y);
    
    void enemy_action(Sound* pSound, Player* pPlayer, u8 collision);
    
    void enemy_damage(u8 damage, u8 damage_type);
    bool enemy_check_hit_collision(s32 attacker_x, s32 attacker_y);
    
    bool enemy_drop_loot();
    
    void enemy_delete_projectile();
    
    // Getters
    s32 enemy_get_x() { return enemy_x; };
    s32 enemy_get_y() { return enemy_y; };
    s32 enemy_get_hitbox_width() { return enemy_hitbox_width; };
    s32 enemy_get_hitbox_height() { return enemy_hitbox_height; };
    s32 enemy_get_damage_hitbox_x() { return enemy_damage_hitbox_x; };
    s32 enemy_get_damage_hitbox_y() { return enemy_damage_hitbox_y; };
    s32 enemy_get_damage_hitbox_width() { return enemy_damage_hitbox_width; };
    s32 enemy_get_damage_hitbox_height() { return enemy_damage_hitbox_height; };
    u32 enemy_get_phase_metroid_flag() { return enemy_phase_metroid_flag; };
    s16 enemy_get_health() { return enemy_health; };
    u16 enemy_get_touch_damage() { return enemy_touch_damage; };
    u16 enemy_get_projectile_damage() { return enemy_projectile_damage; };
    u8 enemy_get_anim() { return enemy_anim; };
    u8 enemy_get_speed() { return enemy_speed; };
    u8 enemy_get_collision() { return enemy_collision; };
    u8 enemy_get_collision_facing();
    u8 enemy_get_facing() { return enemy_facing; };
    u8 enemy_get_size() { return enemy_size; };
    u8 enemy_get_frame() { return enemy_frame; };
    u8 enemy_get_physics() { return enemy_physics; };
    Projectile* enemy_get_projectile() { return pEnemyProjectile; };
    bool enemy_is_metroid() { return enemy_metroid_flag; };
    bool enemy_is_dead() { return enemy_dead; };
    bool enemy_is_frozen() { return enemy_frozen; };
    bool enemy_get_moving() { return enemy_moving; };
    
    u8 enemy_get_loot_chance() { return enemy_loot_chance; };
    std::string enemy_get_loot() { return enemy_loot; };
    s32 enemy_get_loot_spawn_x() { return enemy_loot_spawn_x; };
    s32 enemy_get_loot_spawn_y() { return enemy_loot_spawn_y; };
    
    std::string enemy_get_sprite_path() { return enemy_sprite_path; };
    s32 enemy_get_sprite_width() { return enemy_sprite_width; };
    s32 enemy_get_sprite_height() { return enemy_sprite_height; };
    u8 enemy_get_sprite_frame_speed() { return enemy_sprite_frame_speed; };
    u8 enemy_get_max_spriteline() { return enemy_max_spriteline; };
    
    // Setters
    void enemy_set_x(s32 x) { enemy_x = x; };
    void enemy_set_y(s32 y) { enemy_y = y; };
    void enemy_set_hitbox_widht(s32 width) { enemy_hitbox_width = width; };
    void enemy_set_hitbox_height(s32 height) { enemy_hitbox_height = height; };
    void enemy_set_size(u8 size) { enemy_size = size; };
    void enemy_set_speed(u8 speed) { enemy_speed = speed; };
    void enemy_set_state(u8 state) { enemy_state = state; };
    void enemy_set_anim(u8 anim) { enemy_anim = anim; };
    void enemy_set_collision(u8 collision) { enemy_collision = collision; }
    void enemy_set_facing(u8 facing) { enemy_facing = facing; };
    void enemy_set_frame(u8 frame) { enemy_frame = frame; };
    void enemy_set_damage_hitbox_x(s32 x) { enemy_damage_hitbox_x = x; };
    void enemy_set_damage_hitbox_y(s32 y) { enemy_damage_hitbox_y = y; };
    void enemy_set_damage_hitbox_width(s32 width) { enemy_damage_hitbox_width = width; };
    void enemy_set_damage_hitbox_height(s32 height) { enemy_damage_hitbox_height = height; };
    void enemy_set_health(s16 health) { enemy_health = health; };
    void enemy_set_touch_damage(u16 damage) { enemy_touch_damage = damage; };
    void enemy_set_projectile_damage(u16 damage) { enemy_projectile_damage = damage; };
    void enemy_set_phase_metroid_flag(u32 flag) { enemy_phase_metroid_flag = flag; };
    void enemy_set_moving(bool value) { enemy_moving = value; };
    void enemy_set_physics(u8 physics) { enemy_physics = physics; };
    void enemy_set_frozen(bool value) { enemy_frozen = value; };
    
private:
    void enemy_update_hitbox();
    
    EnemyAI* pAI;
    
    s32 enemy_x;
    s32 enemy_y;
    s32 enemy_hitbox_width;
    s32 enemy_hitbox_height;
    
    s32 enemy_damage_hitbox_x;
    s32 enemy_damage_hitbox_y;
    s32 enemy_damage_hitbox_width;
    s32 enemy_damage_hitbox_height;
    
    u32 enemy_phase_metroid_flag;
    
    s16 enemy_health;
    u16 enemy_touch_damage;
    u16 enemy_projectile_damage;
    
    u8 enemy_ai;
    u8 enemy_physics;
    u8 enemy_state;
    u8 enemy_anim;
    u8 enemy_speed;
    u8 enemy_facing;
    u8 enemy_collision;
    u8 enemy_invincibility_frames;
    u8 enemy_death_frames;
    u8 enemy_size;
    u8 enemy_frame;
    u8 enemy_frame_count;
    
    bool enemy_dead;
    bool enemy_metroid_flag;
    bool enemy_shadow;
    bool enemy_loot_rolled;
    bool enemy_moving;
    bool enemy_frozen;
    
    u8 enemy_loot_chance;
    std::string enemy_loot;
    s32 enemy_loot_spawn_x; // Addition to Object X
    s32 enemy_loot_spawn_y; // Addition to Object Y
    
    std::string enemy_sprite_path;
    s32 enemy_sprite_width;
    s32 enemy_sprite_height;
    u8 enemy_sprite_frame_speed;
    u8 enemy_max_spriteline;
    
    u8 enemy_sound_id;
    
    EnemyPhysics enemy_physics_affected;
    
    Projectile* pEnemyProjectile;
};

#endif /* enemy_h */
