//
//  enemy_ai_yumbo.cpp
//  3ds_metroid
//
//  Created by jbr373 on 04/03/16.
//

#include <stdio.h>

#include "enemy_ai.h"

#define MAX_YUMBO_Y_COUNTER                     60
#define MAX_YUMBO_X_MOVEMENT                    100

/*************************************************************************************************************************/
/*                                                    AI Script                                                          */
/*************************************************************************************************************************/

struct enemy_ai_yumbo : EnemyAI
{
    /****************************/
    /*          Values          */
    /****************************/
    
    u8 enemy_ai_yumbo_y_change_counter;
    
    s32 enemy_ai_yumbo_spawn_x;
    
    bool enemy_ai_yumbo_y_changed_up;
    bool enemy_ai_yumbo_movement_changed;
    
    /****************************/
    /*        Constructor       */
    /****************************/
    
    enemy_ai_yumbo() : EnemyAI()
    {
        // Set Standard Variables
        enemy_ai_set_paused(false);
        enemy_ai_set_movement(true);
        
        // Set Own Variables
        enemy_ai_yumbo_y_change_counter = 0;
        
        enemy_ai_yumbo_spawn_x = 0;
        
        enemy_ai_yumbo_y_changed_up = false;
        enemy_ai_yumbo_movement_changed = true;
    }
    
    /****************************/
    /*         Functions        */
    /****************************/
    
    // Change Y while flying, simulating issues with height control
    void enemy_ai_yumbo_change_y(s32 &enemy_y)
    {
        if (enemy_ai_yumbo_y_change_counter == MAX_YUMBO_Y_COUNTER)
        {
            // Change Y
            if (enemy_ai_yumbo_y_changed_up == true)
            {
                // Set Down Change
                enemy_ai_yumbo_y_changed_up = false;
                
                // Set Enemy Y
                enemy_y -= 2;
                
                // Reset Counter
                enemy_ai_yumbo_y_change_counter = 0;
            }
            else
            {
                // Set Up Change
                enemy_ai_yumbo_y_changed_up = true;
                
                // Set Enemy Y
                enemy_y += 2;
                
                // Reset Counter
                enemy_ai_yumbo_y_change_counter = 0;
            }
        }
        else
        {
            // Increase Counter
            ++enemy_ai_yumbo_y_change_counter;
        }
    }
    
    // Main
    void enemy_ai_update(Player* pPlayer, Projectile* pProjectile, s32 &enemy_x, s32 &enemy_y, s16 &enemy_health, u8 &enemy_speed, u8 &enemy_facing, u8 &enemy_collision, u8 &enemy_anim) override
    {
        // Check if the AI is set to paused
        if (enemy_ai_get_paused() == true)
            return;
        
        // Pause AI on Death
        if (enemy_health <= 0)
        {
            enemy_ai_set_paused(true);
        }
        
        // Set Y Change Effect
        enemy_ai_yumbo_change_y(enemy_y);
        
        // Set X Original
        if (enemy_ai_yumbo_spawn_x == 0)
        {
            enemy_ai_yumbo_spawn_x = enemy_x;
        }
        
        // Set X Movement
        if (enemy_x < enemy_ai_yumbo_spawn_x + MAX_YUMBO_X_MOVEMENT && enemy_ai_yumbo_movement_changed == false)
        {
            // Force Facing
            enemy_facing = FACING_RIGHT;
        }
        else if (enemy_x > enemy_ai_yumbo_spawn_x - MAX_YUMBO_X_MOVEMENT && enemy_ai_yumbo_movement_changed == true)
        {
            // Force Facing
            enemy_facing = FACING_LEFT;
        }
        
        // Change Movement Direction
        if (enemy_x <= enemy_ai_yumbo_spawn_x - MAX_YUMBO_X_MOVEMENT || enemy_collision & COLLISION_LEFT)
        {
            enemy_ai_yumbo_movement_changed = false;
        }
        else if (enemy_x >= enemy_ai_yumbo_spawn_x + MAX_YUMBO_X_MOVEMENT || enemy_collision & COLLISION_RIGHT)
        {
            enemy_ai_yumbo_movement_changed = true;
        }
    }
};

/*************************************************************************************************************************/
/*                                                     Register AI                                                       */
/*************************************************************************************************************************/

EnemyAI* load_enemy_ai_yumbo()
{
    // AI Result
    EnemyAI* pEnemyAI = NULL;
    
    // Allocate Yumbo AI Memory
    pEnemyAI = new enemy_ai_yumbo();
    
    // Return Allocation Result
    return pEnemyAI;
}