//
//  file_operations.cpp
//  3ds_metroid
//
//  Created by jbr373 on 05/02/16.
//

/*
 *
 * A big thank you to User Disch on cpluplus.com!
 * http://www.cplusplus.com/articles/DzywvCM9/
 *
 */

#include <sstream>
#include <string>
#include <string.h>
#include <fstream>
#include <new>
#include <iostream>

#include "file_operations.h"
#include "filepaths.h"

#define TILESET_TILE_DIMENSIONS         128

enum MinimapFileLines
{
    MINIMAPFILE_LINE_ID = 0,
    MINIMAPFILE_LINE_X = 1,
    MINIMAPFILE_LINE_Y = 2,
    MINIMAPFILE_LINE_W = 3,
    MINIMAPFILE_LINE_H = 4
};

enum StringFileLines
{
    STRINGFILE_LINE_X = 0,
    STRINGFILE_LINE_Y = 1,
    STRINGFILE_LINE_TEXT = 2
};

enum ConfigFileLines
{
    CONFIGFILE_LINE_MOVEMENT = 0,
    CONFIGFILE_LINE_SHOOTING = 1,
    CONFIGFILE_LINE_SOUND = 2
};

FileOperations::FileOperations()
{
    fileop_seekg_pos = 0;
    fileop_seekg_pos_c = 0;
}

FileOperations::~FileOperations()
{
}

/*************************************************************************************************************************/
/*                                                  Full Minimap File                                                    */
/*************************************************************************************************************************/

void FileOperations::fileop_load_minimap(MiniMapFileStruct* pMiniMapFileStruct, u8 stage_phase)
{
    // Load Object File
    std::stringstream filepath;
    filepath << FILEPATH_ROOM << (u32)stage_phase << EXTENSIONS_MINIMAP;
    std::string fullpath = filepath.str();

    // Read Map File
    std::ifstream map_minimapfile;
    map_minimapfile.open(fullpath);
    
    // Line Counter
    std::string line;
    u8 i = 0, j = 0;

    u8 roomdata_main = 0;
    bool minimap_load_failed = false;
    
    if (map_minimapfile.is_open())
    {
        // Read all Lines
        while (getline(map_minimapfile, line))
        {
            /* Skip Commented Line */

            // Check for # at the start of the line
            std::string first_line_char(1, line[0]);
            if (strcmp(first_line_char.c_str(), "#") == 0)
            {
                continue;
            }
            
            // Stop loading if array is full!
            if (j > MINIMAP_MAX_MAPS)
            {
                minimap_load_failed = true;
                break;
            }
            
            /* Load Minimap Room Info */
            
            // Get Rooms
            if (j == 0 && roomdata_main == 0)
            {
                pMiniMapFileStruct->minimapfile_rooms = (u8)atoi(line.c_str());
                ++roomdata_main;
            }
            else if (j == 0 && roomdata_main == 1)
            {
                pMiniMapFileStruct->minimapfile_x_max = (s32)atoi(line.c_str());
                ++roomdata_main;
            }
            else if (j == 0 && roomdata_main == 2)
            {
                pMiniMapFileStruct->minimapfile_y_max = (s32)atoi(line.c_str());
                ++roomdata_main;
            }
            else if (j + 1 <= pMiniMapFileStruct->minimapfile_rooms && roomdata_main > 2)
            {
                // Get Room Data
                switch (i)
                {
                    case MINIMAPFILE_LINE_ID:
                        pMiniMapFileStruct->minimapfile_room_id[j] = (u32)atoi(line.c_str());
                        break;
                    case MINIMAPFILE_LINE_X:
                        pMiniMapFileStruct->minimapfile_room_x[j] = (s32)atoi(line.c_str());
                        break;
                    case MINIMAPFILE_LINE_Y:
                        pMiniMapFileStruct->minimapfile_room_y[j] = (s32)atoi(line.c_str());
                        break;
                    case MINIMAPFILE_LINE_W:
                        pMiniMapFileStruct->minimapfile_room_w[j] = (s32)atoi(line.c_str());
                        break;
                    case MINIMAPFILE_LINE_H:
                        pMiniMapFileStruct->minimapfile_room_h[j] = (s32)atoi(line.c_str());
                        break;
                        
                    default:
                        minimap_load_failed = true;
                        break;
                }
                
                // Increment Line Counter
                if (i < MINIMAPFILE_LINE_H)
                {
                    ++i;
                }
                else
                {
                    i = 0;
                    ++j;
                }
            }
            
            // Exit loop on load failed
            if (minimap_load_failed == true)
            {
                break;
            }
        }
        
        // Check for Room Info
        if (j == pMiniMapFileStruct->minimapfile_rooms) // j + 1 is true at 2, so it becomes 3 when the getline loop ends
        {
            minimap_load_failed = false;
        }
        else
        {
            minimap_load_failed = true;
        }
        
        // Flag Struct
        if (minimap_load_failed == true)
        {
            pMiniMapFileStruct->minimapfile_loaded = false;
        }
        else
        {
            pMiniMapFileStruct->minimapfile_loaded = true;
        }
        
        // Close Minimap File
        map_minimapfile.close();
    }
    else // No Minimap File
        pMiniMapFileStruct->minimapfile_loaded = false;
}

/*************************************************************************************************************************/
/*                                                  Full String File                                                     */
/*************************************************************************************************************************/

void FileOperations::fileop_load_strings(StringFileStruct* pStringFileStruct, u8 text_id)
{
    // Load String File with Push Back
    
    // Check for allocated struct
    if (pStringFileStruct == NULL || text_id == 0)
    {
        // Delete Struct on error
        if (pStringFileStruct != NULL)
        {
            pStringFileStruct->stringfile_loaded = false;
        }
        
        return;
    }
    
    // Get String File Path
    std::stringstream filepath;
    filepath << FILEPATH_TEXT << (u32)text_id << EXTENSIONS_TEXT;
    std::string fullpath = filepath.str();
   
    // Read Map File
    std::ifstream stringfile;
    stringfile.open(fullpath);
    
    // Line Counter
    std::string line;
    bool string_load_failed = false;
    u8 i = 0, j = 0;
    s32 x = 0, y = 0;
    
    // Load data from file
    if (stringfile.is_open())
    {
        // Read all Lines
        while (getline(stringfile, line))
        {
            /* Skip Commented Line */
            
            // Check for # at the start of the line
            std::string first_line_char(1, line[0]);
            if (strcmp(first_line_char.c_str(), "#") == 0)
            {
                continue;
            }
            
            /* Write Data */
            
            // Write depending on i
            switch (i)
            {
                case STRINGFILE_LINE_X:
                    x = (s32)atoi(line.c_str());
                    break;
                case STRINGFILE_LINE_Y:
                    y = (s32)atoi(line.c_str());
                    break;
                case STRINGFILE_LINE_TEXT:
                    pStringFileStruct->text_x.push_back(x);
                    pStringFileStruct->text_y.push_back(y);
                    pStringFileStruct->text_string.push_back(line);
                    break;
                    
                default:
                    string_load_failed = true;
                    break;
            }
            
            // Increment Line Count
            if (i == STRINGFILE_LINE_TEXT)
            {
                i = 0;
                ++j;
            }
            else
            {
                ++i;
            }
        }
        
        // Write Text Count
        pStringFileStruct->text_count = pStringFileStruct->text_string.size();
        
        // Set loaded
        if (string_load_failed == false)
        {
            pStringFileStruct->stringfile_loaded = true;
        }
        else
        {
            pStringFileStruct->stringfile_loaded = false;
        }
        
        // Close String File
        stringfile.close();
    }
    else
    {
        // Delete Struct on error
        if (pStringFileStruct != NULL)
        {
            pStringFileStruct->stringfile_loaded = false;
        }
    }
}

/*************************************************************************************************************************/
/*                                                  Full Config File                                                     */
/*************************************************************************************************************************/

inline std::string fileop_convert_boolean_text(bool boolean)
{
    return boolean ? "1" : "0";
}

ConfigFileStruct* FileOperations::fileop_load_config()
{
    ConfigFileStruct* pConfigFileStruct = new ConfigFileStruct();
    
    // Read Map File
    std::ifstream configfile;
    configfile.open(FILE_CONFIG);
    
    // Line Counter
    std::string line;
    u8 i = 0;
    
    // Load data from file
    if (configfile.is_open())
    {
        // Read all Lines
        while (getline(configfile, line))
        {
            if (i == CONFIGFILE_LINE_MOVEMENT)
            {
                pConfigFileStruct->config_movement_changed = (u8)atoi(line.c_str());
            }
            else if (i == CONFIGFILE_LINE_SHOOTING)
            {
                pConfigFileStruct->config_shooting_changed = (u8)atoi(line.c_str());
            }
            else if (i == CONFIGFILE_LINE_SOUND)
            {
                pConfigFileStruct->config_map = (u8)atoi(line.c_str());
            }
            
            // Increment Line Count
            ++i;
        }
        
        // Close Config File
        configfile.close();
    }
    else
    {
        // Reset Values
        pConfigFileStruct->config_movement_changed = false;
        pConfigFileStruct->config_shooting_changed = false;
        pConfigFileStruct->config_map = true;
        
        // Create new Config File
        std::ofstream configfile_new;
        configfile_new.open(FILE_CONFIG);
        
        if (configfile_new.is_open())
        {
            // Write Values
            configfile_new << fileop_convert_boolean_text(pConfigFileStruct->config_movement_changed) << std::endl;
            configfile_new << fileop_convert_boolean_text(pConfigFileStruct->config_shooting_changed) << std::endl;
            configfile_new << fileop_convert_boolean_text(pConfigFileStruct->config_map) << std::endl;
            
            // Close File
            configfile_new.close();
        }
        else
        {
            return NULL;
        }
    }
    
    return pConfigFileStruct;
}

void FileOperations::fileop_save_config(ConfigFileStruct* pConfigFileStruct)
{
    if (pConfigFileStruct != NULL)
    {
        // Save Config File
        std::ofstream configfile;
        configfile.open(FILE_CONFIG);
        
        if (configfile.is_open())
        {
            // Write Values
            configfile << fileop_convert_boolean_text(pConfigFileStruct->config_movement_changed) << std::endl;
            configfile << fileop_convert_boolean_text(pConfigFileStruct->config_shooting_changed) << std::endl;
            configfile << fileop_convert_boolean_text(pConfigFileStruct->config_map) << std::endl;
            
            // Close File
            configfile.close();
        }
    }
}

/*************************************************************************************************************************/
/*                                                  Full Enemy File                                                      */
/*************************************************************************************************************************/

EnemyFileStruct* FileOperations::fileop_load_enemy(std::string enemy_file)
{
    // Define Pointer
    EnemyFileStruct* pEnemyFileStruct = new EnemyFileStruct();
    
    // Read Item File
    std::ifstream enemyfile;
    enemyfile.open(enemy_file, std::ios::in | std::ios::binary);
    
    // Read Save File
    if (enemyfile.is_open())
    {
        // File Pointer Position
        fileop_seekg_resetpos();
        
        // Read Magic & Version
        pEnemyFileStruct->enemy_magic = fileop_read_u32(enemyfile);
        fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_magic));
        
        pEnemyFileStruct->enemy_version = fileop_read_u32(enemyfile);
        fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_version));
        
        // Stop loading on wrong Magic or Version
        if (pEnemyFileStruct->enemy_magic != MAGIC_DECIMAL_ENEMY || pEnemyFileStruct->enemy_version != VERSION_DECIMAL_ENEMY)
        {
            // Close Save File
            enemyfile.close();
            
            // Delete Enemy File
            delete pEnemyFileStruct;
            
            // Move Enemy out of Room
            return NULL;
        }
        else
        {
            /* Get Enemy AI and Physics */
            
            // Get AI
            pEnemyFileStruct->enemy_ai = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_ai));
            
            // Get Physics
            pEnemyFileStruct->enemy_physics = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_physics));
            
            /* Get Enemy Metroid Flag */
            
            // Get Flag
            pEnemyFileStruct->enemy_is_metroid = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_is_metroid));
            
            // Get ID
            pEnemyFileStruct->enemy_metroid_flag = fileop_read_u32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_metroid_flag));
            
            /* Get Enemy Stats */
            
            // Get Health
            pEnemyFileStruct->enemy_health = fileop_read_s16(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_health));
            
            // Get Touching Damage
            pEnemyFileStruct->enemy_touch_damage = fileop_read_u16(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_touch_damage));
            
            // Get Projectile Damage
            pEnemyFileStruct->enemy_projectile_damage = fileop_read_u16(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_projectile_damage));
            
            // Get Speed
            pEnemyFileStruct->enemy_speed = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_speed));
            
            /* Get Enemy Loot Info */
            
            // Get String Length
            pEnemyFileStruct->enemy_loot_filepath_length = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_loot_filepath_length));
            
            // Write each char of length to string
            for (u8 i = 0; i <= pEnemyFileStruct->enemy_loot_filepath_length; ++i)
            {
                // Get Char and set Pointer
                char c = (char)fileop_read_u8(enemyfile);
                fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_loot_filepath_length));
                
                // Add char to string
                pEnemyFileStruct->enemy_loot += c;
            }
            
            // Get Object Loot Chance
            pEnemyFileStruct->enemy_loot_chance = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_loot_chance));
            
            // Get Spawn X Addition
            pEnemyFileStruct->enemy_loot_spawn_x = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_loot_spawn_x));
            
            // Get Spawn Y Addition
            pEnemyFileStruct->enemy_loot_spawn_y = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_loot_spawn_y));
            
            /* Get Enemy Dimensions */
            
            // Get Hitbox Width
            pEnemyFileStruct->enemy_hitbox_width = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_hitbox_width));
            
            // Get Hitbox Height
            pEnemyFileStruct->enemy_hitbox_height = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_hitbox_height));
            
            // Get Size
            pEnemyFileStruct->enemy_size = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_size));
            
            // Get Damage Hitbox X Addition
            pEnemyFileStruct->enemy_damage_hitbox_x = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_damage_hitbox_x));
            
            // Get Damage Hitbox Y Addition
            pEnemyFileStruct->enemy_damage_hitbox_y = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_damage_hitbox_y));
            
            // Get Damage Hitbox Width
            pEnemyFileStruct->enemy_damage_hitbox_width = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_damage_hitbox_width));
            
            // Get Damage Hitbox Height
            pEnemyFileStruct->enemy_damage_hitbox_height = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_damage_hitbox_height));
            
            /* Get Enemy Image Info */
            
            // Get String Length
            pEnemyFileStruct->enemy_spritesheet_filepath_length = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_spritesheet_filepath_length));
            
            // Write each char of length to string
            for (u8 i = 0; i <= pEnemyFileStruct->enemy_spritesheet_filepath_length; ++i)
            {
                // Get Char and set Pointer
                char c = (char)fileop_read_u8(enemyfile);
                fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_spritesheet_filepath_length));
                
                // Add char to string
                pEnemyFileStruct->enemy_spritesheet += c;
            }
            
            // Get Spritesheet Width
            pEnemyFileStruct->enemy_spritesheet_width = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_spritesheet_width));
            
            // Get Spritesheet Height
            pEnemyFileStruct->enemy_spritesheet_height = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_spritesheet_height));
            
            // Get Maximum Sprite Length
            pEnemyFileStruct->enemy_max_spritelines = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_max_spritelines));
            
            // Get Maximum Sprite Animation Speed
            pEnemyFileStruct->enemy_max_frame_speed = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_max_frame_speed));
            
            /* Get Enemy Shadow */
            
            // Get Shadow Flag
            pEnemyFileStruct->enemy_shadow = fileop_read_s32(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_shadow));
            
            /* Get Enemy Sound */
            
            // Get Sound ID
            pEnemyFileStruct->enemy_sound_id = fileop_read_u8(enemyfile);
            fileop_seekg_file(enemyfile, sizeof(pEnemyFileStruct->enemy_sound_id));
            
            // Close Object File
            enemyfile.close();
        }
        
        // File Pointer Reset
        fileop_seekg_resetpos();
    }
    
    // Return Result
    return pEnemyFileStruct;
}

/*************************************************************************************************************************/
/*                                                 Full Item File                                                        */
/*************************************************************************************************************************/

ItemFileStruct* FileOperations::fileop_load_item(std::string item_file)
{
    return NULL;
}

/*************************************************************************************************************************/
/*                                                  Full Object File                                                     */
/*************************************************************************************************************************/

ObjectFileStruct* FileOperations::fileop_load_object(std::string object_file)
{
    // Create File Struct

    ObjectFileStruct* pObjectFileStruct = new ObjectFileStruct();
    
    /* Load Object File */

    // Open Object File
    std::ifstream objectfile;
    objectfile.open(object_file, std::ios::in | std::ios::binary);
    
    // Read Object File
    if (objectfile.is_open())
    {
        // File Pointer Position
        fileop_seekg_resetpos();
        
        // Read Magic & Version
        pObjectFileStruct->object_magic = fileop_read_u32(objectfile);
        fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_magic));
        
        pObjectFileStruct->object_version = fileop_read_u32(objectfile);
        fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_version));
        
        // Create New File if Magic or Version aren't matching
        if (pObjectFileStruct->object_magic != MAGIC_DECIMAL_OBJECT || pObjectFileStruct->object_version != VERSION_DECIMAL_OBJECT)
        {
            // Close Save File
            objectfile.close();
            
            // Delete Object File
            delete pObjectFileStruct;
            
            // Return Failure
            return NULL;
        }
        else
        {
            /* Get Object Type and Phase */
            
            // Get Type
            pObjectFileStruct->object_type = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_type));
            
            // Get Phase
            pObjectFileStruct->object_phase = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_phase));
            
            /* Get Object Type Values */
            
            // Get Value A
            pObjectFileStruct->object_value_a = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_value_a));
            
            // Get Value B
            pObjectFileStruct->object_value_b = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_value_b));
            
            // Get Value C
            pObjectFileStruct->object_value_c = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_value_c));
            
            /* Get Object Loot Info */
            
            // Get String Length
            pObjectFileStruct->object_loot_filepath_length = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_loot_filepath_length));
            
            // Write each char of length to string
            for (u8 i = 0; i <= pObjectFileStruct->object_loot_filepath_length; ++i)
            {
                // Get Char and set Pointer
                char c = (char)fileop_read_u8(objectfile);
                fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_loot_filepath_length));
                
                // Add char to string
                pObjectFileStruct->object_loot += c;
            }
            
            // Get Object Loot Chance
            pObjectFileStruct->object_loot_chance = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_loot_chance));
            
            // Get Spawn X Addition
            pObjectFileStruct->object_loot_spawn_x = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_loot_spawn_x));
            
            // Get Spawn Y Addition
            pObjectFileStruct->object_loot_spawn_y = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_loot_spawn_y));
            
            /* Get Object Dimensions */
            
            // Get Hitbox Width
            pObjectFileStruct->object_hitbox_width = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_hitbox_width));
            
            // Get Hitbox Height
            pObjectFileStruct->object_hitbox_height = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_hitbox_height));
            
            // Get Object Size
            pObjectFileStruct->object_size = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_size));
            
            /* Get Object Image Info */
            
            // Get String Length
            pObjectFileStruct->object_spritesheet_filepath_length = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_spritesheet_filepath_length));
            
            // Write each char of length to string
            for (u8 i = 0; i <= pObjectFileStruct->object_spritesheet_filepath_length; ++i)
            {
                // Get Char and set Pointer
                char c = (char)fileop_read_u8(objectfile);
                fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_spritesheet_filepath_length));
                
                // Add char to string
                pObjectFileStruct->object_spritesheet += c;
            }
            
            // Get Spritesheet Width
            pObjectFileStruct->object_spritesheet_width = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_spritesheet_width));
            
            // Get Spritesheet Height
            pObjectFileStruct->object_spritesheet_height = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_spritesheet_height));
            
            // Get Maximum Sprite Length
            pObjectFileStruct->object_max_spritelines = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_max_spritelines));
            
            // Get Maximum Sprite Animation Speed
            pObjectFileStruct->object_max_frame_speed = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_max_frame_speed));
            
            /* Get Object Light */
            
            // Get Shadow Value
            pObjectFileStruct->object_shadow = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_shadow));
            
            // Get Light Color
            pObjectFileStruct->object_light_color = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_light_color));
            
            // Get Light Size
            pObjectFileStruct->object_light_size = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_light_size));
            
            // Get Light X Addition
            pObjectFileStruct->object_light_spawn_x = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_light_spawn_x));
            
            // Get Light Y Addition
            pObjectFileStruct->object_light_spawn_y = fileop_read_s32(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_light_spawn_y));
            
            /* Get Object Collision */
            
            // Get Collision Value
            pObjectFileStruct->object_collision = fileop_read_u8(objectfile);
            fileop_seekg_file(objectfile, sizeof(pObjectFileStruct->object_collision));
            
            // Close Object File
            objectfile.close();
        }
        
        // File Pointer Reset
        fileop_seekg_resetpos();
    }
    
    // Return File Struct
    return pObjectFileStruct;
}

/*************************************************************************************************************************/
/*                                                  Full Map File                                                        */
/*************************************************************************************************************************/

MapFileStruct* FileOperations::fileop_load_stage(u16 stage_id)
{
    // Define Map File
    MapFileStruct* pMapFileStruct = new MapFileStruct();
    
    // Load Map File
    std::stringstream filepath;
    filepath << FILEPATH_ROOM << stage_id << EXTENSIONS_MAP;
    std::string fullpath = filepath.str();
    
    // Open Map File
    std::ifstream mapfile;
    mapfile.open(fullpath, std::ios::in | std::ios::binary);
    
    // Read Save File
    if (mapfile.is_open())
    {
        // File Pointer Position
        fileop_seekg_resetpos();
        
        // Read Magic & Version
        pMapFileStruct->stage_magic = fileop_read_u32(mapfile);
        fileop_seekg_file(mapfile, sizeof(u32));
        
        pMapFileStruct->stage_version = fileop_read_u32(mapfile);
        fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_version));
        
        // Stop loading on wrong Magic or Version
        if (pMapFileStruct->stage_magic != MAGIC_DECIMAL_MAP || pMapFileStruct->stage_version != VERSION_DECIMAL_MAP)
        {
            // Close Save File
            mapfile.close();
            
            // Delete Map File
            delete pMapFileStruct;
            
            // Move Enemy out of Room
            return NULL;
        }
        else
        {
            /* Get Stage General Data */
            
            // Get Stage ID
            pMapFileStruct->stage_id = fileop_read_u16(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_id));
            
            // Get Stage Phase
            pMapFileStruct->stage_phase = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_phase));
            
            // Get Stage Width
            pMapFileStruct->stage_width = fileop_read_s32(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_width));
            
            // Get Stage Height
            pMapFileStruct->stage_height = fileop_read_s32(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_height));
            
            /* Get Stage Actor Count */
            
            // Get Stage Enemies
            pMapFileStruct->stage_enemies = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_enemies));
            
            // Correct Stage Enemies if needed
            if (pMapFileStruct->stage_enemies > STAGE_MAX_ENEMIES)
            {
                pMapFileStruct->stage_enemies = STAGE_MAX_ENEMIES;
            }
            
            // Get Stage Objects
            pMapFileStruct->stage_objects = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_objects));
            
            // Correct Stage Objects if needed
            if (pMapFileStruct->stage_objects > STAGE_MAX_OBJECTS)
            {
                pMapFileStruct->stage_objects = STAGE_MAX_OBJECTS;
            }
            
            // Get Stage Items
            pMapFileStruct->stage_items = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_items));
            
            // Correct Stage Enemies if needed
            if (pMapFileStruct->stage_items > STAGE_MAX_ITEMS)
            {
                pMapFileStruct->stage_items = STAGE_MAX_ITEMS;
            }
            
            /* Get Stage Minimap */
            
            // Get Stage Minimap ID
            pMapFileStruct->stage_minimap_id = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_minimap_id));
            
            /* Get Stage Liquid Type */
            
            // Get Stage Liquid ID
            pMapFileStruct->stage_liquid_type = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_liquid_type));
            
            /* Get Stage Tileset Info */
            
            // Get Background Path Length
            pMapFileStruct->stage_background_path_length = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_background_path_length));
            
            // Get Background Path
            
            // Write each char of length to string
            for (u8 i = 0; i <= pMapFileStruct->stage_background_path_length; ++i)
            {
                // Get Char and set Pointer
                char c = (char)fileop_read_u8(mapfile);
                fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_background_path_length));
                
                // Add char to string
                pMapFileStruct->stage_background_path += c;
            }
            
            // Get Tileset Path Length
            pMapFileStruct->stage_tileset_path_length = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_tileset_path_length));
            
            // Get Tileset Path
            
            // Write each char of length to string
            for (u8 i = 0; i <= pMapFileStruct->stage_tileset_path_length; ++i)
            {
                // Get Char and set Pointer
                char c = (char)fileop_read_u8(mapfile);
                fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_tileset_path_length));
                
                // Add char to string
                pMapFileStruct->stage_tileset_path += c;
            }
            
            // Get Tileset Foreground Tile Allignment
            
            // Create Tileset Struct
            pMapFileStruct->stage_tileset_layer_foreground = new u8[(pMapFileStruct->stage_width / TILESET_TILE_DIMENSIONS) *
                                                                    (pMapFileStruct->stage_height / TILESET_TILE_DIMENSIONS)];
            
            // Read Tileset Data
            for (u8 i = 0; i < (pMapFileStruct->stage_width / TILESET_TILE_DIMENSIONS) * (pMapFileStruct->stage_height / TILESET_TILE_DIMENSIONS); ++i)
            {
                pMapFileStruct->stage_tileset_layer_foreground[i] = fileop_read_u8(mapfile);
                fileop_seekg_file(mapfile, sizeof(u8));
            }
            
            // Get Tileset Background Layer Flag
            pMapFileStruct->stage_draw_tileset_behind_player = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_draw_tileset_behind_player));
            
            // Get Tileset Background Tile Allignment
            
            // Create Tileset Struct
            pMapFileStruct->stage_tileset_layer_background = new u8[(pMapFileStruct->stage_width / TILESET_TILE_DIMENSIONS) *
                                                                    (pMapFileStruct->stage_height / TILESET_TILE_DIMENSIONS)];
            
            // Read Tileset Data
            for (u8 i = 0; i < (pMapFileStruct->stage_width / TILESET_TILE_DIMENSIONS) * (pMapFileStruct->stage_height / TILESET_TILE_DIMENSIONS); ++i)
            {
                pMapFileStruct->stage_tileset_layer_background[i] = fileop_read_u8(mapfile);
                fileop_seekg_file(mapfile, sizeof(u8));
            }
            
            /* Get Stage Collision Info */
            
            // Get Collision Path Length
            pMapFileStruct->stage_collision_path_length = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_collision_path_length));
            
            // Get Collision Path
            
            // Write each char of length to string
            for (u8 i = 0; i <= pMapFileStruct->stage_collision_path_length; ++i)
            {
                // Get Char and set Pointer
                char c = (char)fileop_read_u8(mapfile);
                fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_collision_path_length));
                
                // Add char to string
                pMapFileStruct->stage_collision_path += c;
            }
            
            /* Get Stage Sound Info */
            
            // Get Collision Path Length
            pMapFileStruct->stage_sound_path_length = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_sound_path_length));
            
            // Get Sound Path
            
            // Write each char of length to string
            for (u8 i = 0; i <= pMapFileStruct->stage_sound_path_length; ++i)
            {
                // Get Char and set Pointer
                char c = (char)fileop_read_u8(mapfile);
                fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_sound_path_length));
                
                // Add char to string
                pMapFileStruct->stage_sound_path += c;
            }
            
            /* Get Stage Light */
            
            // Get Light Color
            pMapFileStruct->stage_light_ambient = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_light_ambient));
            
            /* Get Stage Water Info */
            
            // Get Water X
            pMapFileStruct->stage_liquid_x = fileop_read_s32(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_liquid_x));
            
            // Get Water Y
            pMapFileStruct->stage_liquid_y = fileop_read_s32(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_liquid_y));
            
            // Get Water W
            pMapFileStruct->stage_liquid_w = fileop_read_s32(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_liquid_w));
            
            // Get Water H
            pMapFileStruct->stage_liquid_h = fileop_read_s32(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_liquid_h));
            
            // Get Water Alpha
            pMapFileStruct->stage_liquid_alpha = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_liquid_alpha));
            
            // Get Water Ripples Height
            pMapFileStruct->stage_liquid_ripples_intensity = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_liquid_ripples_intensity));
            
            // Get Water Godray Flag
            pMapFileStruct->stage_liquid_godray = fileop_read_u8(mapfile);
            fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_liquid_godray));
            
            /* Get Stage Actors */
            
            std::string path = "";
            
            /* Get Stage Enemies */
            
            // Do for all Enemies
            for (u8 i = 0; i < pMapFileStruct->stage_enemies; ++i)
            {
                // Get Enemy File Path Length
                pMapFileStruct->stage_enemy_file_path_length[i] = fileop_read_u8(mapfile);
                fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_light_ambient));
                
                // Get Enemy File
                
                // Write each char of length to string
                for (u8 j = 0; j <= pMapFileStruct->stage_enemy_file_path_length[i]; ++j)
                {
                    // Get Char and set Pointer
                    char c = (char)fileop_read_u8(mapfile);
                    fileop_seekg_file(mapfile, sizeof(pMapFileStruct->stage_sound_path_length));
                    
                    // Add char to string
                    path += c;
                }
                
                // Write File Path and reset temporary Path
                pMapFileStruct->stage_enemy_file_path[i] = path;
                path = "";
                
                // Get Enemy X
                pMapFileStruct->stage_enemy_x[i] = fileop_read_s32(mapfile);
                fileop_seekg_file(mapfile, sizeof(s32));
                
                // Get Enemy Y
                pMapFileStruct->stage_enemy_y[i] = fileop_read_s32(mapfile);
                fileop_seekg_file(mapfile, sizeof(s32));
            }
            
            /* Get Stage Objects */
            
            // Do for all Objects
            for (u8 i = 0; i < pMapFileStruct->stage_objects; ++i)
            {
                // Get Object File Path Length
                pMapFileStruct->stage_object_file_path_length[i] = fileop_read_u8(mapfile);
                fileop_seekg_file(mapfile, sizeof(u8));
                
                // Get Object File
                
                // Write each char of length to string
                for (u8 j = 0; j <= pMapFileStruct->stage_object_file_path_length[i]; ++j)
                {
                    // Get Char and set Pointer
                    char c = (char)fileop_read_u8(mapfile);
                    fileop_seekg_file(mapfile, sizeof(u8));
                    
                    // Add char to string
                    path += c;
                }
                
                // Write File Path and reset temporary Path
                pMapFileStruct->stage_object_file_path[i] = path;
                path = "";
                
                // Get Object X
                pMapFileStruct->stage_object_x[i] = fileop_read_s32(mapfile);
                fileop_seekg_file(mapfile, sizeof(s32));
                
                // Get Object Y
                pMapFileStruct->stage_object_y[i] = fileop_read_s32(mapfile);
                fileop_seekg_file(mapfile, sizeof(s32));
            }
            
            /* Get Stage Items */
            
            // Do for all Items
            for (u8 i = 0; i < pMapFileStruct->stage_items; ++i)
            {
                // Get Item File Path Length
                pMapFileStruct->stage_item_file_path_length[i] = fileop_read_u8(mapfile);
                fileop_seekg_file(mapfile, sizeof(u8));
                
                // Get Item File
                
                // Write each char of length to string
                for (u8 j = 0; j <= pMapFileStruct->stage_item_file_path_length[i]; ++j)
                {
                    // Get Char and set Pointer
                    char c = (char)fileop_read_u8(mapfile);
                    fileop_seekg_file(mapfile, sizeof(u8));
                    
                    // Add char to string
                    path += c;
                }
                
                // Write File Path and reset temporary Path
                pMapFileStruct->stage_item_file_path[i] = path;
                path = "";
                
                // Get Item X
                pMapFileStruct->stage_item_x[i] = fileop_read_s32(mapfile);
                fileop_seekg_file(mapfile, sizeof(s32));
                
                // Get Item Y
                pMapFileStruct->stage_item_y[i] = fileop_read_s32(mapfile);
                fileop_seekg_file(mapfile, sizeof(s32));
            }
        }
        
        // Close Map File
        mapfile.close();
        
        // File Pointer Reset
        fileop_seekg_resetpos();
    }
    
    // Return Pointer
    return pMapFileStruct;
}

/*************************************************************************************************************************/
/*                                                  SDMC Check                                                           */
/*************************************************************************************************************************/

bool FileOperations::fileop_file_exists(const char* file)
{
    std::ifstream fileop_file;
    fileop_file.open(file);
    
    // Read Save File
    if (fileop_file.is_open())
    {
        fileop_file.close();
        return true;
    }
    
    return false;
}

bool FileOperations::fileop_file_delete(const char* file)
{
    std::ifstream fileop_file;
    fileop_file.open(file);
    
    // Read Save File
    if (fileop_file.is_open())
    {
        fileop_file.close();
        
        std::remove(file);
        
        return true;
    }
    
    return false;
}

/*************************************************************************************************************************/
/*                                                  Read Binary (C++)                                                    */
/*************************************************************************************************************************/

void FileOperations::fileop_seekg_file(std::ifstream& file, u32 pos_addition)
{
    fileop_seekg_pos += pos_addition;
    file.seekg(fileop_seekg_pos, std::ios::beg);
}

void FileOperations::fileop_seekg_resetpos()
{
    fileop_seekg_pos = 0;
}

void FileOperations::fileop_write_u32(std::ofstream& file, u32 value)
{
    unsigned char bytes[4]; // 4 Bytes = 32 Bit
    
    // extract the individual bytes from our value
    bytes[0] = (value) & 0xFF;  // low byte
    bytes[1] = (value >> 8) & 0xFF;  // high byte
    bytes[2] = (value >> 16) & 0xFF;  // higher byte
    bytes[3] = (value >> 24) & 0xFF;  // highest byte
    
    // write those bytes to the file
    file.write((char*)bytes, 4); // 8 Bit (1 Byte) PTR, size = 4 Bytes
}

void FileOperations::fileop_write_u16(std::ofstream& file, u16 value)
{
    unsigned char bytes[2];
    
    bytes[0] = (value) & 0xFF;
    bytes[1] = (value >> 8) & 0xFF;
    
    file.write((char*)bytes, 2);
}

void FileOperations::fileop_write_u8(std::ofstream& file, u8 value)
{
    unsigned char bytes[1];
    
    bytes[0] = (value) & 0xFF;
    
    file.write((char*)bytes, 1);
}

void FileOperations::fileop_write_s32(std::ofstream& file, s32 value)
{
    signed char bytes[4];
    
    bytes[0] = (value) & 0xFF;
    bytes[1] = (value >> 8) & 0xFF;
    bytes[2] = (value >> 16) & 0xFF;
    bytes[3] = (value >> 24) & 0xFF;
    
    file.write((char*)bytes, 4);
}

void FileOperations::fileop_write_s16(std::ofstream& file, s16 value)
{
    signed char bytes[2];
    
    bytes[0] = (value) & 0xFF;
    bytes[1] = (value >> 8) & 0xFF;
    
    file.write((char*)bytes, 2);
}

void FileOperations::fileop_write_s8(std::ofstream& file, s8 value)
{
    signed char bytes[1];
    
    bytes[0] = (value) & 0xFF;
    
    file.write((char*)bytes, 1);
}

// Read (Binary)

u32 FileOperations::fileop_read_u32(std::ifstream& file)
{
    u32 val;
    unsigned char bytes[4];
    
    file.read( (char*)bytes, 4 );  // read 4 bytes from the file
    val = bytes[0] | (bytes[1] << 8) | (bytes[2] << 16) | (bytes[3] << 24);  // construct the 16-bit value from those bytes
  
    return val;
}

u16 FileOperations::fileop_read_u16(std::ifstream& file)
{
    u16 val;
    unsigned char bytes[2];
    
    file.read( (char*)bytes, 2);
    val = bytes[0] | (bytes[1] << 8);
    
    return val;
}

u8 FileOperations::fileop_read_u8(std::ifstream& file)
{
    u8 val;
    unsigned char bytes[1];
    
    file.read( (char*)bytes, 1);
    val = bytes[0];
    
    return val;
}

s32 FileOperations::fileop_read_s32(std::ifstream& file)
{
    s32 val;
    unsigned char bytes[4];
    
    file.read( (char*)bytes, 4);
    val = bytes[0] | (bytes[1] << 8) | (bytes[2] << 16) | (bytes[3] << 24);
    
    return val;
}

s16 FileOperations::fileop_read_s16(std::ifstream& file)
{
    s16 val;
    unsigned char bytes[2];
    
    file.read( (char*)bytes, 2);
    val = bytes[0] | (bytes[1] << 8);
    
    return val;
}

s8 FileOperations::fileop_read_s8(std::ifstream& file)
{
    s8 val;
    unsigned char bytes[1];
    
    file.read( (char*)bytes, 1);
    val = bytes[0];
    
    return val;
}

/*************************************************************************************************************************/
/*                                                  Read Text (C++)                                                      */
/*************************************************************************************************************************/

std::string FileOperations::fileop_read_string_text(StringFileStruct* pStringFileStruct, u8 pos)
{
    std::string text = "TEXT NOT FOUND";
    
    if (pStringFileStruct != NULL)
    {
        if (pStringFileStruct->text_count - 1 >= pos && pStringFileStruct->stringfile_loaded == true)
        {
            text = pStringFileStruct->text_string[pos];
        }
    }
    
    return text;
}

s32 FileOperations::fileop_read_string_x(StringFileStruct* pStringFileStruct, u8 pos)
{
    s32 x = 0;

    if (pStringFileStruct != NULL)
    {
        if (pStringFileStruct->text_count - 1 >= pos && pStringFileStruct->stringfile_loaded == true)
        {
            x = pStringFileStruct->text_x[pos];
        }
    }
    
    return x;
}

s32 FileOperations::fileop_read_string_y(StringFileStruct* pStringFileStruct, u8 pos)
{
    s32 y = 0;
    
    if (pStringFileStruct != NULL)
    {
        if (pStringFileStruct->text_count - 1 >= pos && pStringFileStruct->stringfile_loaded == true)
        {
            y = pStringFileStruct->text_y[pos];
        }
    }
    
    return y;
}

/*************************************************************************************************************************/
/*                                                  Read Binary (C)                                                      */
/*************************************************************************************************************************/

// Set File Pointer

void FileOperations::fileop_seekg_resetpos_c()
{
    // Reset File Pointer
    fileop_seekg_pos_c = 0;
}

// Read (Binary) (C)
u32 FileOperations::fileop_read_u32(FILE* fp)
{
    // Define Variable
    u32 val;
    
    // Our Bytes
    unsigned char bytes[4];
    
    // Read each Byte
    for (u8 i = 0; i < 4; ++i)
    {
        fread(&bytes[i], 1, 1, fp);
        //++fileop_seekg_pos_c;
        //fseek(fp, fileop_seekg_pos_c, SEEK_SET);
    }
    
    // Combine into u32 Value
    val = bytes[0] | (bytes[1] << 8) | (bytes[2] << 16) | (bytes[3] << 24);  // construct the 16-bit value from those bytes
    
    // Return Value
    return val;
}

u16 FileOperations::fileop_read_u16(FILE* fp)
{
    // Define Variable
    u16 val;
    
    // Our Bytes
    unsigned char bytes[2];
    
    // Read each Byte
    for (u8 i = 0; i < 2; ++i)
    {
        fread(&bytes[i], 1, 1, fp);
        //++fileop_seekg_pos_c;
        //fseek(fp, fileop_seekg_pos_c, SEEK_SET);
    }
    
    // Combine into u16 Value
    val = bytes[0] | (bytes[1] << 8);
    
    // Return Value
    return val;
}

u8 FileOperations::fileop_read_u8(FILE* fp)
{
    // Define Variables
    u8 val;
    
    // Define Bytes
    unsigned char bytes[1];
    
    // Read File
    fread(&bytes[0], 1, 1, fp);
    //++fileop_seekg_pos_c;
    //fseek(fp, fileop_seekg_pos_c, SEEK_SET);
    
    // Write Value
    val = bytes[0];
    
    // Return Value
    return val;
}

s32 FileOperations::fileop_read_s32(FILE* fp)
{
    // Define Variable
    s32 val;
    
    // Our Bytes
    unsigned char bytes[4];
    
    // Read each Byte
    for (u8 i = 0; i < 4; ++i)
    {
        fread(&bytes[i], 1, 1, fp);
        //++fileop_seekg_pos_c;
        //fseek(fp, fileop_seekg_pos_c, SEEK_SET);
    }
    
    // Combine into s32 Value
    val = bytes[0] | (bytes[1] << 8) | (bytes[2] << 16) | (bytes[3] << 24);  // construct the 16-bit value from those bytes
    
    // Return Value
    return val;
}

s16 FileOperations::fileop_read_s16(FILE* fp)
{
    // Define Variable
    s16 val;
    
    // Our Bytes
    unsigned char bytes[2];
    
    // Read each Byte
    for (u8 i = 0; i < 2; ++i)
    {
        fread(&bytes[i], 1, 1, fp);
        //++fileop_seekg_pos_c;
        //fseek(fp, fileop_seekg_pos_c, SEEK_SET);
    }
    
    // Combine into s16 Value
    val = bytes[0] | (bytes[1] << 8);
    
    // Return Value
    return val;
}

s8 FileOperations::fileop_read_s8(FILE* fp)
{
    // Define Variables
    s8 val;
    
    // Define Bytes
    unsigned char bytes[1];
    
    // Read File
    fread(&bytes[0], 1, 1, fp);
    //++fileop_seekg_pos_c;
    //fseek(fp, fileop_seekg_pos_c, SEEK_SET);
    
    // Write Value
    val = bytes[0];
    
    // Return Value
    return val;
}

std::string FileOperations::fileop_read_cstring(FILE* fp, u8 string_length)
{
    // Define String
    char tmp[string_length];
    
    // Copy String from File
    fread(tmp, 1, (string_length - 1), fp);
    tmp[string_length] = '\0';
    
    // Copy to Pointer
    printf("\x1b[22;0H [  TMP %s  ]", tmp);
    
    // Set Position
    //fileop_seekg_pos_c += (string_length - 1);
    //fseek(fp, fileop_seekg_pos_c, SEEK_SET);
    
    std::string string(tmp);
    printf("\x1b[23;0H [  S %s  ]", string.c_str());
    
    // Return Result
    return string;
}
