//
//  file_operations.hpp
//  3ds_metroid
//
//  Created by jbr373 on 05/02/16.
//

#ifndef file_operations_h
#define file_operations_h

#include <3ds.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <fstream>

#include "files.h"

class FileOperations
{
public:
    FileOperations();
    virtual ~FileOperations();
    
    // Minimap
    void fileop_load_minimap(MiniMapFileStruct* pMiniMapFileStruct, u8 stage_phase);
    
    // Text File
    ConfigFileStruct* fileop_load_config();
    void fileop_save_config(ConfigFileStruct* pConfigFileStruct);
    
    void fileop_load_strings(StringFileStruct* pStringFileStruct, u8 text_id);
    
    // Actor Files
    EnemyFileStruct* fileop_load_enemy(std::string enemy_file);
    ItemFileStruct* fileop_load_item(std::string item_file);
    ObjectFileStruct* fileop_load_object(std::string object_file);
    MapFileStruct* fileop_load_stage(u16 stage_id);
    
    // Complete Files
    bool fileop_file_exists(const char* file);
    bool fileop_file_delete(const char* file);
    
    // Write (Binary)
    void fileop_write_u32(std::ofstream& file, u32 value);
    void fileop_write_u16(std::ofstream& file, u16 value);
    void fileop_write_u8(std::ofstream& file, u8 value);
    void fileop_write_s32(std::ofstream& file, s32 value);
    void fileop_write_s16(std::ofstream& file, s16 value);
    void fileop_write_s8(std::ofstream& file, s8 value);
    
    // Read (Binary) (C++)
    void fileop_seekg_file(std::ifstream& file, u32 pos_addition);
    void fileop_seekg_resetpos();
    
    u32 fileop_read_u32(std::ifstream& file);
    u16 fileop_read_u16(std::ifstream& file);
    u8 fileop_read_u8(std::ifstream& file);
    s32 fileop_read_s32(std::ifstream& file);
    s16 fileop_read_s16(std::ifstream& file);
    s8 fileop_read_s8(std::ifstream& file);
    
    // Read (Binary) (C)
    void fileop_seekg_resetpos_c();
    
    u32 fileop_read_u32(FILE* fp);
    u16 fileop_read_u16(FILE* fp);
    u8 fileop_read_u8(FILE* fp);
    s32 fileop_read_s32(FILE* fp);
    s16 fileop_read_s16(FILE* fp);
    s8 fileop_read_s8(FILE* fp);
    std::string fileop_read_cstring(FILE* fp, u8 string_length);
    
    // Write (Text)
    
    // Read (Text)
    std::string fileop_read_string_text(StringFileStruct* pStringFileStruct, u8 pos);
    s32 fileop_read_string_x(StringFileStruct* pStringFileStruct, u8 pos);
    s32 fileop_read_string_y(StringFileStruct* pStringFileStruct, u8 pos);
    
private:
    u32 fileop_seekg_pos;
    u32 fileop_seekg_pos_c;
};

#endif /* file_operations_h */
