//
//  files.h
//  3ds_metroid
//
//  Created by jbr373 on 04/02/16.
//

/*
 
 Magic:
 
 M2E (Anfang jeder Magic für Metroid 2 Engine Files)
 
 0x4D3245[Letztes Char Hex]
 
 M2ES (Metroid 2 Engine Save):
 
 (16) = 0x4D324553
 (10) = 1295140179
 
 M2EM (Metroid 2 Engine Map):
 
 (16) = 0x4D32454D
 (10) = 1295140173
 
 M2EE (Metroid 2 Engine Enemy):
 
 (16) = 0x4D324545
 (10) = 1295140165
 
 M2EI (Metroid 2 Engine Item):
 
 (16) = 0x4D324549
 (10) = 1295140169
 
 M2EO (Metroid 2 Engine Object):
 
 (16) = 0x4D324551
 (10) = 1295140177
 
 */

#ifndef files_h
#define files_h

#include <3ds.h>
#include <string>
#include <vector>

#define MAGIC_DECIMAL_SAVE          1295140179
#define MAGIC_DECIMAL_MAP           1295140173
#define MAGIC_DECIMAL_ENEMY         1295140165
#define MAGIC_DECIMAL_ITEM          1295140169
#define MAGIC_DECIMAL_OBJECT        1295140177

#define VERSION_DECIMAL_SAVE        8
#define VERSION_DECIMAL_MAP         18
#define VERSION_DECIMAL_ENEMY       5
#define VERSION_DECIMAL_ITEM        9
#define VERSION_DECIMAL_OBJECT      7
#define VERSION_DECIMAL_IMAGE       1

#define MINIMAP_MAX_MAPS            32

#define STAGE_MAX_ENEMIES           15
#define STAGE_MAX_ITEMS             15
#define STAGE_MAX_OBJECTS           29

// Minimap and String File does not have a Version or Magic! It's a simple plain-text file!

struct MiniMapFileStruct
{
    MiniMapFileStruct()
    {
        minimapfile_loaded = false;
        minimapfile_rooms = 0;
    }
    
    bool minimapfile_loaded;
    
    u8 minimapfile_rooms;
    s32 minimapfile_x_max;
    s32 minimapfile_y_max;
    
    u32 minimapfile_room_id[MINIMAP_MAX_MAPS];
    s32 minimapfile_room_x[MINIMAP_MAX_MAPS];
    s32 minimapfile_room_y[MINIMAP_MAX_MAPS];
    s32 minimapfile_room_w[MINIMAP_MAX_MAPS];
    s32 minimapfile_room_h[MINIMAP_MAX_MAPS];
};

struct StringFileStruct
{
    StringFileStruct()
    {
        stringfile_loaded = false;
    }
    
    bool stringfile_loaded;
    
    u8 text_count;
    std::vector<std::string> text_string;
    std::vector<s32> text_x;
    std::vector<s32> text_y;
};

struct ConfigFileStruct
{
    ConfigFileStruct()
    {
        config_map = true;
        config_movement_changed = false;
        config_shooting_changed = false;
    }
    
    bool config_map;
    bool config_movement_changed;
    bool config_shooting_changed;
};

struct SaveFileStruct
{
    // File Info
    u32 savefile_magic;
    u32 savefile_version;
    
    // Player Inventory
    u8 savefile_health;
    u8 savefile_healthtanks;
    u8 savefile_healthtanks_available;
    u8 savefile_missles_total;
    u8 savefile_missles_available;
    
    u8 savefile_activebeam;
    
    // Player Position
    u16 savefile_room;
    s32 savefile_x;
    s32 savefile_y;
    
    // Player Objective
    u8 savefile_phase;
    u8 savefile_metroids; // Actual Number
    u32 savefile_metroids_killed_phase; // Comination of the Flags of the killed Metroids -> IN THE CURRENT PHASE <-. Reset to 0 at every new Phase
    
    // Upgrades
    bool savefile_bombs;
    bool savefile_icebeam;
    bool savefile_wavebeam;
    bool savefile_plasmabeam;
    bool savefile_spiderball;
    bool savefile_highjump;
    bool savefile_barriersuit;
    bool savefile_spazerbeam;
    bool savefile_screwattack;
    bool savefile_springball;
    bool savefile_spacejump;
    
    // One-Time Collectible Items
    u32 savefile_collected_missletanks;
    u32 savefile_collected_healthtanks;
    
    // Map
    u8 savefile_phase_stages; // Shows which Stage Phase we're on, p.e. Phase 1 Maps. Does NOT define the current Story Progression!
    
    // Minimap
    u32 savefile_p0_map;
    u32 savefile_p1_map;
    u32 savefile_p2_map;
    u32 savefile_p3_map;
    u32 savefile_p4_map;
    u32 savefile_p5_map;
    u32 savefile_p6_map;
    u32 savefile_p7_map;
    u32 savefile_p8_map;
    u32 savefile_p9_map;
    u32 savefile_p10_map;
};

struct MapFileStruct
{
    // File Info
    u32 stage_magic;
    u32 stage_version;
    
    // Stage General
    u16 stage_id;
    u8 stage_phase;
    s32 stage_width;
    s32 stage_height;
    
    // Actor Count
    u8 stage_enemies;
    u8 stage_items;
    u8 stage_objects;
    
    // Minimap
    u32 stage_minimap_id;
    
    // Water
    u8 stage_liquid_type; // 0 - No Water
    
    // Images
    u8 stage_background_path_length;
    std::string stage_background_path;
    u8 stage_tileset_path_length;
    std::string stage_tileset_path;
    u8* stage_tileset_layer_foreground;
    bool stage_draw_tileset_behind_player;
    u8* stage_tileset_layer_background;
    
    // Collision
    u8 stage_collision_path_length;
    std::string stage_collision_path;
    
    // Sound
    u8 stage_sound_path_length;
    std::string stage_sound_path;
    
    // Light
    u8 stage_light_ambient;
    
    // Water
    s32 stage_liquid_x;
    s32 stage_liquid_y;
    s32 stage_liquid_w;
    s32 stage_liquid_h;
    u8 stage_liquid_alpha;
    u8 stage_liquid_ripples_intensity; // 0 = no ripples
    bool stage_liquid_godray;
    
    // Enemies
    // Enemy String Length before enemy string !
    u8* stage_enemy_file_path_length;
    std::string* stage_enemy_file_path;
    s32* stage_enemy_x;
    s32* stage_enemy_y;
    
    // Items
    // Item String Length before Item String!
    u8* stage_item_file_path_length;
    std::string* stage_item_file_path;
    s32* stage_item_x;
    s32* stage_item_y;
    
    // Objects
    // Object String Length before Object String!
    u8* stage_object_file_path_length;
    std::string* stage_object_file_path;
    s32* stage_object_x;
    s32* stage_object_y;
};

struct EnemyFileStruct
{
    u32 enemy_magic;
    u32 enemy_version;
    
    u8 enemy_ai;
    u8 enemy_physics;
    
    bool enemy_is_metroid;
    u32 enemy_metroid_flag;
    
    s16 enemy_health;
    u16 enemy_touch_damage;
    u16 enemy_projectile_damage;
    u8 enemy_speed;
    
    u8 enemy_loot_filepath_length;
    std::string enemy_loot;
    u8 enemy_loot_chance;
    s32 enemy_loot_spawn_x;
    s32 enemy_loot_spawn_y;
    
    s32 enemy_hitbox_width;
    s32 enemy_hitbox_height;
    
    u8 enemy_size;
    
    s32 enemy_damage_hitbox_x; // Addition to Enemy X!
    s32 enemy_damage_hitbox_y;
    s32 enemy_damage_hitbox_width; // Hitbox explodes Projectiles, Damage hitbox also applies damage
    s32 enemy_damage_hitbox_height;
    
    u8 enemy_spritesheet_filepath_length;
    std::string enemy_spritesheet;
    s32 enemy_spritesheet_width;
    s32 enemy_spritesheet_height;
    u8 enemy_max_spritelines;
    u8 enemy_max_frame_speed;
    
    bool enemy_shadow;
    
    u8 enemy_sound_id;
};

struct ItemFileStruct
{
    // NOTE:
    // Items have only 1 Sprite Line (Idle) and no Collision!
    
    u32 item_magic;
    u32 item_version;
    
    u8 item_type;
    u8 item_action;
    s32 item_action_value;
    u32 item_exist_time; // Time until Item is removed. 0 = No Removal, stays until unloaded by Stage
    u32 item_special_id; // ^2 Value for unique ID - Used for blocking already collected Items (Item Type for Block Type in Savefile)
    
    s32 item_hitbox_width;
    s32 item_hitbox_height;
    
    bool item_has_text; // TRUE = Textbox on Pickup, FALSE = No Textbox
    u8 item_text_id; // Text ID

    bool item_shadow;
    u8 item_light_color;
    u8 item_light_size;
    s32 item_light_spawn_x;
    s32 item_light_spawn_y;
    
    u8 item_sound_id;       // Pickup Sound, normal Pickup or Special Jingle
};

struct ObjectFileStruct
{
    u32 object_magic;
    u32 object_version;
    
    u8 object_type;
    u8 object_phase;
    
    s32 object_value_a;
    s32 object_value_b;
    s32 object_value_c;
    
    u8 object_loot_filepath_length;
    std::string object_loot;
    u8 object_loot_chance;
    s32 object_loot_spawn_x;
    s32 object_loot_spawn_y;
    
    s32 object_hitbox_width; // NOT SPRITESHEET!
    s32 object_hitbox_height; // NOT SPRITESHEET
    
    u8 object_size;
    
    u8 object_spritesheet_filepath_length;
    std::string object_spritesheet;
    s32 object_spritesheet_width; // Divided by 8 for Single W / H
    s32 object_spritesheet_height;
    u8 object_max_spritelines;
    u8 object_max_frame_speed;
    
    bool object_shadow;
    u8 object_light_color;
    u8 object_light_size;
    s32 object_light_spawn_x;
    s32 object_light_spawn_y;
    
    bool object_collision;
};

#endif /* files_h */
