//
//  image.h
//  3ds_metroid
//
//  Created by jbr373 on 16/01/16.
//

#ifndef image_h
#define image_h

#include <3ds.h>
#include <sf2d.h>
#include <stdio.h>
#include <string>

#define MAX_TEXLINE_LENGTH          7 // 0 - 7, 8 Textures
#define MAX_SPRITESHEET_FRAMES      7 // 0 - 7, 8 Frames

#define MAX_WATER_BUBBLES           4
#define MAX_WATER_RIPPLES           6

#define TILESSET_TILE_DIMENSION     128

/* SpriteData */
/* Basically a Spritesheet split at each row */
struct SpriteData
{
    SpriteData()
    {
        spritedata_w = 0;
        spritedata_h = 0;
        spritedata_frame = 0;
        spritedata_startframe = 0;
        spritedata_maxframe = 0;
        spritedata_texture_line = 0;
        spritedata_frame_speed = 2;
        spritedata_frame_speed_current = 0;
        
        for (u8 i = 0; i < MAX_TEXLINE_LENGTH; ++i)
        {
            spritedata_texture[i] = NULL;
        }
    }
    
    s32 spritedata_w; // Per Sprite Frame
    s32 spritedata_h; // Per Sprite Frame
    u8 spritedata_frame;
    u8 spritedata_startframe;
    u8 spritedata_maxframe;
    u8 spritedata_texture_line;
    u8 spritedata_frame_speed;
    u8 spritedata_frame_speed_current;
    
    sf2d_texture* spritedata_texture[MAX_TEXLINE_LENGTH];
};

/* SpriteSheet */
/* A fully assembled Spritesheet */
struct SpriteSheet
{
    SpriteSheet()
    {
        spritesheet_w_single = 0;
        spritesheet_h_single = 0;
        spritesheet_w_full = 0;
        spritesheet_h_full = 0;
        spritesheet_frame = 0;
        spritesheet_startframe = 0;
        spritesheet_endframe = 0;
        spritesheet_sprite_line = 0;
        spritesheet_frame_speed = 2;
        spritesheet_frame_speed_current = 0;
        spritesheet_texture = NULL;
    }
    
    s32 spritesheet_w_single;
    s32 spritesheet_h_single;
    s32 spritesheet_w_full;
    s32 spritesheet_h_full;
    u8 spritesheet_frame;
    u8 spritesheet_startframe;
    u8 spritesheet_endframe;
    u8 spritesheet_sprite_line;
    u8 spritesheet_frame_speed;
    u8 spritesheet_frame_speed_current;
    
    sf2d_texture* spritesheet_texture;
};

#endif /* image_h */
