//
//  item.cpp
//  3ds_metroid
//
//  Created by jbr373 on 16/01/16.
//

#include <sfil.h>
#include <sstream>
#include <string>
#include <fstream>
#include <stdlib.h>

#include "item.h"
#include "filepaths.h"
#include "files.h"

Item::Item()
{
    // General Item Info
    item_x = -400;
    item_y = -240;
    
    item_sound_id = 0;
    
    item_hitbox_width = 0;
    item_hitbox_height = 0;
    
    item_type = 0;
    item_action = ITEMACTION_NONE;
    item_action_value = 0;

    item_exist_time = 0;
    item_special_id = 0;
    
    item_has_text = false;
    item_text_id = 0;
    
    item_collected = false;
    item_shadow = false;
    item_sound_played = false;
}

Item::~Item()
{
}

void Item::item_load(std::string new_item_file, s32 new_item_x, s32 new_item_y)
{
    // Create File Struct
    ItemFileStruct* pItemFileStruct = new ItemFileStruct();
    
    // General Item Info
    item_x = new_item_x;
    item_y = new_item_y;

    // Read Item File
    std::ifstream itemfile;
    itemfile.open (new_item_file);

    if (itemfile.is_open())
    {
        // File Pointer Position
        fileop_seekg_resetpos();
        
        // Read Magic & Version
        pItemFileStruct->item_magic = fileop_read_u32(itemfile);
        fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_magic));
        
        pItemFileStruct->item_version = fileop_read_u32(itemfile);
        fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_version));
        
        // Create New File if Magic or Version aren't matching
        if (pItemFileStruct->item_magic != MAGIC_DECIMAL_ITEM || pItemFileStruct->item_version != VERSION_DECIMAL_ITEM)
        {
            // Close Save File
            itemfile.close();
            
            // Move Item out of Room
            item_x = -400;
            item_y = -240;
            item_collected = true;
        }
        else
        {
            /* Get Item Type */
            
            // Get Type
            pItemFileStruct->item_type = fileop_read_u8(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_type));
            
            /* Get Item Action and Action Value */
            
            // Get Action
            pItemFileStruct->item_action = fileop_read_u8(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_action));
            
            // Get Value
            pItemFileStruct->item_action_value = fileop_read_s32(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_action_value));
            
            /* Get Item Exist Timeframe */
            
            // Get Action
            pItemFileStruct->item_exist_time = fileop_read_u32(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_exist_time));
            
            /* Get Item Unique Special ID */
            
            // Get Special ID
            pItemFileStruct->item_special_id = fileop_read_u32(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_special_id));
      
            /* Get Item Hitbox Dimensions */
            
            // Get Hitbox Width
            pItemFileStruct->item_hitbox_width = fileop_read_s32(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_hitbox_width));
            
            // Get Hitbox Height
            pItemFileStruct->item_hitbox_height = fileop_read_s32(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_hitbox_height));
            
            /* Get Item Text Info */
            
            // Get Item Text Flag
            pItemFileStruct->item_has_text = fileop_read_u8(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_has_text));
            
            // Get Text ID's
            if (pItemFileStruct->item_has_text == true)
            {
                // Get Item Title Text ID
                pItemFileStruct->item_text_id = fileop_read_u8(itemfile);
                fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_text_id));
            }
            else
            {
                pItemFileStruct->item_text_id = 0;
            }
            
            /* Get Item Light */
            
            // Get Shadow Flag
            pItemFileStruct->item_shadow = fileop_read_u8(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_shadow));
            
            // Get Light Color
            pItemFileStruct->item_light_color = fileop_read_u8(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_light_color));
            
            // Get Light Size
            pItemFileStruct->item_light_size = fileop_read_u8(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_light_size));
            
            // Get Light X Addition
            pItemFileStruct->item_light_spawn_x = fileop_read_s32(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_light_spawn_x));
            
            // Get Light Y Addition
            pItemFileStruct->item_light_spawn_y = fileop_read_s32(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_light_spawn_y));
            
            /* Get Item Sound */
            
            // Get Sound ID
            pItemFileStruct->item_sound_id = fileop_read_u8(itemfile);
            fileop_seekg_file(itemfile, sizeof(pItemFileStruct->item_sound_id));
           
            // Close Object File
            itemfile.close();
        }
        
        // File Pointer Reset
        fileop_seekg_resetpos();
        
        /* Write Values to Class and delete the struct */
        
        item_type = pItemFileStruct->item_type;
        item_action = pItemFileStruct->item_action;
        item_action_value = pItemFileStruct->item_action_value;

        item_exist_time = pItemFileStruct->item_exist_time;
        item_special_id = pItemFileStruct->item_special_id;
        
        item_hitbox_width = pItemFileStruct->item_hitbox_width;
        item_hitbox_height = pItemFileStruct->item_hitbox_height;
        
        if (pItemFileStruct->item_has_text == true)
        {
            // Set Flag
            item_has_text = true;
            
            // Load Text
            item_text_id = pItemFileStruct->item_text_id;
        }
        else
        {
            item_has_text = false;
        }
        
        item_shadow = pItemFileStruct->item_shadow;
        
        item_sound_id = pItemFileStruct->item_sound_id;

        // Delete File Struct
        delete pItemFileStruct;
    }
    else
    {
        // Close Save File
        itemfile.close();
        
        // Move Item out of Room
        item_x = -400;
        item_y = -240;
        item_collected = true;
    }
}

bool Item::item_collision_in_range(s32 actor_a_value, s32 actor_b_min, s32 actor_b_max)
{
    // Return Compare Result
    return (actor_a_value >= actor_b_min) && (actor_a_value <= actor_b_max);
}

bool Item::item_check_collection(SaveFile* pSaveFile, Sound* pSound, s32 actor_x, s32 actor_y, s32 actor_w, s32 actor_h)
{
    // Compare Positions
    bool x_overlap = item_collision_in_range(actor_x, item_x, item_x + item_hitbox_width) ||
                     item_collision_in_range(item_x, actor_x, actor_x - (actor_w / 2) + actor_w);
    
    bool y_overlap = item_collision_in_range(actor_y, item_y, item_y + item_hitbox_height) ||
                     item_collision_in_range(item_y, actor_y, actor_y + actor_h);
    
    // Collect Item
    if (x_overlap == true && y_overlap == true)
    {
        // Play Pickup Sound
        if (item_sound_played == false)
        {
            // Play Sound depending on Item Type
            if (item_type == ITEMTYPE_HEALTH20 || item_type == ITEMTYPE_HEALTH50 || item_type == ITEMTYPE_MISSLE5 || item_type == ITEMTYPE_RESTORE)
            {
                pSound->sound_play_fx(SOUND_PLAYER_RESTORE_JINGLE, SOUNDTYPE_PLAYER_SHOOTING, false, true);
            }
            else
            {
                pSound->sound_play_fx(SOUND_PLAYER_UPGRADE_JINGLE, SOUNDTYPE_PLAYER_SHOOTING, false, true);
            }
            
            // Set Sound Played Flag
            item_sound_played = true;
        }
        
        // Check for Type and Unique ID
        
        
        // Flag Item
        item_collected = true;
    }
    
    // Return Result
    if (x_overlap == true && y_overlap == true)
    {
        return true;
    }
    else
    {
        return false;
    }
}
