//
//  item.h
//  3ds_metroid
//
//  Created by jbr373 on 16/01/16.
//

#ifndef item_h
#define item_h

#include <stdio.h>
#include <string>

#include "image.h"
#include "savefile.h"
#include "sound.h"

enum ItemTypes
{
    ITEMTYPE_HEALTH20 = 0,
    ITEMTYPE_HEALTH50 = 1,
    ITEMTYPE_MISSLE5 = 2,
    ITEMTYPE_HEALTHTANK = 3,
    ITEMTYPE_MISSLETANK = 4,
    ITEMTYPE_ICEBEAM = 5,
    ITEMTYPE_WAVEBEAM = 6,
    ITEMTYPE_SPAZERBEAM = 7,
    ITEMTYPE_PLASMABEAM = 8,
    ITEMTYPE_RESTORE = 9,
};

enum ItemActions
{
    ITEMACTION_NONE = 0,
    ITEMACTION_ADDHEALTH = 1,
    ITEMACTION_ADDMISSLES = 2,
    ITEMACTION_INCREASEHEALTHTANKS = 3,
    ITEMACTION_INCREASEMISSLETANKS = 4,
    ITEMACTION_UPGRADE = 5,
};

class Item : private FileOperations
{
public:
    Item();
    virtual ~Item();
    
    void item_load(std::string new_item_file, s32 new_item_x, s32 new_item_y);
    bool item_check_collection(SaveFile* pSaveFile, Sound* pSound, s32 actor_x, s32 actor_y, s32 actor_w, s32 actor_h);
    
    // Getters
    u32 item_get_exist_time() { return item_exist_time; };
    u32 item_get_special_id() { return item_special_id; };
    
    s32 item_get_x() { return item_x; };
    s32 item_get_y() { return item_y; };
    s32 item_get_hitbox_w() { return item_hitbox_width; };
    s32 item_get_hitbox_h() { return item_hitbox_height; };
    s32 item_get_action_value() { return item_action_value; };
    s32 item_get_light_x_addition() { return item_light_x_addition; };
    s32 item_get_light_y_addition() { return item_light_y_addition; };
    
    u8 item_get_type() { return item_type; };
    u8 item_get_action() { return item_action; };
    u8 item_get_sound_id() { return item_sound_id; };
    u8 item_get_text_id() { return item_text_id; };
    
    bool item_get_collected() { return item_collected; };
    bool item_has_text_flag() { return item_has_text; };
    bool item_has_shadow() { return item_shadow; };
    
    // Setters
    void item_set_x(s32 x) { item_x = x; };
    void item_set_y(s32 y) { item_y = y; };
    void item_set_hitbox_w(s32 w) { item_hitbox_width = w; };
    void item_set_hitbox_h(s32 h) { item_hitbox_height = h; };
    void item_set_ligth_x_addition(s32 x) { item_light_x_addition = x; };
    void item_set_ligth_y_addition(s32 y) { item_light_y_addition = y; };
    void item_set_action_value(s32 value) { item_action_value = value; };
    
    void item_set_exist_time(u32 frames) { item_exist_time = frames; };
    void item_set_special_id(u32 special_id) { item_special_id = special_id; };
    
    void item_set_type(u8 type) { item_type = type; };
    void item_set_action(u8 action) { item_action = action; };
    void item_set_sound_id(u8 sound_id) { item_sound_id = sound_id; };
    void item_set_light_type(u8 light_type) { item_light_type = light_type; };
    void item_set_light_size(u8 light_size) { item_light_size = light_size; };
    void item_set_text_id(u8 text_id) { item_text_id = text_id; };
    
    void item_set_collected(bool value) { item_collected = value; };
    void item_set_shadow(bool value) { item_shadow = value; };
    void item_set_text_flag(bool value) { item_has_text = value; };
    
private:
    bool item_collision_in_range(s32 actor_a_value, s32 actor_b_min, s32 actor_b_max);
    
    u8 item_type;
    u8 item_action;
    u8 item_sound_id;
    u8 item_light_type;
    u8 item_light_size;
    u8 item_text_id;
    
    s32 item_x;
    s32 item_y;
    s32 item_hitbox_width;
    s32 item_hitbox_height;
    s32 item_action_value;
    s32 item_light_x_addition;
    s32 item_light_y_addition;
    
    u32 item_exist_time; // 0 = Unlimited, 1 = Despawn
    u32 item_special_id; // 0 = No Special ID
    
    bool item_collected;
    bool item_shadow;
    bool item_sound_played;
    bool item_has_text;
};

#endif /* item_h */
