//
//  menu.cpp
//  3ds_metroid
//
//  Created by jbr373 on 16/01/16.
//

#include <sfil.h>

#include "menu.h"
#include "filepaths.h"

#define GAMEMENU_MAX_SELECTIONS         2 // 0 = New, 1 = Continue, 2 = Options

#define GAMEMENU_STRING_ID              3

#define MAINMENU_TITLE_X                90
#define MAINMENU_TITLE_Y                80

#define MAINMENU_SELECTION_A_X          0
#define MAINMENU_SELECTION_B_X          161
#define MAINMENU_SELECTION_Y            210

#define OPTIONS_SELECTION_A_Y           40
#define OPTIONS_SELECTION_B_Y           81
#define OPTIONS_SELECTION_C_Y           121
#define OPTIONS_SELECTION_D_Y           210

#define GAMEOVER_SELECTION_A_Y          90
#define GAMEOVER_SELECTION_B_Y          120

#define MAINMENU_SELECTION_W            160
#define MAINMENU_SELECTION_H            30
#define OPTIONS_SELECTION_H             40

#define WARNING_TIME                    7

enum MenuSelections
{
    MENU_NEW = 0,
    MENU_CONTINUE = 1,
    MENU_OPTIONS = 2
};

enum OptionSelections
{
    OPTIONS_MOVEMENT = 0,
    OPTIONS_SHOOTING = 1,
    OPTIONS_MAP = 2,
    OPTIONS_RETURN = 3
};

enum GameOverSelections
{
    MENU_GAMEOVER_CONTINUE = 0,
    MENU_GAMEOVER_QUIT = 1
};

GameMenu::GameMenu()
{
    // Menu State
    gamemenu_submenu = GAMEMENU_BOOT;
    gamemenu_next_submenu = GAMEMENU_WARNING;
    
    // Menu Type
    gamemenu_is_main = true;
    gamemenu_ignore_input = false;
    
    // Menu Sound
    gamemenu_started_bgm = false;
    
    // Graphics and Selection set to "Continue"
    gamemenu_selection = MENU_CONTINUE;
    gamemenu_selection_options = OPTIONS_MOVEMENT;
    gamemenu_selection_gameover = MENU_GAMEOVER_CONTINUE;
    
    // Warning Timer
    gamemenu_warning_timer = time(NULL) + WARNING_TIME;
    
    // Set Data
    pGameMenu_Sprite = NULL;
    pGameMenu_Text = NULL;
    pGameConfig = NULL;
    
    // Load Textures
    gamemenu_load_textures();
}

GameMenu::~GameMenu()
{
    // Delete Textures
    if (pGameMenu_Sprite != NULL)
    {
        sf2d_free_texture(pGameMenu_Sprite);
        pGameMenu_Sprite = NULL;
    }
    
    // Delete Text Strings
    if (pGameMenu_Text != NULL)
    {
        delete pGameMenu_Text;
        pGameMenu_Text = NULL;
    }
    
    // Delete Config File
    if (pGameConfig != NULL)
    {
        delete pGameConfig;
        pGameConfig = NULL;
    }
}

void GameMenu::gamemenu_load_textures()
{
    // Load Textures
    pGameMenu_Sprite = sfil_load_PNG_file(IMG_MENU_TITLE, SF2D_PLACE_RAM);
    
    // Load Text Strings
    pGameMenu_Text = new StringFileStruct();
    fileop_load_strings(pGameMenu_Text, GAMEMENU_STRING_ID);
    
    // Load Config File for Options Menu
    pGameConfig = fileop_load_config();
}

void GameMenu::gamemenu_clear_sprites()
{
    if (pGameMenu_Sprite != NULL)
    {
        sf2d_free_texture(pGameMenu_Sprite);
        pGameMenu_Sprite = NULL;
    }
}

/*************************************************************************************************************************/
/*                                                 Main Menu Returns                                                     */
/*************************************************************************************************************************/

u8 GameMenu::gamemenu_main(SaveFile* pSaveFile, Sound* pSound, s32 kdown, touchPosition touch_pos)
{
    /* Main Menu */
 
    // Set Current Menu Type
    gamemenu_is_main = true;
    
    /* Block Input on Conditions */
    
    // Check for not updated Submenu
    if (gamemenu_submenu != gamemenu_next_submenu)
    {
        // Return old SUbmenu
        return gamemenu_submenu;
    }
    
    // Check for Input Block
    if (gamemenu_ignore_input == true)
    {
        // Return old Submenu
        return gamemenu_submenu;
    }
    
    // Start BGM on Splash Screen
    if (gamemenu_started_bgm == false && gamemenu_submenu == GAMEMENU_SPLASH)
    {
        // Load Game Sound
        pSound->sound_play_bgm(SND_BGM_TITLE, true, true);
        
        // Flag BGM Started
        gamemenu_started_bgm = true;
    }
    
    /* Update Selections */
    
    if (gamemenu_submenu != GAMEMENU_GAMEOVER && gamemenu_selection_gameover == MENU_GAMEOVER_CONTINUE)
    {
        gamemenu_selection_gameover = MENU_GAMEOVER_CONTINUE;
    }
    
    if (gamemenu_submenu != GAMEMENU_SAVEFILES && gamemenu_selection_gameover != MENU_CONTINUE)
    {
        gamemenu_selection = MENU_CONTINUE;
    }
    
    /* Menues */
    
    // Switch depending on Menu State
    if (gamemenu_submenu == GAMEMENU_WARNING)
    {
        // Display Software Warning
        gamemenu_warning_screen();
    }
    else if (gamemenu_submenu == GAMEMENU_SPLASH)
    {
        // Splash Screen Input
        gamemenu_splash_screen(pSound, pSaveFile, kdown, touch_pos);
    }
    else if (gamemenu_submenu == GAMEMENU_SAVEFILES)
    {
        // Savefile Selection
        gamemenu_saves_screen(pSaveFile, pSound, kdown, touch_pos);
    }
    else if (gamemenu_submenu == GAMEMENU_OPTIONS)
    {
        // Savefile Selection
        gamemenu_options_screen(pSound, kdown, touch_pos);
    }
    
    /* Result */
    
    // Return Current Menu State
    return gamemenu_submenu;
}

u8 GameMenu::gamemenu_gameover(Sound* pSound, u32 kdown, touchPosition touch_pos)
{
    /* Game Over Menu */
    
    // Set Current Menu Type
    gamemenu_is_main = false;
    gamemenu_selection = 0;
    
    // Check Menu
    gamemenu_gameover_screen(pSound, kdown, touch_pos);

    // Return Menu State
    return gamemenu_submenu;
}

/*************************************************************************************************************************/
/*                                               Specific Menu Screens                                                   */
/*************************************************************************************************************************/

void GameMenu::gamemenu_splash_screen(Sound* pSound, SaveFile* pSaveFile, s32 kdown, touchPosition touch_pos)
{
    /* Touch Screen Input */
    
    // Touch Registered on Splash
    if (touch_pos.px != 0 && touch_pos.py != 0)
    {
        // Set Next Menu
        if (pSaveFile->savefile_get_new_savefile() == true)
        {
            // New Save, start new Game
            gamemenu_next_submenu = GAMEMENU_START;
        }
        else
        {
            // Jump to Savefile Menu
            gamemenu_next_submenu = GAMEMENU_SAVEFILES;
        }
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
    }
    
    /* Buttons */
    
    if (kdown & KEY_A)
    {
        // Set Game Menu
        if (pSaveFile->savefile_get_new_savefile() == true)
        {
            // New Save, start new Game
            gamemenu_next_submenu = GAMEMENU_START;
        }
        else
        {
            // Jump to Savefile Menu
            gamemenu_next_submenu = GAMEMENU_SAVEFILES;
        }
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
    }
    else if (kdown & KEY_START)
    {
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
        
        // Set Next Menu
        if (pSaveFile->savefile_get_new_savefile() == true)
        {
            // New Save, start new Game
            gamemenu_next_submenu = GAMEMENU_START;
        }
        else
        {
            // Jump to Savefile Menu
            gamemenu_next_submenu = GAMEMENU_SAVEFILES;
        }
        
    }
    else if (kdown & KEY_B)
    {
        // Return to Home Menu / Homebrew Launcher
        gamemenu_next_submenu = GAMEMENU_QUIT;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
}

void GameMenu::gamemenu_saves_screen(SaveFile* pSaveFile, Sound* pSound, s32 kdown, touchPosition touch_pos)
{
    /* Touch Input */
    
    if (touch_pos.py > MAINMENU_SELECTION_Y && touch_pos.py <= (MAINMENU_SELECTION_Y + MAINMENU_SELECTION_H))
    {
        // Set Sprite Position for selection
        if (touch_pos.px > MAINMENU_SELECTION_A_X && touch_pos.px <= (MAINMENU_SELECTION_A_X + MAINMENU_SELECTION_W))
        {
            // Set Selection
            gamemenu_selection = MENU_NEW;
            
            // Overwrite Savefile
            pSaveFile->savefile_new();
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
            
            // Set Next Submenu
            gamemenu_next_submenu = GAMEMENU_START;
        }
        else if (touch_pos.px > MAINMENU_SELECTION_B_X && touch_pos.py <= (MAINMENU_SELECTION_B_X + MAINMENU_SELECTION_W))
        {
            // Set Selection
            gamemenu_selection = MENU_CONTINUE;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
            
            // Set Next Submenu
            gamemenu_next_submenu = GAMEMENU_START;
        }
    }
    else if (touch_pos.py > 0 && touch_pos.py <= MAINMENU_SELECTION_H && touch_pos.px > 1 && touch_pos.px <= (MAINMENU_SELECTION_W * 2))
    {
        // Set Selection
        gamemenu_selection = MENU_OPTIONS;
        
        // Change to Options Menu
        gamemenu_next_submenu = GAMEMENU_OPTIONS;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
    }
    
    /* Button Input */
    
    // Directional Pad
    if (kdown & KEY_LEFT && gamemenu_selection > MENU_NEW && gamemenu_selection < GAMEMENU_MAX_SELECTIONS)
    {
        // Change Selection
        --gamemenu_selection;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
    else if (kdown & KEY_RIGHT && gamemenu_selection < MENU_CONTINUE)
    {
        // Change Selection
        ++gamemenu_selection;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
    else if (kdown & KEY_UP && gamemenu_selection < GAMEMENU_MAX_SELECTIONS)
    {
        // Change Selection
        gamemenu_selection = GAMEMENU_MAX_SELECTIONS;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
    else if (kdown & KEY_DOWN && gamemenu_selection == GAMEMENU_MAX_SELECTIONS)
    {
        // Change Selection
        --gamemenu_selection;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
    
    // Buttons
    if (kdown & KEY_A)
    {
        // Action depending on Submenu
        if (gamemenu_selection == MENU_NEW)
        {
            // Overwrite Savefile
            pSaveFile->savefile_new();
            
            // Set to start
            gamemenu_next_submenu = GAMEMENU_START;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
        }
        else if (gamemenu_selection == MENU_OPTIONS)
        {
            // Change to Options Menu
            gamemenu_next_submenu = GAMEMENU_OPTIONS;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
        else
        {
            // Continue from old Savefile
            gamemenu_next_submenu = GAMEMENU_START;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
        }
    }
    else if (kdown & KEY_B)
    {
        // Return to Splash Screen
        gamemenu_next_submenu = GAMEMENU_SPLASH;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
}

void GameMenu::gamemenu_options_screen(Sound* pSound, s32 kdown, touchPosition touch_pos)
{
    /* Touch Input */
    
    static u16 old_touch_x = 0, old_touch_y = 0;
    
    if (touch_pos.py > OPTIONS_SELECTION_D_Y && touch_pos.py <= (OPTIONS_SELECTION_D_Y + MAINMENU_SELECTION_H) &&
        touch_pos.px > 1 && touch_pos.px <= MAINMENU_SELECTION_W * 2)
    {
        // Save Config File
        fileop_save_config(pGameConfig);
        
        // Return to File Select Screen
        gamemenu_next_submenu = GAMEMENU_SAVEFILES;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
    }
    else if (touch_pos.py > OPTIONS_SELECTION_A_Y && touch_pos.py <= (OPTIONS_SELECTION_A_Y + OPTIONS_SELECTION_H))
    {
        // Set Current Option
        gamemenu_selection_options = OPTIONS_MOVEMENT;
        
        // Set Option On or Off
        if (touch_pos.px > MAINMENU_SELECTION_W && touch_pos.px <= MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2))
        {
            // Set Option
            pGameConfig->config_movement_changed = false;
            
            // Play Sound
            if (old_touch_x == 0 && old_touch_y == 0)
                pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
        else if (touch_pos.px > MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2) && touch_pos.px <= MAINMENU_SELECTION_W * 2)
        {
            // Set Option
            pGameConfig->config_movement_changed = true;
            
            // Play Sound
            if (old_touch_x == 0 && old_touch_y == 0)
                pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
    }
    else if (touch_pos.py > OPTIONS_SELECTION_B_Y && touch_pos.py <= (OPTIONS_SELECTION_B_Y + OPTIONS_SELECTION_H))
    {
        // Set Current Option
        gamemenu_selection_options = OPTIONS_SHOOTING;
        
        // Set Option On or Off
        if (touch_pos.px > MAINMENU_SELECTION_W && touch_pos.px <= MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2))
        {
            // Set Option
            pGameConfig->config_shooting_changed = false;
            
            // Play Sound
            if (old_touch_x == 0 && old_touch_y == 0)
                pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
        else if (touch_pos.px > MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2) && touch_pos.px <= MAINMENU_SELECTION_W * 2)
        {
            // Set Option
            pGameConfig->config_shooting_changed = true;
            
            // Play Sound
            if (old_touch_x == 0 && old_touch_y == 0)
                pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
    }
    else if (touch_pos.py > OPTIONS_SELECTION_C_Y && touch_pos.py <= (OPTIONS_SELECTION_C_Y + OPTIONS_SELECTION_H))
    {
        // Set Current Option
        gamemenu_selection_options = OPTIONS_MAP;
        
        // Set Option On or Off
        if (touch_pos.px > MAINMENU_SELECTION_W && touch_pos.px <= MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2))
        {
            // Set Option
            pGameConfig->config_map = false;
            
            // Play Sound
            if (old_touch_x == 0 && old_touch_y == 0)
                pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
        else if (touch_pos.px > MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2) && touch_pos.px <= MAINMENU_SELECTION_W * 2)
        {
            // Set Option
            pGameConfig->config_map = true;
            
            // Play Sound
            if (old_touch_x == 0 && old_touch_y == 0)
                pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
    }
    
    // Update Touch Pos for Sound Block
    old_touch_x = touch_pos.px, old_touch_y = touch_pos.py;
    
    /* Button Input */
    
    // Directional Pad
    if (kdown & KEY_LEFT)
    {
        // Change Options
        if (gamemenu_selection_options == OPTIONS_MOVEMENT && pGameConfig->config_movement_changed == true)
        {
            // Set Option
            pGameConfig->config_movement_changed = false;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
        else if (gamemenu_selection_options == OPTIONS_SHOOTING && pGameConfig->config_shooting_changed == true)
        {
            // Set Option
            pGameConfig->config_shooting_changed = false;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
        else if (gamemenu_selection_options == OPTIONS_MAP && pGameConfig->config_map == true)
        {
            // Set Option
            pGameConfig->config_map = false;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
    }
    else if (kdown & KEY_RIGHT)
    {
        // Change Options
        if (gamemenu_selection_options == OPTIONS_MOVEMENT && pGameConfig->config_movement_changed == false)
        {
            // Set Option
            pGameConfig->config_movement_changed = true;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
        else if (gamemenu_selection_options == OPTIONS_SHOOTING && pGameConfig->config_shooting_changed == false)
        {
            // Set Option
            pGameConfig->config_shooting_changed = true;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
        else if (gamemenu_selection_options == OPTIONS_MAP && pGameConfig->config_map == false)
        {
            // Set Option
            pGameConfig->config_map = true;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
    }
    else if (kdown & KEY_UP)
    {
        // Change Selection
        if (gamemenu_selection_options > 0)
        {
            --gamemenu_selection_options;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
    }
    else if (kdown & KEY_DOWN)
    {
        // Change Selection
        if (gamemenu_selection_options < GAMEMENU_MAX_SELECTIONS + 1)
        {
            ++gamemenu_selection_options;
            
            // Play Sound
            pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
        }
    }
    else if (kdown & KEY_A && gamemenu_selection_options == GAMEMENU_MAX_SELECTIONS + 1)
    {
        // Save Config File
        fileop_save_config(pGameConfig);
        
        // Return to File Select Screen
        gamemenu_next_submenu = GAMEMENU_SAVEFILES;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
    else if (kdown & KEY_B)
    {
        // Cancel out to File Select Screen
        gamemenu_next_submenu = GAMEMENU_SAVEFILES;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
}

void GameMenu::gamemenu_gameover_screen(Sound* pSound, s32 kdown, touchPosition touch_pos)
{
    /* Touch Screen Input */
    
    // Touch Registered
    if (touch_pos.px != 0 && touch_pos.py > GAMEOVER_SELECTION_A_Y && touch_pos.py <= (GAMEOVER_SELECTION_A_Y + MAINMENU_SELECTION_H))
    {
        // Set Selection
        gamemenu_selection_gameover = MENU_GAMEOVER_CONTINUE;
        
        // Set to Game Start
        gamemenu_next_submenu = GAMEMENU_START;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
    }
    else if (touch_pos.px != 0 && touch_pos.py > GAMEOVER_SELECTION_B_Y && touch_pos.py <= (GAMEOVER_SELECTION_B_Y + MAINMENU_SELECTION_H))
    {
        // Set Selection
        gamemenu_selection_gameover = MENU_GAMEOVER_QUIT;
        
        // Return to Home Menu / Homebrew Launcher
        gamemenu_next_submenu = GAMEMENU_QUIT;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
    
    /* Buttons */
    
    // Movement
    if (kdown & KEY_UP && gamemenu_selection_gameover == MENU_GAMEOVER_QUIT)
    {
        // Set Selection
        gamemenu_selection_gameover = MENU_GAMEOVER_CONTINUE;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
    else if (kdown & KEY_DOWN && gamemenu_selection_gameover == MENU_GAMEOVER_CONTINUE)
    {
        // Set Selection
        gamemenu_selection_gameover = MENU_GAMEOVER_QUIT;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
    
    // Buttons
    if (kdown & KEY_A)
    {
        // Choose selected Option
        if (gamemenu_selection_gameover == MENU_GAMEOVER_QUIT)
        {
            // Return to Home Menu / Homebrew Launcher
            gamemenu_next_submenu = GAMEMENU_QUIT;
            
            // Reset GameOver Selection
            gamemenu_selection_gameover = MENU_GAMEOVER_CONTINUE;
        }
        else
        {
            // Restart from last Save
            gamemenu_next_submenu = GAMEMENU_START;
        }
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_SHOOT, SOUNDTYPE_PLAYER_SHOOTING, false, true);
    }
    else if (kdown & KEY_B)
    {
        // Return to Home Menu / Homebrew Launcher
        gamemenu_next_submenu = GAMEMENU_QUIT;
        
        // Play Sound
        pSound->sound_play_fx(SOUND_PLAYER_STANDINGMOVE, SOUNDTYPE_PLAYER_MOVING, false, true);
    }
}

void GameMenu::gamemenu_warning_screen()
{
    // Display Warning about Software for a certain time then advance to splash screen
    if (time(NULL) > gamemenu_warning_timer)
    {
        // Advance to Splash Screen
        gamemenu_next_submenu = GAMEMENU_SPLASH;
    }
}

/*************************************************************************************************************************/
/*                                                 Menu Info Setters                                                     */
/*************************************************************************************************************************/

bool GameMenu::gamemenu_get_submenu_switch()
{
    // Change our current Submenu to the next one
    if (gamemenu_submenu != gamemenu_next_submenu)
    {
        return true;
    }
    else
    {
        return false;
    }
}
