//
//  menu.h
//  3ds_metroid
//
//  Created by jbr373 on 16/01/16.
//

#ifndef menu_h
#define menu_h

#include <3ds.h>
#include <stdio.h>
#include <time.h>

#include "file_operations.h"
#include "files.h"
#include "savefile.h"
#include "sound.h"

enum GameSubMenues
{
    GAMEMENU_BOOT = 0,
    GAMEMENU_WARNING = 1,
    GAMEMENU_SPLASH = 2,
    GAMEMENU_SAVEFILES = 3,
    GAMEMENU_QUIT = 4,
    GAMEMENU_START = 5,
    GAMEMENU_GAMEOVER = 6,
    GAMEMENU_OPTIONS = 7,
    GAMEMENU_TRANSITION = 8
};

enum GameMenuText
{
    TEXT_SPLASH_START = 0,
    TEXT_SAVEFILE_NEW = 1,
    TEXT_SAVEFILE_CONTINUE = 2,
    TEXT_GAMEOVER_GAMEOVER = 3,
    TEXT_GAMEOVER_CONTINUE = 4,
    TEXT_GAMEOVER_QUIT = 5,
    TEXT_SAVEFILE_OPTIONS = 6,
    TEXT_SAVEFILE_FILE = 7,
    TEXT_SAVEFILE_METROIDS = 8,
    TEXT_SAVEFILE_MISSLES = 9,
    TEXT_OPTIONS_MOVEMENT = 10,
    TEXT_MOVEMENT_ORIGINAL = 11,
    TEXT_MOVEMENT_DPAD = 12,
    TEXT_OPTIONS_SHOOTING = 13,
    TEXT_SHOOTING_ORIGINAL = 14,
    TEXT_SHOOTING_Y = 15,
    TEXT_OPTIONS_MAP = 16,
    TEXT_MAP_AUTO = 17,
    TEXT_MAP_MANUAL = 18,
    TEXT_OPTIONS_RETURN = 19
};

class GameMenu : private FileOperations
{
public:
    GameMenu();
    virtual ~GameMenu();
    
    u8 gamemenu_main(SaveFile* pSaveFile, Sound* pSound, s32 kdown, touchPosition touch_pos);
    u8 gamemenu_gameover(Sound* pSound, u32 kdown, touchPosition touch_pos);
    void gamemenu_clear_sprites();
    
    // Getters
    sf2d_texture* gamemenu_get_sprite() { return pGameMenu_Sprite; };
    std::string gamemenu_get_text_string(u8 text_slot) { return fileop_read_string_text(pGameMenu_Text, text_slot); };
    ConfigFileStruct* gamemenu_get_config_file() { return pGameConfig; };
    
    u8 gamemenu_get_submenu() { return gamemenu_submenu; };
    u8 gamemenu_get_next_submenu() { return gamemenu_next_submenu; };
    u8 gamemenu_get_selection() { return gamemenu_selection; };
    u8 gamemenu_get_selection_options() { return gamemenu_selection_options; };
    u8 gamemenu_get_selection_gameover() { return gamemenu_selection_gameover; };
    
    s32 gamemenu_get_text_x(u8 text_slot) { return fileop_read_string_x(pGameMenu_Text, text_slot); };
    s32 gamemenu_get_text_y(u8 text_slot) { return fileop_read_string_y(pGameMenu_Text, text_slot); };
    
    bool gamemenu_is_main_menu() { return gamemenu_is_main; };
    bool gamemenu_get_input_block() { return gamemenu_ignore_input; };
    bool gamemenu_get_submenu_switch();
    
    // Setters
    void gamemenu_set_input_block(bool block) { gamemenu_ignore_input = block; };
    void gamemenu_update_next_submenu() { gamemenu_submenu = gamemenu_next_submenu; };
    void gamemenu_set_next_submenu(u8 submenu) { gamemenu_next_submenu = submenu; };
    void gamemenu_set_submenu(u8 submenu) { gamemenu_submenu = submenu; };
    void gamemenu_set_gameover(bool value) { gamemenu_is_main = value; };
    
private:
    // Loading
    void gamemenu_load_textures();
    
    void gamemenu_warning_screen();
    void gamemenu_splash_screen(Sound* pSound, SaveFile* pSaveFile, s32 kdown, touchPosition touch_pos);
    void gamemenu_saves_screen(SaveFile* pSaveFile, Sound* pSound, s32 kdown, touchPosition touch_pos);
    void gamemenu_options_screen(Sound* pSound, s32 kdown, touchPosition touch_pos);
    void gamemenu_gameover_screen(Sound* pSound, s32 kdown, touchPosition touch_pos);
    
    // Menues
    u8 gamemenu_selection;
    u8 gamemenu_selection_options;
    u8 gamemenu_selection_gameover;
    u8 gamemenu_submenu;
    u8 gamemenu_next_submenu;
    
    // Warning Screen Timer
    time_t gamemenu_warning_timer;
    
    // Flags
    bool gamemenu_is_main;
    bool gamemenu_ignore_input;
    bool gamemenu_started_bgm;
    
    // Data Types
    sf2d_texture* pGameMenu_Sprite;
    StringFileStruct* pGameMenu_Text;
    ConfigFileStruct* pGameConfig;
};

#endif /* menu_h */
