//
//  menu_game.cpp
//  3ds_metroid
//
//  Created by jbr373 on 24/01/16.
//

#include <sfil.h>

#include "menu_game.h"
#include "filepaths.h"

#define MAX_TEXLINE_SPRITE              4

#define TEXT_ID_UPGRADES                2

enum TextSlots
{
    TEXTSLOT_BEAMS = 0,
    TEXTSLOT_CHARGE = 1,
    TEXTSLOT_ICE = 2,
    TEXTSLOT_WAVE = 3,
    TEXTSLOT_SPAZER = 4,
    TEXTSLOT_PLASMA = 5,
    TEXTSLOT_LAUNCHER = 6,
    TEXTSLOT_MISSLE = 7,
    TEXTSLOT_SUIT = 8,
    TEXTSLOT_BARRIER = 9,
    TEXTSLOT_MISC = 10,
    TEXTSLOT_MORPHBALL = 11,
    TEXTSLOT_BOMB = 12,
    TEXTSLOT_SPRINGBALL = 13,
    TEXTSLOT_SPIDERBALL = 14,
    TEXTSLOT_BOOTS = 15,
    TEXTSLOT_HIGHJUMP = 16,
    TEXTSLOT_SPACEJUMP = 17,
    TEXTSLOT_SCREW = 18
};

enum DimensionData
{
    // For all Icons
    ICON_WIDTH = 37,
    ICON_HEIGHT = 35,
    
    // Top Bottom Menu
    BG_TOP_WIDTH = 320,
    BG_TOP_HEIGHT = 49,
    
    // Lower Bottom Menu
    BG_BTM_WIDTH = 320,
    BG_BTM_HEIGHT = 55,
    
    // Beams
    BEAMSWITCHER_WIDTH = 178,
    BEAMSWITCHER_HEIGHT = 35,
    
    // Beams
    BEAMS_WIDTH = 46,
    BEAMS_HEIGHT = 40,
    
    // Upgrades
    UPGRADES_WIDTH = 72,
    UPGRADES_HEIGHT = 49,
    
    // Map Scrolling Area
    MAPSCROLL_WIDTH = 320,
    MAPSCROLL_HEIGHT = 135,
};

enum BeamPositions
{
    BEAMSWITCHERPOS_Y = 7, // Was 9
    BEAMSWITCHERPOS_X = 129,
    
    BEAMPOS_ICE_X = 129,
    BEAMPOS_WAVE_X = 176,
    BEAMPOS_SPAZER_X = 223,
    BEAMPOS_PLASMA_X = 270
};

enum NumberPositions
{
    NUMBERPOS_Y = 15,
    NUMBERPOS_X = 71,
};

enum StatusPositions
{
    STATUSPOS_UPGRADES_Y = 88,
    STATUSPOS_BEAMS_X = 66,
    STATUSPOS_ENCHANCMENTS_X = 206,
    STATUSPOS_MAPSCROLL_Y = 50,
};

BottomMenu::BottomMenu()
{
    // Current Bottom Menu State
    gamemenu_submenu = SUBMENU_MAP;
    
    // Menu Control
    gamemenu_input_blocked = false;
    gamemenu_minimap_follow_auto = false;
    gamemenu_minimap_moved_manual = false;
    gamemenu_actions = BOTTOMMENU_SETMAP;
    
    // Selection Icon
    gamemenu_sel_icon_x = BEAMSWITCHERPOS_X;
    gamemenu_sel_icon_y = BEAMSWITCHERPOS_Y;

    // Minimap File
    pMiniMapFileStruct = new MiniMapFileStruct();
    pMiniMapFileStruct->minimapfile_loaded = false;
    
    // Minimap Coordinates
    gamemenu_minimap_shift_x = 10;
    gamemenu_minimap_shift_y = 10;
    gamemenu_minimap_autofollow_x = 0;
    gamemenu_minimap_autofollow_y = 0;
    gamemenu_last_touch_pos_x = 0;
    gamemenu_last_touch_pos_y = 0;
    
    // Load Sprites
    gamemenu_load_sprites();
    
    // Load Text
    pStringFileStruct = new StringFileStruct();
    fileop_load_strings(pStringFileStruct, TEXT_ID_UPGRADES);
}

BottomMenu::~BottomMenu()
{
    if (pMiniMapFileStruct != NULL)
    {
        delete pMiniMapFileStruct;
    }
    
    if (pStringFileStruct != NULL)
    {
        delete pStringFileStruct;
    }
}

void BottomMenu::gamemenu_load_sprites()
{
    /* Sprite Classes */

    // Create Sprite Class
    gamemenu_sprites = new SpriteData();
    
    /* Menu Sprites */
 
    // Load Menu Backgrounds
    gamemenu_sprites->spritedata_texture[MENU_TEXLINE_TOPBG] = sfil_load_PNG_file(IMG_BTMM_TOP_BG, SF2D_PLACE_RAM);
    gamemenu_sprites->spritedata_texture[MENU_TEXLINE_BTMBG] = sfil_load_PNG_file(IMG_BTMM_BTM_BG, SF2D_PLACE_RAM);
    gamemenu_sprites->spritedata_texture[MENU_TEXLINE_BEAMS] = sfil_load_PNG_file(IMG_BTMM_STATUS_BEAM, SF2D_PLACE_RAM);
    gamemenu_sprites->spritedata_texture[MENU_TEXLINE_SUIT] = sfil_load_PNG_file(IMG_BTMM_STATUS_SUIT, SF2D_PLACE_RAM);
    gamemenu_sprites->spritedata_texture[MENU_TEXLINE_BEAMSWITCHER] = sfil_load_PNG_file(IMG_BTMM_BEAMSWITCHER, SF2D_PLACE_RAM);

    // Frame Info
    gamemenu_sprites->spritedata_frame = 0;
    gamemenu_sprites->spritedata_startframe = 0;
    gamemenu_sprites->spritedata_maxframe = 0;
    gamemenu_sprites->spritedata_frame_speed_current = 0;
    gamemenu_sprites->spritedata_frame_speed = 0;
    
    // Start on Map Background
    gamemenu_sprites->spritedata_texture_line = MENU_TEXLINE_TOPBG;
    gamemenu_sprites->spritedata_w = BG_TOP_WIDTH;
    gamemenu_sprites->spritedata_h = BG_TOP_HEIGHT;
}

u8 BottomMenu::gamemenu_get_max_texline()
{
    // Return Texline Max
    return MAX_TEXLINE_SPRITE;
}

void BottomMenu::gamemenu_reload_minimap(u8 stage_phase)
{
    // Load Minimap
    fileop_load_minimap(pMiniMapFileStruct, stage_phase);
    
    // Reset Last Touch Position
    gamemenu_last_touch_pos_x = 0;
    gamemenu_last_touch_pos_y = 0;
    
    // Reset Minimap Position Shift
    gamemenu_minimap_shift_x = 0;
    gamemenu_minimap_shift_y = 0;
}

void BottomMenu::gamemenu_force_minimap_shift(u32 minimap_room_id)
{
    // Reset if moved manually
    if (gamemenu_minimap_moved_manual == true)
    {
        // Reset Minimap Shift
        gamemenu_minimap_shift_x = gamemenu_minimap_autofollow_x;
        gamemenu_minimap_shift_y = gamemenu_minimap_autofollow_y;
        
        // Reset Manual Movement Flag
        gamemenu_minimap_moved_manual = false;
    }
    
    // Set Follow X
    if (pMiniMapFileStruct->minimapfile_room_x[minimap_room_id / 2] < pMiniMapFileStruct->minimapfile_room_x[minimap_room_id])
    {
        gamemenu_minimap_shift_x -= pMiniMapFileStruct->minimapfile_room_x[minimap_room_id] - pMiniMapFileStruct->minimapfile_room_x[minimap_room_id / 2];
        gamemenu_minimap_autofollow_x = gamemenu_minimap_shift_x;
    }
    else if (pMiniMapFileStruct->minimapfile_room_x[minimap_room_id / 2] > pMiniMapFileStruct->minimapfile_room_x[minimap_room_id])
    {
        gamemenu_minimap_shift_x += pMiniMapFileStruct->minimapfile_room_x[minimap_room_id / 2] - pMiniMapFileStruct->minimapfile_room_x[minimap_room_id];
        gamemenu_minimap_autofollow_x = gamemenu_minimap_shift_x;
    }
    
    // Set Follow Y
    if (pMiniMapFileStruct->minimapfile_room_y[minimap_room_id / 2] < pMiniMapFileStruct->minimapfile_room_y[minimap_room_id])
    {
        gamemenu_minimap_shift_y -= pMiniMapFileStruct->minimapfile_room_y[minimap_room_id] - pMiniMapFileStruct->minimapfile_room_y[minimap_room_id / 2];
        gamemenu_minimap_autofollow_y = gamemenu_minimap_shift_y;
    }
    else if (pMiniMapFileStruct->minimapfile_room_y[minimap_room_id / 2] > pMiniMapFileStruct->minimapfile_room_y[minimap_room_id])
    {
        gamemenu_minimap_shift_y += pMiniMapFileStruct->minimapfile_room_y[minimap_room_id / 2] - pMiniMapFileStruct->minimapfile_room_y[minimap_room_id];
        gamemenu_minimap_autofollow_y = gamemenu_minimap_shift_y;
    }
}

SpriteData* BottomMenu::gamemenu_get_sprite(u8 texline)
{
    // Set Tex Line
    gamemenu_sprites->spritedata_texture_line = texline;
       
    // Set matching Width and Height
    if (texline == MENU_TEXLINE_TOPBG)
    {
        gamemenu_sprites->spritedata_w = BG_TOP_WIDTH;
        gamemenu_sprites->spritedata_h = BG_TOP_HEIGHT;
    }
    else if (texline == MENU_TEXLINE_BTMBG)
    {
        gamemenu_sprites->spritedata_w = BG_BTM_WIDTH;
        gamemenu_sprites->spritedata_h = BG_BTM_HEIGHT;
    }
    else if (texline == MENU_TEXLINE_BEAMS)
    {
        gamemenu_sprites->spritedata_w = BEAMS_WIDTH;
        gamemenu_sprites->spritedata_h = BEAMS_HEIGHT;
    }
    else if (texline == MENU_TEXLINE_BEAMSWITCHER)
    {
        gamemenu_sprites->spritedata_w = BEAMSWITCHER_WIDTH;
        gamemenu_sprites->spritedata_h = BEAMSWITCHER_HEIGHT;
    }
    else if (texline == MENU_TEXLINE_SEL)
    {
        gamemenu_sprites->spritedata_w = ICON_WIDTH;
        gamemenu_sprites->spritedata_h = ICON_HEIGHT;
    }
    else
    {
        gamemenu_sprites->spritedata_w = UPGRADES_WIDTH;
        gamemenu_sprites->spritedata_h = UPGRADES_HEIGHT;
    }
    
    // Block wrong call
    if (texline > MAX_TEXLINE_SPRITE)
        return NULL;
    
    // Return Sprite Data Pointer
    return gamemenu_sprites;
}

s32 BottomMenu::gamemenu_get_sprite_x(u8 texline)
{
    // X Position
    if (texline == MENU_TEXLINE_TOPBG || texline == MENU_TEXLINE_BTMBG)
        return 0;
    else if (texline == MENU_TEXLINE_BEAMS)
        return STATUSPOS_BEAMS_X;
    else if (texline == MENU_TEXLINE_BEAMSWITCHER)
        return BEAMSWITCHERPOS_X;
    else if (texline == MENU_TEXLINE_SEL)
        return gamemenu_sel_icon_x;
    else if (texline == ICON_TEXLINE_NUMONE)
        return NUMBERPOS_X;
    else
        return STATUSPOS_ENCHANCMENTS_X;
}

s32 BottomMenu::gamemenu_get_sprite_y(u8 texline)
{
    // X Position
    if (texline == MENU_TEXLINE_TOPBG)
        return TOPBGPOS_Y;
    else if (texline == MENU_TEXLINE_BTMBG)
        return BTMBGPOS_Y;
    else if (texline == MENU_TEXLINE_BEAMSWITCHER)
        return BEAMSWITCHERPOS_Y;
    else if (texline == MENU_TEXLINE_SEL)
        return gamemenu_sel_icon_y;
    else if (texline == ICON_TEXLINE_NUMONE)
        return NUMBERPOS_Y;
    else
        return STATUSPOS_UPGRADES_Y;
}

BottomMenuActions BottomMenu::gamemenu_action(touchPosition touch_pos)
{
    /* Return Action depending on Touch Position */
    
    if (touch_pos.py >= BEAMSWITCHERPOS_Y && touch_pos.py <= BEAMSWITCHERPOS_Y + ICON_HEIGHT)
    {
        /* Beams */
         
        if (touch_pos.px >= BEAMPOS_ICE_X && touch_pos.px <= BEAMPOS_ICE_X + ICON_WIDTH)
            gamemenu_actions = BOTTOMMENU_ICEBEAM;
        else if (touch_pos.px >= BEAMPOS_WAVE_X && touch_pos.px <= BEAMPOS_WAVE_X + ICON_WIDTH)
            gamemenu_actions = BOTTOMMENU_WAVEBEAM;
        else if (touch_pos.px >= BEAMPOS_SPAZER_X && touch_pos.px <= BEAMPOS_SPAZER_X + ICON_WIDTH)
            gamemenu_actions = BOTTOMMENU_SPAZERBEAM;
        else if (touch_pos.px >= BEAMPOS_PLASMA_X && touch_pos.px <= BEAMPOS_PLASMA_X + ICON_WIDTH)
            gamemenu_actions = BOTTOMMENU_PLASMABEAM;
    }
    else if (touch_pos.py >= SUBMENUPOS_ALL_Y && touch_pos.py <= SUBMENUPOS_ALL_Y + ICON_HEIGHT)
    {
        /* Map Screens */
        
        if (touch_pos.px >= SUBMENUPOS_STATUS_X && touch_pos.px <= SUBMENUPOS_STATUS_X + ICON_WIDTH)
        {
            if (gamemenu_actions != BOTTOMMENU_SETSTATUS)
                gamemenu_actions = BOTTOMMENU_SETSTATUS;
        }
        else if (touch_pos.px >= SUBMENUPOS_MAP_X && touch_pos.px <= SUBMENUPOS_MAP_X + ICON_WIDTH)
        {
            if (gamemenu_actions != BOTTOMMENU_SETMAP)
                gamemenu_actions = BOTTOMMENU_SETMAP;
        }
    }
    
    // Perform Map Shift on Map State
    if (gamemenu_submenu == SUBMENU_MAP)
    {
        gamemenu_get_minimap_shift(touch_pos);
    }
    
    return gamemenu_actions;
}

void BottomMenu::gamemenu_get_minimap_shift(touchPosition touch_pos)
{
    // Get Touch Input
    if (touch_pos.px == 0 && touch_pos.py == 0)
    {
        // Reset Touch Position when we're not getting any Input
        gamemenu_last_touch_pos_x = 0;
        gamemenu_last_touch_pos_y = 0;
    }
    else
    {
        // Check if we're inside the Map Moving Area
        if (touch_pos.px > 0 && touch_pos.px < MAPSCROLL_WIDTH && touch_pos.py > STATUSPOS_MAPSCROLL_Y && touch_pos.py < STATUSPOS_MAPSCROLL_Y + MAPSCROLL_HEIGHT)
        {
            // Last Touch Positions have been set again
            if (gamemenu_last_touch_pos_x != 0 && gamemenu_last_touch_pos_y != 0)
            {
                // Shift X
                if (gamemenu_minimap_shift_x + (touch_pos.px - gamemenu_last_touch_pos_x) < 0 &&
                    gamemenu_minimap_shift_x + (touch_pos.px - gamemenu_last_touch_pos_x) > pMiniMapFileStruct->minimapfile_x_max * (-1))
                {
                    gamemenu_minimap_shift_x += touch_pos.px - gamemenu_last_touch_pos_x;
                }
        
                // Shift Y
                if (gamemenu_minimap_shift_y + (touch_pos.py - gamemenu_last_touch_pos_y) < 0 &&
                    gamemenu_minimap_shift_y + (touch_pos.py - gamemenu_last_touch_pos_y) > pMiniMapFileStruct->minimapfile_y_max * (-1))
                {
                    gamemenu_minimap_shift_y += touch_pos.py - gamemenu_last_touch_pos_y;
                }
                
                // We Moved the Minimap, flag for auto-follow
                if (gamemenu_minimap_follow_auto == true)
                {
                    gamemenu_minimap_moved_manual = true;
                }
            
                // Set last Touch Position
                gamemenu_last_touch_pos_x = touch_pos.px;
                gamemenu_last_touch_pos_y = touch_pos.py;
            }
            else // Last Touch Positions are still reset
            {
                // Set last Touch Position
                gamemenu_last_touch_pos_x = touch_pos.px;
                gamemenu_last_touch_pos_y = touch_pos.py;
            }
        }
        else
        {
            // Reset Touch Position if we're outside the scrollable area
            gamemenu_last_touch_pos_x = 0;
            gamemenu_last_touch_pos_y = 0;
        }
    }
}

u8 BottomMenu::gamemenu_get_minimap_slot(u32 flag)
{
    u8 minimap_slot = 0;
    
    while (pMiniMapFileStruct->minimapfile_room_id[minimap_slot] != flag)
    {
        ++minimap_slot;
    }
    
    return minimap_slot;
}

void BottomMenu::gamemenu_set_sel_pos(u8 active_beam)
{
    // Set X
    switch (active_beam)
    {
        case BOTTOMMENU_ICEBEAM:
            gamemenu_sel_icon_x = BEAMPOS_ICE_X;
            break;
        case BOTTOMMENU_WAVEBEAM:
            gamemenu_sel_icon_x = BEAMPOS_WAVE_X;
            break;
        case BOTTOMMENU_SPAZERBEAM:
            gamemenu_sel_icon_x = BEAMPOS_SPAZER_X;
            break;
        case BOTTOMMENU_PLASMABEAM:
            gamemenu_sel_icon_x = BEAMPOS_PLASMA_X;
            break;
            
        default:
            gamemenu_sel_icon_x = BEAMPOS_ICE_X;
            break;
    }
    
    // Set Y
    gamemenu_sel_icon_y = BEAMSWITCHERPOS_Y;
}

u8 BottomMenu::gamemenu_get_stringcount()
{
    if (pStringFileStruct->stringfile_loaded == false)
    {
        return 0;
    }
    else
    {
        return pStringFileStruct->text_count;
    }
}

std::string BottomMenu::gamemenu_get_upgrade_string(SaveFile* pSaveFile, u8 text_slot)
{
    // Not aquired Upgrade
    std::string upgrade_text;

    if (text_slot > pStringFileStruct->text_count - 1 || pStringFileStruct->stringfile_loaded == false)
    {
        upgrade_text = "NONE";
    }
    else
    {
        switch (text_slot)
        {
            case TEXTSLOT_ICE:
                if (pSaveFile->savefile_get_icebeam() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
            case TEXTSLOT_WAVE:
                if (pSaveFile->savefile_get_wavebeam() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
            case TEXTSLOT_SPAZER:
                if (pSaveFile->savefile_get_spazerbeam() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
            case TEXTSLOT_PLASMA:
                if (pSaveFile->savefile_get_plasmabeam() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
            case TEXTSLOT_BARRIER:
                if (pSaveFile->savefile_get_barriersuit() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
            case TEXTSLOT_BOMB:
                if (pSaveFile->savefile_get_bombs() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
            case TEXTSLOT_SPRINGBALL:
                if (pSaveFile->savefile_get_springball() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
            case TEXTSLOT_SPIDERBALL:
                if (pSaveFile->savefile_get_spiderball() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
            case TEXTSLOT_HIGHJUMP:
                if (pSaveFile->savefile_get_highjump() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
            case TEXTSLOT_SPACEJUMP:
                if (pSaveFile->savefile_get_spacejump() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
            case TEXTSLOT_SCREW:
                if (pSaveFile->savefile_get_screwattack() == true)
                    upgrade_text = pStringFileStruct->text_string[text_slot];
                else
                    upgrade_text = "NONE";
                break;
                
            default:
                upgrade_text = pStringFileStruct->text_string[text_slot];
                break;
        }
    }
    
    return upgrade_text;
}

s32 BottomMenu::gamemenu_get_upgrade_x(u8 text_slot)
{
    if (text_slot > pStringFileStruct->text_count - 1 || pStringFileStruct->stringfile_loaded == false)
    {
        return -400;
    }
    else
    {
        return pStringFileStruct->text_x[text_slot];
    }
}

s32 BottomMenu::gamemenu_get_upgrade_y(u8 text_slot)
{
    if (text_slot > pStringFileStruct->text_count - 1 || pStringFileStruct->stringfile_loaded == false)
    {
        return -240;
    }
    else
    {
        return pStringFileStruct->text_y[text_slot];
    }
}
