//
//  menu_game.h
//  3ds_metroid
//
//  Created by jbr373 on 24/01/16.
//

#ifndef menu_game_h
#define menu_game_h

#include <3ds.h>
#include <stdio.h>

#include "image.h"
#include "files.h"
#include "file_operations.h"
#include "savefile.h"

enum SubMenus
{
    SUBMENU_MAP = 0,
    SUBMENU_STATUS = 1
};

enum TextureLinesMenu
{
    MENU_TEXLINE_TOPBG = 0,
    MENU_TEXLINE_BTMBG = 1,
    MENU_TEXLINE_BEAMS = 2,
    MENU_TEXLINE_SUIT = 3,
    MENU_TEXLINE_BEAMSWITCHER = 4,
    
    // Blocked by Get Max, but used by X
    ICON_TEXLINE_NUMONE = 7,
    ICON_TEXLINE_NUMTWO = 8,
    MENU_TEXLINE_SEL = 9
};

enum BottomMenuActions
{
    BOTTOMMENU_NOCHANGE = 0,
    
    BOTTOMMENU_SETMAP = 1,
    BOTTOMMENU_SETSTATUS = 2,
    BOTTOMMENU_ICEBEAM = 3,
    BOTTOMMENU_WAVEBEAM = 4,
    BOTTOMMENU_SPAZERBEAM = 5,
    BOTTOMMENU_PLASMABEAM = 6,
};

enum SubMenuPositions
{
    TOPBGPOS_X = 0,
    TOPBGPOS_Y = 0,
    
    BTMBGPOS_X = 0,
    BTMBGPOS_Y = 185,
    
    SUBMENUPOS_ALL_Y = 200,
    SUBMENUPOS_MAP_X = 11,
    SUBMENUPOS_STATUS_X = 272
};

class BottomMenu : private FileOperations
{
public:
    BottomMenu();
    virtual ~BottomMenu();
    
    BottomMenuActions gamemenu_action(touchPosition touch_pos);
    void gamemenu_set_sel_pos(u8 active_beam);
    void gamemenu_reload_minimap(u8 stage_phase);
    void gamemenu_force_minimap_shift(u32 minimap_room_id);
    
    // Getters
    SpriteData* gamemenu_get_sprite(u8 texline);
    MiniMapFileStruct* gamemenu_get_minimap() { return pMiniMapFileStruct; };
    
    std::string gamemenu_get_upgrade_string(SaveFile* pSaveFile, u8 text_slot);
    
    u8 gamemenu_get_submenu() { return gamemenu_submenu; };
    u8 gamemenu_get_action() { return (u8)gamemenu_actions; };
    u8 gamemenu_get_max_texline();
    u8 gamemenu_get_minimap_slot(u32 flag);
    u8 gamemenu_get_stringcount();
    
    bool gamemenu_get_minimap_follow_auto() { return gamemenu_minimap_follow_auto; };
    
    s32 gamemenu_get_sprite_x(u8 texline);
    s32 gamemenu_get_sprite_y(u8 texline);
    s32 gamemenu_get_minimap_x() { return gamemenu_minimap_shift_x; };
    s32 gamemenu_get_minimap_y() { return gamemenu_minimap_shift_y; };
    s32 gamemenu_get_upgrade_x(u8 text_slot);
    s32 gamemenu_get_upgrade_y(u8 text_slot);
    
    // Setters
    void gamemenu_set_submenu(u8 submenu) { gamemenu_submenu = submenu; };
    void gamemenu_set_action(u8 action) { gamemenu_actions = (BottomMenuActions)action; };
    
    void gamemenu_set_minimap_follow_auto(bool enable) { gamemenu_minimap_follow_auto = enable; };
    
    void gamemenu_set_minimap_x(s32 x) { gamemenu_minimap_shift_x = x; };
    void gamemenu_set_minimap_y(s32 y) { gamemenu_minimap_shift_y = y; };
    
private:
    void gamemenu_load_sprites();
    void gamemenu_get_minimap_shift(touchPosition touch_pos);
    
    // General
    u8 gamemenu_submenu;
    
    bool gamemenu_input_blocked;
    bool gamemenu_minimap_follow_auto;
    bool gamemenu_minimap_moved_manual;
    
    s32 gamemenu_sel_icon_x;
    s32 gamemenu_sel_icon_y;
    
    s32 gamemenu_minimap_shift_x;
    s32 gamemenu_minimap_shift_y;
    s32 gamemenu_minimap_autofollow_x;
    s32 gamemenu_minimap_autofollow_y;
    s32 gamemenu_last_touch_pos_x;
    s32 gamemenu_last_touch_pos_y;
    
    BottomMenuActions gamemenu_actions;
    
    SpriteData* gamemenu_sprites;
    MiniMapFileStruct* pMiniMapFileStruct;
    StringFileStruct* pStringFileStruct;
};

#endif /* menu_game_h */
