//
//  object.cpp
//  3ds_metroid
//
//  Created by jbr373 on 21/01/16.
//

#include <stdlib.h>
#include <string>
#include <fstream>
#include <sfil.h>

#include "object.h"
#include "filepaths.h"

/*
 *      TODO
 * ---------------
 * - This Chain of Read and then Set Filepos is ugly, implement special load function to class FileOperations
 */

Object::Object()
{
    // General Item Info
    object_phase = 0;
    object_type = OBJECTTYPE_NONE;
    object_value_a = 0;
    object_value_b = 0;
    object_value_c = 0;
    
    // Coordinates
    object_x = 0;
    object_y = 0;
    object_width = 0;
    object_height = 0;
    object_size = 1;
    
    // Sprite Info
    object_sprite_path = LINE_DO_NOT_USE;
    object_sprite_width = 0;
    object_sprite_height = 0;
    object_sprite_frame_speed = 0;
    object_max_spriteline = 0;
    object_anim = OBJECT_SPRITELINE_IDLE;
    object_frame = 0;
    object_frame_count = 0;
 
    // Flags
    object_destroy = false;
    object_destroy_completed = false;
    object_draw_shadow = false;
    object_collision = false;
}

Object::~Object()
{

}

void Object::object_load(ObjectFileStruct* pObjectFileStruct, s32 new_object_x, s32 new_object_y)
{
    // General Item Info
    object_x = new_object_x;
    object_y = new_object_y;

    /* Load Item File */

    // Read Save File
    if (pObjectFileStruct != NULL)
    {
        /* Write Values to Class and delete the struct */
        
        object_size = pObjectFileStruct->object_size;
        
        object_phase = pObjectFileStruct->object_phase;
        object_type = pObjectFileStruct->object_type;
        
        object_value_a = pObjectFileStruct->object_value_a;
        object_value_b = pObjectFileStruct->object_value_b;
        object_value_c = pObjectFileStruct->object_value_c;
        
        object_respawn_time = pObjectFileStruct->object_value_b;
        
        // NOTE: This might seem strange, but is useful for scaling from Sprite!
        object_width = pObjectFileStruct->object_hitbox_width;
        object_height = pObjectFileStruct->object_hitbox_height;

        object_draw_shadow = pObjectFileStruct->object_shadow;
        
        object_light_color = pObjectFileStruct->object_light_color;
        object_light_size = pObjectFileStruct->object_light_size;
        object_light_spawn_x = pObjectFileStruct->object_light_spawn_x;
        object_light_spawn_y = pObjectFileStruct->object_light_spawn_y;
        
        object_collision = pObjectFileStruct->object_collision;
        
        object_loot_chance = pObjectFileStruct->object_loot_chance;
        object_loot = pObjectFileStruct->object_loot;
        object_loot_spawn_x = pObjectFileStruct->object_loot_spawn_x;
        object_loot_spawn_y = pObjectFileStruct->object_loot_spawn_y;
        
        object_sprite_path = pObjectFileStruct->object_spritesheet;
        object_sprite_width = pObjectFileStruct->object_spritesheet_width;
        object_sprite_height = pObjectFileStruct->object_spritesheet_height;
        object_sprite_frame_speed = pObjectFileStruct->object_max_frame_speed;
        object_max_spriteline = pObjectFileStruct->object_max_spritelines;
        
        // Check Hitbox Scale
        object_update_hitbox();
    }
    else
    {
        // Move Object out of Room
        object_x = -400;
        object_y = -240;
        object_destroy = true;
        object_destroy_completed = true;
    }
}

void Object::object_action(s32 player_x, s32 player_y, s32 player_w, s32 player_h, u8 player_facing, s32 stage_width, s32 stage_height)
{
    // Check Hitbox Scale
    object_update_hitbox();
    
    // Update Frame
    if (object_frame_count == object_sprite_frame_speed)
    {
        if (object_frame < MAX_SPRITESHEET_FRAMES)
        {
            ++object_frame;
        }
        else
        {
            object_frame = 0;
        }
        
        object_frame_count = 0;
    }
    else
    {
        ++object_frame_count;
    }
    
    // Check for Object Respawn
    if (object_destroy_completed == true && object_value_b > 0)
    {
        if (object_respawn_time == 0)
        {
            object_destroy_completed = false;
            object_frame = 0;
            object_frame_count = 0;
            object_destroy = false;
        }
        else
        {
            --object_respawn_time;
        }
    }

    // Check for Object Destruction
    if (object_destroy == true)
    {
        if (object_frame == MAX_SPRITESHEET_FRAMES)
        {
            object_destroy_completed = true;
            object_anim = OBJECT_SPRITELINE_DESTROY;
        }
    }
    
    // Check Item Bounds
    if (object_collision == true)
    {
        // Set Animation
        if (object_max_spriteline > OBJECT_SPRITELINE_DESTROY && object_destroy == false && object_destroy_completed == false)
        {
            // Set Frame Info
            object_anim = OBJECT_SPRITELINE_USE;
        }
    }
}

bool Object::object_collision_in_range(s32 actor_a_value, s32 actor_b_min, s32 actor_b_max)
{
    // Return Compare Result
    return (actor_a_value >= actor_b_min) && (actor_a_value <= actor_b_max);
}

bool Object::object_is_collision(s32 actor_x, s32 actor_y, s32 actor_w, s32 actor_h, u8 actor_facing)
{
    // Room Triggers
    if (object_type == OBJECTTYPE_ROOMTRIGGER)
    {
        if (actor_x >= object_x && actor_x <= object_x + object_width &&
            actor_y >= object_y && actor_y <= object_y + object_height)
        {
            // Roomtrigger Collision
            return true;
        }
        else
        {
            // No Roomtrigger Collision
            return false;
        }
    }
    
    // No Collision on Destroy Animation or missing Collision Flag
    if (object_destroy == true || object_collision == false)
        return false;
    
    // All other Object Types

    // Compare Positions
    bool x_overlap = object_collision_in_range(actor_x, object_x, object_x + object_width) ||
                     object_collision_in_range(object_x, actor_x, actor_x/* - (actor_w / 2) + actor_w*/);
    
    bool y_overlap = object_collision_in_range(actor_y, object_y, object_y + object_height) ||
                     object_collision_in_range(object_y, actor_y, actor_y/* + actor_h*/);
    
    // Return Result
    if (x_overlap == true && y_overlap == true)
    {
        return true;
    }
    else
    {
        return false;
    }
}

bool Object::object_is_collision(s32 actor_x, s32 actor_y, s32 actor_w, s32 actor_h, u8 actor_facing, bool is_attack, u8 attack_type)
{
    // No Collision on Destroy Animation
    if (object_destroy == true || object_collision == false)
        return false;
    
    // Compare Positions
    bool x_overlap = object_collision_in_range(actor_x, object_x, object_x + object_width) ||
                     object_collision_in_range(object_x, actor_x, actor_x - (actor_w / 2) + actor_w);
    
    bool y_overlap = object_collision_in_range(actor_y, object_y, object_y + object_height) ||
                     object_collision_in_range(object_y, actor_y, actor_y + actor_h);
    
    // Destroy Object
    if ((x_overlap && y_overlap) == true)
    {
        // Flag Destroyed
        if (is_attack == true && object_destroy_completed == false && object_destroy == false)
        {
            if (object_type == OBJECTTYPE_DESTRUCTABLE || object_type == OBJECTTYPE_DESTRUCTABLE_LOOT)
            {
                object_destroy = true;
                object_anim = OBJECT_SPRITELINE_DESTROY;
                object_frame = 0;
                object_sprite_frame_speed = 2;
                object_frame_count = 0;
            }
            else if (object_type == OBJECTTYPE_DESTRUCTABLE_MISSLE && attack_type == OBJECTATTACKTYPE_MISSLE)
            {
                object_destroy = true;
                object_anim = OBJECT_SPRITELINE_DESTROY;
                object_frame = 0;
                object_sprite_frame_speed = 2;
                object_frame_count = 0;
            }
        }
    }
    
    // Return Result
    if (x_overlap == true && y_overlap == true)
    {
        return true;
    }
    else
    {
        return false;
    }
}

void Object::object_update_hitbox()
{
    // Check for special Hitbox sizes
    if (object_size == 0 || object_size == 1 || object_size == 100)
        return;
    
    // Object Hitbox
    object_width = object_width * (object_size / 100);
    object_height = object_height * (object_size / 100);
    
    // Object Loot Addition
    object_loot_spawn_x = object_loot_spawn_x * (object_size / 100);
    object_loot_spawn_y = object_loot_spawn_y * (object_size / 100);
}

void Object::object_set_destroy(bool destroyed)
{
    // Same Value changes nothing
    if (destroyed == object_destroy)
        return;
    
    // Set Value
    object_destroy = destroyed;
    
    if (destroyed == true && object_max_spriteline > OBJECT_SPRITELINE_IDLE)
    {
        // Set Frame Info
        object_anim = OBJECT_SPRITELINE_DESTROY;
        object_frame = 0;
        object_sprite_frame_speed = 2;
        object_frame_count = 0;
    }
    else
    {
        // Set Frame Info
        object_anim = OBJECT_SPRITELINE_IDLE;
        object_frame = 0;
        object_frame_count = 0;
    }
}
