//
//  object.h
//  3ds_metroid
//
//  Created by jbr373 on 21/01/16.
//

#ifndef object_h
#define object_h

#include <3ds.h>
#include <stdio.h>

#include "image.h"
#include "files.h"

enum ObjectTypes
{
    OBJECTTYPE_NONE = 0,                // Spawned Prob for Design          (Value = Collision)
    OBJECTTYPE_ROOMTRIGGER = 1,         // Teleport to next Room            (Value = Room ID)
    OBJECTTYPE_DESTRUCTABLE = 2,        // Something like Blocks (NO LOOT)  (Value A = HP) (Value B = Respawn Time [0 = No Respawn])
    OBJECTTYPE_DESTRUCTABLE_LOOT = 3,   // Destructable with Loot           (Value A = HP) (Value B = Respawn Time [0 = No Respawn])
    OBJECTTYPE_DESTRUCTABLE_MISSLE = 4, // Destructable With Missles        (Value A = HP) (Value B = Respawn Time [0 = No Respawn])
    OBJECTTYPE_USABLE = 5,              // ??? (Not Used)                   (Value = Not Used)
    OBJECTTYPE_SAVESTATION = 6,         // Saves Player                     (Value = Restore HP and/or Missles)
    OBJECTTYPE_DAMAGE = 7,              // Causes Damage                    (Value = Damage)
    OBJECTTYPE_LIGHT = 8                // Spawns Light                     (Value = Not Used)
};

enum ObjectPhases
{
    // Every Phase
    OBJECTPHASE_ALL = 0,
    
    // Only in the selected Phase
    OBJECTPHASE_1 = 1,
    OBJECTPHASE_2 = 2,
    OBJECTPHASE_3 = 3,
    OBJECTPHASE_4 = 4,
    OBJECTPHASE_5 = 5,
    OBJECTPHASE_6 = 6,
    OBJECTPHASE_7 = 7,
    OBJECTPHASE_8 = 8,
    OBJECTPHASE_9 = 9,
    OBJECTPHASE_10 = 10
};

enum ObjectAttackTypes
{
    OBJECTATTACKTYPE_MISS = 0,
    
    OBJECTATTACKTYPE_MELEE = 1,
    OBJECTATTACKTYPE_BEAM = 2,
    OBJECTATTACKTYPE_MISSLE = 3
};

enum ObjectSpriteLines
{
    OBJECT_SPRITELINE_IDLE = 0,
    OBJECT_SPRITELINE_DESTROY = 1,
    OBJECT_SPRITELINE_USE = 2
};

class Object
{
public:
    Object();
    virtual ~Object();
    
    // Object Actions
    void object_clean();
    void object_load(ObjectFileStruct* pObjectFileStruct, s32 new_object_x, s32 new_object_y);
    
    // This can also manupulate Coordinades, depending on the Object Type
    void object_action(s32 player_x, s32 player_y, s32 player_w, s32 player_h, u8 player_facing, s32 stage_width, s32 stage_height);
    
    bool object_is_collision(s32 actor_x, s32 actor_y, s32 actor_w, s32 actor_h, u8 actor_facing, bool is_attack, u8 attack_type);
    bool object_is_collision(s32 actor_x, s32 actor_y, s32 actor_w, s32 actor_h, u8 actor_facing);
    
    // Getters
    u8 object_get_phase() { return object_phase; };
    u8 object_get_type() { return object_type; };
    s32 object_get_respawn_time() { return object_respawn_time; };
    s32 object_get_value_a() { return object_value_a; };
    s32 object_get_value_b() { return object_value_b; };
    s32 object_get_value_c() { return object_value_c; };
    
    s32 object_get_x() { return object_x; };
    s32 object_get_y() { return object_y; };
    s32 object_get_w() { return object_width; };
    s32 object_get_h() { return object_height; };
    u8 object_get_size() { return object_size; };
    
    bool object_is_destroy_set() { return object_destroy; };
    bool object_is_destroy_completed() { return object_destroy_completed; };
    bool object_get_draw_shadow() { return object_draw_shadow; };
    bool object_get_collision() { return object_collision; };
    
    u8 object_get_loot_chance() { return object_loot_chance; };
    std::string object_get_loot() { return object_loot; };
    s32 object_get_loot_spawn_x() { return object_loot_spawn_x; };
    s32 object_get_loot_spawn_y() { return object_loot_spawn_y; };
    
    std::string object_get_sprite_path() { return object_sprite_path; };
    s32 object_get_sprite_width() { return object_sprite_width; };
    s32 object_get_sprite_height() { return object_sprite_height; };
    u8 object_get_sprite_frame_speed() { return object_sprite_frame_speed; };
    u8 object_get_max_spriteline() { return object_max_spriteline; };
    u8 object_get_anim() { return object_anim; };
    u8 object_get_frame() { return object_frame; };
    
    // Setters
    void object_set_phase(u8 phase) { object_phase = phase; };
    void object_set_type(u8 type) { object_type = type; };
    void object_set_respawn_time(s32 frames) { object_respawn_time = frames; };
    
    void object_set_value_a(s32 value) { object_value_a = value; };
    void object_set_value_b(s32 value) { object_value_b = value; };
    void object_set_value_c(s32 value) { object_value_c = value; };
    
    void object_set_loot_chance(u8 percent) { object_loot_chance = percent; };
    void object_set_loot(std::string file) { object_loot = file; };
    void object_set_loot_spawn_x(s32 x) { object_loot_spawn_x = x; };
    void object_set_loot_spawn_y(s32 y) { object_loot_spawn_y = y; };
    
    void object_set_x(s32 x) { object_x = x; };
    void object_set_y(s32 y) { object_y = y; };
    void object_set_w(s32 width) { object_width = width; };
    void object_set_h(s32 height) { object_height = height; };
    void object_set_size(u8 size) { object_size = size; };
    
    void object_set_destroy(bool destroyed);
    void object_set_destroy_completed(bool destroy_completed) { object_destroy_completed = destroy_completed; };
    void object_set_draw_shadow(bool draw_shadow) { object_draw_shadow = draw_shadow; };
    void object_set_collision(bool has_collision) { object_collision = has_collision; };
    
    void object_set_anim(u8 anim) { object_anim = anim; };
    void object_set_frame(u8 frame) { object_frame = frame; };
    
private:
    bool object_collision_in_range(s32 actor_a_value, s32 actor_b_min, s32 actor_b_max);
    
    void object_update_hitbox();
    
    u8 object_phase;
    u8 object_type;
    u8 object_frame;
    u8 object_frame_count;
    u8 object_anim;
    
    u8 object_size;
    
    s32 object_value_a;
    s32 object_value_b;
    s32 object_value_c;
    
    s32 object_x;
    s32 object_y;
    s32 object_width;
    s32 object_height;
    
    s32 object_respawn_time;
    
    bool object_destroy;
    bool object_destroy_completed;
    
    bool object_draw_shadow;
    u8 object_light_color;
    u8 object_light_size;
    s32 object_light_spawn_x; // Addition to Object X
    s32 object_light_spawn_y; // Addition to Y
    
    bool object_collision;
    
    u8 object_loot_chance;
    std::string object_loot;
    s32 object_loot_spawn_x; // Addition to Object X
    s32 object_loot_spawn_y; // Addition to Object Y
    
    std::string object_sprite_path;
    s32 object_sprite_width;
    s32 object_sprite_height;
    u8 object_sprite_frame_speed;
    u8 object_max_spriteline;
};

#endif /* object_h */
