//
//  physics.cpp
//  3ds_metroid
//
//  Created by jbr373 on 21/01/16.
//

#include "physics.h"
#include "stage_collision.h"
#include "actor_facing.h"

/*
 *      TODO
 * ---------------
 * - Jump Up Y checks with Object Height -> new Y is applied wrong
 * - Lots of +1 used because the loops in collision_distance Function return the collision Pixel Y
 * - Slopes fail because of collision_distance pretty often, hacky fix used for now
 */

#define WATER_MOVEMENT_MODIFIER     2

#define JUMP_MOVEMENT_NORMAL        32//33
#define JUMP_MOVEMENT_HIGH          40
#define JUMP_MOVEMENT_SPACE         40

#define JUMP_MOVEMENT_SPEED         4

// This fixes some collision problems with uneven slopes,
//but it causes sort of a "hover" effect on long slopes!
#define RUN_Y_FAILURE_CORRECTION    2 //4

// This is the Sprite Height of the Ledge Grabing Sprite
#define LEDGE_GRAB_Y_CORRECTION     76

// Ledge Climbing Counter
#define LEDGE_CLIMBING_COUNTER      9
#define LEDGE_CLIMB_Y_UP_FRAMES     6
#define LEDGE_CLIMB_ANIM_FRAME      1
#define LEDGE_CLIMB_LOCK_FRAMES     15

// This is the Speed Multiplier when a Jump failes because of Collision
// Used to make sure Players don't get stuck in Collision
// Needed since the current Collision detection is kind of unprecise and done wierdly
#define JUMP_COL_SPEED_MULTIPLIER   3

enum JumpStates
{
    JUMP_NONE = 0,
    JUMP_LIFTOFF = 1,
    JUMP_FALL = 2,
    JUMP_LAND = 3
};

Physics::Physics()
{
    // Counters
    physics_old_player_speed = 0;
    physics_player_jump_counter = 0;
    physics_x_falling_modifier = 0;
    physics_player_jump_state = JUMP_NONE;
    physics_enemy_jump_state = JUMP_NONE;
    physics_object_collision_id = 0;
    physics_enemy_object_collision_id = 0;
    physics_old_player_facing = 0;
    physics_climbing_counter = 0;
    
    // Coordinates
    physics_climbing_y = 0;
    
    // Flags
    physics_player_object_id_set = false;
    physics_player_speed_modified = false;
    physics_enemy_object_id_set = false;
}

Physics::~Physics()
{
    
}

/*************************************************************************************************************************/
/*                                                   Player Physics                                                      */
/*************************************************************************************************************************/

void Physics::physics_player_physics(Player* pPlayer, Stage* pStage, SaveFile* pSaveFile, u8 collision)
{
    /* Water Speed */
    if (pStage->stage_is_in_water(pPlayer->player_get_x(), pPlayer->player_get_y(), pPlayer->player_get_hitbox_height()) == true)
    {
        // Reset Speed if required
        if (physics_player_speed_modified == false)
        {
            physics_old_player_speed = pPlayer->player_get_speed();
            pPlayer->player_set_speed(pPlayer->player_get_speed() / WATER_MOVEMENT_MODIFIER);
            physics_player_speed_modified = true;
        }
    }
    else
    {
        // Reset Speed if required
        if (pPlayer->player_get_speed() != physics_old_player_speed && physics_player_speed_modified == true)
        {
            pPlayer->player_set_speed(physics_old_player_speed);
            physics_player_speed_modified = false;
        }
    }
    
    if (pPlayer->player_get_state() == PLAYERSTATE_CLIMBING)
    {
        physics_player_climbing(pPlayer, pStage, collision);
    }
    else
    {
        /* Running */
        
        physics_player_running(pPlayer, pStage, collision);
        
        /* Jumping */
        
        physics_player_jumping(pPlayer, pStage, pSaveFile, collision);
        
        /* Falling */
        
        physics_player_falling(pPlayer, pStage, pSaveFile, collision);
    }
}

/***************************/
/*         CLIMBING        */
/***************************/

void Physics::physics_player_climbing(Player* pPlayer, Stage* pStage, u8 collision)
{
    // Check for collision hit from falling X addition
    s32 x_ledge = 0;
    
    if (pPlayer->player_get_facing() < 3)
    {
        x_ledge = pPlayer->player_get_x() + pPlayer->player_get_speed();
    }
    else if (pPlayer->player_get_facing() > 2 && pPlayer->player_get_facing() < 6)
    {
        x_ledge = pPlayer->player_get_x() - pPlayer->player_get_speed();
    }

    // Set Y
    if (physics_climbing_counter == 0)
    {
        // Get Y
        physics_climbing_y = pStage->stage_check_specific_collision_distance(x_ledge,
                                                                             pPlayer->player_get_y() + pPlayer->player_get_hitbox_height(),
                                                                             pPlayer->player_get_hitbox_width(),
                                                                             1,
                                                                             pPlayer->player_get_hitbox_height() / 2,
                                                                             COLLISION_DOWN);
        
        // Set X
        pPlayer->player_set_x(x_ledge);
        
        // Set Y
        pPlayer->player_set_y(physics_climbing_y - (pPlayer->player_get_hitbox_height() / 2));
        
        // Increase Y
        physics_climbing_y -= (pPlayer->player_get_hitbox_height() / 2);
        
        // Set Counter
        ++physics_climbing_counter;
    }
    else if (physics_climbing_counter >= LEDGE_CLIMBING_COUNTER - LEDGE_CLIMB_Y_UP_FRAMES && physics_climbing_counter <= LEDGE_CLIMBING_COUNTER - 1)
    {
        // Set X
        pPlayer->player_set_x(x_ledge);
        
        // Set Y
        pPlayer->player_set_y(physics_climbing_y);
        
        // Update Climbing Y
        if (physics_climbing_counter != LEDGE_CLIMBING_COUNTER - 1)
        {
            physics_climbing_y += (pPlayer->player_get_hitbox_height() / 2) / LEDGE_CLIMB_Y_UP_FRAMES;
        }
        
        // Increment Climbing Counter
        ++physics_climbing_counter;
    }
    else if (physics_climbing_counter == LEDGE_CLIMBING_COUNTER)
    {
        // Get Y
        physics_climbing_y = pStage->stage_check_specific_collision_distance(x_ledge,
                                                                             pPlayer->player_get_y() + pPlayer->player_get_hitbox_height(),
                                                                             pPlayer->player_get_hitbox_width(),
                                                                             1,
                                                                             pPlayer->player_get_hitbox_height() / 2,
                                                                             COLLISION_DOWN);
        
        // Set X
        pPlayer->player_set_x(x_ledge);
        
        // Set Y
        pPlayer->player_set_y(physics_climbing_y + 1);
        
        // Set Player State and Animation
        pPlayer->player_set_anim(PLAYERANIM_STANDING);
        pPlayer->player_set_state(PLAYERSTATE_IDLE);
        
        // Reset Counter
        physics_climbing_counter = 0;
    }
    else
    {
        // Set Y
        pPlayer->player_set_y(physics_climbing_y);
        
        // Increment Climbing Counter
        ++physics_climbing_counter;
    }
}

/**************************/
/*         RUNNING        */
/**************************/

void Physics::physics_player_running(Player* pPlayer, Stage* pStage, u8 collision)
{
    /* Running */
    
    if (pPlayer->player_get_state() == PLAYERSTATE_RUNNING && physics_player_jump_state == JUMP_NONE)
    {
        // Normal Movement, no Spiderball
        if (pPlayer->player_get_spiderball() == false)
        {
            /* X Position */
            
            // Set Variables
            s32 x_addition = 0;
            bool block_y = false;
            
            // Running Right
            if (pPlayer->player_get_facing() < FACING_LEFT)
            {
                // Get X
                x_addition = pStage->stage_check_specific_collision_distance(pPlayer->player_get_x(),
                                                                             pPlayer->player_get_y() + (pPlayer->player_get_speed() + 1),
                                                                             pPlayer->player_get_hitbox_width(),
                                                                             pPlayer->player_get_hitbox_height(),
                                                                             pPlayer->player_get_speed(),
                                                                             COLLISION_RIGHT);
                
                physics_old_player_facing = FACING_RIGHT;
            }
            
            // Running Left
            if (pPlayer->player_get_facing() > FACING_RIGHTDOWN && pPlayer->player_get_facing() < FACING_UP)
            {
                // Get X
                x_addition = pStage->stage_check_specific_collision_distance(pPlayer->player_get_x(),
                                                                             pPlayer->player_get_y() + (pPlayer->player_get_speed() + 1),
                                                                             pPlayer->player_get_hitbox_width(),
                                                                             pPlayer->player_get_hitbox_height(),
                                                                             pPlayer->player_get_speed(),
                                                                             COLLISION_LEFT);
                
                physics_old_player_facing = FACING_LEFT;
            }
            
            // Apply Position Change
            if (x_addition == pPlayer->player_get_x())
            {
                // X is the same, no need to check Y on Running Physics
                block_y = true;
            }
            else
            {
                // Set X if no collision Hit
                if (physics_is_object_collision(pStage,
                                                x_addition,
                                                pPlayer->player_get_y(),
                                                pPlayer->player_get_hitbox_width(),
                                                pPlayer->player_get_hitbox_height(),
                                                pPlayer->player_get_facing()) == false)
                {
                    // Apply X Change
                    pPlayer->player_set_x(x_addition);
                }
                else
                {
                    // Check Player on Object
                    /*if (pPlayer->player_get_y() <= physics_get_object_collision_y(pStage, physics_object_collision_id, COLLISION_DOWN) + 1 &&
                        physics_player_object_id_set == true)*/
                    if (pPlayer->player_get_x() != physics_get_object_collision_x(pStage, physics_object_collision_id, physics_old_player_facing) &&
                        physics_player_object_id_set == true)
                    {
                        // Set X on Object
                        pPlayer->player_set_x(x_addition);
                    }
                    
                    // Block Y on Collision Hit
                    block_y = true;
                }
            }
            
            /* Y Position */

            // Y Movement Position
            s32 y_addition = pStage->stage_check_specific_collision_distance(pPlayer->player_get_x(),
                                                                             pPlayer->player_get_y(),
                                                                             pPlayer->player_get_hitbox_width(),
                                                                             0,
                                                                             pPlayer->player_get_speed() * 2,
                                                                             COLLISION_UP);

            // Down Y has no way upwards, check downwards
            if (y_addition == pPlayer->player_get_y() + ((pPlayer->player_get_speed() * 2) + 1))
            {
                // Check Collision Down
                y_addition = pStage->stage_check_specific_collision_distance(pPlayer->player_get_x(),
                                                                             pPlayer->player_get_y(),
                                                                             pPlayer->player_get_hitbox_width(),
                                                                             0,
                                                                             pPlayer->player_get_speed() * 2,
                                                                             COLLISION_DOWN);

                // Either Move Up Y or don't change Y and block X
                if (block_y == false)
                {
                    // Check if we're falling!
                    if (y_addition == pPlayer->player_get_y() - (pPlayer->player_get_speed() * 2))
                    {
                        // Set X if no collision Hit
                        if (physics_is_object_collision(pStage,
                                                        pPlayer->player_get_x(),
                                                        pPlayer->player_get_y() - 1,
                                                        pPlayer->player_get_hitbox_width(),
                                                        pPlayer->player_get_hitbox_height(),
                                                        pPlayer->player_get_facing()) == true)
                        {
                            // Set Y 1 Pixel Above Object
                            // We Use Collision Up because we can only walk on top of the Object without Spiderball
                            if (physics_player_object_id_set == true)
                            {
                                pPlayer->player_set_y(physics_get_object_collision_y(pStage, physics_object_collision_id, COLLISION_DOWN) + 1);
                            }
                        }
                        else
                        {
                            // Set Falling from running
                            physics_x_falling_modifier = 1;
                            pPlayer->player_set_state(PLAYERSTATE_FALLING);
                        
                            // Change our Animation (except if we're in Morph Ball State)
                            if (pPlayer->player_get_morphball() == false)
                            {
                                pPlayer->player_set_anim(PLAYERANIM_FALLING);
                            }
                        
                            // Set Falling
                            physics_player_jump_state = JUMP_FALL;
                        }
                    }
                    
                    // Set Y
                    // +1 Because y_addition gets the position of the Collision Pixel,
                    // And we want to be 1 Pixel above the Collision Pixel
                    pPlayer->player_set_y(y_addition + 1);
                }
            }
            else
            {
                if (block_y == false && y_addition <= pPlayer->player_get_y() + (pPlayer->player_get_speed() + 1))
                {
                    pPlayer->player_set_y(y_addition + 1);
                }
            }
        }
        else
        {
            // Spiderball Physics
        }
    }
    else if (pPlayer->player_get_state() == PLAYERSTATE_IDLE && physics_player_jump_state == JUMP_NONE)
    {
        // No Running, no Collision below
        s32 y_addition = pPlayer->player_get_y();
        
        // Check Collision Down
        y_addition = pStage->stage_check_specific_collision_distance(pPlayer->player_get_x(),
                                                                     pPlayer->player_get_y(),
                                                                     pPlayer->player_get_hitbox_width(),
                                                                     0,
                                                                     pPlayer->player_get_speed() * 2,
                                                                     COLLISION_DOWN);
        
        // Check if we're falling!
        if (y_addition == pPlayer->player_get_y() - (pPlayer->player_get_speed() * 2))
        {
            // Set X if no collision Hit
            if (physics_is_object_collision(pStage,
                                            pPlayer->player_get_x(),
                                            pPlayer->player_get_y() - 1,
                                            pPlayer->player_get_hitbox_width(),
                                            pPlayer->player_get_hitbox_height(),
                                            pPlayer->player_get_facing()) == true)
            {
                // Set Y 1 Pixel Above Object
                // We Use Collision Up because we can only walk on top of the Object without Spiderball
                if (physics_player_object_id_set == true)
                {
                    pPlayer->player_set_y(physics_get_object_collision_y(pStage, physics_object_collision_id, COLLISION_DOWN));
                }
            }
            else
            {
                // Set Falling from standing
                physics_x_falling_modifier = 0;
                pPlayer->player_set_state(PLAYERSTATE_FALLING);
                
                // Change our Animation (except if we're in Morph Ball State)
                if (pPlayer->player_get_morphball() == false)
                {
                    pPlayer->player_set_anim(PLAYERANIM_FALLING);
                }
                
                // Set Falling
                physics_player_jump_state = JUMP_FALL;
            }
        }
    }
}

/**************************/
/*    JUMPING & FALLING   */
/**************************/

void Physics::physics_player_jumping(Player* pPlayer, Stage* pStage, SaveFile* pSaveFile, u8 collision)
{
    /* Jumping */
    
    if (pPlayer->player_get_state() == PLAYERSTATE_JUMPING && physics_player_jump_state == JUMP_NONE)
    {
        // Reset for Space Jump
        pPlayer->player_set_state(PLAYERSTATE_JUMPING);
        physics_player_jump_state = JUMP_NONE;
    }
    if (pPlayer->player_get_state() == PLAYERSTATE_JUMPING && physics_player_jump_state == JUMP_NONE)
    {
        // Height Manipulation
        float jump90_manipulation = 1;
        if (pPlayer->player_get_anim() == PLAYERANIM_JUMP90)
            jump90_manipulation = 0.65;
        
        // Set Jump Frames
        if (pSaveFile->savefile_get_highjump() == true || pSaveFile->savefile_get_spacejump() == true)
        {
            physics_player_jump_counter = JUMP_MOVEMENT_HIGH * jump90_manipulation;
        }
        else
        {
            physics_player_jump_counter = JUMP_MOVEMENT_NORMAL * jump90_manipulation;
        }
        
        // Set State
        physics_player_jump_state = JUMP_LIFTOFF;
    }
    else if (pPlayer->player_get_state() == PLAYERSTATE_JUMPING && physics_player_jump_state == JUMP_LIFTOFF)
    {
        /* Set Speed */
        
        // Set Y Jumping Speed
        u8 elevation_speed = pPlayer->player_get_speed() + (pPlayer->player_get_speed() / 4);
        
        /* Y Coordinates */
        
        // Get Y
        s32 y_new = pStage->stage_check_specific_collision_distance(pPlayer->player_get_x(),
                                                                    pPlayer->player_get_y(),
                                                                    pPlayer->player_get_hitbox_width(),
                                                                    pPlayer->player_get_hitbox_height(),
                                                                    elevation_speed,
                                                                    COLLISION_UP);
        
        // Set Y
        if (y_new == pPlayer->player_get_y() + elevation_speed + 1)
        {
            // Set Y if there's no Object above Player
            if (physics_is_object_collision(pStage,
                                            pPlayer->player_get_x(),
                                            y_new + pPlayer->player_get_hitbox_height(),
                                            pPlayer->player_get_hitbox_width(),
                                            pPlayer->player_get_hitbox_height(),
                                            pPlayer->player_get_facing()) == false)
            {
                // Update Y for Jump Angle
                if (physics_player_jump_counter <= 3)
                {
                    
                    // Correct Y
                    --y_new;
                }
                
                // Set Y
                pPlayer->player_set_y(y_new);
            }
            else
            {
                // Set Falling
                physics_player_jump_counter = 0;
            }
        }
        else
        {
            // Set Y
            pPlayer->player_set_y(pPlayer->player_get_y() + (pPlayer->player_get_hitbox_height() / 4));
            
            // Set Falling
            physics_player_jump_counter = 0;
        }
        
        // X Movement while jumping
        if (pPlayer->player_get_anim() == PLAYERANIM_JUMPSPIN  ||
            pPlayer->player_get_anim() == PLAYERANIM_JUMPSCREW ||
            pPlayer->player_is_morphball_moving_jump() == true)
        {
            // Get X
            s32 x_new = pStage->stage_check_specific_collision_distance(pPlayer->player_get_x(),
                                                                        pPlayer->player_get_y(),
                                                                        pPlayer->player_get_hitbox_width(),
                                                                        pPlayer->player_get_hitbox_height() / 2,
                                                                        pPlayer->player_get_speed(),
                                                                        pPlayer->player_get_collision_facing());
            // Set X if no collision Hit
            if (physics_is_object_collision(pStage,
                                            x_new,
                                            pPlayer->player_get_y() - 1,
                                            pPlayer->player_get_hitbox_width(),
                                            pPlayer->player_get_hitbox_height(),
                                            pPlayer->player_get_facing()) == false)
            {
                
                // Set X
                pPlayer->player_set_x(x_new);
                physics_x_falling_modifier = 1;
            }
            else
            {
                physics_x_falling_modifier = 0;
            }
        }
        else
        {
            physics_x_falling_modifier = 0;
        }
        
        // Set State or reduce Jump Counter
        if (physics_player_jump_counter <= 0)
        {
            pPlayer->player_set_state(PLAYERSTATE_FALLING);
            physics_player_jump_state = JUMP_FALL;
        }
        else
        {
            --physics_player_jump_counter;
        }
    }
    else if (pPlayer->player_has_taken_damage() == true && physics_player_jump_state == JUMP_LIFTOFF)
    {
        // Set Falling X Change
        physics_x_falling_modifier = 0;
        
        // Reset Jump Counter
        physics_player_jump_counter = 0;
        
        // Set Physics Falling
        physics_player_jump_state = JUMP_FALL;
        
        // Set Player Falling
        pPlayer->player_set_state(PLAYERSTATE_FALLING);
        
        if (pPlayer->player_get_anim() != PLAYERANIM_DAMAGE)
        {
            pPlayer->player_set_anim(PLAYERANIM_FALLING);
        }
    }
    else if (pPlayer->player_get_state() == PLAYERSTATE_FALLING && physics_player_jump_state == JUMP_LIFTOFF)
    {
        // Reset Jump Counter
        physics_player_jump_counter = 0;
        
        // Set Movement Modifier for falling
        if (pPlayer->player_is_jump_movement_cancel() == false && pPlayer->player_get_anim() == PLAYERANIM_JUMP90)
        {
            // Set Falling X Change
            physics_x_falling_modifier = 0;
        }
        else
        {
            // Set Falling X Change
            physics_x_falling_modifier = 1;
            
            // Reset JUMP90 Flag
            pPlayer->player_set_jump_movement_cancel(false);
        }
        
        // Set Jump State
        physics_player_jump_state = JUMP_FALL;
    }
}

void Physics::physics_player_falling(Player* pPlayer, Stage* pStage, SaveFile* pSaveFile, u8 collision)
{
    /* Falling */
    
    if (collision & COLLISION_DOWN && pPlayer->player_get_state() == PLAYERSTATE_FALLING && physics_player_jump_state == JUMP_FALL)
    {
        // We can't go into Morphball while falling, so we've must been in morphball already!
        if (pPlayer->player_get_morphball() == false)
        {
            pPlayer->player_set_anim(PLAYERANIM_STANDING);
        }
        
        // Reset State
        pPlayer->player_set_state(PLAYERSTATE_IDLE);
        pPlayer->player_set_y(pPlayer->player_get_y() + RUN_Y_FAILURE_CORRECTION);
        
        // Reset Physics Jump Flag
        physics_player_jump_state = JUMP_NONE;
    }
    else if (physics_is_object_collision(pStage,
                                         pPlayer->player_get_x(),
                                         pPlayer->player_get_y() - 1,
                                         pPlayer->player_get_hitbox_width(),
                                         pPlayer->player_get_hitbox_height(),
                                         pPlayer->player_get_facing()) == true &&
             pPlayer->player_get_state() == PLAYERSTATE_FALLING &&
             physics_player_jump_state == JUMP_FALL)
    {
        // We can't go into Morphball while falling, so we've must been in morphball already!
        if (pPlayer->player_get_morphball() == false)
        {
            pPlayer->player_set_anim(PLAYERANIM_STANDING);
        }
        
        // Reset State
        pPlayer->player_set_state(PLAYERSTATE_IDLE);
        
        // Set New Player Y
        if (physics_player_object_id_set == true)
        {
            pPlayer->player_set_y(physics_get_object_collision_y(pStage, physics_object_collision_id, COLLISION_DOWN));
        }
        
        // Reset Physics Jump Flag
        physics_player_jump_state = JUMP_NONE;
    }
    else if (pPlayer->player_get_state() == PLAYERSTATE_FALLING && physics_player_jump_state == JUMP_FALL)
    {
        if (pPlayer->player_is_morphball_moving_jump() == false &&
            pPlayer->player_get_anim() != PLAYERANIM_JUMPSPIN   &&
            pPlayer->player_get_anim() != PLAYERANIM_JUMPSCREW  &&
            physics_x_falling_modifier == 0)
        {
            // Set Animation to Falling or Morph Ball
            if (pPlayer->player_get_morphball() == false)
            {
                pPlayer->player_set_anim(PLAYERANIM_FALLING);
            }
        }
        else
        {
            /* Ledge Grabbing */
            
            // Check if we can grab Ledges!
            if (pPlayer->player_get_ledge_block_frames() == 0 && pPlayer->player_get_morphball() == false)
            {
                // Check for collision hit from falling X addition
                s32 x_ledge = 0;
            
                if (pPlayer->player_get_facing() < 3)
                {
                    x_ledge = pPlayer->player_get_x() + pPlayer->player_get_speed();
                }
                else if (pPlayer->player_get_facing() > 2 && pPlayer->player_get_facing() < 6)
                {
                    x_ledge = pPlayer->player_get_x() - pPlayer->player_get_speed();
                }
            
                // The Y Height of the next possible ledge
                s32 y_ledge = pStage->stage_check_specific_collision_distance(x_ledge,
                                                                              pPlayer->player_get_y() + pPlayer->player_get_hitbox_height(),
                                                                              pPlayer->player_get_hitbox_width(),
                                                                              1,
                                                                              pPlayer->player_get_hitbox_height() / 2,
                                                                              COLLISION_DOWN);
                
                // The Y Height of the ground below us
                s32 y_ground = pStage->stage_check_specific_collision_distance(pPlayer->player_get_x(),
                                                                               y_ledge - LEDGE_GRAB_Y_CORRECTION,
                                                                               pPlayer->player_get_hitbox_width(),
                                                                               0,
                                                                               2,
                                                                               COLLISION_DOWN);
            
                if (y_ledge < pPlayer->player_get_y() + pPlayer->player_get_hitbox_height() - 5 &&
                    y_ledge > pPlayer->player_get_y() + pPlayer->player_get_hitbox_height() / 2 + 5 &&
                    y_ledge != (pPlayer->player_get_y() + pPlayer->player_get_hitbox_height()) - pPlayer->player_get_hitbox_height() / 2 &&
                    y_ground == (y_ledge - LEDGE_GRAB_Y_CORRECTION) - 2)
                {
                    // Set Facing
                    pPlayer->player_set_shooting_facing(FACING_DOWN);
                    
                    // Correct Y to Ledge Top Y
                    pPlayer->player_set_y(y_ledge - LEDGE_GRAB_Y_CORRECTION);
                    
                    // Check if we weren't hanging before!
                    if (pPlayer->player_get_state() != PLAYERSTATE_LEDGE && pPlayer->player_get_anim() != PLAYERANIM_HANGING)
                    {
                        // Set Block Frames
                        pPlayer->player_set_ledge_block_frames(LEDGE_CLIMB_LOCK_FRAMES);
                    }
                    
                    // Set Hanging State
                    pPlayer->player_set_state(PLAYERSTATE_LEDGE);
                
                    // Set Animation
                    pPlayer->player_set_anim(PLAYERANIM_HANGING);
                
                    // Set Modifier
                    physics_x_falling_modifier = 0;
                    
                    // Set Jump Counter
                    physics_player_jump_counter = 0;
                    
                    // Stop Jump State
                    physics_player_jump_state = JUMP_NONE;
                }
            }
            
            /* Falling */
            
            // Check if we're now grabbing a Ledge!
            if (pPlayer->player_get_state() != PLAYERSTATE_LEDGE || pPlayer->player_get_state() != PLAYERSTATE_CLIMBING)
            {
                /* X Change while Falling */
                
                // Set Falling Speed
                u8 falling_speed = pPlayer->player_get_speed();
                
                if (pPlayer->player_get_anim() != PLAYERANIM_JUMPSPIN && pPlayer->player_get_anim() != PLAYERANIM_JUMPSCREW)
                {
                    falling_speed = falling_speed * 0.75;
                }
                
                // Get X
                s32 x_new = pStage->stage_check_specific_collision_distance(pPlayer->player_get_x(),
                                                                            pPlayer->player_get_y(),
                                                                            pPlayer->player_get_hitbox_width() + 1,
                                                                            pPlayer->player_get_hitbox_height() + 1,
                                                                            falling_speed,
                                                                            pPlayer->player_get_collision_facing());
                
                // Check for collision hit from falling X addition
                u8 falling_collision = 0;
                
                if (pPlayer->player_get_facing() < 3)
                {
                    falling_collision = COLLISION_RIGHT;
                }
                else if (pPlayer->player_get_facing() > 2 && pPlayer->player_get_facing() < 6)
                {
                    falling_collision = COLLISION_LEFT;
                }
                
                if (pStage->stage_check_collision(x_new,
                                                  pPlayer->player_get_y(),
                                                  pPlayer->player_get_hitbox_width(),
                                                  pPlayer->player_get_hitbox_height()) & falling_collision)
                {
                    // Set Falling Modifier
                    physics_x_falling_modifier = 0;
                }
                else
                {
                    // Set X if no collision Hit
                    if (physics_is_object_collision(pStage,
                                                    x_new,
                                                    pPlayer->player_get_y() - 1,
                                                    pPlayer->player_get_hitbox_width(),
                                                    pPlayer->player_get_hitbox_height(),
                                                    pPlayer->player_get_facing()) == false)
                    {
                        // Set New Player X
                        pPlayer->player_set_x(x_new);
                    }
                    else
                    {
                        // Set Falling Modifier
                        physics_x_falling_modifier = 0;
                    }
                }
            }
        }
        
        if (pPlayer->player_get_state() != PLAYERSTATE_LEDGE || pPlayer->player_get_state() != PLAYERSTATE_CLIMBING)
        {
            // Get Y
            s32 y_new = pStage->stage_check_specific_collision_distance(pPlayer->player_get_x(),
                                                                        pPlayer->player_get_y(),
                                                                        pPlayer->player_get_hitbox_width(),
                                                                        pPlayer->player_get_hitbox_height(),
                                                                        pPlayer->player_get_speed() * 1.25,
                                                                        COLLISION_DOWN);
        
            // Set Y
            pPlayer->player_set_y(y_new);
        }
    }
    else if (pPlayer->player_get_state() == PLAYERSTATE_IDLE && physics_player_jump_state == JUMP_FALL)
    {
        // Reset State
        pPlayer->player_set_anim(PLAYERANIM_FALLING);
        
        // Reset State
        pPlayer->player_set_state(PLAYERSTATE_FALLING);
    }
}

/*************************************************************************************************************************/
/*                                                    Enemy Physics                                                      */
/*************************************************************************************************************************/

void Physics::physics_enemy_physics(Enemy* pEnemy, Stage* pStage, u8 collision)
{
    if (pEnemy->enemy_get_moving() == true)
    {
        if (pEnemy->enemy_get_physics() == ENEMYPHYSICS_WALLWALKING)
        {
            // Walking up Walls
            //physics_enemy_wallwalking(pEnemy, pStage, collision);
        }
        else
        {
            // Normal Running
            physics_enemy_running(pEnemy, pStage, collision);
        }
    }
    
    // Falling Physics
    if (physics_enemy_jump_state == JUMP_FALL)
    {
        physics_enemy_falling(pEnemy, pStage, collision);
    }
}

void Physics::physics_enemy_wallwalking(Enemy* pEnemy, Stage* pStage, u8 collision)
{
    /* X Position */
    
    // Set Variables
    s32 x_addition = 0, y_addition = 0;
    bool no_wall = false;
    
    if (no_wall == true)
    {
        // Set Falling State
        physics_enemy_jump_state = JUMP_FALL;
        
        // Force Facing
        pEnemy->enemy_set_facing(FACING_RIGHT);
        
        // Return
        return;
    }
    else if (physics_enemy_jump_state == JUMP_FALL)
    {
        // Reset falling State
        physics_enemy_jump_state = JUMP_NONE;
    }
    
    /********************************/
    /* Walking on Ground or Ceiling */
    /********************************/
    
    // Wallwalking Right
    if (pEnemy->enemy_get_facing() == FACING_RIGHT)
    {
        // Get X
        x_addition = pStage->stage_check_specific_collision_distance(pEnemy->enemy_get_x(),
                                                                     pEnemy->enemy_get_y() + (pEnemy->enemy_get_speed() + 1),
                                                                     pEnemy->enemy_get_hitbox_width(),
                                                                     pEnemy->enemy_get_hitbox_height(),
                                                                     pEnemy->enemy_get_speed(),
                                                                     COLLISION_RIGHT);
    }
    
    // Wallwalking Left
    if (pEnemy->enemy_get_facing() == FACING_LEFT)
    {
        // Get X
        x_addition = pStage->stage_check_specific_collision_distance(pEnemy->enemy_get_x(),
                                                                     pEnemy->enemy_get_y() + (pEnemy->enemy_get_speed() + 1),
                                                                     pEnemy->enemy_get_hitbox_width(),
                                                                     pEnemy->enemy_get_hitbox_height(),
                                                                     pEnemy->enemy_get_speed(),
                                                                     COLLISION_LEFT);
    }
    
    // Apply Position Change
    if (x_addition == pEnemy->enemy_get_x())
    {
        // Change Facing
        /*
        if (pEnemy->enemy_get_facing() == FACING_RIGHT)
        {
            pEnemy->enemy_set_facing(FACING_UP);
        }
        else if (pEnemy->enemy_get_facing() == FACING_LEFT)
        {
            pEnemy->enemy_set_facing(FACING_DOWN);
        }
        */
    }
    else
    {
        // Set X if no collision Hit
        if (physics_enemy_is_object_collision(pStage,
                                              x_addition,
                                              pEnemy->enemy_get_y(),
                                              pEnemy->enemy_get_hitbox_width(),
                                              pEnemy->enemy_get_hitbox_height(),
                                              pEnemy->enemy_get_facing()) == false)
        {
            // Apply X Change
            pEnemy->enemy_set_x(x_addition);
        }
        else if (pEnemy->enemy_get_y() <= physics_enemy_get_object_collision_y(pStage, physics_enemy_object_collision_id, COLLISION_DOWN) + 1 &&
                physics_enemy_object_id_set == true)
        {
            // Set X on Object
            pEnemy->enemy_set_x(x_addition);
        }
    }
    
    /*********************************/
    /* Walking on Left or Right Wall */
    /*********************************/
    
    // Wallwalking Up
    if (pEnemy->enemy_get_facing() == FACING_UP)
    {
        // Get Y
        y_addition = pStage->stage_check_specific_collision_distance(pEnemy->enemy_get_x(),
                                                                     pEnemy->enemy_get_y(),
                                                                     pEnemy->enemy_get_hitbox_width(),
                                                                     0,
                                                                     pEnemy->enemy_get_speed() * 2,
                                                                     COLLISION_UP);
    }
    
    // Wallwalking Down
    if (pEnemy->enemy_get_facing() == FACING_DOWN)
    {
        // Get Y
        y_addition = pStage->stage_check_specific_collision_distance(pEnemy->enemy_get_x(),
                                                                     pEnemy->enemy_get_y(),
                                                                     pEnemy->enemy_get_hitbox_width(),
                                                                     0,
                                                                     pEnemy->enemy_get_speed() * 2,
                                                                     COLLISION_DOWN);
    }
    
    // Apply Position Change
    
    // TEMP
    pEnemy->enemy_set_y(y_addition);
}

void Physics::physics_enemy_running(Enemy* pEnemy, Stage* pStage, u8 collision)
{
    /* X Position */
    
    // Set Variables
    s32 x_addition = 0;
    bool block_y = false;
    
    // Running Right
    if (pEnemy->enemy_get_facing() < FACING_LEFT)
    {
        // Get X
        x_addition = pStage->stage_check_specific_collision_distance(pEnemy->enemy_get_x(),
                                                                     pEnemy->enemy_get_y() + (pEnemy->enemy_get_speed() + 1),
                                                                     pEnemy->enemy_get_hitbox_width(),
                                                                     pEnemy->enemy_get_hitbox_height(),
                                                                     pEnemy->enemy_get_speed(),
                                                                     COLLISION_RIGHT);
    }
    
    // Running Left
    if (pEnemy->enemy_get_facing() > FACING_RIGHTDOWN && pEnemy->enemy_get_facing() < FACING_UP)
    {
        // Get X
        x_addition = pStage->stage_check_specific_collision_distance(pEnemy->enemy_get_x(),
                                                                     pEnemy->enemy_get_y() + (pEnemy->enemy_get_speed() + 1),
                                                                     pEnemy->enemy_get_hitbox_width(),
                                                                     pEnemy->enemy_get_hitbox_height(),
                                                                     pEnemy->enemy_get_speed(),
                                                                     COLLISION_LEFT);
    }
    
    // Apply Position Change
    if (x_addition == pEnemy->enemy_get_x())
    {
        // X is the same, no need to check Y on Running Physics
        block_y = true;
        
        // Change Facing
        if (pEnemy->enemy_get_facing() == FACING_LEFT)
        {
            pEnemy->enemy_set_facing(FACING_RIGHT);
        }
        else if (pEnemy->enemy_get_facing() == FACING_RIGHT)
        {
            pEnemy->enemy_set_facing(FACING_LEFT);
        }
    }
    else
    {
        // Set X if no collision Hit
        if (physics_enemy_is_object_collision(pStage,
                                        x_addition,
                                        pEnemy->enemy_get_y(),
                                        pEnemy->enemy_get_hitbox_width(),
                                        pEnemy->enemy_get_hitbox_height(),
                                        pEnemy->enemy_get_facing()) == false)
        {
            // Apply X Change
            pEnemy->enemy_set_x(x_addition);
        }
        else
        {
            // Check Player on Object
            if (pEnemy->enemy_get_y() <= physics_enemy_get_object_collision_y(pStage, physics_enemy_object_collision_id, COLLISION_DOWN) + 1 &&
                physics_enemy_object_id_set == true)
            {
                // Set X on Object
                pEnemy->enemy_set_x(x_addition);
            }
            
            // Block Y on Collision Hit
            block_y = true;
        }
    }
    
    /* Y Position */
    
    if (pEnemy->enemy_get_physics() != ENEMYPHYSICS_FLYING && pEnemy->enemy_get_physics() != ENEMYPHYSICS_NONE)
    {
        // Y Movement Position
        s32 y_addition = pStage->stage_check_specific_collision_distance(pEnemy->enemy_get_x(),
                                                                     pEnemy->enemy_get_y(),
                                                                     pEnemy->enemy_get_hitbox_width(),
                                                                     0,
                                                                     pEnemy->enemy_get_speed() * 2,
                                                                     COLLISION_UP);
    
        // Down Y has no way upwards, check downwards
        if (y_addition == pEnemy->enemy_get_y() + ((pEnemy->enemy_get_speed() * 2) + 1))
        {
            // Check Collision Down
            y_addition = pStage->stage_check_specific_collision_distance(pEnemy->enemy_get_x(),
                                                                     pEnemy->enemy_get_y(),
                                                                     pEnemy->enemy_get_hitbox_width(),
                                                                     0,
                                                                     pEnemy->enemy_get_speed() * 2,
                                                                     COLLISION_DOWN);
        
            // Either Move Up Y or don't change Y and block X
            if (block_y == false)
            {
                // Check if we're falling!
                if (y_addition == pEnemy->enemy_get_y() - (pEnemy->enemy_get_speed() * 2))
                {
                    // Set X if no collision Hit
                    if (physics_enemy_is_object_collision(pStage,
                                                pEnemy->enemy_get_x(),
                                                pEnemy->enemy_get_y() - 1,
                                                pEnemy->enemy_get_hitbox_width(),
                                                pEnemy->enemy_get_hitbox_height(),
                                                pEnemy->enemy_get_facing()) == true)
                    {
                        // Set Y 1 Pixel Above Object
                        // We Use Collision Up because we can only walk on top of the Object without Spiderball
                        if (physics_enemy_object_id_set == true)
                        {
                            pEnemy->enemy_set_y(physics_enemy_get_object_collision_y(pStage, physics_enemy_object_collision_id, COLLISION_DOWN) + 1);
                        }
                    }
                    else
                    {
                        // Set Falling from running
                        //physics_x_falling_modifier = 1;
                        pEnemy->enemy_set_state(PLAYERSTATE_FALLING);
                    
                        // Change our Animation (except if we're in Morph Ball State)
                        pEnemy->enemy_set_anim(PLAYERANIM_FALLING);
                    
                        // Set Falling
                        //physics_player_jump_state = JUMP_FALL;
                    }
                }
            
                // Set Y
                // +1 Because y_addition gets the position of the Collision Pixel,
                // And we want to be 1 Pixel above the Collision Pixel
                pEnemy->enemy_set_y(y_addition + 1);
            }
        }
        else
        {
            if (block_y == false && y_addition <= pEnemy->enemy_get_y() + (pEnemy->enemy_get_speed() + 1))
            {
                pEnemy->enemy_set_y(y_addition + 1);
            }
        }
    }
    else if (pEnemy->enemy_get_physics() != ENEMYPHYSICS_NONE && pEnemy->enemy_get_physics() != ENEMYPHYSICS_FLYING)
    {
        // No Running, no Collision below
        s32 y_addition = pEnemy->enemy_get_y();
        
        // Check Collision Down
        y_addition = pStage->stage_check_specific_collision_distance(pEnemy->enemy_get_x(),
                                                                     pEnemy->enemy_get_y(),
                                                                     pEnemy->enemy_get_hitbox_width(),
                                                                     0,
                                                                     pEnemy->enemy_get_speed() * 2,
                                                                     COLLISION_DOWN);
        
        // Check if we're falling!
        if (y_addition == pEnemy->enemy_get_y() - (pEnemy->enemy_get_speed() * 2))
        {
            // Set X if no collision Hit
            if (physics_enemy_is_object_collision(pStage,
                                                  pEnemy->enemy_get_x(),
                                                  pEnemy->enemy_get_y() - 1,
                                                  pEnemy->enemy_get_hitbox_width(),
                                                  pEnemy->enemy_get_hitbox_height(),
                                                  pEnemy->enemy_get_facing()) == true)
            {
                // Set Y 1 Pixel Above Object
                // We Use Collision Up because we can only walk on top of the Object without Spiderball
                if (physics_enemy_object_id_set == true)
                {
                    pEnemy->enemy_set_y(physics_enemy_get_object_collision_y(pStage, physics_enemy_object_collision_id, COLLISION_DOWN));
                }
            }
            else
            {
                // Set Falling
                physics_enemy_jump_state = JUMP_FALL;
            }
        }
    }
}

void Physics::physics_enemy_falling(Enemy* pEnemy, Stage* pStage, u8 collision)
{
    /* Falling */
    
    if (collision & COLLISION_DOWN && physics_enemy_jump_state == JUMP_FALL)
    {
        // Set Y
        pEnemy->enemy_set_y(pEnemy->enemy_get_y() + RUN_Y_FAILURE_CORRECTION);
        
        // Reset Physics Jump Flag
        physics_enemy_jump_state = JUMP_NONE;
    }
    else if (physics_is_object_collision(pStage,
                                         pEnemy->enemy_get_x(),
                                         pEnemy->enemy_get_y() - 1,
                                         pEnemy->enemy_get_hitbox_width(),
                                         pEnemy->enemy_get_hitbox_height(),
                                         pEnemy->enemy_get_facing()) == true &&
             physics_enemy_jump_state == JUMP_FALL)
    {
        // Set New Player Y
        if (physics_enemy_object_id_set == true)
        {
            pEnemy->enemy_set_y(physics_enemy_get_object_collision_y(pStage, physics_enemy_object_collision_id, COLLISION_DOWN));
        }
        
        // Reset Physics Jump Flag
        physics_enemy_jump_state = JUMP_NONE;
    }
    else if (physics_enemy_jump_state == JUMP_FALL)
    {
        // Get Y
        s32 y_new = pStage->stage_check_specific_collision_distance(pEnemy->enemy_get_x(),
                                                                    pEnemy->enemy_get_y(),
                                                                    pEnemy->enemy_get_hitbox_width(),
                                                                    pEnemy->enemy_get_hitbox_height(),
                                                                    pEnemy->enemy_get_speed(),
                                                                    COLLISION_DOWN);
        
        // Set Y
        pEnemy->enemy_set_y(y_new);
    }
}

/*************************************************************************************************************************/
/*                                                 Actor Collision                                                       */
/*************************************************************************************************************************/

bool Physics::physics_collision_in_range(s32 actor_a_value, s32 actor_b_min, s32 actor_b_max)
{
    // Return Compare Result
    return (actor_a_value >= actor_b_min) && (actor_a_value <= actor_b_max);
}

bool Physics::physics_actor_collision(s32 actor_a_x, s32 actor_a_y, s32 actor_a_w, s32 actor_a_h, s32 actor_b_x, s32 actor_b_y, s32 actor_b_w, s32 actor_b_h)
{
    // Compare Positions
    bool x_overlap = physics_collision_in_range(actor_a_x, actor_b_x, actor_b_x + actor_b_w) ||
                     physics_collision_in_range(actor_b_x, actor_a_x, actor_a_x + actor_a_w);
    
    bool y_overlap = physics_collision_in_range(actor_a_y, actor_b_y, actor_b_y + actor_b_h) ||
                     physics_collision_in_range(actor_b_y, actor_a_y, actor_a_y + actor_a_h);
    
    // Return Result
    return x_overlap && y_overlap;
}

/*************************************************************************************************************************/
/*                                            Player Object Collision                                                    */
/*************************************************************************************************************************/

bool Physics::physics_is_object_collision(Stage* pStage, s32 actor_x, s32 actor_y, s32 actor_w, s32 actor_h, u8 actor_f)
{
    // Flag for Collision Hit
    bool object_collision_hit = false;
    
    // Reset Object ID
    physics_player_object_id_set = false;
    physics_object_collision_id = 0;
    
    // Check Objects with collision
    if (pStage->stage_get_object_count() > 0)
    {
        for (u8 k = 0; k <= pStage->stage_get_object_count() - 1; ++k)
        {
            // Only Perform
            if (pStage->stage_get_object(k)->object_is_destroy_completed() == false &&
                pStage->stage_get_object(k)->object_get_collision() == true)
            {
                // Set Collision Flag on Collision Hit
                object_collision_hit = pStage->stage_get_object(k)->object_is_collision(actor_x,
                                                                                        actor_y,
                                                                                        actor_w,
                                                                                        actor_h,
                                                                                        actor_f);
                
                // Break out of Loop once we hit a collision
                if (object_collision_hit == true)
                {
                    physics_player_object_id_set = true;
                    physics_object_collision_id = k;
                    break;
                }
            }
        }
    }
    
    return object_collision_hit;
}

s32 Physics::physics_get_object_collision_x(Stage* pStage, u8 object_id, u8 collision)
{
    if (physics_player_object_id_set == false || object_id > pStage->stage_get_object_count() - 1)
        return 0;
        
    if (collision & COLLISION_LEFT)
    {
        return pStage->stage_get_object(object_id)->object_get_x();
    }
    else if (collision & COLLISION_RIGHT)
    {
        return pStage->stage_get_object(object_id)->object_get_x() + pStage->stage_get_object(object_id)->object_get_w();
    }
    else
    {
        return 0;//pStage->stage_get_object(object_id)->object_get_x() + pStage->stage_get_object(object_id)->object_get_w();
    }
}

s32 Physics::physics_get_object_collision_y(Stage* pStage, u8 object_id, u8 collision)
{
    if (physics_player_object_id_set == false || object_id > pStage->stage_get_object_count() - 1)
        return 0;
    
    if (collision & COLLISION_DOWN)
    {
        return pStage->stage_get_object(object_id)->object_get_y() + pStage->stage_get_object(object_id)->object_get_h();
    }
    else if (collision & COLLISION_UP)
    {
        return pStage->stage_get_object(object_id)->object_get_y();
    }
    else
    {
        return pStage->stage_get_object(object_id)->object_get_y() + pStage->stage_get_object(object_id)->object_get_h();
    }
}

/*************************************************************************************************************************/
/*                                             Enemy Object Collision                                                    */
/*************************************************************************************************************************/

bool Physics::physics_enemy_is_object_collision(Stage* pStage, s32 actor_x, s32 actor_y, s32 actor_w, s32 actor_h, u8 actor_f)
{
    // Flag for Collision Hit
    bool object_collision_hit = false;
    
    // Reset Object ID
    physics_enemy_object_id_set = false;
    physics_enemy_object_collision_id = 0;
    
    // Check Objects with collision
    if (pStage->stage_get_object_count() > 0)
    {
        for (u8 k = 0; k <= pStage->stage_get_object_count() - 1; ++k)
        {
            // Only Perform
            if (pStage->stage_get_object(k)->object_is_destroy_completed() == false &&
                pStage->stage_get_object(k)->object_get_collision() == true)
            {
                // Set Collision Flag on Collision Hit
                object_collision_hit = pStage->stage_get_object(k)->object_is_collision(actor_x,
                                                                                        actor_y,
                                                                                        actor_w,
                                                                                        actor_h,
                                                                                        actor_f);
                
                // Break out of Loop once we hit a collision
                if (object_collision_hit == true)
                {
                    physics_enemy_object_id_set = true;
                    physics_enemy_object_collision_id = k;
                    break;
                }
            }
        }
    }
    
    return object_collision_hit;
}

s32 Physics::physics_enemy_get_object_collision_x(Stage* pStage, u8 object_id, u8 collision)
{
    if (physics_enemy_object_id_set == false || object_id > pStage->stage_get_object_count() - 1)
        return 0;
    
    if (collision & COLLISION_LEFT)
    {
        return pStage->stage_get_object(object_id)->object_get_x();
    }
    else if (collision & COLLISION_RIGHT)
    {
        return pStage->stage_get_object(object_id)->object_get_x() + pStage->stage_get_object(object_id)->object_get_w();
    }
    else
    {
        return pStage->stage_get_object(object_id)->object_get_x() + pStage->stage_get_object(object_id)->object_get_w();
    }
}

s32 Physics::physics_enemy_get_object_collision_y(Stage* pStage, u8 object_id, u8 collision)
{
    if (physics_enemy_object_id_set == false || object_id > pStage->stage_get_object_count() - 1)
        return 0;
    
    if (collision & COLLISION_DOWN)
    {
        return pStage->stage_get_object(object_id)->object_get_y() + pStage->stage_get_object(object_id)->object_get_h();
    }
    else if (collision & COLLISION_UP)
    {
        return pStage->stage_get_object(object_id)->object_get_y();
    }
    else
    {
        return pStage->stage_get_object(object_id)->object_get_y() + pStage->stage_get_object(object_id)->object_get_h();
    }
}
