//
//  physics.h
//  3ds_metroid
//
//  Created by jbr373 on 21/01/16.
//

#ifndef physics_h
#define physics_h

#include <stdio.h>
#include <3ds.h>

#include "stage.h"
#include "player.h"
#include "enemy.h"
#include "savefile.h"

enum PhysicsType
{
    PHYSICS_NONE = 0, // Normal
    PHYSICS_NOGRAVITY = 1,
    PHYSICS_LIQUID = 2, // Water, Acid, Lava
    PHYSICS_CURRENT = 3 // Increases Movement NYI
};

class Physics
{
public:
    Physics();
    virtual ~Physics();
    
    // Physics affecting the Player
    void physics_player_physics(Player* pPlayer, Stage* pStage, SaveFile* pSaveFile, u8 collision);
    
    // Physics affecting Enemies
    void physics_enemy_physics(Enemy* pEnemy, Stage* pStage, u8 collision);
    
    // Collision with Stage Objects
    bool physics_actor_collision(s32 actor_a_x, s32 actor_a_y, s32 actor_a_w, s32 actor_a_h, s32 actor_b_x, s32 actor_b_y, s32 actor_b_w, s32 actor_b_h);
    
    // Getters
    u8 physics_get_player_jump_counter() { return physics_player_jump_counter; };
    
    // Setters
    void physics_set_player_jump_counter(u8 value) { physics_player_jump_counter = value; };
    
private:
    // Player Physics
    void physics_player_running(Player* pPlayer, Stage* pStage, u8 collision);
    void physics_player_jumping(Player* pPlayer, Stage* pStage, SaveFile* pSaveFile, u8 collision);
    void physics_player_falling(Player* pPlayer, Stage* pStage, SaveFile* pSaveFile, u8 collision);
    void physics_player_climbing(Player* pPlayer, Stage* pStage, u8 collision);
    
    // Enemy Physics
    void physics_enemy_running(Enemy* pEnemy, Stage* pStage, u8 collision);
    void physics_enemy_wallwalking(Enemy* pEnemy, Stage* pStage, u8 collision);
    void physics_enemy_falling(Enemy* pEnemy, Stage* pStage, u8 collision);
    
    bool physics_is_object_collision(Stage* pStage, s32 actor_x, s32 actor_y, s32 actor_w, s32 actor_h, u8 actor_f);
    bool physics_enemy_is_object_collision(Stage* pStage, s32 actor_x, s32 actor_y, s32 actor_w, s32 actor_h, u8 actor_f);
    
    // General Physics
    bool physics_collision_in_range(s32 actor_a_value, s32 actor_b_min, s32 actor_b_max);
    
    s32 physics_get_object_collision_x(Stage* pStage, u8 object_id, u8 collision);
    s32 physics_get_object_collision_y(Stage* pStage, u8 object_id, u8 collision);
    s32 physics_enemy_get_object_collision_x(Stage* pStage, u8 object_id, u8 collision);
    s32 physics_enemy_get_object_collision_y(Stage* pStage, u8 object_id, u8 collision);
 
    // Counters
    u8 physics_old_player_speed;
    u8 physics_player_jump_counter;
    u8 physics_x_falling_modifier;
    u8 physics_player_jump_state;
    u8 physics_enemy_jump_state;
    u8 physics_object_collision_id;
    u8 physics_enemy_object_collision_id;
    u8 physics_old_player_facing;
    u8 physics_climbing_counter;
    
    // Coordinates
    s32 physics_climbing_y;
    
    // Flags
    bool physics_player_object_id_set;
    bool physics_player_speed_modified;
    bool physics_enemy_object_id_set;
};

#endif /* physics_h */
