//
//  player.h
//  3ds_metroid
//
//  Created by jbr373 on 16/01/16.
//

#ifndef player_h
#define player_h

#include <stdio.h>
//#include <vector>

#include "savefile.h"
#include "image.h"
#include "projectile.h"
#include "sound.h"

#define DEFAULT_PLAYER_CHARGE_BEAM              60
#define PLAYER_MAX_PROJECTILE                   19

#define PLAYER_SPRITEFRAME_W                    100
#define PLAYER_SPRITEFRAME_H                    90

enum PlayerStates
{
    PLAYERSTATE_IDLE = 0,
    PLAYERSTATE_JUMPING = 1,
    PLAYERSTATE_RUNNING = 2,
    PLAYERSTATE_FALLING = 3,
    PLAYERSTATE_LEDGE = 4,
    PLAYERSTATE_CLIMBING = 5
};

enum PlayerAnim
{
    PLAYERANIM_STANDING = 0,
    PLAYERANIM_CROUCHED = 1,
    PLAYERANIM_JUMP90 = 2,
    PLAYERANIM_JUMPSPIN = 3,
    PLAYERANIM_JUMPSCREW = 4,
    PLAYERANIM_MORPHBALL = 5,
    PLAYERANIM_SPIDERBALL = 6,
    PLAYERANIM_RUN = 7,
    PLAYERANIM_HANGING = 8,
    PLAYERANIM_CLIMBING = 9, // Happens only after hanging
    PLAYERANIM_FALLING = 10,
    PLAYERANIM_DAMAGE = 11,
    PLAYERANIM_DEATH = 12,
    PLAYERANIM_FRONT = 13,
    PLAYERANIM_CHANGESTATE = 14
};


enum PlayerSpritePosition
{
    RUNPOS_RIGHT = 0,
    RUNPOS_LEFT = 1,
    RUNPOS_RIGHT_AIM = 2,
    RUNPOS_LEFT_AIM = 3,
    RUNPOS_JUMP90 = 4,
    RUNPOS_JUMP = 5,
    RUNPOS_MORPHBALL = 6,
    RUNPOS_RIGHT_AIMUP = 7,
    RUNPOS_LEFT_AIMUP = 8,
    RUNPOS_SPIDERBALL = 9,
    
    STANDINGPOS_STANDING = 0,
    STANDINGPOS_CROUCHED = 1,
    STANDINGPOS_CHANGESTATE = 2,
    STANDINGPOS_FRONT = 2,
    STANDINGPOS_LEDGE = 3,
    STANDINGPOS_DAMAGE = 4,
    STANDINGPOS_RIGHT_DEATH = 5,
    STANDINGPOS_LEFT_DEATH = 6
};

class Player
{
public:
    Player(SaveFile* pSaveFile);
    virtual ~Player();
    
    void player_save(SaveFile* pSaveFile);
    void player_reload(SaveFile* pSaveFile);
    
    void player_set_controls(bool movement_changed, bool shooting_changed);
    
    void player_action(SaveFile* pSaveFile, Sound* pSound, u32 kdown, u32 kheld, u32 kup, u8 collision, bool is_in_water);
    
    bool player_get_enemy_collision(s32 enemy_x, s32 enemy_y, s32 enemy_width, s32 enemy_height);
    
    // Getters
    s32 player_get_x() { return player_x; };
    s32 player_get_y() { return player_y; };
    s32 player_get_width() { return player_width; };
    s32 player_get_height() { return player_height; };
    u8 player_get_speed() { return player_speed; };
    u8 player_get_collision() { return player_collision; };
    u8 player_get_collision_facing();
    
    s32 player_get_hitbox_width() { return player_hitbox_width; };
    s32 player_get_hitbox_height() { return player_hitbox_height; };
    s32 player_get_hitbox_x() { return player_hitbox_x; };
    
    bool player_get_crouched() { return player_crouched; };
    bool player_get_morphball() { return player_morphball; };
    bool player_is_morphball_moving_jump() { return player_morphball_running_jump; }; // Kind of lazy, but it won't require much rework of the physics code
    bool player_is_jump_movement_cancel() { return player_jump_movement_cancel; };
    bool player_has_taken_damage() { return player_took_damage; };
    bool player_get_barriersuit() { return player_barriersuit; };
    bool player_get_spiderball() { return player_spiderball; };
    u8 player_get_used_projectile() { return player_last_projectile; };
    
    u8 player_get_state() { return player_state; };
    u8 player_get_facing() { return player_facing; };
    u8 player_get_shooting_facing() { return player_shoot_facing; };
    u8 player_get_anim() { return player_anim; };
    u8 player_get_ledge_block_frames() { return player_ledge_grab_block_frames; };
    
    u8 player_get_health() { return player_health; };
    bool player_is_dead() { return player_dead; };
    
    u8 player_get_healthtanks_total() { return player_healthtanks_total; };
    u8 player_get_healthtanks_available() { return player_healthtanks_available; };
    
    u8 player_get_missles_total() { return player_missles_total; };
    u8 player_get_missles_available() { return player_missles_available; };
    
    u8 player_get_charge_beam_counter() { return player_charge_beam; };
    u8 player_get_projectile_count() { return player_projectiles_count; };
    Projectile* player_get_projectile(u8 projectile);
    bool player_get_shooting() { return player_shooting; };
    
    // Setters
    void player_damage(u8 damage);
    void player_heal(u8 heal);
    void player_heal_full();
    
    void player_increase_healthtanks();
    void player_increase_total_missles(u8 missles);
    void player_increase_available_missles(u8 missles);
    void player_set_barriersuit(bool is_barrier_suit) { player_barriersuit = is_barrier_suit; };
    void player_set_spiderball(bool is_spiderball) { player_spiderball = is_spiderball; };
    
    void player_set_metroids(u8 metroids) { player_metroids = metroids; };
    void player_set_phase(u8 phase) { player_phase = phase; };
    
    void player_set_x(s32 x) { player_x = x; };
    void player_set_y(s32 y) { player_y = y; };
    void player_set_speed(u8 speed) { player_speed = speed; };
    void player_set_state(u8 state) { player_state = state; };
    void player_set_anim(u8 anim) { player_anim = anim; };
    void player_set_collision(u8 collision) { player_collision = collision; }
    void player_set_facing(u8 facing) { player_facing = facing; };
    void player_set_shooting_facing(u8 facing) { player_shoot_facing = facing; };
    void player_set_jump_movement_cancel(bool value) { player_jump_movement_cancel = value; }
    void player_set_ledge_block_frames(u8 frames) { player_ledge_grab_block_frames = frames; };
    
    void player_set_projectile_count(u8 count) { player_projectiles_count = count; };
    void player_delete_projectile(u8 projectile);
    void player_delete_all_projectiles();
    
private:
    void player_add_projectile(ProjectileType projectile_type);
    void player_check_shooting_lock();
    ProjectileType player_check_projectile_type(u8 projectile_used, bool projectile_charged);
    
    void player_update_hitbox();
    bool player_collision_in_range(s32 actor_a_value, s32 actor_b_min, s32 actor_b_max);
    
    void player_checkmovement_jumping(Sound* pSound, u8 collision, bool is_running, SaveFile* pSaveFile);
    void player_checkmovement_running(Sound* pSound, u8 collision, bool is_in_water);
    void player_checkmovement_change(Sound* pSound, u8 collision, bool moved_up, bool spiderball, SaveFile* pSaveFile);
    void player_checkshooting(Sound* pSound, bool is_missle, SaveFile* pSaveFile);
    void player_checkfacing(u32 kdown, u32 kheld);
    
    // Controls
    u32 movement_up;
    u32 movement_down;
    u32 movement_left;
    u32 movement_right;
    
    u32 beam_up;
    u32 beam_down;
    u32 beam_left;
    u32 beam_right;
    
    u32 shooting_button;
    
    // Player States
    u8 player_anim;
    u8 player_facing;
    u8 player_shoot_facing;
    u8 player_speed;
    u8 player_state;
    u8 player_collision;
    u8 player_invincibility_frames;
    u8 player_shooting_lock_frames;
    u8 player_ledge_grab_block_frames;
    u8 player_low_health_counter;
    u8 player_damage_frames;
    u8 player_charge_beam;
    u8 player_last_projectile;
    
    s32 player_x;
    s32 player_y;
    s32 player_width;
    s32 player_height;
    
    s32 player_hitbox_x;
    s32 player_hitbox_width;
    s32 player_hitbox_height;
    
    bool player_dead;
    bool player_took_damage;
    bool player_shooting;
    bool player_morphball;
    bool player_morphball_running_jump;
    bool player_jump_movement_cancel;
    bool player_crouched;
    bool player_spiderball;
    bool player_barriersuit;
    
    // Player Stats
    u8 player_health;
    u8 player_healthtanks_total;
    u8 player_healthtanks_available;
    u8 player_missles_total;
    u8 player_missles_available;
    u8 player_metroids;
    u8 player_phase;
    
    // Player Projectiles
    s8 player_projectiles_count;
    Projectile* player_projectile[PLAYER_MAX_PROJECTILE];
};

#endif /* player_h */
