//
//  projectile.cpp
//  3ds_metroid
//
//  Created by jbr373 on 22/01/16.
//

#include "projectile.h"
#include "filepaths.h"
#include "actor_facing.h"

/*
 *      TODO
 * ---------------
 * - This Stuff should be loaded by file, and not by copy & paste and enums!
 */

#define PROJECTILE_TEXTURELINE_PROJECTILE       0
#define PROJECTILE_TEXTURELINE_EXPLOSION        1

#define PROJECTILE_MAX_DISTANCE_X               450
#define PROJECTILE_MAX_DISTANCE_Y               320

/* All Player Beams */
#define BEAM_FRAMES_CHARGE_MIN                  4
#define BEAM_FRAMES_CHARGE_MAX                  6
#define BEAM_FRANES_CHARGED_MIN                 6
#define BEAM_FRAMES_CHARGED_MAX                 7

#define BEAM_FRAMES_SHOT_HORIZONTAL             0
#define BEAM_FRAMES_CHARGEDSHOT_HORIZONTAL      1
#define BEAM_FRAMES_SHOT_DIAGONAL               2
#define BEAM_FRAMES_CHARGEDSHOT_DIAGONAL        3

/* Missle */
#define MISSLE_FRAMES_TRAVEL_MIN                4
#define MISSLE_FRAMES_TRAVEL_MAX                7

#define MISSLE_FRAMES_SHOT_HORIZONTAL_MIN       0
#define MISSLE_FRAMES_SHOT_HORIZONTAL_MAX       1
#define MISSLE_FRAMES_SHOT_DIAGONAL_MIN         2
#define MISSLE_FRAMES_SHOT_DIAGONAL_MAX         3

/* Bomb */
#define BOMB_FRAMES_LAUNCH_MIN                  0
#define BOMB_FRAMES_LAUNCH_MAX                  7

#define BOMB_EXPLOSION_TIME                     35 // Frames

/* Projectile Explosions */
#define EXPLOSION_FRAMES_MIN                    0
#define EXPLOSION_FRAMES_MAX                    7

enum BeamProjectile
{
    BEAM_PROJECTILE_DAMAGE_POWER = 1,
    BEAM_PROJECTILE_DAMAGE_ICE = 2,
    BEAM_PROJECTILE_DAMAGE_WAVE = 3,
    BEAM_PROJECTILE_DAMAGE_SPAZER = 4,
    BEAM_PROJECTILE_DAMAGE_PLASMA = 5,
    BEAM_PROJECTILE_SPEED = 8,
    BEAM_PROJECTILE_WIDTH = 30,
    BEAM_PROJECTILE_HEIGHT = 30,
    BEAM_PROJECTILE_HITBOX_X = 0,
    BEAM_PROJECTILE_HITBOX_Y = 0,
    BEAM_PROJECTILE_HITBOX_W = 20,
    BEAM_PROJECTILE_HITBOX_H = 4,
};

enum MissleProjectile
{
    MISSLE_PROJECTILE_DAMAGE = 3,
    MISSLE_PROJECTILE_SPEED = 8,
    MISSLE_PROJECTILE_WIDTH = 30,
    MISSLE_PROJECTILE_HEIGHT = 30,
    MISSLE_PROJECTILE_HITBOX_X = 0,
    MISSLE_PROJECTILE_HITBOX_Y = 0,
    MISSLE_PROJECTILE_HITBOX_W = 26,
    MISSLE_PROJECTILE_HITBOX_H = 6,
};

enum BombProjectile
{
    BOMB_PROJECTILE_DAMAGE = 4,
    BOMB_PROJECTILE_SPEED = 0,
    BOMB_PROJECTILE_WIDTH = 30,
    BOMB_PROJECTILE_HEIGHT = 30,
    BOMB_PROJECTILE_HITBOX_X = 0,
    BOMB_PROJECTILE_HITBOX_Y = 0,
    BOMB_PROJECTILE_HITBOX_W = 0,
    BOMB_PROJECTILE_HITBOX_H = 0,
};

enum EnemyProjectile
{
    
};

enum MetroidProjectile
{
    
};

Projectile::Projectile(ProjectileType type, ProjectileLaunchState launch_state, s32 start_x, s32 start_y, u8 start_facing)
{
    /* Default Init */
    
    // Allocate Memory for Sprites

    // Set Default Info
    projectile_type = type;
    projectile_launched_state = launch_state;
    projectile_x = start_x;
    projectile_y = start_y;
    projectile_facing = start_facing;
    
    projectile_explode = false;
    projectile_sound_explosion = false;
    projectile_explode_finished = false;
    projectile_charged = false;
    
    projectile_bomb_timer = BOMB_EXPLOSION_TIME;
    projectile_explosion_frames = 0;
    
    // Set Type-Specific Info
    projectile_set_type();
}

Projectile::~Projectile()
{}

void Projectile::projectile_update(Sound* pSound, u8 collision)
{
    /* Check Projectile Collision */
    
    // Only applies to Power Beam, Ice Beam and Missles
    if (collision != 0 &&
        projectile_type != PROJECTILE_WAVEBEAM && projectile_type != PROJECTILE_WAVEBEAM_CHARGED &&
        projectile_type != PROJECTILE_SPAZERBEAM && projectile_type != PROJECTILE_SPAZERBEAM_CHARGED &&
        projectile_type != PROJECTILE_PLASMABEAM && projectile_type != PROJECTILE_PLASMABEAM_CHARGED)
    {
        projectile_explode = true;
    }
    
    /* Projectile Explosion */
    
    if (projectile_explode == true)
    {
        // Remove Hitbox
        projectile_hitbox_height = 0;
        projectile_hitbox_width = 0;
        projectile_hitbox_y = 0;
        projectile_hitbox_x = 0;
        
        // Remove Speed
        projectile_speed = 0;
        
        // Play Sound
        if (projectile_sound_explosion == false)
        {
            // Explosion Sounds for special Projectiles
            if (projectile_type == PROJECTILE_MISSLE || projectile_type == PROJECTILE_BOMB)
            {
                pSound->sound_play_fx(SOUND_PLAYER_EXPLOSION, SOUNDTYPE_PLAYER_SHOOTING, false, true);
            }
            else if (projectile_type == PROJECTILE_ICEBEAM || projectile_type == PROJECTILE_ICEBEAM_CHARGED)
            {
                pSound->sound_play_fx(SOUND_PLAYER_EXPLOSION, SOUNDTYPE_PLAYER_SHOOTING, false, true);
            }
            
            // Set Sound Played
            projectile_sound_explosion = true;
        }
        
        // Increment Frame
        ++projectile_explosion_frames;
    }
    
    /* Movement */
    
    // Coordinates Change depending on Facing
    switch (projectile_facing)
    {
        case FACING_RIGHT:
            projectile_x += projectile_speed;
            break;
        case FACING_RIGHTUP:
            projectile_x += projectile_speed;
            projectile_y += projectile_speed;
            break;
        case FACING_RIGHTDOWN:
            projectile_x += projectile_speed;
            projectile_y -= projectile_speed;
            break;
        case FACING_LEFT:
            projectile_x -= projectile_speed;
            break;
        case FACING_LEFTUP:
            projectile_x -= projectile_speed;
            projectile_y += projectile_speed;
            break;
        case FACING_LEFTDOWN:
            projectile_x -= projectile_speed;
            projectile_y -= projectile_speed;
            break;
        case FACING_UP:
            projectile_y += projectile_speed;
            break;
        case FACING_DOWN:
            projectile_y -= projectile_speed;
            break;
            
        default:
            projectile_x += projectile_speed;
            break;
    }
    
    /* Bomb, Timed Projectiles Special */
    
    if (projectile_type == PROJECTILE_BOMB)
    {
        if (projectile_bomb_timer <= 0)
            projectile_explode = true;
        else
            --projectile_bomb_timer;
    }
}

void Projectile::projectile_set_exploded(bool explode)
{
    // Bombs can not be forced to explode
    if (projectile_type == PROJECTILE_BOMB)
        return;
    
    // Set Explode
    projectile_explode = explode;
}

void Projectile::projectile_set_type()
{
    if (projectile_type == PROJECTILE_POWERBEAM || projectile_type == PROJECTILE_POWERBEAM_CHARGED)
    {
        projectile_hitbox_x = BEAM_PROJECTILE_HITBOX_X;
        projectile_hitbox_y = BEAM_PROJECTILE_HITBOX_Y;
        projectile_hitbox_width = BEAM_PROJECTILE_HITBOX_W;
        projectile_hitbox_height = BEAM_PROJECTILE_HITBOX_H;
        projectile_speed = BEAM_PROJECTILE_SPEED;
        
        // Charged or non-charged
        if (projectile_type == PROJECTILE_POWERBEAM)
            projectile_damage = BEAM_PROJECTILE_DAMAGE_POWER;
        else if (projectile_type == PROJECTILE_POWERBEAM_CHARGED)
        {
            projectile_damage = BEAM_PROJECTILE_DAMAGE_POWER * 2;
            projectile_charged = true;
        }
    }
    else if (projectile_type == PROJECTILE_ICEBEAM || projectile_type == PROJECTILE_ICEBEAM_CHARGED)
    {
        projectile_hitbox_x = BEAM_PROJECTILE_HITBOX_X;
        projectile_hitbox_y = BEAM_PROJECTILE_HITBOX_Y;
        projectile_hitbox_width = BEAM_PROJECTILE_HITBOX_W;
        projectile_hitbox_height = BEAM_PROJECTILE_HITBOX_H;
        projectile_speed = BEAM_PROJECTILE_SPEED;
        
        // Charged or non-charged
        if (projectile_type == PROJECTILE_ICEBEAM)
            projectile_damage = BEAM_PROJECTILE_DAMAGE_ICE;
        else if (projectile_type == PROJECTILE_ICEBEAM_CHARGED)
        {
            projectile_damage = BEAM_PROJECTILE_DAMAGE_ICE * 2;
            projectile_charged = true;
        }
    }
    else if (projectile_type == PROJECTILE_WAVEBEAM || projectile_type == PROJECTILE_WAVEBEAM_CHARGED)
    {
        projectile_hitbox_x = BEAM_PROJECTILE_HITBOX_X;
        projectile_hitbox_y = BEAM_PROJECTILE_HITBOX_Y;
        projectile_hitbox_width = BEAM_PROJECTILE_HITBOX_W;
        projectile_hitbox_height = BEAM_PROJECTILE_HITBOX_H;
        projectile_speed = BEAM_PROJECTILE_SPEED;
        
        // Charged or non-charged
        if (projectile_type == PROJECTILE_WAVEBEAM)
            projectile_damage = BEAM_PROJECTILE_DAMAGE_WAVE;
        else if (projectile_type == PROJECTILE_WAVEBEAM_CHARGED)
        {
            projectile_damage = BEAM_PROJECTILE_DAMAGE_WAVE * 2;
            projectile_charged = true;
        }
    }
    else if (projectile_type == PROJECTILE_SPAZERBEAM || projectile_type == PROJECTILE_SPAZERBEAM_CHARGED)
    {
        projectile_hitbox_x = BEAM_PROJECTILE_HITBOX_X;
        projectile_hitbox_y = BEAM_PROJECTILE_HITBOX_Y;
        projectile_hitbox_width = BEAM_PROJECTILE_HITBOX_W;
        projectile_hitbox_height = BEAM_PROJECTILE_HITBOX_H;
        projectile_speed = BEAM_PROJECTILE_SPEED;
        
        // Charged or non-charged
        if (projectile_type == PROJECTILE_SPAZERBEAM)
            projectile_damage = BEAM_PROJECTILE_DAMAGE_SPAZER;
        else if (projectile_type == PROJECTILE_SPAZERBEAM_CHARGED)
        {
            projectile_damage = BEAM_PROJECTILE_DAMAGE_SPAZER * 2;
            projectile_charged = true;
        }
    }
    else if (projectile_type == PROJECTILE_PLASMABEAM || projectile_type == PROJECTILE_PLASMABEAM_CHARGED)
    {
        projectile_hitbox_x = BEAM_PROJECTILE_HITBOX_X;
        projectile_hitbox_y = BEAM_PROJECTILE_HITBOX_Y;
        projectile_hitbox_width = BEAM_PROJECTILE_HITBOX_W;
        projectile_hitbox_height = BEAM_PROJECTILE_HITBOX_H;
        projectile_speed = BEAM_PROJECTILE_SPEED;
        
        // Charged or non-charged
        if (projectile_type == PROJECTILE_PLASMABEAM)
            projectile_damage = BEAM_PROJECTILE_DAMAGE_PLASMA;
        else if (projectile_type == PROJECTILE_PLASMABEAM_CHARGED)
        {
            projectile_damage = BEAM_PROJECTILE_DAMAGE_PLASMA * 2;
            projectile_charged = true;
        }
    }
    else if (projectile_type == PROJECTILE_MISSLE)
    {
        projectile_hitbox_x = MISSLE_PROJECTILE_HITBOX_X;
        projectile_hitbox_y = MISSLE_PROJECTILE_HITBOX_Y;
        projectile_hitbox_width = MISSLE_PROJECTILE_HITBOX_W;
        projectile_hitbox_height = MISSLE_PROJECTILE_HITBOX_H;
        projectile_speed = MISSLE_PROJECTILE_SPEED;
        projectile_damage = MISSLE_PROJECTILE_DAMAGE;
    }
    else if (projectile_type == PROJECTILE_BOMB)
    {
        projectile_hitbox_x = BOMB_PROJECTILE_HITBOX_X;
        projectile_hitbox_y = BOMB_PROJECTILE_HITBOX_Y;
        projectile_hitbox_width = BOMB_PROJECTILE_HITBOX_W;
        projectile_hitbox_height = BOMB_PROJECTILE_HITBOX_H;
        projectile_speed = BOMB_PROJECTILE_SPEED;
        projectile_damage = BOMB_PROJECTILE_DAMAGE;
    }
    else
    {
        projectile_hitbox_x = BEAM_PROJECTILE_HITBOX_X;
        projectile_hitbox_y = BEAM_PROJECTILE_HITBOX_Y;
        projectile_hitbox_width = BEAM_PROJECTILE_HITBOX_W;
        projectile_hitbox_height = BEAM_PROJECTILE_HITBOX_H;
        projectile_speed = BEAM_PROJECTILE_SPEED;
        
        // Charged or non-charged
        if (projectile_type == PROJECTILE_POWERBEAM)
            projectile_damage = BEAM_PROJECTILE_DAMAGE_POWER;
        else if (projectile_type == PROJECTILE_POWERBEAM_CHARGED)
            projectile_damage = BEAM_PROJECTILE_DAMAGE_POWER * 2;
    }
}

bool Projectile::projectile_is_outside_screen(s32 player_x, s32 player_y)
{
    // Check Outside Possibilities
    
    if (projectile_x >= player_x + PROJECTILE_MAX_DISTANCE_X)
        return true;
    
    if (projectile_x <= player_x - PROJECTILE_MAX_DISTANCE_X)
        return true;
    
    if (projectile_y >= player_y + PROJECTILE_MAX_DISTANCE_Y)
        return true;
    
    if (projectile_y <= player_y - PROJECTILE_MAX_DISTANCE_Y)
        return true;
    
    // Return inside Screen Bounds
    
    return false;
}
