//
//  projectile.h
//  3ds_metroid
//
//  Created by jbr373 on 22/01/16.
//

#ifndef projectile_h
#define projectile_h

#include <3ds.h>
#include <stdio.h>

#include "image.h"
#include "sound.h"

#define PROJECTILE_SPRITE_DIMENSIONS        30

enum ProjectileType
{
    PROJECTILE_POWERBEAM = 0,
    PROJECTILE_POWERBEAM_CHARGED = 1,
    PROJECTILE_ICEBEAM = 2,
    PROJECTILE_ICEBEAM_CHARGED = 3,
    PROJECTILE_WAVEBEAM = 4,
    PROJECTILE_WAVEBEAM_CHARGED = 5,
    PROJECTILE_SPAZERBEAM = 6,
    PROJECTILE_SPAZERBEAM_CHARGED = 7,
    PROJECTILE_PLASMABEAM = 8,
    PROJECTILE_PLASMABEAM_CHARGED = 9,
    PROJECTILE_MISSLE = 10,
    PROJECTILE_BOMB = 11,
    PROJECTILE_ENEMY = 12,
    PROJECTILE_METROID = 13
};

enum ProjectileLaunchState
{
    PROJECTILE_LAUNCHSTATE_STANDING = 0,
    PROJECTILE_LAUNCHSTATE_CROUCHED = 1,
    PROJECTILE_LAUNCHSTATE_MORPHBALL = 2,
};

class Projectile
{
public:
    Projectile(ProjectileType type, ProjectileLaunchState launch_state, s32 start_x, s32 start_y, u8 start_facing);
    virtual ~Projectile();
    
    void projectile_update(Sound* pSound, u8 collision);
    bool projectile_is_in_hitbox(s32 object_hitbox_x, s32 object_hitbox_y, s32 object_hitbox_w, s32 object_hitbox_h);
    
    // Getters
    u8 projectile_get_type() { return (u8)projectile_type; };
    bool projectile_is_charged() { return projectile_charged; };

    s32 projectile_get_x() { return projectile_x; }
    s32 projectile_get_y() { return projectile_y; }
    s32 projectile_get_width() { return PROJECTILE_SPRITE_DIMENSIONS; }
    s32 projectile_get_height() { return PROJECTILE_SPRITE_DIMENSIONS; }
    s32 projectile_get_hitbox_width() { return projectile_hitbox_width; };
    s32 projectile_get_hitbox_height() { return projectile_hitbox_height; };
    u8 projectile_get_facing() { return projectile_facing; };
    u8 projectile_get_damage() { return projectile_damage; };
    
    bool projectile_is_outside_screen(s32 player_x, s32 player_y);
    bool projectile_get_explode() { return projectile_explode; };
    bool projectile_get_explode_finished() { return projectile_explode_finished; };
    u8 projectile_get_explosion_frame() { return projectile_explosion_frames; };
    ProjectileLaunchState projectile_get_launch_state() { return projectile_launched_state; };
    
    // Setters
    void projectile_set_exploded(bool explode);
    void projectile_set_explode_finished(bool finished) { projectile_explode_finished = finished; };
    void projectile_set_x(s32 x) { projectile_x = x; }
    void projectile_set_y(s32 y) { projectile_y = y; }
    void projectile_set_facing(u8 facing) { projectile_facing = facing; };
    
private:
    void projectile_set_frame(u8 facing);
    void projectile_set_type();
    
    ProjectileType projectile_type;
    
    s32 projectile_x;
    s32 projectile_y;
    s32 projectile_hitbox_x;
    s32 projectile_hitbox_y;
    s32 projectile_hitbox_width;
    s32 projectile_hitbox_height;
    
    u8 projectile_speed;
    u8 projectile_damage;
    u8 projectile_facing;
    u8 projectile_bomb_timer;
    u8 projectile_explosion_frames; // Needed since the render class resets for each projectile
    
    ProjectileLaunchState projectile_launched_state;
    
    bool projectile_explode;
    bool projectile_sound_explosion;
    bool projectile_explode_finished;
    bool projectile_charged;
};

#endif /* projectile_h */
