//
//  render.cpp
//  3ds_metroid
//
//  Created by jbr373 on 15/01/16.
//

#include <stdlib.h>
#include <sfil.h>

#include "render.h"
#include "render_poscorrection.h"
#include "filepaths.h"
#include "actor_facing.h"
#include "files.h"
#include "system.h"

/*******************/
/* Stereoscopic 3D */
/*******************/

#define CONFIG_3D_SLIDERSTATE (*(float *)0x1FF81080)

/**************/
/* Pause Menu */
/**************/

#define PAUSE_TEXT_ID                   4

/*************/
/* Main Menu */
/*************/

#define MAINMENU_SELECTION_W            160
#define MAINMENU_SELECTION_H            30

#define MAINMENU_GLOW_MAX_ALPHA         150
#define MAINMENU_GLOW_ALPHA_STEP        2

#define MAINMENU_MISSION_BORDER_W       220
#define MAINMENU_MISSION_BORDER_H       26
#define MAINMENU_MISSION_BORDER_Y       50

#define MAINMENU_HEALTH_X               10
#define MAINMENU_HEALTH_Y               90

#define MAINMENU_BAR_X                  38
#define MAINMENU_BAR_Y                  95

#define MAINMENU_HEALTHTANK_X           45
#define MAINMENU_HEALTHTANK_Y           86

#define MAINMENU_MISSLE_CURRENT_X       110
#define MAINMENU_MISSLE_FULL_X          160
#define MAINMENU_MISSLE_Y               120
#define MAINMENU_MISSLE_SEPERATE_X      147
#define MAINMENU_MISSLE_SEPERATE_D      8

#define MAINMENU_METROIDS_NUM_X         255
#define MAINMENU_METROIDS_Y             150

#define MAINMENU_OPTIONS_H              40

#define MAINMENU_GAMEOVER_Y             90

/**************/
/* Sprite HUD */
/**************/

/* Top Screen HUD */
#define HUD_NUMBER_X            3 // Old: 60 //10  //56
#define HUD_NUMBER_2_X          14 // Old: 72 //22 //68
#define HUD_NUMBER_Y            11

#define HUD_BAR_X               27
#define HUD_BAR_Y               16

#define HUD_HT_X                27 // Old: 95 //87 //37 //84
#define HUD_HT_Y                8  // Old: 7
#define HUD_HT_WIDTH            6 // Old: 4
#define HUD_HT_HEIGHT           6 // Old: 7
#define HUD_HT_SPACE            2

#define HUD_BG_X                0
#define HUD_BG_Y                0
#define HUD_BG_WIDTH            215
#define HUD_BG_HEIGHT           30

#define HUD_BAR_HEIGHT          2

#define HUD_MISSLE_NUM_X        150
#define HUD_MISSLE_NUM_Y        11

/* Bottom Screen HUD */
#define BTM_UPGRADE_SINGLE_Y    8
#define BTM_SELECTION_WIDTH     37
#define BTM_SELECTION_HEIGHT    35

/* Stage Effects */
#define STAGE_EFFECTS_WIDTH     400
#define STAGE_EFFECTS_HEIGHT    240
#define STAGE_EFFECTS_RAIN_Y    10   // Movement Y
#define STAGE_EFFECTS_RAIN_MAX  240

/* Animation */
#define FADE_ANIM_MAX           20 // 25
#define SPRITE_MAX_FRAME        7
#define MAX_STARS               7

enum HudSprites
{
    HUDSPRITE_HT_FULL = 0,
    HUDSPRITE_HT_EMPTY = 1,
    HUDSPRITE_BAR = 2,
    HUDSPRITE_BG = 3
};

enum StageEffects
{
    STAGEEFFECT_NONE = 0,
    
    STAGEEFFECT_RAIN = 1,
    STAGEEFFECT_RAIN_B = 2, // LOADING ONLY! NEVER ON MAPFILE!
    STAGEEFFECT_FOG = 3,
    STAGEEFFECT_HEAT = 4,
    STAGEEFFECT_DROPS = 5
};

enum FullScreenEffects
{
    SCREENEFFECT_NONE = 0,
    
    SCREENEFFECT_FADEOUT = 1,
    SCREENEFFECT_FADEIN = 2,
};

extern bool game_debug;

/**********************************/
/*         CLASS RENDER           */
/**********************************/

Render::Render()
{
    render_camera_x = 0;
    render_camera_y = 0;
    render_camera_player_x = 0;
    render_camera_player_y = 0;
    
    render_fade_drawcount = FADE_ANIM_MAX;
    
    render_hudcolor = RGBA8(109, 175, 26, 95);
    render_mm_playercolor = RGBA8(196, 44, 0, 215);
    render_mm_mapcolor = RGBA8(255, 148, 0, 195);
    render_mm_mapbordercolor = RGBA8(112, 15, 0, 210);
    render_upgrade_bordercolor = RGBA8(50, 77, 15, 255);
    render_upgrade_bgcolor = RGBA8(109, 175, 26, 255);

    // HUD
    pTexture_HealthTankFull = sfil_load_PNG_file(IMG_HUD_HT_FULL, SF2D_PLACE_RAM);
    pTexture_HealthTankEmpty = sfil_load_PNG_file(IMG_HUD_HT_EMPTY, SF2D_PLACE_RAM);
    pTexture_HudBackground = sfil_load_PNG_file(IMG_HUD_BG, SF2D_PLACE_RAM);
    pTexture_HudBarFull = sfil_load_PNG_file(IMG_HUD_BAR, SF2D_PLACE_RAM);
    pTexture_HudBarEmpty = sfil_load_PNG_file(IMG_HUD_BAR_EMPTY, SF2D_PLACE_RAM);
    
    // Stereoscopic 3D
    render_offset_3d = 0.0f;
    
    // Load Pause Text
    pRenderPause_Text = new StringFileStruct();
    fileop_load_strings(pRenderPause_Text, PAUSE_TEXT_ID);
    
    // Define Glow
    render_calc_starglow();

    // Load Font
    renderfont_load();
    
    // Load Beams
    renderprojectile_load();
    
    // Load Item Sprite
    renderitem_load();
}

Render::~Render()
{
}

/**********************/
/*      DEFAULT       */
/**********************/

void Render::render_draw(SpriteHandler* pSpriteHandler, Stage* pStage, Player* pPlayer, BottomMenu* pMenu, SaveFile* pSaveFile, bool draw_game_pause)
{
    /* Set Frame Animation depending on Game Pause */
    
    bool animate;
    bool anim = false;
    u8 screen = 0;
    
    if (draw_game_pause == true)
    {
        animate = false;
    }
    else
    {
        animate = true;
    }
    
    /* Calculate Camera */
    
    render_calc_camera(pPlayer->player_get_x(),
                       pPlayer->player_get_y(),
                       pPlayer->player_get_width(),
                       pPlayer->player_get_height(),
                       pStage->stage_get_width(),
                       pStage->stage_get_height());

    /* Top Screen */
    
    while (screen < 2)
    {
        // Define Screen
        
        if (screen == 0)
        {
            // Define 3D Offset
            render_offset_3d = 0; //(CONFIG_3D_SLIDERSTATE * 1.0f);
            
            // Top, Left
            sf2d_start_frame(GFX_TOP, GFX_LEFT);
        }
        else if (screen == 1)
        {
            // Set Animation off
            animate = false;
            
            // Define 3D Offset
            render_offset_3d = 0;
            
            // Top, Right
            sf2d_start_frame(GFX_TOP, GFX_RIGHT);
        }
        
        // Animation
        if (screen == 0)
        {
            anim = true;
        }
        else
        {
            anim = false;
        }
        
        /**********************************/
        /*        ROOM: BACKGROUND        */
        /**********************************/
        
        // Draw Room Background Elements
        render_draw_room_background(pStage, render_offset_3d);
        
        /*************************/
        /*        SHADOWS        */
        /*************************/
        
        // Draw Item Shadows
        
        // Draw Object Shadows
        
        // Draw Enemy Shadows
        
        // Draw Player Shadow
        
        /***********************/
        /*        ITEMS        */
        /***********************/
        
        // Reset Frame Increment Happened
        renderitem_set_frame_increment_happened(false);
        
        // Draw Items
        if (pStage->stage_get_items_count() > 0)
        {
            for (u8 i = 0; i <= pStage->stage_get_items_count() - 1; ++i)
            {
                // Collected Items are not drawn
                if (renderitem_get_sprite(pPlayer, pStage->stage_get_item(i), false) != NULL)
                {
                    render_draw_spritesheet(renderitem_get_sprite(pPlayer, pStage->stage_get_item(i), animate),
                                            false,
                                            render_calc_camera_actor_x(pPlayer,
                                                                       pStage->stage_get_item(i)->item_get_x(),
                                                                       pStage->stage_get_item(i)->item_get_y(),
                                                                       SPRITESHEET_ITEM_WIDTH_SINGLE,
                                                                       SPRITESHEET_ITEM_HEIGHT_SINGLE),
                                            render_calc_camera_actor_y(pPlayer,
                                                                       pStage->stage_get_item(i)->item_get_x(),
                                                                       pStage->stage_get_item(i)->item_get_y(),
                                                                       SPRITESHEET_ITEM_WIDTH_SINGLE,
                                                                       SPRITESHEET_ITEM_HEIGHT_SINGLE),
                                            100,
                                            false,
                                            0/*render_offset_3d * 2*/);
                }
            }
        }
        
        /*************************/
        /*        ENEMIES        */
        /*************************/
        
        // Draw Enemies
        if (pStage->stage_get_enemy_count() > 0)
        {
            for (u8 i = 0; i <= pStage->stage_get_enemy_count() - 1; ++i)
            {
                // Collected Items are not drawn
                if (pSpriteHandler->spritehandler_get_sprite(ACTORSPRITE_TYPE_ENEMY,
                                                             pStage->stage_get_enemy(i)->enemy_get_sprite_path(),
                                                             pStage->stage_get_enemy(i)->enemy_get_anim(),
                                                             pStage->stage_get_enemy(i)->enemy_get_facing(),
                                                             pStage->stage_get_enemy(i)->enemy_get_frame()) != NULL &&
                    pStage->stage_get_enemy(i)->enemy_is_dead() == false)
                {
                    render_draw_spritesheet(pSpriteHandler->spritehandler_get_sprite(ACTORSPRITE_TYPE_ENEMY,
                                                                                     pStage->stage_get_enemy(i)->enemy_get_sprite_path(),
                                                                                     pStage->stage_get_enemy(i)->enemy_get_anim(),
                                                                                     pStage->stage_get_enemy(i)->enemy_get_facing(),
                                                                                     pStage->stage_get_enemy(i)->enemy_get_frame()),
                                            false,
                                            render_calc_camera_actor_x(pPlayer,
                                                                       pStage->stage_get_enemy(i)->enemy_get_x(),
                                                                       pStage->stage_get_enemy(i)->enemy_get_y(),
                                                                       pStage->stage_get_enemy(i)->enemy_get_sprite_width() / 8,
                                                                       pStage->stage_get_enemy(i)->enemy_get_sprite_height() / (pStage->stage_get_enemy(i)->enemy_get_max_spriteline() + 1)),
                                            render_calc_camera_actor_y(pPlayer,
                                                                       pStage->stage_get_enemy(i)->enemy_get_x(),
                                                                       pStage->stage_get_enemy(i)->enemy_get_y(),
                                                                       pStage->stage_get_enemy(i)->enemy_get_sprite_width() / 8,
                                                                       pStage->stage_get_enemy(i)->enemy_get_sprite_height() / (pStage->stage_get_enemy(i)->enemy_get_max_spriteline() + 1)),
                                            pStage->stage_get_enemy(i)->enemy_get_size(),
                                            false,
                                            0/*render_offset_3d * 2*/);
                }
            }
        }
        
        /****************************/
        /*        PROJECILES        */
        /****************************/
        
        // Draw Projectiles
        for (u8 k = 0; k <= pPlayer->player_get_projectile_count(); ++k)
        {
            if (pPlayer->player_get_projectile(k) != NULL)
            {
                // Calculate Position
                renderprojectile_set_projectile_screen_pos(pPlayer, pPlayer->player_get_projectile(k), render_camera_player_x, render_camera_player_y);
                
                // Draw Sprite
                render_draw_sprite_rotate(renderprojectile_get_sprite(pPlayer->player_get_projectile(k), true),
                                          false,
                                          renderprojectile_get_projectile_x(),
                                          renderprojectile_get_projectile_y(),
                                          false,
                                          render_calc_facing_angle(pPlayer->player_get_projectile(k)->projectile_get_facing()),
                                          0/*render_offset_3d * 2*/);
                
                // Signal Explode once Sprite Animation ends
                if (pPlayer->player_get_projectile(k)->projectile_get_explode() == true && renderprojectile_get_sprite(pPlayer->player_get_projectile(k), true)->spritesheet_frame == SPRITE_MAX_FRAME)
                {
                    // Signal finished for Object delete
                    pPlayer->player_get_projectile(k)->projectile_set_explode_finished(true);
                }
            }
        }
        
        /************************/
        /*        PLAYER        */
        /************************/
        
        // Draw Player Effect
        if (pSpriteHandler->spritehandler_get_effect_sprite(pPlayer) != NULL)
        {
            // Draw First Effect
            render_draw_spritesheet(pSpriteHandler->spritehandler_get_effect_sprite(pPlayer),
                                    false,
                                    render_camera_player_x,
                                    render_camera_player_y + (pPlayer->player_get_height() * 0.15),
                                    100,
                                    false,
                                    0/*render_offset_3d * 2*/);
            
            // Special: Jumping
            if (pPlayer->player_get_anim() == PLAYERANIM_JUMP90 && pPlayer->player_get_state() == PLAYERSTATE_JUMPING)
            {
                render_draw_spritesheet(pSpriteHandler->spritehandler_get_effect_sprite(pPlayer),
                                        false,
                                        render_camera_player_x,
                                        render_camera_player_y + (pPlayer->player_get_height() * 0.40),
                                        100,
                                        false,
                                        0/*render_offset_3d * 2*/);
            }
        }
        
        // Draw Black Background on Player Death
        if (pPlayer->player_is_dead() == true)
        {
            sf2d_draw_rectangle(0, 0, 420, 240, RGBA8(0, 0, 0, 255));
        }
        
        // Draw Player
        if (pSpriteHandler->spritehandler_get_sprite(pPlayer, false)->spritesheet_texture != NULL)
        {
            render_draw_spritesheet(pSpriteHandler->spritehandler_get_sprite(pPlayer,
                                                                             pStage->stage_is_in_water(pPlayer->player_get_x(),
                                                                                                       pPlayer->player_get_y(),
                                                                                                       pPlayer->player_get_hitbox_height())),
                                    animate,
                                    render_camera_player_x,
                                    render_camera_player_y,
                                    100,
                                    false,
                                    0/*render_offset_3d * 2*/);
        }
        
        /*****************************/
        /*        CHARGE BEAM        */
        /*****************************/
        
        // Draw Missle Smoke
        if (pPlayer->player_get_used_projectile() == PROJECTILE_MISSLE && pPlayer->player_is_dead() == false)
        {
            if (pPlayer->player_get_anim() != PLAYERANIM_CLIMBING && pPlayer->player_get_anim() != PLAYERANIM_DAMAGE && pPlayer->player_get_anim() != PLAYERANIM_JUMPSCREW && pPlayer->player_get_anim() != PLAYERANIM_JUMPSPIN && pPlayer->player_get_morphball() != true)
            {
                // Calculate Position
                renderprojectile_set_chargebeam_screen_pos(pPlayer,
                                                           render_camera_player_x,
                                                           render_camera_player_y,
                                                           pSpriteHandler->spritehandler_get_sprite(pPlayer, false)->spritesheet_frame);

                // Draw Missle Smoke Animation
                render_draw_sprite_rotate(renderprojectile_get_smoke_sprite(anim),
                                          false,
                                          renderprojectile_get_chargebeam_x(),
                                          renderprojectile_get_chargebeam_y(),
                                          false,
                                          0,
                                          0/*render_offset_3d*/);
            }
            else
            {
                // Reset Missle Smoke
                renderprojectile_reset_missle_smoke();
            }
        }
        else
        {
            // Reset Missle Smoke
            renderprojectile_reset_missle_smoke();
        }
        
        // Draw Charge Beam
        if (pPlayer->player_get_charge_beam_counter() > 0 && pPlayer->player_is_dead() == false)
        {
            if (pPlayer->player_get_anim() != PLAYERANIM_CLIMBING && pPlayer->player_get_anim() != PLAYERANIM_DAMAGE && pPlayer->player_get_anim() != PLAYERANIM_JUMPSCREW && pPlayer->player_get_anim() != PLAYERANIM_JUMPSPIN && pPlayer->player_get_morphball() != true)
            {
                // Calculate Position
                renderprojectile_set_chargebeam_screen_pos(pPlayer, render_camera_player_x, render_camera_player_y, pSpriteHandler->spritehandler_get_sprite(pPlayer, false)->spritesheet_frame);
                
                // Draw Charge Beam Animation
                // Frame increment is handled by Projectile Render Class
                render_draw_sprite_rotate(renderprojectile_get_chargebeam_sprite(pPlayer->player_get_used_projectile(),
                                                                                 pPlayer->player_get_charge_beam_counter(),
                                                                                 anim),
                                          false,
                                          renderprojectile_get_chargebeam_x(),
                                          renderprojectile_get_chargebeam_y(),
                                          false,
                                          0,
                                          0/*render_offset_3d * 2*/);
            }
            else
            {
                renderprojectile_reset_chargebeam_frame();
            }
        }
        else
        {
            renderprojectile_reset_chargebeam_frame();
        }
        
        /*************************/
        /*        OBJECTS        */
        /*************************/
        
        // Draw Objects
        if (pStage->stage_get_object_count() > 0 && pPlayer->player_is_dead() == false)
        {
            for (u8 i = 0; i <= pStage->stage_get_object_count() - 1; ++i)
            {
                // Collected Items are not drawn
                if (pSpriteHandler->spritehandler_get_sprite(ACTORSPRITE_TYPE_OBJECT,
                                                             pStage->stage_get_object(i)->object_get_sprite_path(),
                                                             pStage->stage_get_object(i)->object_get_anim(),
                                                             0,
                                                             pStage->stage_get_object(i)->object_get_frame()) != NULL &&
                    pStage->stage_get_object(i)->object_get_type() != OBJECTTYPE_ROOMTRIGGER &&
                    pStage->stage_get_object(i)->object_get_type() != OBJECTTYPE_LIGHT)
                {
                    render_draw_spritesheet(pSpriteHandler->spritehandler_get_sprite(ACTORSPRITE_TYPE_OBJECT,
                                                                                     pStage->stage_get_object(i)->object_get_sprite_path(),
                                                                                     pStage->stage_get_object(i)->object_get_anim(),
                                                                                     0,
                                                                                     pStage->stage_get_object(i)->object_get_frame()),
                                            false,
                                            render_calc_camera_actor_x(pPlayer,
                                                                       pStage->stage_get_object(i)->object_get_x(),
                                                                       pStage->stage_get_object(i)->object_get_y(),
                                                                       pStage->stage_get_object(i)->object_get_sprite_width(),
                                                                       pStage->stage_get_object(i)->object_get_sprite_height() / (pStage->stage_get_object(i)->object_get_max_spriteline() + 1)),
                                            render_calc_camera_actor_y(pPlayer,
                                                                       pStage->stage_get_object(i)->object_get_x(),
                                                                       pStage->stage_get_object(i)->object_get_y(),
                                                                       pStage->stage_get_object(i)->object_get_sprite_width() / 8,
                                                                       pStage->stage_get_object(i)->object_get_sprite_height() / (pStage->stage_get_object(i)->object_get_max_spriteline() + 1)),
                                            pStage->stage_get_object(i)->object_get_size(),
                                            false,
                                            0/*render_offset_3d*/);
                }
            }
        }
        
        /**********************************/
        /*        ROOM: FOREGROUND        */
        /**********************************/
        
        // Draw Room Foreground Graphics
        if (pPlayer->player_is_dead() == false)
        {
            // Draw Room Foreground Elements
            render_draw_room_foreground(pStage, pPlayer, render_offset_3d);
            
            // Draw Room Effect
            render_draw_room_effect(pStage->stage_get_light(), render_offset_3d);
        }
        
        /*********************************/
        /*        HUD AND EFFECTS        */
        /*********************************/
        
        // Draw HUD
        if (pPlayer->player_is_dead() == false)
        {
            render_draw_hud(pPlayer, render_offset_3d);
        }
        
        // Draw Pause Screen
        if (draw_game_pause == true && pPlayer->player_is_dead() == false)
        {
            render_draw_pause(true, render_offset_3d);
        }
        
        // Define Fade-Out Drawing
        bool fade_increment = false;
        
        if (game_debug == true && screen == 1)
        {
            fade_increment = true;
        }
        
        // Draw Effect
        if (pStage->stage_get_room_change() == STAGE_ROOMCHANGE_FADEOUT)
        {
            // Fade Out
            render_reset_fullscreen_effect(SCREENEFFECT_FADEOUT);
            render_draw_fullscreen_effect(SCREENEFFECT_FADEOUT, true, fade_increment);
            
            // Set Menu Switch
            if (render_fade_drawcount == FADE_ANIM_MAX)
            {
                pStage->stage_set_room_change(STAGE_ROOMCHANGE_LOAD);
            }
        }
        else if (pStage->stage_get_room_change() == STAGE_ROOMCHANGE_FADEIN)
        {
            // Fade back in
            render_reset_fullscreen_effect(SCREENEFFECT_FADEIN);
            render_draw_fullscreen_effect(SCREENEFFECT_FADEIN, true, fade_increment);
            
            // Set Menu Input
            if (render_fade_drawcount == 0)
            {
                pStage->stage_set_room_change(STAGE_ROOMCHANGE_NOCHANGE);
            }
        }
        
        // End Frame for our current Screen Side
        sf2d_end_frame();
        
        // Increment Screen Counter
        ++screen;
    }
    
    // Reset Screen Counter
    screen = 0;
    
    if (game_debug == true)
    {
        return;
    }
    
    /* Draw Bottom Screen */
    
    sf2d_start_frame(GFX_BOTTOM, GFX_LEFT);
    
    // Draw Bottom Menu
    render_draw_bottom_menu(pPlayer, pStage, pMenu, pSaveFile);
    
    // Draw Pause Screen
    if (draw_game_pause == true)
    {
        render_draw_pause(false, 0);
    }
    
    // Draw Effect
    if (pStage->stage_get_room_change() == STAGE_ROOMCHANGE_FADEOUT)
    {
        // Fade Out
        render_reset_fullscreen_effect(SCREENEFFECT_FADEOUT);
        render_draw_fullscreen_effect(SCREENEFFECT_FADEOUT, false, true);
        
        // Set Menu Switch
        if (render_fade_drawcount == FADE_ANIM_MAX)
        {
            pStage->stage_set_room_change(STAGE_ROOMCHANGE_LOAD);
        }
    }
    else if (pStage->stage_get_room_change() == STAGE_ROOMCHANGE_FADEIN)
    {
        // Fade back in
        render_reset_fullscreen_effect(SCREENEFFECT_FADEIN);
        render_draw_fullscreen_effect(SCREENEFFECT_FADEIN, false, true);
        
        // Set Menu Input
        if (render_fade_drawcount == 0)
        {
            pStage->stage_set_room_change(STAGE_ROOMCHANGE_NOCHANGE);
        }
    }
    
    sf2d_end_frame();
}

void Render::render_draw_pause(bool is_topscreen, float offset_3d)
{
    if (is_topscreen == true)
    {
        // Draw Top Background
        sf2d_draw_rectangle(0 + offset_3d, 0, TOPSCREEN_X, TOPSCREEN_Y, RGBA8(0, 0, 0, 175));

        // Draw all Text String
        for (u8 i = 0; i <= 1; ++i)
        {
            // Draw Text String
            renderfont_draw_text(fileop_read_string_text(pRenderPause_Text, i).c_str(),
                                 fileop_read_string_x(pRenderPause_Text, i) + offset_3d,
                                 fileop_read_string_y(pRenderPause_Text, i));
        }
    }
    else
    {
        // Draw Bottom Background
        sf2d_draw_rectangle(0, 0, BOTTOMSCREEN_X, BOTTOMSCREEN_Y, RGBA8(0, 0, 0, 175));
    }
}


void Render::render_draw(GameMenu* pMenu, SaveFile* pSaveFile)
{
    /* Star Glow */
    
    // Recalculate Stars
    render_calc_starglow();
    
    /* Top Screen */
 
    u8 screen = 0;
    static u8 menu_static_line = 0;
    
    while (screen != 2)
    {
        // Define Screen
        
        if (screen == 0)
        {
            // Define 3D Offset
            render_offset_3d = -(CONFIG_3D_SLIDERSTATE * 0.5f);
            
            // Top, Left
            sf2d_start_frame(GFX_TOP, GFX_LEFT);
        }
        else if (screen == 1)
        {
            // Define 3D Offset
            render_offset_3d = CONFIG_3D_SLIDERSTATE * 0.5f;
            
            // Top, Right
            sf2d_start_frame(GFX_TOP, GFX_RIGHT);
        }
        
        // Draw Menu
        if (pMenu->gamemenu_get_submenu() != GAMEMENU_BOOT)
        {
            // Draw Background Effect
            if (pMenu->gamemenu_get_submenu() != GAMEMENU_WARNING && pMenu->gamemenu_get_submenu() != GAMEMENU_TRANSITION)
            {
                // Draw Blue Background
                sf2d_draw_rectangle(0 + render_offset_3d, 0, TOPSCREEN_X, TOPSCREEN_Y, RGBA8(0, 0, 0, 255));
                sf2d_draw_rectangle(0 + render_offset_3d, 0, TOPSCREEN_X, TOPSCREEN_Y, RGBA8(19, 12, 75, 25));
                
                // Draw Star Glow
                for (u8 s = 0; s < RENDER_MAX_STARGLOW; ++s)
                {
                    // Only draw allocated Stars
                    if (render_starglow[s] != NULL)
                    {
                        if (render_starglow[s]->star_topscreen == true && render_starglow[s]->star_loaded == true)
                        {
                            sf2d_draw_fill_circle(render_starglow[s]->star_x + render_offset_3d,
                                                  render_starglow[s]->star_y,
                                                  render_starglow[s]->star_size,
                                                  RGBA8(255, 255, 255, render_starglow[s]->star_alpha));
                        }
                    }
                }
            }
            
            // Draw Image or Text depending on Menu
            if (pMenu->gamemenu_is_main_menu() == true)
            {
                if (pMenu->gamemenu_get_submenu() == GAMEMENU_WARNING && pMenu->gamemenu_get_submenu() != GAMEMENU_TRANSITION)
                {
                    // Draw all Warning Text
                    renderfont_draw_text(METROID_WARNING_A, METROID_WARNING_X, METROID_WARNING_A_Y);
                    renderfont_draw_text(METROID_WARNING_B, METROID_WARNING_X, METROID_WARNING_A_Y + 12);
                    renderfont_draw_text(METROID_WARNING_C, METROID_WARNING_X, METROID_WARNING_A_Y + 24);
                    renderfont_draw_text(METROID_WARNING_D, METROID_WARNING_X, METROID_WARNING_A_Y + 36);
                    renderfont_draw_text(METROID_WARNING_E, METROID_WARNING_X, METROID_WARNING_B_Y);
                    renderfont_draw_text(METROID_WARNING_F, METROID_WARNING_X, METROID_WARNING_B_Y + 12);
                    renderfont_draw_text(METROID_WARNING_G, METROID_WARNING_X, METROID_WARNING_B_Y + 24);
                    renderfont_draw_text(METROID_WARNING_H, METROID_WARNING_X, METROID_WARNING_B_Y + 36);
                    renderfont_draw_text(METROID_WARNING_I, METROID_WARNING_X, METROID_WARNING_B_Y + 68);
                    renderfont_draw_text(METROID_WARNING_J, METROID_WARNING_X, METROID_WARNING_B_Y + 80);
                    renderfont_draw_text(METROID_WARNING_K, METROID_WARNING_X, METROID_WARNING_B_Y + 92);
                    renderfont_draw_text(METROID_WARNING_L, METROID_WARNING_X, METROID_WARNING_C_Y);
                }
                else
                {
                    // Draw Title Image
                    if (pMenu->gamemenu_get_sprite() != NULL)
                    {
                        sf2d_draw_texture_part(pMenu->gamemenu_get_sprite(), 90 - render_offset_3d * 2, 70, 0, 0, 213, 130);
                    }
                    
                    // DEMO: Draw Sample Version Text
                    renderfont_draw_text(METROID_VERSION_TEXT, 0, 0);
                }
            }
            else  if (pMenu->gamemenu_get_submenu() != GAMEMENU_TRANSITION)
            {
                // Draw Game Over Image
                if (pMenu->gamemenu_get_sprite() != NULL)
                {
                    sf2d_draw_texture_part(pMenu->gamemenu_get_sprite(), 90 - render_offset_3d * 2, 85, 0, 131, 213, 70);
                }
            }
            
            // Draw Effect
            if (pMenu->gamemenu_get_submenu_switch() == true)
            {
                // Fade Out
                render_reset_fullscreen_effect(SCREENEFFECT_FADEOUT);
                render_draw_fullscreen_effect(SCREENEFFECT_FADEOUT, true, false);
                pMenu->gamemenu_set_input_block(true);
                
                // Set Menu Switch
                if (render_fade_drawcount == FADE_ANIM_MAX)
                {
                    pMenu->gamemenu_update_next_submenu();
                }
            }
            else if (pMenu->gamemenu_get_input_block() == true && pMenu->gamemenu_get_submenu_switch() == false)
            {
                // Fade back in
                render_reset_fullscreen_effect(SCREENEFFECT_FADEIN);
                render_draw_fullscreen_effect(SCREENEFFECT_FADEIN, true, false);
                
                // Set Menu Input
                if (render_fade_drawcount == 0)
                {
                    pMenu->gamemenu_set_input_block(false);
                }
            }
        }
        
        // Increment Screen Counter
        ++screen;
        
        // End Current Frame
        sf2d_end_frame();
    }
    
    // Reset Screen Counter
    screen = 0;
    
    if (game_debug == true)
    {
        return;
    }
    
    /* Draw Bottom Screen */
    
    sf2d_start_frame(GFX_BOTTOM, GFX_LEFT);
    
    // Draw Menu
    if (pMenu->gamemenu_get_submenu() != GAMEMENU_BOOT)
    {
        // Draw Background
        if (pMenu->gamemenu_get_submenu() != GAMEMENU_WARNING && pMenu->gamemenu_get_submenu() != GAMEMENU_TRANSITION)
        {
            sf2d_draw_rectangle(0, 0, BOTTOMSCREEN_X, BOTTOMSCREEN_Y, RGBA8(0, 0, 0, 255));
            sf2d_draw_rectangle(0, 0, BOTTOMSCREEN_X, BOTTOMSCREEN_Y, RGBA8(19, 12, 75, 25));
        
            // Draw Star Effect
            for (u8 s = 0; s < RENDER_MAX_STARGLOW; ++s)
            {
                // Only draw allocated Stars
                if (render_starglow[s] != NULL)
                {
                    if (render_starglow[s]->star_topscreen == false && render_starglow[s]->star_loaded == true)
                    {
                        sf2d_draw_fill_circle(render_starglow[s]->star_x,
                                              render_starglow[s]->star_y,
                                              render_starglow[s]->star_size,
                                              RGBA8(255, 255, 255, render_starglow[s]->star_alpha));
                    }
                }
            }
        }
    
        if (pMenu->gamemenu_is_main_menu() == true)
        {
            // Main Menu Bottom Screen
            if (pMenu->gamemenu_get_submenu() == GAMEMENU_SPLASH)
            {
                /*************************/
                /* Draw String File Text */
                /*************************/
                
                // Draw "Press Start Button" Text
                renderfont_draw_text(pMenu->gamemenu_get_text_string(TEXT_SPLASH_START).c_str(),
                                     pMenu->gamemenu_get_text_x(TEXT_SPLASH_START),
                                     pMenu->gamemenu_get_text_y(TEXT_SPLASH_START));
            }
            else if (pMenu->gamemenu_get_submenu() == GAMEMENU_OPTIONS)
            {
                /*************************/
                /* Draw Background Boxes */
                /*************************/
                
                s32 sel_x;
                
                // Draw Background Boxes
                for (u8 i = 0; i < 3; ++i)
                {
                    /* Get Info for Selected Option */
                    if (i == 0)
                    {
                        if (pMenu->gamemenu_get_config_file()->config_movement_changed == false)
                            sel_x = 1 + MAINMENU_SELECTION_W;
                        else
                            sel_x = 1 + MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2);
                    }
                    else if (i == 1)
                    {
                        if (pMenu->gamemenu_get_config_file()->config_shooting_changed == false)
                            sel_x = 1 + MAINMENU_SELECTION_W;
                        else
                            sel_x = 1 + MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2);
                    }
                    else
                    {
                        if (pMenu->gamemenu_get_config_file()->config_map == false)
                            sel_x = 1 + MAINMENU_SELECTION_W;
                        else
                            sel_x = 1 + MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2);
                    }
                    
                    /* Descriptions */
                        
                    // Draw Background
                    render_draw_rectangle(0, (MAINMENU_OPTIONS_H * i) + 40, MAINMENU_SELECTION_W, MAINMENU_OPTIONS_H, RGBA8(96, 139, 50, 175), 0);
                    sf2d_draw_rectangle(1, (MAINMENU_OPTIONS_H * i) + 41, MAINMENU_SELECTION_W - 1, MAINMENU_OPTIONS_H - 1, RGBA8(134, 195, 81, 35));
                    
                    /* Default Options */
                    
                    // Draw Background
                    render_draw_rectangle(MAINMENU_SELECTION_W, (MAINMENU_OPTIONS_H * i) + 40, (MAINMENU_SELECTION_W / 2), MAINMENU_OPTIONS_H, RGBA8(96, 139, 50, 175), 0);
                    sf2d_draw_rectangle(1 + MAINMENU_SELECTION_W, (MAINMENU_OPTIONS_H * i) + 41, (MAINMENU_SELECTION_W / 2) - 1, MAINMENU_OPTIONS_H - 1, RGBA8(134, 195, 81, 35));
                    
                    /* Changed Option */
                    
                    // Draw Background
                    render_draw_rectangle(MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2), (MAINMENU_OPTIONS_H * i) + 40, (MAINMENU_SELECTION_W / 2), MAINMENU_OPTIONS_H, RGBA8(96, 139, 50, 175), 0);
                    sf2d_draw_rectangle(1 + MAINMENU_SELECTION_W + (MAINMENU_SELECTION_W / 2), (MAINMENU_OPTIONS_H * i) + 41, (MAINMENU_SELECTION_W / 2) - 1, MAINMENU_OPTIONS_H - 1, RGBA8(134, 195, 81, 35));
                    
                    /* Draw Selected Config Options */
                    
                    // Draw Options Rectangle
                    sf2d_draw_rectangle(sel_x,
                                        (MAINMENU_OPTIONS_H * i) + 41,
                                        (MAINMENU_SELECTION_W / 2) - 1,
                                        MAINMENU_OPTIONS_H - 1,
                                        RGBA8(134, 195, 81, 65));
                }
                
                /* Return Button */
                
                // Draw Background
                render_draw_rectangle(0, BOTTOMSCREEN_Y - MAINMENU_SELECTION_H, BOTTOMSCREEN_X, MAINMENU_SELECTION_H, RGBA8(96, 139, 50, 175), 0);
                sf2d_draw_rectangle(1, BOTTOMSCREEN_Y - MAINMENU_SELECTION_H + 1, BOTTOMSCREEN_X - 1, MAINMENU_SELECTION_H - 1, RGBA8(134, 195, 81, 35));
                
                /* Draw Selected Options Menu Point */
                
                if (pMenu->gamemenu_get_selection_options() < 3)
                {
                    sf2d_draw_rectangle(0,
                                        (MAINMENU_OPTIONS_H * pMenu->gamemenu_get_selection_options()) + 41,
                                        MAINMENU_SELECTION_W - 1,
                                        MAINMENU_OPTIONS_H - 1,
                                        RGBA8(134, 195, 81, 65));
                }
                else
                {
                    sf2d_draw_rectangle(1, BOTTOMSCREEN_Y - MAINMENU_SELECTION_H + 1, BOTTOMSCREEN_X - 1, MAINMENU_SELECTION_H-1, RGBA8(134, 195, 81, 65));
                }
                
                /*************************/
                /* Draw String File Text */
                /*************************/
                
                for (u8 i = TEXT_OPTIONS_MOVEMENT; i <= TEXT_OPTIONS_RETURN; ++i)
                {
                    // Draw Option Text Strings
                    renderfont_draw_text(pMenu->gamemenu_get_text_string(i).c_str(),
                                         pMenu->gamemenu_get_text_x(i),
                                         pMenu->gamemenu_get_text_y(i));
                }
                
            }
            else if (pMenu->gamemenu_get_submenu() == GAMEMENU_SAVEFILES)
            {
                
                // Draw Scan Line
                
                if (menu_static_line == BOTTOMSCREEN_Y)
                {
                    menu_static_line = 0;
                }
                else
                {
                    ++menu_static_line;
                }
                
                // Draw Line Effect
                if (menu_static_line < BOTTOMSCREEN_Y - (1 + MAINMENU_SELECTION_H) &&
                    menu_static_line > MAINMENU_MISSION_BORDER_Y + MAINMENU_MISSION_BORDER_H + 1)
                {
                    sf2d_draw_line(0,
                                   menu_static_line,
                                   BOTTOMSCREEN_X,
                                   menu_static_line,
                                   RGBA8(134, 195, 81, 65));
                }
                
                // TODO: In desperate need of cleanup!
                
                /*************************/
                /* Draw Background Boxes */
                /*************************/
                
                // Draw Mission Log Rectangle
                render_draw_rectangle(0,
                                      MAINMENU_MISSION_BORDER_Y,
                                      MAINMENU_MISSION_BORDER_W,
                                      MAINMENU_MISSION_BORDER_H,
                                      RGBA8(96, 139, 50, 175), 0);
                
                // Draw Mission Log Box
                sf2d_draw_rectangle(1,
                                    MAINMENU_MISSION_BORDER_Y - 1,
                                    MAINMENU_MISSION_BORDER_W - 1 ,
                                    MAINMENU_MISSION_BORDER_H -1,
                                    RGBA8(134, 195, 81, 35));
                
                // Draw File Info Background Borders
                sf2d_draw_line(0,
                               MAINMENU_MISSION_BORDER_Y + MAINMENU_MISSION_BORDER_H + 1,
                               BOTTOMSCREEN_X,
                               MAINMENU_MISSION_BORDER_Y + MAINMENU_MISSION_BORDER_H + 1,
                               RGBA8(96, 139, 50, 175));
                
                sf2d_draw_line(0,
                               BOTTOMSCREEN_Y - (1 + MAINMENU_SELECTION_H),
                               BOTTOMSCREEN_X,
                               BOTTOMSCREEN_Y - (1 + MAINMENU_SELECTION_H),
                               RGBA8(96, 139, 50, 175));
                
                // Draw File Info Background
                sf2d_draw_rectangle(0,
                                    MAINMENU_MISSION_BORDER_Y + MAINMENU_MISSION_BORDER_H + 2,
                                    BOTTOMSCREEN_X,
                                    BOTTOMSCREEN_Y - (2 + MAINMENU_SELECTION_H),
                                    RGBA8(134, 195, 81, 35));
                
                // Draw New Rectangle
                render_draw_rectangle(0,
                                      BOTTOMSCREEN_Y - MAINMENU_SELECTION_H,
                                      MAINMENU_SELECTION_W,
                                      MAINMENU_SELECTION_H,
                                      RGBA8(96, 139, 50, 175), 0);
                
                // Draw New Background
                sf2d_draw_rectangle(1,
                                    BOTTOMSCREEN_Y - MAINMENU_SELECTION_H + 1,
                                    MAINMENU_SELECTION_W - 1,
                                    MAINMENU_SELECTION_H - 1,
                                    RGBA8(134, 195, 81, 35));
                
                // Draw Continue Rectangle
                render_draw_rectangle(MAINMENU_SELECTION_W + 1,
                                      BOTTOMSCREEN_Y - MAINMENU_SELECTION_H,
                                      MAINMENU_SELECTION_W,
                                      MAINMENU_SELECTION_H,
                                      RGBA8(96, 139, 50, 175), 0);
                
                // Draw Continue Background
                sf2d_draw_rectangle(MAINMENU_SELECTION_W + 2,
                                    BOTTOMSCREEN_Y - MAINMENU_SELECTION_H + 1,
                                    MAINMENU_SELECTION_W - 1,
                                    MAINMENU_SELECTION_H - 1,
                                    RGBA8(134, 195, 81, 35));
                
                // Draw Options Rectangle
                render_draw_rectangle(0,
                                      0,
                                      MAINMENU_SELECTION_W * 2,
                                      MAINMENU_SELECTION_H,
                                      RGBA8(96, 139, 50, 175), 0);
                
                // Draw Options Background
                sf2d_draw_rectangle(0 + 1,
                                    1,
                                    (MAINMENU_SELECTION_W * 2) - 1,
                                    MAINMENU_SELECTION_H - 1,
                                    RGBA8(134, 195, 81, 35));
                
                // Draw Health Separator Line
                sf2d_draw_line(0,
                               MAINMENU_HEALTH_Y + 20,
                               BOTTOMSCREEN_X,
                               MAINMENU_HEALTH_Y + 20,
                               RGBA8(96, 139, 50, 175));
                
                // Draw Missle Seperator Line
                sf2d_draw_line(0,
                               MAINMENU_MISSLE_Y + 20,
                               BOTTOMSCREEN_X,
                               MAINMENU_MISSLE_Y + 20,
                               RGBA8(96, 139, 50, 175));
                
                // Draw Metroids Seperator Line
                sf2d_draw_line(0,
                               MAINMENU_METROIDS_Y + 20,
                               BOTTOMSCREEN_X,
                               MAINMENU_METROIDS_Y + 20,
                               RGBA8(96, 139, 50, 175));
                
                // Define Selection Background
                s32 sel_x, sel_y, sel_w;
                
                if (pMenu->gamemenu_get_selection() == 2)
                {
                    sel_x = 1; sel_y = 0; sel_w = (MAINMENU_SELECTION_W * 2) - 1;
                }
                else if (pMenu->gamemenu_get_selection() == 0)
                {
                    sel_x = 1; sel_y = (BOTTOMSCREEN_Y - MAINMENU_SELECTION_H) + 1; sel_w = MAINMENU_SELECTION_W - 1;
                }
                else
                {
                    sel_x = MAINMENU_SELECTION_W + 1; sel_y = (BOTTOMSCREEN_Y - MAINMENU_SELECTION_H) + 1; sel_w = MAINMENU_SELECTION_W - 1;
                }
                
                // Draw Options Background
                sf2d_draw_rectangle(sel_x,
                                    sel_y,
                                    sel_w,
                                    MAINMENU_SELECTION_H - 1,
                                    RGBA8(134, 195, 81, 65));
                
                /*************************/
                /* Draw String File Text */
                /*************************/
                
                // Draw "New" Text
                renderfont_draw_text(pMenu->gamemenu_get_text_string(TEXT_SAVEFILE_NEW).c_str(),
                                     pMenu->gamemenu_get_text_x(TEXT_SAVEFILE_NEW),
                                     pMenu->gamemenu_get_text_y(TEXT_SAVEFILE_NEW));
                
                // Draw "Continue" Text
                renderfont_draw_text(pMenu->gamemenu_get_text_string(TEXT_SAVEFILE_CONTINUE).c_str(),
                                     pMenu->gamemenu_get_text_x(TEXT_SAVEFILE_CONTINUE),
                                     pMenu->gamemenu_get_text_y(TEXT_SAVEFILE_CONTINUE));
                
                // Draw "Options" Text
                renderfont_draw_text(pMenu->gamemenu_get_text_string(TEXT_SAVEFILE_OPTIONS).c_str(),
                                     pMenu->gamemenu_get_text_x(TEXT_SAVEFILE_OPTIONS),
                                     pMenu->gamemenu_get_text_y(TEXT_SAVEFILE_OPTIONS));
                
                // Draw "Mission Log" Text
                renderfont_draw_text(pMenu->gamemenu_get_text_string(TEXT_SAVEFILE_FILE).c_str(),
                                     pMenu->gamemenu_get_text_x(TEXT_SAVEFILE_FILE),
                                     pMenu->gamemenu_get_text_y(TEXT_SAVEFILE_FILE));
                
                // Draw "Metroids Remaining" Text
                renderfont_draw_text(pMenu->gamemenu_get_text_string(TEXT_SAVEFILE_METROIDS).c_str(),
                                     pMenu->gamemenu_get_text_x(TEXT_SAVEFILE_METROIDS),
                                     pMenu->gamemenu_get_text_y(TEXT_SAVEFILE_METROIDS));
                
                // Draw "Missles" Text
                renderfont_draw_text(pMenu->gamemenu_get_text_string(TEXT_SAVEFILE_MISSLES).c_str(),
                                     pMenu->gamemenu_get_text_x(TEXT_SAVEFILE_MISSLES),
                                     pMenu->gamemenu_get_text_y(TEXT_SAVEFILE_MISSLES));
                
                /*************************/
                /* Draw Health and Tanks */
                /*************************/
                
                // Draw Health Number
                renderfont_draw_number(pSaveFile->savefile_get_health(), MAINMENU_HEALTH_X, MAINMENU_HEALTH_Y, 2);

                // Draw all Health Tanks if available
                for (u8 i = 1; i <= pSaveFile->savefile_get_healthtanks(); ++i)
                {
                    // Draw Full or Empty Health Tanks
                    if (i <= pSaveFile->savefile_get_healthtanks_available())
                    {
                        sf2d_draw_texture(pTexture_HealthTankFull, (MAINMENU_HEALTHTANK_X + ((HUD_HT_WIDTH + HUD_HT_SPACE) * i)), MAINMENU_HEALTHTANK_Y);
                    }
                    else
                    {
                        sf2d_draw_texture(pTexture_HealthTankEmpty, (MAINMENU_HEALTHTANK_X + ((HUD_HT_WIDTH + HUD_HT_SPACE) * i)), MAINMENU_HEALTHTANK_Y);
                    }
                }
                
                // Draw Health Bar (Full)
                if (pSaveFile->savefile_get_health() > 0)
                {
                    sf2d_draw_texture_scale(pTexture_HudBarFull, MAINMENU_BAR_X, MAINMENU_BAR_Y, pSaveFile->savefile_get_health(), HUD_BAR_HEIGHT);
                }
                
                // Health Bar (Empty)
                if (pSaveFile->savefile_get_health() < 99)
                {
                    sf2d_draw_texture_scale(pTexture_HudBarEmpty,
                                            pSaveFile->savefile_get_health() + MAINMENU_BAR_X,
                                            MAINMENU_BAR_Y,
                                            99 - pSaveFile->savefile_get_health(),
                                            HUD_BAR_HEIGHT);
                }
                
                // Health Bar Edges are "Full" Health Bar Lines
                sf2d_draw_texture_scale(pTexture_HudBarFull, MAINMENU_BAR_X, MAINMENU_BAR_Y, 1, HUD_BAR_HEIGHT);
                sf2d_draw_texture_scale(pTexture_HudBarFull, MAINMENU_BAR_X + 99, MAINMENU_BAR_Y, 1, HUD_BAR_HEIGHT);
                
                /********************/
                /* Draw Missle Info */
                /********************/
                
                // Draw Available Missles
                renderfont_draw_number(pSaveFile->savefile_get_missles_available(),
                                       MAINMENU_MISSLE_CURRENT_X,
                                       MAINMENU_MISSLE_Y,
                                       3);
                
                // Seperation Line
                renderfont_draw_text(")",
                                     MAINMENU_MISSLE_SEPERATE_X,
                                     MAINMENU_MISSLE_Y);
                
                // Draw Total Missles
                renderfont_draw_number(pSaveFile->savefile_get_missles_total(),
                                       MAINMENU_MISSLE_FULL_X,
                                       MAINMENU_MISSLE_Y,
                                       3);
                
                /***********************/
                /* Draw Metroid Number */
                /***********************/
                
                // Draw Metroid Number
                renderfont_draw_number(pSaveFile->savefile_get_metroids(),
                                       MAINMENU_METROIDS_NUM_X,
                                       MAINMENU_METROIDS_Y,
                                       2);
            }
        }
        else if (pMenu->gamemenu_get_submenu() == GAMEMENU_GAMEOVER)
        {
            /*************************/
            /* Draw Background Boxes */
            /*************************/
            
            // Draw Background Boxes
            for (u8 i = 0; i <= 1; ++i)
            {
                // Draw Button Rectangle
                render_draw_rectangle(0,
                                      MAINMENU_GAMEOVER_Y + (MAINMENU_SELECTION_H * i),
                                      BOTTOMSCREEN_X,
                                      MAINMENU_SELECTION_H,
                                      RGBA8(96, 139, 50, 175), 0);
                
                // Draw Button Background
                sf2d_draw_rectangle(1,
                                    MAINMENU_GAMEOVER_Y + (MAINMENU_SELECTION_H * i) + 1,
                                    BOTTOMSCREEN_X - 1,
                                    MAINMENU_SELECTION_H - 1,
                                    RGBA8(134, 195, 81, 35));
            }
            
            // Draw Selection Background
            sf2d_draw_rectangle(1,
                                MAINMENU_GAMEOVER_Y + (MAINMENU_SELECTION_H * pMenu->gamemenu_get_selection_gameover()) + 1,
                                BOTTOMSCREEN_X - 1,
                                MAINMENU_SELECTION_H - 1,
                                RGBA8(134, 195, 81, 65));
            
            /*************************/
            /* Draw String File Text */
            /*************************/
            
            // Draw "Continue" Text
            renderfont_draw_text(pMenu->gamemenu_get_text_string(TEXT_GAMEOVER_CONTINUE).c_str(),
                                 pMenu->gamemenu_get_text_x(TEXT_GAMEOVER_CONTINUE),
                                 pMenu->gamemenu_get_text_y(TEXT_GAMEOVER_CONTINUE));
            
            // Draw "Quit" Text
            renderfont_draw_text(pMenu->gamemenu_get_text_string(TEXT_GAMEOVER_QUIT).c_str(),
                                 pMenu->gamemenu_get_text_x(TEXT_GAMEOVER_QUIT),
                                 pMenu->gamemenu_get_text_y(TEXT_GAMEOVER_QUIT));
        }
    }
    
    // Draw Effect
    if (pMenu->gamemenu_get_submenu_switch() == true)
    {
        // Fade Out
        render_reset_fullscreen_effect(SCREENEFFECT_FADEOUT);
        render_draw_fullscreen_effect(SCREENEFFECT_FADEOUT, false, true);
        pMenu->gamemenu_set_input_block(true);
        
        // Set Menu Switch
        if (render_fade_drawcount == FADE_ANIM_MAX)
        {
            pMenu->gamemenu_update_next_submenu();
        }
    }
    else if (pMenu->gamemenu_get_input_block() == true && pMenu->gamemenu_get_submenu_switch() == false)
    {
        // Fade back in
        render_reset_fullscreen_effect(SCREENEFFECT_FADEIN);
        render_draw_fullscreen_effect(SCREENEFFECT_FADEIN, false, true);
        
        // Set Menu Input
        if (render_fade_drawcount == 0)
        {
            pMenu->gamemenu_set_input_block(false);
        }
    }
    
    sf2d_end_frame();
}

/*****************************/
/*      SPRITE DRAWING       */
/*****************************/

void Render::render_draw_rectangle(s32 x, s32 y, s32 w, s32 h, u32 color, float offset_3d)
{
    // Draw Top Line of our Rectangle
    sf2d_draw_line(x + offset_3d, y, x + w + offset_3d, y, color);
    
    // Draw Bottom Line
    sf2d_draw_line(x + offset_3d, y + h, x + w + offset_3d, y + h, color);
    
    // Draw Left Border
    sf2d_draw_line(x + offset_3d, y, x + offset_3d, y + h, color);
    
    // Draw Right Border
    sf2d_draw_line(x + w + offset_3d, y, x + w + offset_3d, y + h, color);
}

void Render::render_draw_sprite(SpriteData* pSprite, bool frame_increment, s32 sprite_x, s32 sprite_y, bool force, float offset_3d)
{
    if (pSprite == NULL)
        return;

    // Check if we actually see the Sprite!
    if (force == false)
    {
        if (sprite_x + pSprite->spritedata_w < 0 ||
            sprite_x > TOPSCREEN_X ||
            sprite_y + pSprite->spritedata_h < 0 ||
            sprite_y > TOPSCREEN_Y)
        { return; }
    }

    // Draw Current Texture Frame
    sf2d_draw_texture_part(pSprite->spritedata_texture[pSprite->spritedata_texture_line],
                           sprite_x + offset_3d,
                           sprite_y,
                           (pSprite->spritedata_w * pSprite->spritedata_frame),
                           0,
                           pSprite->spritedata_w,
                           pSprite->spritedata_h);
    
    // Increment Frame if wanted
    if (frame_increment == true)
    {
        if (pSprite->spritedata_frame_speed_current > pSprite->spritedata_frame_speed)
        {
            if (pSprite->spritedata_frame == pSprite->spritedata_maxframe)
            {
                pSprite->spritedata_frame = pSprite->spritedata_startframe;
            }
            else
            {
                pSprite->spritedata_frame += 1;
            }
            
            // Change Frame Counter
            pSprite->spritedata_frame_speed_current = 0;
        }
        else
        {
            pSprite->spritedata_frame_speed_current += 1;
        }
    }
}

void Render::render_draw_spritesheet(SpriteSheet* pSprite, bool frame_increment, s32 sprite_x, s32 sprite_y, u8 size, bool force, float offset_3d)
{
    if (pSprite == NULL || pSprite->spritesheet_texture == NULL ||
        pSprite->spritesheet_h_full == 0 || pSprite->spritesheet_w_full == 0 ||
        pSprite->spritesheet_h_single == 0 || pSprite->spritesheet_w_single == 0)
        return;
    
    // Check if we actually see the Sprite!
    if (force == false)
    {
        if (sprite_x + pSprite->spritesheet_w_single < 0 || sprite_x > TOPSCREEN_X ||
            sprite_y + pSprite->spritesheet_h_single < 0 || sprite_y > TOPSCREEN_Y)
        { return; }
    }
    
    // Check for wrong Positions
    if ((pSprite->spritesheet_w_single * pSprite->spritesheet_frame) > pSprite->spritesheet_w_full)
        return;
    
    if ((pSprite->spritesheet_h_single * pSprite->spritesheet_sprite_line) > pSprite->spritesheet_h_full)
        return;
    
    // Get Size Float
    float float_size = 1;
    
    if (size != 1)
    {
        float_size = size / 100;
    }
    
    // Draw Current Texture Frame
    sf2d_draw_texture_part_scale(pSprite->spritesheet_texture,
                                 sprite_x + offset_3d,
                                 sprite_y,
                                 (pSprite->spritesheet_w_single * pSprite->spritesheet_frame),
                                 (pSprite->spritesheet_h_single * pSprite->spritesheet_sprite_line),
                                 pSprite->spritesheet_w_single,
                                 pSprite->spritesheet_h_single,
                                 float_size,
                                 float_size);
    
    // Increment Frame if wanted
    if (frame_increment == true)
    {
        if (pSprite->spritesheet_frame_speed_current > pSprite->spritesheet_frame_speed)
        {
            if (pSprite->spritesheet_frame == pSprite->spritesheet_endframe)
            {
                pSprite->spritesheet_frame = pSprite->spritesheet_startframe;
            }
            else
            {
                pSprite->spritesheet_frame += 1;
            }
            
            // Change Frame Counter
            pSprite->spritesheet_frame_speed_current = 0;
        }
        else
        {
            pSprite->spritesheet_frame_speed_current += 1;
        }
    }
}

void Render::render_draw_sprite_rotate(SpriteSheet* pSprite, bool frame_increment, s32 sprite_x, s32 sprite_y, bool force, float angle, float offset_3d)
{
    if (pSprite == NULL)
        return;
    
    // Check if we actually see the Sprite!
    if (force == false)
    {
        if (sprite_x + pSprite->spritesheet_w_single < 0 ||
            sprite_x > TOPSCREEN_X ||
            sprite_y + pSprite->spritesheet_h_single < 0 ||
            sprite_y > TOPSCREEN_Y)
        {
            return;
        }
    }
    
    // Draw Current Texture Frame
    sf2d_draw_texture_part_rotate_scale(pSprite->spritesheet_texture,
                                        sprite_x + offset_3d,
                                        sprite_y,
                                        (float)angle,
                                        (pSprite->spritesheet_w_single * pSprite->spritesheet_frame),
                                        (pSprite->spritesheet_h_single * pSprite->spritesheet_sprite_line),
                                        pSprite->spritesheet_w_single,
                                        pSprite->spritesheet_h_single,
                                        1,
                                        1);
    
    // Increment Frame if wanted
    if (frame_increment == true)
    {
        if (pSprite->spritesheet_frame_speed_current > pSprite->spritesheet_frame_speed)
        {
            if (pSprite->spritesheet_frame == pSprite->spritesheet_endframe)
            {
                pSprite->spritesheet_frame = pSprite->spritesheet_startframe;
            }
            else
            {
                pSprite->spritesheet_frame += 1;
            }
            
            // Change Frame Counter
            pSprite->spritesheet_frame_speed_current = 0;
        }
        else
        {
            pSprite->spritesheet_frame_speed_current++;
        }
    }
}

void Render::render_draw_still(SpriteData* pSprite, s32 sprite_x, s32 sprite_y, bool force, float offset_3d)
{
    if (pSprite == NULL)
        return;
    
    // Check if we actually see the Sprite!
    if (force == false)
    {
        if (sprite_x + pSprite->spritedata_w < 0 ||
            sprite_x > TOPSCREEN_X ||
            sprite_y + pSprite->spritedata_h < 0 ||
            sprite_y > TOPSCREEN_Y)
        {
            return;
        }
    }
    
    // Draw Current Texture Frame
    sf2d_draw_texture_part(pSprite->spritedata_texture[pSprite->spritedata_texture_line],
                           sprite_x + offset_3d,
                           sprite_y,
                           0,
                           0,
                           pSprite->spritedata_w,
                           pSprite->spritedata_h);
}

void Render::render_draw_still(SpriteData* pSprite, s32 sprite_x, s32 sprite_y, s32 sprite_part_x, s32 sprite_part_y, s32 sprite_part_w, s32 sprite_part_h, bool force, float offset_3d)
{
    if (pSprite == NULL)
        return;
    
    // Check if we actually see the Sprite!
    if (force == false)
    {
        if (sprite_x + pSprite->spritedata_w < 0 ||
            sprite_x > TOPSCREEN_X ||
            sprite_y + pSprite->spritedata_h < 0 ||
            sprite_y > TOPSCREEN_Y)
        {
            return;
        }
    }
    
    // Draw Current Texture Frame
    sf2d_draw_texture_part(pSprite->spritedata_texture[pSprite->spritedata_texture_line],
                           sprite_x + offset_3d,
                           sprite_y,
                           sprite_part_x,
                           sprite_part_y,
                           sprite_part_w,
                           sprite_part_h);
}

void Render::render_draw_fullscreen_effect(u8 effect, bool is_topscreen, bool increment)
{
    /* Draw Fullscreen Effects */
    
    if (effect == SCREENEFFECT_FADEOUT)
    {
        // Draw Texture
        for (u8 i = 0 ; i <= render_fade_drawcount; ++i)
        {
            if (is_topscreen == true)
            {
                sf2d_draw_rectangle(0, 0, TOPSCREEN_X, TOPSCREEN_Y, RGBA8(0, 0, 0, 10 * render_fade_drawcount));
            }
            else
                sf2d_draw_rectangle(0, 0, BOTTOMSCREEN_X, BOTTOMSCREEN_Y, RGBA8(0, 0, 0, 10 * render_fade_drawcount));
        }
        
        // Increase Draw Counters
        if (render_fade_drawcount < FADE_ANIM_MAX && increment == true)
        {
            ++render_fade_drawcount;
        }
    }
    else if (effect == SCREENEFFECT_FADEIN)
    {
        // Draw Texture
        for (u8 i = render_fade_drawcount ; i > 0; --i)
        {
            if (is_topscreen == true)
            {
                sf2d_draw_rectangle(0, 0, TOPSCREEN_X, TOPSCREEN_Y, RGBA8(0, 0, 0, 10 * render_fade_drawcount));
            }
            else
                sf2d_draw_rectangle(0, 0, BOTTOMSCREEN_X, BOTTOMSCREEN_Y, RGBA8(0, 0, 0, 10 * render_fade_drawcount));
        }
        
        // Increase Draw Counters
        if (render_fade_drawcount > 0 && increment == true)
        {
            --render_fade_drawcount;
        }
    }
}

/***************************/
/*      ROOM DRAWING       */
/***************************/

void Render::render_draw_room_background(Stage* pStage, float offset_3d)
{
    /* Check if our Room is completely loaded */
    
    // Return if not loaded
    if (pStage->stage_get_loaded() == false)
    {
        return;
    }
    
    /* The order of drawing Images is important here! */

    // Draw Background
    if (pStage->stage_get_background() != NULL)
    {
        sf2d_draw_texture(pStage->stage_get_background(), 0 - (offset_3d * 2), 0);
    }
    
    // Draw Background Tileset
    if (pStage->stage_get_tileset() != NULL && pStage->stage_get_background_tileset_draw_flag() == true)
    {
        // Define Coordinates of our Tile
        s32 x = 0, y = 0, tex_x = 0, tex_y = 0, tileset_id = 0;
        
        // Draw all Tiles
        for (u32 i = 0; i < pStage->stage_get_tile_count(); ++i)
        {
            // Shoud we draw this Tile?
            if (pStage->stage_get_layer_tile(true, i) != 0)
            {
                // Get Tileset Teture X and Y
                tileset_id = pStage->stage_get_layer_tile(true, i);
                
                // Set Texture Y
                while (tileset_id > 8)
                {
                    tileset_id -= 8;
                    ++tex_y;
                }
                
                // Set Texture X
                tex_x = tileset_id - 1;
                
                // Draw Tile
                sf2d_draw_texture_part(pStage->stage_get_tileset(),
                                       -render_camera_x + (x * TILESSET_TILE_DIMENSION) - offset_3d,
                                       -render_camera_y + (y * TILESSET_TILE_DIMENSION),
                                       tex_x * TILESSET_TILE_DIMENSION,
                                       tex_y * TILESSET_TILE_DIMENSION,
                                       TILESSET_TILE_DIMENSION,
                                       TILESSET_TILE_DIMENSION);
                
                // Reset X and Y of Tileset Tile
                tex_x = 0;
                tex_y = 0;
            }
            
            // Count drawn Tiles
            if (x == pStage->stage_get_tile_layout_width() - 1)
            {
                // Increase Y and reset X
                x = 0;
                ++y;
            }
            else
            {
                // Increase counter for width drawn
                ++x;
            }
        }
    }
}

void Render::render_draw_room_foreground(Stage* pStage, Player* pPlayer, float offset_3d)
{
    /* Check if our Room is completely loaded */
    
    // Return if not loaded
    if (pStage->stage_get_loaded() == false)
    {
        return;
    }
    
    /* The Order of Drawing Images is important here! */

    // Draw Liquid
    if (pStage->stage_get_water_type() != 0)
    {
        renderwater_draw(pStage->stage_get_water_type(),
                         render_camera_player_x,
                         render_camera_player_y,
                         pPlayer->player_get_x(),
                         pPlayer->player_get_y(),
                         pStage->stage_get_water_x(),
                         pStage->stage_get_water_y(),
                         pStage->stage_get_water_w(),
                         pStage->stage_get_water_h());
    }
    
    // Draw Foreground Tileset
    if (pStage->stage_get_background() != NULL)
    {
        // Define Coordinates of our Tile
        s32 x = 0, y = 0, tex_x = 0, tex_y = 0, tileset_id = 0;
        
        // Draw all Tiles
        for (u8 i = 0; i < pStage->stage_get_tile_count(); ++i)
        {
            // Shoud we draw this Tile?
            if (pStage->stage_get_layer_tile(false, i) != 0)
            {
                // Get Tileset Teture X and Y
                tileset_id = pStage->stage_get_layer_tile(false, i);
                
                // Set Texture Y
                while (tileset_id > 8)
                {
                    tileset_id -= 8;
                    ++tex_y;
                }
                
                // Set Texture X
                tex_x = tileset_id - 1;
                
                // Draw Tile
                sf2d_draw_texture_part(pStage->stage_get_tileset(),
                                       -render_camera_x + (x * TILESSET_TILE_DIMENSION) + offset_3d, // + offset_3d,
                                       -render_camera_y + (y * TILESSET_TILE_DIMENSION),
                                       tex_x * TILESSET_TILE_DIMENSION,
                                       tex_y * TILESSET_TILE_DIMENSION,
                                       TILESSET_TILE_DIMENSION,
                                       TILESSET_TILE_DIMENSION);
                
                // Reset X and Y of Tileset Tile
                tex_x = 0;
                tex_y = 0;
            }
            
            // Count drawn Tiles
            if (x == pStage->stage_get_tile_layout_width() - 1)
            {
                // Increase Y and reset X
                x = 0;
                ++y;
            }
            else
            {
                // Increase counter for width drawn
                ++x;
            }
        }
    }
}

void Render::render_draw_room_effect(u8 effect, float offset_3d)
{
    // TODO: ROOM EFFECT WITH SF2D LIB FUNCTIONS NO PNG
    return;
}

/*****************************/
/*      USER INTERFACE       */
/*****************************/

void Render::render_draw_hud(Player* pPlayer, float offset_3d)
{
    // Draw HUD Background
    sf2d_draw_texture(pTexture_HudBackground, HUD_BG_X + offset_3d, HUD_BG_Y);
    
    // Draw Health
    renderfont_draw_number(pPlayer->player_get_health(), HUD_NUMBER_X + offset_3d, HUD_NUMBER_Y, 2);
    
    // Draw Missles
    renderfont_draw_number(pPlayer->player_get_missles_available(), HUD_MISSLE_NUM_X + offset_3d, HUD_MISSLE_NUM_Y, 3);

    // Draw Health Tanks
    for (u8 i = 1; i <= pPlayer->player_get_healthtanks_total(); ++i)
    {
        if (i <= pPlayer->player_get_healthtanks_available())
        {
            sf2d_draw_texture(pTexture_HealthTankFull, (HUD_HT_X + ((HUD_HT_WIDTH + HUD_HT_SPACE) * i)) + offset_3d, HUD_HT_Y);
        }
        else
        {
            sf2d_draw_texture(pTexture_HealthTankEmpty, (HUD_HT_X + ((HUD_HT_WIDTH + HUD_HT_SPACE) * i)) + offset_3d, HUD_HT_Y);
        }
    }
    
    // Draw Health Bar
    if (pPlayer->player_get_health() > 0)
    {
        sf2d_draw_texture_scale(pTexture_HudBarFull, HUD_BAR_X + offset_3d, HUD_BAR_Y, pPlayer->player_get_health(), HUD_BAR_HEIGHT);
    }
    
    if (pPlayer->player_get_health() < 99)
    {
        sf2d_draw_texture_scale(pTexture_HudBarEmpty, pPlayer->player_get_health() + HUD_BAR_X + offset_3d, HUD_BAR_Y, 99 - pPlayer->player_get_health(), HUD_BAR_HEIGHT);
    }
    
    // Health Bar Edges are "Full" Health Bar Lines
    sf2d_draw_texture_scale(pTexture_HudBarFull, HUD_BAR_X + offset_3d, HUD_BAR_Y, 1, HUD_BAR_HEIGHT);
    sf2d_draw_texture_scale(pTexture_HudBarFull, HUD_BAR_X + 99 + offset_3d, HUD_BAR_Y, 1, HUD_BAR_HEIGHT);
}

void Render::render_draw_bottom_menu(Player* pPlayer, Stage* pStage, BottomMenu* pMenu, SaveFile* pSaveFile)
{
    /* Draw Bottom Menu depending on Submenu */

    /* Draw Background Lines */
    
    // Draw Diagonal Lines
    for (u16 i = 10; i < 240; i += 10)
    {
        sf2d_draw_line(0, i, 320, i, render_hudcolor);
    }
    
    // Draw Horizontal Lines
    for (u16 i = 10; i < 320; i += 10)
    {
        sf2d_draw_line(i, 0, i, 240, render_hudcolor);
    }
    
    /* Draw Bottom Menu Graphics depending on Status or Map State */
    
    // A Wron Submenu will result in the Map Menu
    if (pMenu->gamemenu_get_submenu() == SUBMENU_STATUS)
    {
        // Draw Upgrades
        
        // Check if we have text strings
        if (pMenu->gamemenu_get_stringcount() > 0)
        {
            for (u8 i = 0; i <= pMenu->gamemenu_get_stringcount() - 1; ++i)
            {
                // Check if we used that Upgrade
                if (pMenu->gamemenu_get_upgrade_string(pSaveFile, i) != "NONE")
                {
                    // Draw background
                    sf2d_draw_rectangle(pMenu->gamemenu_get_upgrade_x(i) - 20,
                                        pMenu->gamemenu_get_upgrade_y(i) - 1,
                                        160,
                                        13,
                                        render_upgrade_bordercolor);
                    
                    // Draw background
                    if (i != 0 && i != 6 && i != 8 && i != 10 && i != 15)
                    {
                        sf2d_draw_rectangle(pMenu->gamemenu_get_upgrade_x(i) - 19,
                                            pMenu->gamemenu_get_upgrade_y(i),
                                            158,
                                            11,
                                            render_upgrade_bgcolor);
                    }
                    
                    // Draw Text String
                    renderfont_draw_text(pMenu->gamemenu_get_upgrade_string(pSaveFile, i).c_str(),
                                         pMenu->gamemenu_get_upgrade_x(i),
                                         pMenu->gamemenu_get_upgrade_y(i) + 1);
                }
            }
        }
    }
    else if (pMenu->gamemenu_get_minimap() != NULL)
    {
        if (pMenu->gamemenu_get_minimap()->minimapfile_loaded == true)
        {
            // Define Start Coordinates
            s32 mm_x = BOTTOMSCREEN_X / 4;
            s32 mm_y = BOTTOMSCREEN_Y / 2;
            
            // Draw all Rooms
            for (u8 i = 0; i <= pMenu->gamemenu_get_minimap()->minimapfile_rooms - 1; ++i)
            {
                // Check if we explored this Map
                if (pSaveFile->savefile_get_minimap_flag() & pMenu->gamemenu_get_minimap()->minimapfile_room_id[i])
                {
                    // Draw Border
                    sf2d_draw_rectangle((mm_x + pMenu->gamemenu_get_minimap()->minimapfile_room_x[i]) + pMenu->gamemenu_get_minimap_x(),
                                        (mm_y + pMenu->gamemenu_get_minimap()->minimapfile_room_y[i]) + pMenu->gamemenu_get_minimap_y(),
                                        pMenu->gamemenu_get_minimap()->minimapfile_room_w[i],
                                        pMenu->gamemenu_get_minimap()->minimapfile_room_h[i],
                                        render_mm_mapbordercolor);
                    
                    // Draw Map
                    sf2d_draw_rectangle((mm_x + pMenu->gamemenu_get_minimap()->minimapfile_room_x[i]) + pMenu->gamemenu_get_minimap_x() + 1,
                                        (mm_y + pMenu->gamemenu_get_minimap()->minimapfile_room_y[i]) + pMenu->gamemenu_get_minimap_y() + 1,
                                        pMenu->gamemenu_get_minimap()->minimapfile_room_w[i] - 2,
                                        pMenu->gamemenu_get_minimap()->minimapfile_room_h[i] - 2,
                                        render_mm_mapcolor);
                }
            }
            
            // Draw Player Position
            mm_x = (((BOTTOMSCREEN_X / 4) + pMenu->gamemenu_get_minimap()->minimapfile_room_x[pMenu->gamemenu_get_minimap_slot(pStage->stage_get_minimap_id())]) + pMenu->gamemenu_get_minimap_x()) + (pPlayer->player_get_x() / 20);
            
            mm_y = (((BOTTOMSCREEN_Y / 2) +
                pMenu->gamemenu_get_minimap()->minimapfile_room_y[pMenu->gamemenu_get_minimap_slot(pStage->stage_get_minimap_id())]) +
                pMenu->gamemenu_get_minimap_y()) +
                (pMenu->gamemenu_get_minimap()->minimapfile_room_h[pMenu->gamemenu_get_minimap_slot(pStage->stage_get_minimap_id())] - (pPlayer->player_get_y() / 20));
            
            sf2d_draw_fill_circle(mm_x,
                                  mm_y,
                                  2,
                                  render_mm_playercolor);
        }
        else
        {
            // Draw Load Failed Text
            renderfont_draw_text("NO MAP FOUND", (BOTTOMSCREEN_X / 2) - 60, (BOTTOMSCREEN_Y / 2) - 5);
        }
    }
    else
    {
        // Draw Load Failed Text
        renderfont_draw_text("NO MAP FOUND", (BOTTOMSCREEN_X / 2) - 60, (BOTTOMSCREEN_Y / 2) - 5);
    }
    
    /* Draw Bottom Menu Background */
    
    // Draw BottomMenu Top
    if (pMenu->gamemenu_get_sprite(MENU_TEXLINE_TOPBG)->spritedata_texture[MENU_TEXLINE_TOPBG] != NULL)
    {
        render_draw_still(pMenu->gamemenu_get_sprite(MENU_TEXLINE_TOPBG),
                          pMenu->gamemenu_get_sprite_x(MENU_TEXLINE_TOPBG),
                          pMenu->gamemenu_get_sprite_y(MENU_TEXLINE_TOPBG),
                          true,
                          0);
    }
    
    // Draw BottomMenu Lower
    if (pMenu->gamemenu_get_sprite(MENU_TEXLINE_BTMBG)->spritedata_texture[MENU_TEXLINE_BTMBG] != NULL)
    {
        render_draw_still(pMenu->gamemenu_get_sprite(MENU_TEXLINE_BTMBG),
                          pMenu->gamemenu_get_sprite_x(MENU_TEXLINE_BTMBG),
                          pMenu->gamemenu_get_sprite_y(MENU_TEXLINE_BTMBG),
                          true,
                          0);
    }
    
    // Draw Selection
    if (pMenu->gamemenu_get_submenu() == SUBMENU_STATUS)
    {
        // Draw Selection
        sf2d_draw_rectangle(SUBMENUPOS_STATUS_X,
                            SUBMENUPOS_ALL_Y,
                            BTM_SELECTION_WIDTH,
                            BTM_SELECTION_HEIGHT,
                            render_hudcolor);
    }
    else
    {
        // Draw Selection
        sf2d_draw_rectangle(SUBMENUPOS_MAP_X,
                            SUBMENUPOS_ALL_Y,
                            BTM_SELECTION_WIDTH,
                            BTM_SELECTION_HEIGHT,
                            render_hudcolor);
    }
    
    /* Draw Bottom Menu Icons */
    
    // Beam Icons
    s32 beamline_w = 0;
    
    if (pSaveFile->savefile_get_plasmabeam() == true)
    {
        beamline_w = pMenu->gamemenu_get_sprite(MENU_TEXLINE_BEAMSWITCHER)->spritedata_w;
    }
    else if (pSaveFile->savefile_get_spazerbeam() == true)
    {
        beamline_w = (pMenu->gamemenu_get_sprite(MENU_TEXLINE_BEAMSWITCHER)->spritedata_w / 4) * 3;
    }
    else if (pSaveFile->savefile_get_wavebeam() == true)
    {
        beamline_w = (pMenu->gamemenu_get_sprite(MENU_TEXLINE_BEAMSWITCHER)->spritedata_w / 4) * 2;
    }
    else if (pSaveFile->savefile_get_icebeam() == true)
    {
        beamline_w = (pMenu->gamemenu_get_sprite(MENU_TEXLINE_BEAMSWITCHER)->spritedata_w / 4);
    }
    
    // Draw Beams Line
    if (beamline_w > 0)
    {
        if (pMenu->gamemenu_get_sprite(MENU_TEXLINE_BEAMSWITCHER)->spritedata_texture[MENU_TEXLINE_BEAMSWITCHER] != NULL)
        {
            render_draw_still(pMenu->gamemenu_get_sprite(MENU_TEXLINE_BEAMSWITCHER),
                      pMenu->gamemenu_get_sprite_x(MENU_TEXLINE_BEAMSWITCHER),
                      pMenu->gamemenu_get_sprite_y(MENU_TEXLINE_BEAMSWITCHER),
                      0,
                      0,
                      beamline_w,
                      pMenu->gamemenu_get_sprite(MENU_TEXLINE_BEAMSWITCHER)->spritedata_h,
                      true,
                      0);
        }
    }
    
    // Only draw Selection Icon if ve've collected the Ice Beam(Required)
    if (pSaveFile->savefile_get_icebeam() == true)
    {
        // Selection Icon - Beams
        sf2d_draw_rectangle(pMenu->gamemenu_get_sprite_x(MENU_TEXLINE_SEL),
                            pMenu->gamemenu_get_sprite_y(MENU_TEXLINE_BEAMSWITCHER),
                            BTM_SELECTION_WIDTH,
                            BTM_SELECTION_HEIGHT,
                            render_hudcolor);
    }
    
    /* Metroids */
    
    // Draw Metroid Numbers
    renderfont_draw_number(pSaveFile->savefile_get_metroids(),
                           pMenu->gamemenu_get_sprite_x(ICON_TEXLINE_NUMONE),
                           pMenu->gamemenu_get_sprite_y(ICON_TEXLINE_NUMONE),
                           2);
}

/**************************/
/*      CALCULATION       */
/**************************/

void Render::render_calc_camera(s32 player_x, s32 player_y, s32 player_width, s32 player_height, s32 stage_width, s32 stage_height)
{
    // Camera X
    if (player_x <= TOPSCREEN_X / 2)
    {
        render_camera_x = 0;
        render_camera_player_x = player_x - player_width;
    }
    else if (player_x > stage_width - (TOPSCREEN_X / 2))
    {
        render_camera_x = stage_width - TOPSCREEN_X;
        // 100 = Sprite Coordinates Fix
        render_camera_player_x = (TOPSCREEN_X - 100) - ((stage_width - player_x) - player_width);
    }
    else
    {
        render_camera_x = player_x - (TOPSCREEN_X / 2);
        render_camera_player_x = (TOPSCREEN_X / 2) - player_width;
    }
    
    // Camera Y
    if ((player_y + player_height) <= TOPSCREEN_Y / 2)
    {
        render_camera_y = stage_height - TOPSCREEN_Y;
        render_camera_player_y = TOPSCREEN_Y - (player_y + player_height);
    }
    else if ((player_y + player_height) > stage_height - (TOPSCREEN_Y / 2))
    {
        render_camera_y = 0;
        render_camera_player_y = stage_height - (player_y + player_height);
    }
    else
    {
        render_camera_y = stage_height - ((player_y + player_height) + (TOPSCREEN_Y / 2));
        render_camera_player_y = TOPSCREEN_Y / 2;
    }
}

s32 Render::render_calc_camera_actor_x(Player* pPlayer, s32 actor_x, s32 actor_y, s32 actor_width, s32 actor_height)
{
    return render_camera_player_x + (actor_x - (pPlayer->player_get_x() - pPlayer->player_get_width()));
}

s32 Render::render_calc_camera_actor_y(Player* pPlayer, s32 actor_x, s32 actor_y, s32 actor_width, s32 actor_height)
{
    return (render_camera_player_y + PLAYER_SPRITEFRAME_H) - ((actor_y + actor_height) - pPlayer->player_get_y());
}

void Render::render_calc_starglow()
{
    for (u8 i = 0; i < RENDER_MAX_STARGLOW; ++i)
    {
        // Check if Star is allocated
        if (render_starglow[i] != NULL)
        {
            if (render_starglow[i]->star_loaded == false)
            {
                // Set Size, Alpha, Fadeing and Position
                render_starglow[i]->star_alpha = rand() % MAINMENU_GLOW_MAX_ALPHA;
                render_starglow[i]->star_size = rand() % 2 + 1;
                render_starglow[i]->star_topscreen = rand() % 2;
                render_starglow[i]->star_dying = rand() % 2;
                
                // Set X and Y depending on Screen Size
                if (render_starglow[i]->star_topscreen == true)
                {
                    // Set Top Screen Coordinates
                    render_starglow[i]->star_x = rand() % TOPSCREEN_X;
                    render_starglow[i]->star_y = rand() % TOPSCREEN_Y;
                }
                else
                {
                    // Set Bottom Screen Coordinates
                    render_starglow[i]->star_x = rand() % BOTTOMSCREEN_X;
                    render_starglow[i]->star_y = rand() % BOTTOMSCREEN_Y;
                }
                
                // Set Star loaded
                render_starglow[i]->star_loaded = true;
            }
            else
            {
                // Either Increase or Decrease Star Alpha
                if (render_starglow[i]->star_dying == false)
                {
                    // Set Star Alpha
                    if (render_starglow[i]->star_alpha + MAINMENU_GLOW_ALPHA_STEP < MAINMENU_GLOW_MAX_ALPHA)
                    {
                        // Increase Star Alpha
                        render_starglow[i]->star_alpha += MAINMENU_GLOW_ALPHA_STEP;
                    }
                    else
                    {
                        // Set Star to decrease Alpha
                        render_starglow[i]->star_dying = true;
                    }
                }
                else
                {
                    // Set Star Alpha
                    if (render_starglow[i]->star_alpha - MAINMENU_GLOW_ALPHA_STEP > 0)
                    {
                        // Decrease Star Alpha
                        render_starglow[i]->star_alpha -= MAINMENU_GLOW_ALPHA_STEP;
                    }
                    else
                    {
                        // Set to increase Alpha
                        render_starglow[i]->star_dying = false;
                    }
                }
            }
        }
        else
        {
            // Create Star
            render_starglow[i] = new StarGlow;
            
            // Set Star not loaded
            render_starglow[i]->star_loaded = false;
        }
    }
}

float Render::render_calc_facing_angle(u8 facing)
{
    switch (facing)
    {
        case FACING_LEFT:
            return 3.10;
            break;
        case FACING_RIGHT:
            return 0;
            break;
        case FACING_DOWN:
            return 1.50;
            break;
        case FACING_UP:
            return 4.62;
            break;
        case FACING_LEFTUP:
            return 4.62;
            break;
        case FACING_RIGHTUP:
            return 0;
            break;
        case FACING_LEFTDOWN:
            return 3.14;
            break;
        case FACING_RIGHTDOWN:
            return 1.54;
            break;
            
        default:
            return 0;
    }
}

void Render::render_reset_fullscreen_effect(u8 effect)
{
    /* Reset Fullscreen Effects */
    
    switch (effect)
    {
        case SCREENEFFECT_FADEOUT:
            if (render_fade_drawcount == FADE_ANIM_MAX)
                render_fade_drawcount = 0;
            break;
        case SCREENEFFECT_FADEIN:
            if (render_fade_drawcount == 0)
                render_fade_drawcount = FADE_ANIM_MAX;
            break;
    }
}
