//
//  render.h
//  3ds_metroid
//
//  Created by jbr373 on 15/01/16.
//

#ifndef render_h
#define render_h

#include <3ds.h>
#include <sf2d.h>
#include <stdio.h>
#include <string>

#include "image.h"
#include "stage.h"
#include "player.h"
#include "item.h"
#include "menu.h"
#include "menu_game.h"
#include "render_projectile.h"
#include "render_font.h"
#include "render_item.h"
#include "render_water.h"
#include "sprite_handler.h"

#define RENDER_MAX_STARGLOW     30

struct StarGlow
{
    bool star_loaded;
    
    u8 star_alpha;
    u8 star_size;
    s32 star_x;
    s32 star_y;
    bool star_topscreen;
    bool star_dying;
};

class Render : private RenderFont, private RenderProjectile, private RenderItem, private RenderWater, private FileOperations
{
public:
    Render();
    virtual ~Render();
    
    // Gameplay
    void render_draw(SpriteHandler* pSpriteHandler, Stage* pStage, Player* pPlayer, BottomMenu* pMenu, SaveFile* pSaveFile, bool draw_game_pause);
    
    // Cutscenes
    
    // Main Menu & Game Over
    void render_draw(GameMenu* pMenu, SaveFile* pSaveFile);
    
private:
    // Calculation
    float render_calc_facing_angle(u8 facing);
    void render_calc_camera(s32 player_x, s32 player_y, s32 player_width, s32 player_height, s32 stage_width, s32 stage_height);
    void render_reset_fullscreen_effect(u8 effect);
    void render_calc_starglow();
    
    s32 render_calc_camera_actor_x(Player* pPlayer, s32 actor_x, s32 actor_y, s32 actor_width, s32 actor_height);
    s32 render_calc_camera_actor_y(Player* pPlayer, s32 actor_x, s32 actor_y, s32 actor_width, s32 actor_height);
    
    // General Drawing
    void render_draw_pause(bool is_topscreen, float offset_3d);
    void render_draw_rectangle(s32 x, s32 y, s32 w, s32 h, u32 color, float offset_3d);
    void render_draw_sprite(SpriteData* pSprite, bool frame_increment, s32 sprite_x, s32 sprite_y, bool force, float offset_3d);
    void render_draw_sprite_rotate(SpriteSheet* pSprite, bool frame_increment, s32 sprite_x, s32 sprite_y, bool force, float angle, float offset_3d);
    void render_draw_spritesheet(SpriteSheet* pSprite, bool frame_increment, s32 sprite_x, s32 sprite_y, u8 size, bool force, float offset_3d);
    void render_draw_still(SpriteData* pSprite, s32 sprite_x, s32 sprite_y, bool force, float offset_3d);
    void render_draw_still(SpriteData* pSprite, s32 sprite_x, s32 sprite_y, s32 sprite_part_x, s32 sprite_part_y, s32 sprite_part_w, s32 sprite_part_h, bool force, float offset_3d);
    
    // User Interface
    void render_draw_hud(Player* pPlayer, float offset_3d);
    void render_draw_bottom_menu(Player* pPlayer, Stage* pStage, BottomMenu* pMenu, SaveFile* pSaveFile);
    
    // Room
    void render_draw_room_background(Stage* pStage, float offset_3d);
    void render_draw_room_foreground(Stage* pStage, Player* pPlayer, float offset_3d);
    
    // Effects
    void render_draw_room_effect(u8 effect, float offset_3d);
    void render_draw_fullscreen_effect(u8 effect, bool is_topscreen, bool increment);
    
    StarGlow* render_starglow[RENDER_MAX_STARGLOW];
    
    // Stereoscopic 3D
    float render_offset_3d;
    
    // Cameras
    s32 render_camera_x;
    s32 render_camera_y;
    s32 render_camera_player_x;
    s32 render_camera_player_y;
    
    // Special: Fade Effect
    u8 render_fade_drawcount;
    
    // Pause Text
    StringFileStruct* pRenderPause_Text;
    
    // Colors
    u32 render_hudcolor;
    u32 render_mm_playercolor;
    u32 render_mm_mapcolor;
    u32 render_mm_mapbordercolor;
    u32 render_upgrade_bordercolor;
    u32 render_upgrade_bgcolor;
    
    // Textures
    sf2d_texture* pTexture_HealthTankFull;
    sf2d_texture* pTexture_HealthTankEmpty;
    sf2d_texture* pTexture_HudBackground;
    sf2d_texture* pTexture_HudBarFull;
    sf2d_texture* pTexture_HudBarEmpty;
};

#endif /* render_h */
