//
//  render_font.cpp
//  3ds_metroid
//
//  Created by jbr373 on 01/02/16.
//

#include <sfil.h>

#include "render_font.h"
#include "filepaths.h"

#define FONTPOS_Y_ROW_ONE           0
#define FONTPOS_Y_ROW_TWO           11
#define FONTPOS_Y_ROW_THREE         22
#define FONTPOS_Y_ROW_FOUR          33
#define FONTPOS_Y_ROW_FIVE          44

#define FONTPOS_CHAR_DIMENSION      11
#define FONTPOS_CHAR_SPACE          1

enum FontXPos
{
    // X Row Pos
    X_POS_0 = 0,
    X_POS_1 = 11,
    X_POS_2 = 22,
    X_POS_3 = 33,
    X_POS_4 = 44,
    X_POS_5 = 55,
    X_POS_6 = 66,
    X_POS_7 = 77,
    X_POS_8 = 88,
    X_POS_9 = 99,
};

RenderFont::RenderFont()
{
    // Coordinates
    renderfont_pos_x = 0;
    renderfont_pos_y = 0;
    
    // Texture
    pFontTexture = NULL;
}

RenderFont::~RenderFont()
{
    // Free Texture
    if (pFontTexture != NULL)
    {
        sf2d_free_texture(pFontTexture);
        pFontTexture = NULL;
    }
}

void RenderFont::renderfont_load()
{
    // Load Texture
    pFontTexture = sfil_load_PNG_file(IMG_FONT, SF2D_PLACE_RAM);
}

void RenderFont::renderfont_draw_number(u8 number, s32 font_x, s32 font_y, u8 draw_numbers)
{
    // Return on wrong input
    if (draw_numbers == 0 || draw_numbers > 3)
        return;

    // Draw Number 1
    if (draw_numbers == 1)
        renderfont_draw_number_single(number, font_x, font_y);

    // Draw Number 2
    if (draw_numbers == 2)
        renderfont_draw_number_tens(number, font_x, font_y);
    
    // Draw Number 3
    if (draw_numbers == 3)
        renderfont_draw_number_hundreds(number, font_x, font_y);
}

void RenderFont::renderfont_draw_number_hundreds(u8 number, s32 font_x, s32 font_y)
{
    // Get Numbers
    u8 num_one = 0, num_two = 0, num_three = 0, num_full = number;
    
    // Check Counter
    if (num_full > 0) // Missles 0 = 00
    {
        /* Numbers over 100 */
        
        // Get First Number (No Bigger than 255 -> u8)
        if (num_full >= 200)
        {
            num_one = 2;
            num_full -= 200;
        }
        else if (num_full >= 100 && num_full < 200)
        {
            num_one = 1;
            num_full -= 100;
        }
        
        /* Numbers over 10 */
        
        if (num_full >= 10)
        {
            // Get Second Number
            while (num_full >= 10)
            {
                num_full -= 10;
                ++num_two;
            }
            
            // Set Third Number
            num_three = num_full;
        }
        else
            num_three = num_full; // Number below 10
    }
    
    // Draw Number 1
    renderfont_get_font_pos_u8(num_one);
    sf2d_draw_texture_part(pFontTexture, font_x, font_y, renderfont_pos_x, FONTPOS_Y_ROW_FOUR, FONTPOS_CHAR_DIMENSION, FONTPOS_CHAR_DIMENSION);
    
    // Draw Number 2
    renderfont_get_font_pos_u8(num_two);
    sf2d_draw_texture_part(pFontTexture, font_x + (FONTPOS_CHAR_DIMENSION + FONTPOS_CHAR_SPACE), font_y, renderfont_pos_x, FONTPOS_Y_ROW_FOUR, FONTPOS_CHAR_DIMENSION, FONTPOS_CHAR_DIMENSION);
    
    // Draw Number 3
    renderfont_get_font_pos_u8(num_three);
    sf2d_draw_texture_part(pFontTexture, font_x + ((FONTPOS_CHAR_DIMENSION + FONTPOS_CHAR_SPACE) * 2), font_y, renderfont_pos_x, FONTPOS_Y_ROW_FOUR, FONTPOS_CHAR_DIMENSION, FONTPOS_CHAR_DIMENSION);
}

void RenderFont::renderfont_draw_number_tens(u8 number, s32 font_x, s32 font_y)
{
    // Get Numbers
    u8 num_one = 0, num_two = 0, num_full = number;
    
    // Check Counter
    if (num_full > 0)
    {
        /* Numbers over 10 */
        
        if (num_full >= 10)
        {
            // Get Second Number
            while (num_full >= 10)
            {
                num_full -= 10;
                ++num_one;
            }
            
            // Set Second Number
            num_two = num_full;
        }
        else
            num_two = num_full; // Number below 10
    }
    
    // Draw Number 1
    renderfont_get_font_pos_u8(num_one);
    sf2d_draw_texture_part(pFontTexture, font_x, font_y, renderfont_pos_x, FONTPOS_Y_ROW_FOUR, FONTPOS_CHAR_DIMENSION, FONTPOS_CHAR_DIMENSION);
    
    // Draw Number 2
    renderfont_get_font_pos_u8(num_two);
    sf2d_draw_texture_part(pFontTexture, font_x + (FONTPOS_CHAR_DIMENSION + FONTPOS_CHAR_SPACE), font_y, renderfont_pos_x, FONTPOS_Y_ROW_FOUR, FONTPOS_CHAR_DIMENSION, FONTPOS_CHAR_DIMENSION);
}

void RenderFont::renderfont_draw_number_single(u8 number, s32 font_x, s32 font_y)
{
    // Draw Number 1
    renderfont_get_font_pos_u8(number);
    sf2d_draw_texture_part(pFontTexture, font_x, font_y, renderfont_pos_x, FONTPOS_Y_ROW_FOUR, FONTPOS_CHAR_DIMENSION, FONTPOS_CHAR_DIMENSION);
}

/***********************/
/*      Draw Text      */
/***********************/

void RenderFont::renderfont_draw_text(const char* text, s32 font_x, s32 font_y)
{
    // Variables
    size_t length = strlen(text);
    s32 textpos_x = font_x;
    
    // Write each Letter
    for (size_t i = 0; i <= length - 1; ++i)
    {
        // Get Font File Position
        renderfont_get_font_pos_char(&text[i]);
        
        // Draw Text
        sf2d_draw_texture_part(pFontTexture, textpos_x, font_y, renderfont_pos_x, renderfont_pos_y, FONTPOS_CHAR_DIMENSION, FONTPOS_CHAR_DIMENSION);
        
        // Increase Text Position X
        textpos_x += (FONTPOS_CHAR_DIMENSION + FONTPOS_CHAR_SPACE);
    }
}

void RenderFont::renderfont_get_font_pos_char(const char* character)
{
    // Get Position
    char c = *character;
    
    // X
    if (c == '+' || c == 'b' || c == 'l' || c == 'v' || c == 'B' || c == 'L' || c == 'V' || c == '1')
        renderfont_pos_x = X_POS_1;
    else if (c == '!' || c == 'c' || c == 'm' || c == 'w' || c == 'C' || c == 'M' || c == 'W' || c == '2')
        renderfont_pos_x = X_POS_2;
    else if (c == '(' || c == 'd' || c == 'n' || c == 'x' || c == 'D' || c == 'N' || c == 'X' || c == '3')
        renderfont_pos_x = X_POS_3;
    else if (c == ')' || c == 'e' || c == 'o' || c == 'y' || c == 'E' || c == 'O' || c == 'Y' || c == '4')
        renderfont_pos_x = X_POS_4;
    else if (c == '$' || c == 'f' || c == 'p' || c == 'z' || c == 'F' || c == 'P' || c == 'Z' || c == '5')
        renderfont_pos_x = X_POS_5;
    else if (c == '%' || c == 'g' || c == 'q' || c == '.' || c == 'G' || c == 'Q' || c == '6')
        renderfont_pos_x = X_POS_6;
    else if (c == ':' || c == 'h' || c == 'r' || c == ',' || c == 'H' || c == 'R' || c == '7')
        renderfont_pos_x = X_POS_7;
    else if (c == 'i' || c == 's' || c == 'I' || c == 'S' || c == ' ' || c == '8')
        renderfont_pos_x = X_POS_8;
    else if (c == 'j' || c == 't' || c == 'J' || c == 'T' || c == '\'' || c == '9')
        renderfont_pos_x = X_POS_9;
    else
        renderfont_pos_x = X_POS_0;
    
    // Y
    if (c == '.' || c == ',' || c == ' ' || c == '\'')
    {
        renderfont_pos_y = FONTPOS_Y_ROW_THREE;
    }
    else if (c == '*' || c == '+' || c == '!' || c == '(' || c == ')' || c == '$' || c == '%' || c == ':')
    {
        // * = Ä , + = Ö, ! = Ü, ( = Copyright, ) = /, $ = French Left, % = French Right
        renderfont_pos_y = FONTPOS_Y_ROW_FIVE;
    }
    else
    {
        if (c == 'k' || c == 'l' || c == 'm' || c == 'n' || c == 'o' || c == 'p' || c == 'q' || c == 'r' || c == 's' || c == 't')
            renderfont_pos_y = FONTPOS_Y_ROW_TWO;
        else if (c == 'K' || c == 'L' || c == 'M' || c == 'N' || c == 'O' || c == 'P' || c == 'Q' || c == 'R' || c == 'S' || c == 'T')
            renderfont_pos_y = FONTPOS_Y_ROW_TWO;
        else if (c == 'u' || c == 'v' || c == 'w' || c == 'x' || c == 'y' || c == 'z')
            renderfont_pos_y = FONTPOS_Y_ROW_THREE;
        else if (c == 'U' || c == 'V' || c == 'W' || c == 'X' || c == 'Y' || c == 'Z')
            renderfont_pos_y = FONTPOS_Y_ROW_THREE;
        else if (c == '0' || c == '1' || c == '2' || c == '3' || c == '4' || c == '5' || c == '6' || c == '7' || c == '8' || c == '9')
            renderfont_pos_y = FONTPOS_Y_ROW_FOUR;
        else
            renderfont_pos_y = FONTPOS_Y_ROW_ONE;
    }
}

void RenderFont::renderfont_get_font_pos_u8(u8 number)
{
    // Get Position
    switch (number)
    {
        case 1:
            renderfont_pos_x = X_POS_1;
            break;
        case 2:
            renderfont_pos_x = X_POS_2;
            break;
        case 3:
            renderfont_pos_x = X_POS_3;
            break;
        case 4:
            renderfont_pos_x = X_POS_4;
            break;
        case 5:
            renderfont_pos_x = X_POS_5;
            break;
        case 6:
            renderfont_pos_x = X_POS_6;
            break;
        case 7:
            renderfont_pos_x = X_POS_7;
            break;
        case 8:
            renderfont_pos_x = X_POS_8;
            break;
        case 9:
            renderfont_pos_x = X_POS_9;
            break;
            
        default:
            renderfont_pos_x = X_POS_0;
            break;
    }
}
