//
//  render_item.cpp
//  3ds_metroid
//
//  Created by jbr373 on 15/02/16.
//

#include <sf2d.h>
#include <sfil.h>

#include "render_item.h"
#include "render_poscorrection.h"
#include "filepaths.h"

//#include "item_spritesheet_items.h"

#define SPRITESHEET_ITEM_DEFAULT_SPEED              4

#define SPRITEHSEET_ITEM_ENDFRAME_LEFT              2
#define SPRITESHEET_ITEM_ENDFRAME_RIGHT             5

RenderItem::RenderItem()
{
    // Frame
    renderitem_frame = 0;
    frame_increment_happened = false;
    
    // Sprite
    pItemSprite = NULL;
}

RenderItem::~RenderItem()
{
    if (pItemSprite != NULL)
    {
        if (pItemSprite->spritesheet_texture != NULL)
            sf2d_free_texture(pItemSprite->spritesheet_texture);
        
        delete pItemSprite;
    }
}

void RenderItem::renderitem_load()
{
    /* Create Items Texture */
    
    pItemSprite = new SpriteSheet();
    
    /* Load Textures */
    
    /*
    pItemSprite->spritesheet_texture = sf2d_create_texture_mem_RGBA8(item_spritesheet_items.pixel_data,
                                                                     item_spritesheet_items.width,
                                                                     item_spritesheet_items.height,
                                                                     TEXFMT_RGBA8,
                                                                     SF2D_PLACE_RAM);
    */

    /* Set Frame Data */
    
    pItemSprite->spritesheet_texture = sfil_load_PNG_file(IMG_PLAYER_ITEMS, SF2D_PLACE_RAM);

    pItemSprite->spritesheet_frame = 0;
    pItemSprite->spritesheet_frame_speed = SPRITESHEET_ITEM_DEFAULT_SPEED;
    pItemSprite->spritesheet_frame_speed_current = 0;
    pItemSprite->spritesheet_endframe = SPRITEHSEET_ITEM_ENDFRAME_LEFT;
    pItemSprite->spritesheet_startframe = 0;
    pItemSprite->spritesheet_sprite_line = 0;
    pItemSprite->spritesheet_w_single = SPRITESHEET_ITEM_WIDTH_SINGLE;
    pItemSprite->spritesheet_h_single = SPRITESHEET_ITEM_HEIGHT_SINGLE;
    pItemSprite->spritesheet_w_full = SPRITESHEET_ITEM_WIDTH_FULL;
    pItemSprite->spritesheet_h_full = SPRITESHEET_ITEM_HEIGHT_FULL;
}

SpriteSheet* RenderItem::renderitem_get_sprite(Player* pPlayer, Item* pItem, bool animate)
{
    if (pItem->item_get_collected() == true)
    {
        return NULL;
    }
    
    // Increase Frame if allowed
    if (animate == true && frame_increment_happened == false)
    {
        // Increment Frame
        if (pItemSprite->spritesheet_frame_speed_current >= pItemSprite->spritesheet_frame_speed)
        {
            //Increment Frame to the max of the Left Sprite side
            if (renderitem_frame < SPRITEHSEET_ITEM_ENDFRAME_LEFT)
            {
                ++renderitem_frame;
            }
            else
            {
                renderitem_frame = 0;
            }
            
            // Reset Frame Counter
            pItemSprite->spritesheet_frame_speed_current = 0;
        }
        else
        {
            // Increase Frame Counter
            pItemSprite->spritesheet_frame_speed_current += 1;
        }
        
        // Set Frame Increment Happened
        frame_increment_happened = true;
    }
    
    // Set Item Spritesheet Values
    switch (pItem->item_get_type())
    {
        case ITEMTYPE_HEALTH20:
            pItemSprite->spritesheet_endframe = SPRITEHSEET_ITEM_ENDFRAME_LEFT;
            pItemSprite->spritesheet_frame = renderitem_frame;
            pItemSprite->spritesheet_startframe = 0;
            pItemSprite->spritesheet_sprite_line = 6;
            break;
        case ITEMTYPE_HEALTH50:
            pItemSprite->spritesheet_endframe = SPRITEHSEET_ITEM_ENDFRAME_LEFT;
            pItemSprite->spritesheet_frame = renderitem_frame;
            pItemSprite->spritesheet_startframe = 0;
            pItemSprite->spritesheet_sprite_line = 7;
            break;
        case ITEMTYPE_MISSLE5:
            pItemSprite->spritesheet_endframe = SPRITESHEET_ITEM_ENDFRAME_RIGHT;
            pItemSprite->spritesheet_frame = renderitem_frame + SPRITEHSEET_ITEM_ENDFRAME_LEFT + 1;
            pItemSprite->spritesheet_startframe = SPRITEHSEET_ITEM_ENDFRAME_LEFT + 1;
            pItemSprite->spritesheet_sprite_line = 6;
            break;
        case ITEMTYPE_HEALTHTANK:
            pItemSprite->spritesheet_endframe = SPRITEHSEET_ITEM_ENDFRAME_LEFT;
            pItemSprite->spritesheet_frame = renderitem_frame;
            pItemSprite->spritesheet_startframe = 0;
            pItemSprite->spritesheet_sprite_line = 0;
            break;
        case ITEMTYPE_MISSLETANK:
            pItemSprite->spritesheet_endframe = SPRITESHEET_ITEM_ENDFRAME_RIGHT;
            pItemSprite->spritesheet_frame = renderitem_frame + SPRITEHSEET_ITEM_ENDFRAME_LEFT + 1;
            pItemSprite->spritesheet_startframe = SPRITEHSEET_ITEM_ENDFRAME_LEFT + 1;
            pItemSprite->spritesheet_sprite_line = 0;
            break;
        case ITEMTYPE_ICEBEAM:
            pItemSprite->spritesheet_endframe = SPRITEHSEET_ITEM_ENDFRAME_LEFT;
            pItemSprite->spritesheet_frame = renderitem_frame;
            pItemSprite->spritesheet_startframe = 0;
            pItemSprite->spritesheet_sprite_line = 1;
            break;
        case ITEMTYPE_WAVEBEAM:
            pItemSprite->spritesheet_endframe = SPRITESHEET_ITEM_ENDFRAME_RIGHT;
            pItemSprite->spritesheet_frame = renderitem_frame + SPRITEHSEET_ITEM_ENDFRAME_LEFT + 1;
            pItemSprite->spritesheet_startframe = SPRITEHSEET_ITEM_ENDFRAME_LEFT + 1;
            pItemSprite->spritesheet_sprite_line = 1;
            break;
        case ITEMTYPE_SPAZERBEAM:
            pItemSprite->spritesheet_endframe = SPRITEHSEET_ITEM_ENDFRAME_LEFT;
            pItemSprite->spritesheet_frame = renderitem_frame;
            pItemSprite->spritesheet_startframe = 0;
            pItemSprite->spritesheet_sprite_line = 2;
            break;
        case ITEMTYPE_PLASMABEAM:
            pItemSprite->spritesheet_endframe = SPRITESHEET_ITEM_ENDFRAME_RIGHT;
            pItemSprite->spritesheet_frame = renderitem_frame + SPRITEHSEET_ITEM_ENDFRAME_LEFT + 1;
            pItemSprite->spritesheet_startframe = SPRITEHSEET_ITEM_ENDFRAME_LEFT + 1;
            pItemSprite->spritesheet_sprite_line = 2;
            break;
            
        default:
            break;
    }
    
    // Return Item Spritesheet
    return pItemSprite;
}
