//
//  render_projectile.cpp
//  3ds_metroid
//
//  Created by jbr373 on 01/02/16.
//

#include <sfil.h>

#include "render_projectile.h"
#include "filepaths.h"
#include "render_poscorrection.h"
#include "actor_facing.h"

/*
 *      TODO
 * ---------------
 * - Works, but is really ugly in terms of code style. Also way too much copying.
 */

/* General */
#define PROJECTILE_DEFAULT_FRAME_SPEED          1
#define PROJECTILE_SPRITE_WIDTH                 30
#define PROJECTILE_SPRITE_HEIGHT                30
#define PROJECTILE_SPRITESHEET_WIDTH            240
#define PROJECTILE_SPRITESHEET_HEIGHT           420

/* All Player Beams */
#define BEAM_FRAMES_SHOT_HORIZONTAL             0
#define BEAM_FRAMES_CHARGEDSHOT_HORIZONTAL      1
#define BEAM_FRAMES_SHOT_DIAGONAL               2
#define BEAM_FRAMES_CHARGEDSHOT_DIAGONAL        3

#define BEAM_FRAMES_CHARGE_MIN                  4
#define BEAM_FRAMES_CHARGE_MAX                  6
#define BEAM_FRAMES_CHARGED_MIN                 6
#define BEAM_FRAMES_CHARGED_MAX                 7

#define CHARGEBEAM_COUNTER_MAX                  1

/* Missle */
#define MISSLE_FRAMES_TRAVEL_MIN                4
#define MISSLE_FRAMES_TRAVEL_MAX                7

#define MISSLE_FRAMES_SHOT_HORIZONTAL_MIN       0
#define MISSLE_FRAMES_SHOT_HORIZONTAL_MAX       1
#define MISSLE_FRAMES_SHOT_DIAGONAL_MIN         2
#define MISSLE_FRAMES_SHOT_DIAGONAL_MAX         3

#define MISSLE_COUNTER_MAX                      1

#define MISSLE_FRAME_SPEED                      2

/* Bombs */
#define PROJECTILE_BOMB_FRAME_MAX               7

#define BOMB_FRAME_SPEED                        2

/* Explosion */
#define PROJECTILE_EXPLOSION_FRAME_START        0
#define PROJECTILE_EXPLOSION_FRAME_END          7

enum ProjectileTextureLines
{
    PROJECTILE_TEXLINE_POWERBEAM_TRAVEL = 0,
    PROJECTILE_TEXLINE_POWERBEAM_EXPLOSION = 1,
    PROJECTILE_TEXLINE_ICEBEAM_TRAVEL = 2,
    PROJECTILE_TEXLINE_ICEBEAM_EXPLOSION = 3,
    PROJECTILE_TEXLINE_WAVEBEAM_TRAVEL = 4,
    PROJECTILE_TEXLINE_WAVEBEAM_EXPLOSION = 5,
    PROJECTILE_TEXLINE_SPAZERBEAM_TRAVEL = 6,
    PROJECTILE_TEXLINE_SPAZERBEAM_EXPLOSION = 7,
    PROJECTILE_TEXLINE_PLASMABEAM_TRAVEL = 8,
    PROJECTILE_TEXLINE_PLASMABEAM_EXPLOSION = 9,
    PROJECTILE_TEXLINE_MISSLE_TRAVEL = 10,
    PROJECTILE_TEXLINE_MISSLE_EXPLOSION = 11,
    PROJECTILE_TEXLINE_BOMB_TRAVEL = 12,
    PROJECTILE_TEXLINE_BOMB_EXPLOSION = 13,
};

enum ProjectileBeams
{
    PROJECTILE_POWERBEAM_ACTIVE = 0,
    PROJECTILE_ICEBEAM_ACTIVE = 1,
    PROJECTILE_WAVEBEAM_ACTIVE = 2,
    PROJECTILE_SPAZERBEAM_ACTIVE = 3,
    PROJECTILE_PLASMABEAM_ACTIVE = 4
};

RenderProjectile::RenderProjectile()
{
    // Camera
    renderprojectile_projectile_x = 0;
    renderprojectile_projectile_y = 0;
    renderprojectile_chargebeam_x = 0;
    renderprojectile_chargebeam_y = 0;
    
    // Charge Beam
    renderprojectile_chargebeam_frame = BEAM_FRAMES_CHARGE_MIN;
    renderprojectile_chargebeam_counter = 0;
    
    // Missle Smoke
    renderprojectile_missle_smoke_frame = MISSLE_FRAMES_TRAVEL_MIN;
    renderprojectile_missle_smoke_counter = 0;
    
    // SpriteSheet Struct
    pPlayerProjectileTex = NULL;
}

RenderProjectile::~RenderProjectile()
{
    // Player Projectiles
    if (pPlayerProjectileTex != NULL)
    {
        if (pPlayerProjectileTex->spritesheet_texture != NULL)
        {
            sf2d_free_texture(pPlayerProjectileTex->spritesheet_texture);
        }
    
        delete pPlayerProjectileTex;
    }
}

void RenderProjectile::renderprojectile_load()
{
    /* Create Texture Objects */
    
    pPlayerProjectileTex = new SpriteSheet();
    
    /* Load Textures */
    
    pPlayerProjectileTex->spritesheet_texture = sfil_load_PNG_file(IMG_PROJECTILE_PLAYER, SF2D_PLACE_RAM);
    
    /* Set Frame Data */
    
    pPlayerProjectileTex->spritesheet_frame = BEAM_FRAMES_CHARGE_MIN;
    pPlayerProjectileTex->spritesheet_frame_speed = PROJECTILE_DEFAULT_FRAME_SPEED;
    pPlayerProjectileTex->spritesheet_frame_speed_current = 0;
    pPlayerProjectileTex->spritesheet_endframe = 0;
    pPlayerProjectileTex->spritesheet_startframe = 0;
    pPlayerProjectileTex->spritesheet_sprite_line = PROJECTILE_TEXLINE_POWERBEAM_TRAVEL;
    pPlayerProjectileTex->spritesheet_w_single = PROJECTILE_SPRITE_WIDTH;
    pPlayerProjectileTex->spritesheet_h_single = PROJECTILE_SPRITE_HEIGHT;
    pPlayerProjectileTex->spritesheet_w_full = PROJECTILE_SPRITESHEET_WIDTH;
    pPlayerProjectileTex->spritesheet_h_full = PROJECTILE_SPRITESHEET_HEIGHT;
}

SpriteSheet* RenderProjectile::renderprojectile_get_sprite(Projectile* pProjectile, bool is_right_screen)
{
    // Check for valid Pointer
    if (pProjectile == NULL)
        return NULL;

    // Update Data
    if (is_right_screen == true)
    {
        renderprojectile_update(pProjectile);
    }
    
    // Return Default
    return pPlayerProjectileTex;
}

void RenderProjectile::renderprojectile_reset_missle_smoke()
{
    // Reset Smoke Frame
    renderprojectile_missle_smoke_frame = MISSLE_FRAMES_TRAVEL_MIN;
    renderprojectile_missle_smoke_counter = 0;
}

void RenderProjectile::renderprojectile_reset_chargebeam_frame()
{
    renderprojectile_chargebeam_frame = BEAM_FRAMES_CHARGE_MIN;
    renderprojectile_chargebeam_counter = 0;
}


SpriteSheet* RenderProjectile::renderprojectile_get_smoke_sprite(bool increment_frame)
{
    // Update Sprite Line
    pPlayerProjectileTex->spritesheet_sprite_line = PROJECTILE_TEXLINE_MISSLE_TRAVEL;
    
    // Set Smoke Frame
    if (increment_frame == true)
    {
        if (renderprojectile_missle_smoke_counter == MISSLE_COUNTER_MAX)
        {
            // Reset Smoke Counter
            renderprojectile_missle_smoke_counter = 0;
            
            // Set Frame
            if (renderprojectile_missle_smoke_frame < MISSLE_FRAMES_TRAVEL_MAX)
            {
                // Increment Smoke Frame
                ++renderprojectile_missle_smoke_frame;
            }
        }
        else
        {
            // Increment Smoke Counter
            ++renderprojectile_missle_smoke_counter;
        }
    }
    
    // Set SpriteData
    pPlayerProjectileTex->spritesheet_frame = renderprojectile_missle_smoke_frame;
    pPlayerProjectileTex->spritesheet_startframe = MISSLE_FRAMES_TRAVEL_MIN;
    pPlayerProjectileTex->spritesheet_endframe = MISSLE_FRAMES_TRAVEL_MAX;
    
    // Return Sprite Data
    return pPlayerProjectileTex;
}

SpriteSheet* RenderProjectile::renderprojectile_get_chargebeam_sprite(u8 active_beam, u8 charge_beam_counter, bool increment_frame)
{
    /* Update Sprite Data and return updated Sprite */

    /* Set Sprite Line from Active Beam */
    
    if (active_beam == PROJECTILE_POWERBEAM_ACTIVE || active_beam == PROJECTILE_ICEBEAM_ACTIVE || active_beam == PROJECTILE_WAVEBEAM_ACTIVE || active_beam == PROJECTILE_SPAZERBEAM_ACTIVE || active_beam == PROJECTILE_PLASMABEAM_ACTIVE)
    {
        // Reset Frame
        if (charge_beam_counter == 0)
        {
            renderprojectile_chargebeam_frame = BEAM_FRAMES_CHARGE_MIN;
        }
        
        switch (active_beam)
        {
            case PROJECTILE_ICEBEAM_ACTIVE:
                pPlayerProjectileTex->spritesheet_sprite_line = PROJECTILE_TEXLINE_ICEBEAM_TRAVEL;
                break;
            case PROJECTILE_WAVEBEAM_ACTIVE:
                pPlayerProjectileTex->spritesheet_sprite_line = PROJECTILE_TEXLINE_WAVEBEAM_TRAVEL;
                break;
            case PROJECTILE_SPAZERBEAM_ACTIVE:
                pPlayerProjectileTex->spritesheet_sprite_line = PROJECTILE_TEXLINE_SPAZERBEAM_TRAVEL;
                break;
            case PROJECTILE_PLASMABEAM_ACTIVE:
                pPlayerProjectileTex->spritesheet_sprite_line = PROJECTILE_TEXLINE_PLASMABEAM_TRAVEL;
                break;
                
            default:
                pPlayerProjectileTex->spritesheet_sprite_line = PROJECTILE_TEXLINE_POWERBEAM_TRAVEL;
                break;
        }
        
        /* Set Animation Data */
        
        // Set Charge Beam Frame
        if (increment_frame == true)
        {
            if (renderprojectile_chargebeam_counter == CHARGEBEAM_COUNTER_MAX)
            {
                // Reset Smoke Counter
                renderprojectile_chargebeam_counter = 0;
                
                // Charge States depending on Counter
                if (charge_beam_counter < DEFAULT_PLAYER_CHARGE_BEAM)
                {
                    // Beam charging
                    
                    // Custom Frame handling
                    if (renderprojectile_chargebeam_frame < BEAM_FRAMES_CHARGE_MAX)
                        ++renderprojectile_chargebeam_frame;
                    else if (renderprojectile_chargebeam_frame == 0)
                        renderprojectile_chargebeam_frame = BEAM_FRAMES_CHARGE_MIN;
                    else
                        renderprojectile_chargebeam_frame = BEAM_FRAMES_CHARGE_MIN;
                    
                    // Set SpriteData for charging
                    pPlayerProjectileTex->spritesheet_frame = renderprojectile_chargebeam_frame;
                    pPlayerProjectileTex->spritesheet_startframe = BEAM_FRAMES_CHARGE_MIN;
                    pPlayerProjectileTex->spritesheet_endframe = BEAM_FRAMES_CHARGE_MAX;
                }
                else
                {
                    // Beam fully charged
                    
                    // Custom Frame handling
                    if (renderprojectile_chargebeam_frame < BEAM_FRAMES_CHARGED_MAX)
                        ++renderprojectile_chargebeam_frame;
                    else if (renderprojectile_chargebeam_frame == 0)
                        renderprojectile_chargebeam_frame = BEAM_FRAMES_CHARGED_MIN;
                    else
                        renderprojectile_chargebeam_frame = BEAM_FRAMES_CHARGED_MIN;
                }
            }
            else
            {
                // Increment Smoke Counter
                ++renderprojectile_chargebeam_counter;
            }
        }
        
        // Set SpriteData for charged
        pPlayerProjectileTex->spritesheet_frame = renderprojectile_chargebeam_frame;
        pPlayerProjectileTex->spritesheet_startframe = BEAM_FRAMES_CHARGED_MIN;
        pPlayerProjectileTex->spritesheet_endframe = BEAM_FRAMES_CHARGED_MAX;
    }
    
    // Return Sprite Data
    return pPlayerProjectileTex;
}

void RenderProjectile::renderprojectile_set_projectile_screen_pos(Player* pPlayer, Projectile* pProjectile, s32 player_camera_x, s32 player_camera_y)
{
    // Camera X
    renderprojectile_projectile_x = (player_camera_x + PROJECTILE_X_CORRECT) + (pProjectile->projectile_get_x() - (pPlayer->player_get_x() - (pPlayer->player_get_width() / 2)));
    
    // Camera Y
    if (pProjectile->projectile_get_launch_state() == PROJECTILE_LAUNCHSTATE_CROUCHED)
    {
        renderprojectile_projectile_y = (player_camera_y - (pProjectile->projectile_get_y() - (pPlayer->player_get_y() + pPlayer->player_get_height())));
    }
    else if (pProjectile->projectile_get_launch_state() == PROJECTILE_LAUNCHSTATE_MORPHBALL)
    {
        renderprojectile_projectile_y = (player_camera_y - (pProjectile->projectile_get_y() - (pPlayer->player_get_y() + pPlayer->player_get_height())));
    }
    else
    {
        renderprojectile_projectile_y = (player_camera_y - (pProjectile->projectile_get_y() - (pPlayer->player_get_y() + pPlayer->player_get_height())));
    }
}

void RenderProjectile::renderprojectile_set_chargebeam_screen_pos(Player* pPlayer, s32 player_camera_x, s32 player_camera_y, u8 player_frame)
{
    /* Camera X */
    
    if (pPlayer->player_get_shooting_facing() < FACING_LEFT &&
        pPlayer->player_get_anim() != PLAYERANIM_FALLING && pPlayer->player_get_anim() != PLAYERANIM_JUMP90)
    {
        renderprojectile_chargebeam_x = player_camera_x + PROJECTILE_CHARGEBEAM_CORRECT_RIGHT;
    }
    else if (pPlayer->player_get_shooting_facing() > FACING_RIGHTDOWN && pPlayer->player_get_shooting_facing() < FACING_UP &&
             pPlayer->player_get_anim() != PLAYERANIM_FALLING && pPlayer->player_get_anim() != PLAYERANIM_JUMP90)
    {
        renderprojectile_chargebeam_x = player_camera_x + PROJECTILE_CHARGEBEAM_CORRECT_LEFT;
    }
    else
    {
        renderprojectile_chargebeam_x = player_camera_x + PROJECTILE_CHARGEBEAM_CORRECT_CENTER;
        
        if (pPlayer->player_get_anim() != PLAYERANIM_FALLING && pPlayer->player_get_anim() != PLAYERANIM_JUMP90)
        {
            if (pPlayer->player_get_shooting_facing() == FACING_RIGHTDOWN || pPlayer->player_get_shooting_facing() == FACING_RIGHTUP)
            {
                renderprojectile_chargebeam_x += PROJECTILE_X_CORRECT_VERTICAL;
            }
            else if (pPlayer->player_get_shooting_facing() == FACING_LEFTDOWN || pPlayer->player_get_shooting_facing() == FACING_LEFTUP)
            {
                renderprojectile_chargebeam_x -= PROJECTILE_X_CORRECT_VERTICAL;
            }
        }
        else if (pPlayer->player_get_anim() == PLAYERANIM_FALLING || pPlayer->player_get_anim() == PLAYERANIM_JUMP90)
        {
            if (pPlayer->player_get_shooting_facing() == FACING_RIGHT)
            {
                renderprojectile_chargebeam_x += PROJECTILE_X_CORRECT_VERTICAL * 2;
            }
            else if (pPlayer->player_get_shooting_facing() == FACING_LEFT)
            {
                renderprojectile_chargebeam_x -= PROJECTILE_X_CORRECT_VERTICAL * 2;
            }
        }
        else if (pPlayer->player_get_anim() == PLAYERANIM_HANGING)
        {
            if (player_frame < 2)
            {
                renderprojectile_chargebeam_x -= PROJECTILE_X_CORRECT_VERTICAL * 2;
            }
            else
            {
                renderprojectile_chargebeam_x += PROJECTILE_X_CORRECT_VERTICAL * 2;
            }
        }
        else
        {
            /*
            if (player_frame == 3)
            {
                renderprojectile_chargebeam_x += PROJECTILE_X_CORRECT_VERTICAL;
            }
            else if (player_frame == 7)
            {
                renderprojectile_chargebeam_x -= PROJECTILE_X_CORRECT_VERTICAL;
            }
            */
        }
    }
    
    
    /* Camera Y */
    
    if (pPlayer->player_get_crouched() == true)
    {
        renderprojectile_chargebeam_y = player_camera_y + PROJECTILE_CHARGEBEAM_CORRECT_CROUCH_Y;
        
        // Left X
        if (pPlayer->player_get_facing() > FACING_RIGHTDOWN && pPlayer->player_get_facing() < FACING_UP)
        {
            renderprojectile_chargebeam_x += PROJECTILE_CHARGEBEAM_CORRECT_LEFT / 10;
        }
    }
    else
    {
        // Set Y Standing
        renderprojectile_chargebeam_y = player_camera_y + PROJECTILE_CHARGEBEAM_CORRECT_IDLE_Y;
    }
    
    /* Increase or Reduce Y and X for Up/Down Aiming */
    
    if (pPlayer->player_get_shooting_facing() == FACING_RIGHTUP || pPlayer->player_get_shooting_facing() == FACING_LEFTUP)
    {
        if (pPlayer->player_get_crouched() == true)
            renderprojectile_chargebeam_y -= PROJECTILE_CHARGEBEAM_CORRECT_CROUCHED_Y_LRUP;
        else
            renderprojectile_chargebeam_y -= PROJECTILE_CHARGEBEAM_CORRECT_IDLE_Y_ALL;
    }
    else if (pPlayer->player_get_shooting_facing() == FACING_RIGHTDOWN || pPlayer->player_get_shooting_facing() == FACING_LEFTDOWN)
    {
        if (pPlayer->player_get_crouched() == true)
            renderprojectile_chargebeam_y += PROJECTILE_CHARGEBEAM_CORRECT_CROUCHED_Y_LRDOWN;
        else
            renderprojectile_chargebeam_y += PROJECTILE_CHARGEBEAM_CORRECT_IDLE_Y_ALL;
    }
    else if (pPlayer->player_get_shooting_facing() == FACING_UP)
    {
        if (pPlayer->player_get_crouched() == true)
            renderprojectile_chargebeam_y -= PROJECTILE_CHARGEBEAM_CORRECT_CROUCHED_Y_LRUP + PROJECTILE_CHARGEBEAM_CORRECT_CROUCHED_Y_LRUP / 2;
        else
            renderprojectile_chargebeam_y -= PROJECTILE_CHARGEBEAM_CORRECT_IDLE_Y_ALL + PROJECTILE_CHARGEBEAM_CORRECT_IDLE_Y_ALL / 2;
    }
    else if (pPlayer->player_get_shooting_facing() == FACING_DOWN)
    {
        if (pPlayer->player_get_crouched() == true)
            renderprojectile_chargebeam_y += PROJECTILE_CHARGEBEAM_CORRECT_CROUCHED_Y_LRUP + PROJECTILE_CHARGEBEAM_CORRECT_CROUCHED_Y_LRUP / 2;
        else
            renderprojectile_chargebeam_y += PROJECTILE_CHARGEBEAM_CORRECT_IDLE_Y_ALL + PROJECTILE_CHARGEBEAM_CORRECT_IDLE_Y_ALL / 2;
    }
    
    // Down Fix
    if (pPlayer->player_get_anim() == PLAYERANIM_JUMP90 || pPlayer->player_get_anim() == PLAYERANIM_FALLING)
        renderprojectile_chargebeam_y += 5;
}

u8 RenderProjectile::renderprojectile_getspriteline(Projectile* pProjectile)
{
    if (pProjectile->projectile_get_explode() == false)
    {
        if (pProjectile->projectile_get_type() == PROJECTILE_ICEBEAM || pProjectile->projectile_get_type() == PROJECTILE_ICEBEAM_CHARGED)
            return PROJECTILE_TEXLINE_ICEBEAM_TRAVEL;
        else if (pProjectile->projectile_get_type() == PROJECTILE_WAVEBEAM || pProjectile->projectile_get_type() == PROJECTILE_WAVEBEAM_CHARGED)
            return PROJECTILE_TEXLINE_WAVEBEAM_TRAVEL;
        else if (pProjectile->projectile_get_type() == PROJECTILE_SPAZERBEAM || pProjectile->projectile_get_type() == PROJECTILE_SPAZERBEAM_CHARGED)
            return PROJECTILE_TEXLINE_SPAZERBEAM_TRAVEL;
        else if (pProjectile->projectile_get_type() == PROJECTILE_PLASMABEAM || pProjectile->projectile_get_type() == PROJECTILE_PLASMABEAM_CHARGED)
            return PROJECTILE_TEXLINE_PLASMABEAM_TRAVEL;
        else if (pProjectile->projectile_get_type() == PROJECTILE_MISSLE)
            return PROJECTILE_TEXLINE_MISSLE_TRAVEL;
        else if (pProjectile->projectile_get_type() == PROJECTILE_BOMB)
            return PROJECTILE_TEXLINE_BOMB_TRAVEL;
        else
            return PROJECTILE_TEXLINE_POWERBEAM_TRAVEL;
    }
    else
    {
        if (pProjectile->projectile_get_type() == PROJECTILE_ICEBEAM || pProjectile->projectile_get_type() == PROJECTILE_ICEBEAM_CHARGED)
            return PROJECTILE_TEXLINE_ICEBEAM_EXPLOSION;
        else if (pProjectile->projectile_get_type() == PROJECTILE_WAVEBEAM || pProjectile->projectile_get_type() == PROJECTILE_WAVEBEAM_CHARGED)
            return PROJECTILE_TEXLINE_WAVEBEAM_EXPLOSION;
        else if (pProjectile->projectile_get_type() == PROJECTILE_SPAZERBEAM || pProjectile->projectile_get_type() == PROJECTILE_SPAZERBEAM_CHARGED)
            return PROJECTILE_TEXLINE_SPAZERBEAM_EXPLOSION;
        else if (pProjectile->projectile_get_type() == PROJECTILE_PLASMABEAM || pProjectile->projectile_get_type() == PROJECTILE_PLASMABEAM_CHARGED)
            return PROJECTILE_TEXLINE_PLASMABEAM_EXPLOSION;
        else if (pProjectile->projectile_get_type() == PROJECTILE_MISSLE)
            return PROJECTILE_TEXLINE_MISSLE_EXPLOSION;
        else if (pProjectile->projectile_get_type() == PROJECTILE_BOMB)
            return PROJECTILE_TEXLINE_BOMB_EXPLOSION;
        else
            return PROJECTILE_TEXLINE_POWERBEAM_EXPLOSION;
    }
    
    return PROJECTILE_TEXLINE_POWERBEAM_TRAVEL;
}

void RenderProjectile::renderprojectile_update(Projectile* pProjectile)
{
    // Set Frames when exploding
    if (pProjectile->projectile_get_explode() == true)
    {
        if (pProjectile->projectile_get_type() == PROJECTILE_BOMB)
        {
            pPlayerProjectileTex->spritesheet_sprite_line = renderprojectile_getspriteline(pProjectile);
            pPlayerProjectileTex->spritesheet_frame = pProjectile->projectile_get_explosion_frame();
            pPlayerProjectileTex->spritesheet_startframe = PROJECTILE_EXPLOSION_FRAME_START;
            pPlayerProjectileTex->spritesheet_endframe = PROJECTILE_EXPLOSION_FRAME_END;
        }
        else
        {
            pPlayerProjectileTex->spritesheet_sprite_line = renderprojectile_getspriteline(pProjectile);
            pPlayerProjectileTex->spritesheet_frame = pProjectile->projectile_get_explosion_frame();
            pPlayerProjectileTex->spritesheet_startframe = PROJECTILE_EXPLOSION_FRAME_START;
            pPlayerProjectileTex->spritesheet_endframe = PROJECTILE_EXPLOSION_FRAME_END;
        }
        
        // Stop Updating
        return;
    }
    
    // Set Frame depending on Facing
    if (pProjectile->projectile_get_facing() == FACING_RIGHT || pProjectile->projectile_get_facing() == FACING_LEFT || pProjectile->projectile_get_facing() == FACING_UP || pProjectile->projectile_get_facing() == FACING_DOWN)
    {
        if (pProjectile->projectile_get_type() == PROJECTILE_MISSLE)
        {
            pPlayerProjectileTex->spritesheet_frame = MISSLE_FRAMES_SHOT_HORIZONTAL_MIN;
            pPlayerProjectileTex->spritesheet_sprite_line = renderprojectile_getspriteline(pProjectile);
            pPlayerProjectileTex->spritesheet_startframe = MISSLE_FRAMES_SHOT_HORIZONTAL_MIN;
            pPlayerProjectileTex->spritesheet_endframe = MISSLE_FRAMES_SHOT_HORIZONTAL_MIN;
        }
        else if (pProjectile->projectile_get_type() == PROJECTILE_BOMB)
        {
            pPlayerProjectileTex->spritesheet_sprite_line = renderprojectile_getspriteline(pProjectile);
            pPlayerProjectileTex->spritesheet_startframe = 0;
            pPlayerProjectileTex->spritesheet_endframe = PROJECTILE_BOMB_FRAME_MAX;
        }
        else
        {
            pPlayerProjectileTex->spritesheet_sprite_line = renderprojectile_getspriteline(pProjectile);

            if (pProjectile->projectile_is_charged() == false)
            {
                pPlayerProjectileTex->spritesheet_frame = BEAM_FRAMES_SHOT_HORIZONTAL;
                pPlayerProjectileTex->spritesheet_startframe = BEAM_FRAMES_SHOT_HORIZONTAL;
                pPlayerProjectileTex->spritesheet_endframe = BEAM_FRAMES_SHOT_HORIZONTAL;
            }
            else
            {
                pPlayerProjectileTex->spritesheet_frame = BEAM_FRAMES_CHARGEDSHOT_HORIZONTAL;
                pPlayerProjectileTex->spritesheet_startframe = BEAM_FRAMES_CHARGEDSHOT_HORIZONTAL;
                pPlayerProjectileTex->spritesheet_endframe = BEAM_FRAMES_CHARGEDSHOT_HORIZONTAL;
            }
        }
    }
    else
    {
        if (pProjectile->projectile_get_type() == PROJECTILE_MISSLE)
        {
            pPlayerProjectileTex->spritesheet_frame = MISSLE_FRAMES_SHOT_DIAGONAL_MIN;
            pPlayerProjectileTex->spritesheet_sprite_line = renderprojectile_getspriteline(pProjectile);
            pPlayerProjectileTex->spritesheet_startframe = MISSLE_FRAMES_SHOT_DIAGONAL_MIN;
            pPlayerProjectileTex->spritesheet_endframe = MISSLE_FRAMES_SHOT_DIAGONAL_MIN;
        }
        else if (pProjectile->projectile_get_type() == PROJECTILE_BOMB)
        {
            pPlayerProjectileTex->spritesheet_sprite_line = renderprojectile_getspriteline(pProjectile);
            pPlayerProjectileTex->spritesheet_startframe = 0;
            pPlayerProjectileTex->spritesheet_endframe = PROJECTILE_BOMB_FRAME_MAX;
        }
        else
        {
            pPlayerProjectileTex->spritesheet_sprite_line = renderprojectile_getspriteline(pProjectile);

            if (pProjectile->projectile_is_charged() == false)
            {
                pPlayerProjectileTex->spritesheet_frame = BEAM_FRAMES_SHOT_DIAGONAL;
                pPlayerProjectileTex->spritesheet_startframe = BEAM_FRAMES_SHOT_DIAGONAL;
                pPlayerProjectileTex->spritesheet_endframe = BEAM_FRAMES_SHOT_DIAGONAL;
            }
            else
            {
                pPlayerProjectileTex->spritesheet_frame = BEAM_FRAMES_CHARGEDSHOT_DIAGONAL;
                pPlayerProjectileTex->spritesheet_startframe = BEAM_FRAMES_CHARGEDSHOT_DIAGONAL;
                pPlayerProjectileTex->spritesheet_endframe = BEAM_FRAMES_CHARGEDSHOT_DIAGONAL;
            }
        }
    }
}