//
//  render_water.cpp
//  3ds_metroid
//
//  Created by jbr373 on 21/03/16.
//

#include <sf2d.h>

#include "render_water.h"

/*
 *      TODO
 * ---------------
 * - Water should be loaded from File - Remove Hardcoded Values!
 */

enum WaterTypes
{
    WATERTYPE_NONE = 0,
    
    WATERTYPE_WATER_WAVES = 1,
    WATERTYPE_WATER_STILL = 2,
    WATERTYPE_WATER_WAVES_LIGHT = 3,
    WATERTYPE_WATER_STILL_LIGHT = 4,
    WATERTYPE_ACID_BUBBLES = 5,
    WATERTYPE_ACID_STILL = 6,
    WATERTYPE_LAVA_BUBBLES = 7,
    WATERTYPE_LAVA_STILL = 8
};

RenderWater::RenderWater()
{
    // Flags
    renderwater_loaded = true;
    
    // Water Temp
    renderwater_counter = 0;
    
    /* Water Data */
    
    renderwater_water_type = 0;
    
    /* Water Surface */
    
    /*
    // Surface Line
    renderwater_water_surface_alpha_min = 0;
    renderwater_water_surface_alpha_max = 0;
    renderwater_water_surface_alpha_step = 0;
    
    renderwater_water_surface_line_length = 0;
    renderwater_water_surface_line_alpha_min = 0;
    renderwater_water_surface_line_alpha_max = 0;
    renderwater_water_surface_line_alpha_step = 0;
    
    // Surface Base Color
    renderwater_water_surface_r = 0;
    renderwater_water_surface_g = 0;
    renderwater_water_surface_b = 0;
    */
    renderwater_water_surface_color = 0;
    
    /* Water Body */
    
    // Water Body
    renderwater_water_body_alpha_min = 0;
    renderwater_water_body_alpha_max = 0;
    renderwater_water_body_alpha_step = 0;
    
    // Body Base Color
    renderwater_water_body_r = 0;
    renderwater_water_body_g = 0;
    renderwater_water_body_b = 0;
    
    /* Water Light Lines */
    
    // Water Light Info
    renderwater_water_draw_light = 0;
    renderwater_water_light_line_base_width = 0;
    renderwater_water_light_line_base_length = 0;
    renderwater_water_light_line_alpha_difference = 0;
    renderwater_water_light_line_difference = 0;
    renderwater_water_light_line_angle = 0;
    renderwater_water_light_line_count = 0;
    renderwater_water_light_x_min = 0;
    renderwater_water_light_x_max = 0;
    
    // Water Light Color
    renderwater_water_light_r = 0;
    renderwater_water_light_g = 0;
    renderwater_water_light_b = 0;
    
    /* Water Waves */
    
    // Water Wave Info
    renderwater_water_wave_size_up = 0;
    renderwater_water_wave_size_down = 0;
    renderwater_water_wave_x_difference = 0;
    renderwater_water_wave_falling_speed = 0;
    renderwater_water_wave_draw_idle = 0;
}

RenderWater::~RenderWater()
{
}

/*************************************************************************************************************************/
/*                                                    Water Loading                                                      */
/*************************************************************************************************************************/

void RenderWater::renderwater_load(u8 water_type)
{
    // Only load water if we get a valid type
    if (water_type == WATERTYPE_NONE || renderwater_water_type == water_type)
    {
        return;
    }
    
    // Load Water depending on Type
    switch (water_type)
    {
        case WATERTYPE_WATER_STILL:
            renderwater_load_water_still();
            break;
            
        default:
            renderwater_load_water_still();
            break;
    }
}

void RenderWater::renderwater_load_water_still()
{
    /* Water Surface */
    
    // Surface Line
    //renderwater_water_surface_alpha_min = 5;
    //renderwater_water_surface_alpha_max = 65;
    //renderwater_water_surface_alpha_step = 50;
    
    renderwater_water_surface_line_length = 17;
    
    // Surface Base Color
    //renderwater_water_surface_r = 211;
    //renderwater_water_surface_g = 243;
    //renderwater_water_surface_b = 255;
    renderwater_water_surface_color = RGBA8(211, 243, 255, 65);
    
    /* Water Body */
    
    // Water Body
    renderwater_water_body_alpha_min = 50;
    renderwater_water_body_alpha_max = 80;
    renderwater_water_body_alpha_step = 5;

    // Body Base Color
    renderwater_water_body_r = 98;
    renderwater_water_body_g = 222;
    renderwater_water_body_b = 253;
    
    /* Water Light */
    
    // Set No Light
    renderwater_water_draw_light = false;
    
    /* General */
    
    // Updated current Water Type
    renderwater_water_type = WATERTYPE_WATER_STILL;
    
    // Set Loaded
    renderwater_loaded = true;
}

/*************************************************************************************************************************/
/*                                                    Water Drawing                                                      */
/*************************************************************************************************************************/

void RenderWater::renderwater_draw(u8 water_type, s32 camera_player_x, s32 camera_player_y, s32 player_x, s32 player_y, s32 x, s32 y, s32 w, s32 h)
{
    // (Re)Load Water if our type Changed
    if (renderwater_loaded == false || renderwater_water_type != water_type)
    {
        renderwater_load(water_type);
    }
    
    /***************/
    /* Water Setup */
    /***************/
    
    // Get Camera Position
    s32 water_camera_x = renderwater_get_camera_x(x, camera_player_x, player_x);
    s32 water_camera_y = renderwater_get_camera_y(y, h, camera_player_y, player_y) + h;
    
    // Set Default Variables
    u8 i = 1;
    
    /**********************/
    /* Draw Water Surface */
    /**********************/

    // Draw Water Surface Lines
    sf2d_draw_line(water_camera_x, water_camera_y, water_camera_x + w, water_camera_y, renderwater_water_surface_color);
    
    /*******************/
    /* Draw Water Body */
    /*******************/
    
    // Set Alpha
    u8 a = renderwater_water_body_alpha_min;
    
    // Draw Water Body Beginning
    while (i < (renderwater_water_body_alpha_max - renderwater_water_body_alpha_min) / renderwater_water_body_alpha_step)
    {
        // Draw Water Body Line
        sf2d_draw_line(water_camera_x, water_camera_y + i, water_camera_x + w, water_camera_y + i,
                       RGBA8(renderwater_water_body_r, renderwater_water_body_g, renderwater_water_body_b, a));
        
        // Increment Position
        ++i;
        
        // Update Alpha
        a += renderwater_water_body_alpha_step;
    }
    
    // Reset Alpha (Body Rectangle)
    a = renderwater_water_body_alpha_max * 1.25;
    
    // Draw Body
    sf2d_draw_rectangle(water_camera_x, water_camera_y + i, w, h - i,
                        RGBA8(renderwater_water_body_r, renderwater_water_body_g, renderwater_water_body_b, a));
}

/*************************************************************************************************************************/
/*                                                 Water Coordinates                                                     */
/*************************************************************************************************************************/

s32 RenderWater::renderwater_get_camera_x(s32 x, s32 camera_player_x, s32 player_x)
{
    return camera_player_x + (x - (player_x - 100));
}

s32 RenderWater::renderwater_get_camera_y(s32 y, s32 h, s32 camera_player_y, s32 player_y)
{
    return (camera_player_y + 90) - ((y + h) - player_y);
}