//
//  render_water.hpp
//  3ds_metroid
//
//  Created by jbr373 on 21/03/16.
//

#ifndef render_water_h
#define render_water_h

#include <3ds.h>
#include <stdio.h>

class RenderWater
{
public:
    RenderWater();
    virtual ~RenderWater();
    
    // Draw Water
    void renderwater_draw(u8 water_type, s32 camera_player_x, s32 camera_player_y, s32 player_x, s32 player_y, s32 x, s32 y, s32 w, s32 h);
    
private:
    // Load Water Types
    void renderwater_load(u8 water_type);
    void renderwater_load_water_still();
    
    // Coordinates
    s32 renderwater_get_camera_x(s32 x, s32 camera_player_x, s32 player_x);
    s32 renderwater_get_camera_y(s32 y, s32 h, s32 camera_player_y, s32 player_y);
    
    // Water Loaded
    bool renderwater_loaded;
    
    // Water Temp
    u8 renderwater_counter;
    
    /* Water Data */
    
    u8 renderwater_water_type;
    
    /* Water Surface */
    
    // Surface Line
    //u8 renderwater_water_surface_alpha_min;
    //u8 renderwater_water_surface_alpha_max;
    //u8 renderwater_water_surface_alpha_step;
    
    u8 renderwater_water_surface_line_length; // left to right Alpha Difference (Alpha distracted from current Y line alpha)
    //u8 renderwater_water_surface_line_alpha_min;
    //u8 renderwater_water_surface_line_alpha_max;
    //u8 renderwater_water_surface_line_alpha_step;
    
    // Surface Base Color
    //u8 renderwater_water_surface_r;
    //u8 renderwater_water_surface_g;
    //u8 renderwater_water_surface_b;
    u32 renderwater_water_surface_color;
    
    /* Water Body */
    
    // Water Body
    u8 renderwater_water_body_alpha_min; // Min to max = Line Count of the Water Surface
    u8 renderwater_water_body_alpha_max;
    u8 renderwater_water_body_alpha_step;
    
    // Body Base Color
    u8 renderwater_water_body_r;
    u8 renderwater_water_body_g;
    u8 renderwater_water_body_b;
    
    /* Water Light Lines */
    
    // Water Light Info
    bool renderwater_water_draw_light;
    u8 renderwater_water_light_line_base_width;
    u8 renderwater_water_light_line_base_length;
    u8 renderwater_water_light_line_alpha_difference; // Difference of Line Alpha (L 0 = 0, 1 = Alpha + Difference;
    u8 renderwater_water_light_line_difference; // used with rand() %! DIFFERENCE OF LINE LENGTH
    s8 renderwater_water_light_line_angle; // Y
    u8 renderwater_water_light_line_count; // Light Raus
    s32 renderwater_water_light_x_min; // starting from 0 (left side of the water
    s32 renderwater_water_light_x_max; // x_max - x_min / count = distance between light lines
    
    // Water Light Color
    u8 renderwater_water_light_r;
    u8 renderwater_water_light_g;
    u8 renderwater_water_light_b;
    
    /* Water Waves */
    
    // Water Wave Info
    u8 renderwater_water_wave_size_up; // Displacement from y = 0; (up 2 = 0 - 2)
    u8 renderwater_water_wave_size_down; // Displacement y (bsp 2 - 0 + 2)
    u8 renderwater_water_wave_x_difference; // Each Wave up - down - up - down is seperated by this difference
    u8 renderwater_water_wave_falling_speed; // - of the current Wave Size (Every second count: up - speed, down + speed
    bool renderwater_water_wave_draw_idle; // Draw Ripples when there is no impact
};

#endif /* render_water_h */
