//
//  savefile.cpp
//  3ds_metroid
//
//  Created by jbr373 on 15/01/16.
//

#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <sstream>

#include "savefile.h"
#include "filepaths.h"

SaveFile::SaveFile()
{
    pSaveFileStruct = new SaveFileStruct();
    
    pSaveFileStruct->savefile_health = 99;
    pSaveFileStruct->savefile_healthtanks = 0;
    pSaveFileStruct->savefile_healthtanks_available = 0;
    pSaveFileStruct->savefile_missles_total = 30;
    pSaveFileStruct->savefile_missles_available = 30;
    
    pSaveFileStruct->savefile_room = 1;
    pSaveFileStruct->savefile_x = 625;
    pSaveFileStruct->savefile_y = 27;
    pSaveFileStruct->savefile_phase = 1;
    pSaveFileStruct->savefile_metroids = 39; // 39 with Metroid Queen
    pSaveFileStruct->savefile_metroids_killed_phase = 0;
    
    pSaveFileStruct->savefile_activebeam = BEAM_STANDARD;
    pSaveFileStruct->savefile_icebeam = false;
    pSaveFileStruct->savefile_plasmabeam = false;
    pSaveFileStruct->savefile_wavebeam = false;
    pSaveFileStruct->savefile_spazerbeam = false;
    
    pSaveFileStruct->savefile_highjump = false;
    pSaveFileStruct->savefile_spacejump = false;
    pSaveFileStruct->savefile_screwattack = false;
    
    pSaveFileStruct->savefile_bombs = false;
    pSaveFileStruct->savefile_springball = false;
    pSaveFileStruct->savefile_spiderball = false;
    
    pSaveFileStruct->savefile_barriersuit = false;
    
    pSaveFileStruct->savefile_collected_healthtanks = 0;
    pSaveFileStruct->savefile_collected_missletanks = 0;
    
    pSaveFileStruct->savefile_phase_stages = 0;
    
    pSaveFileStruct->savefile_p0_map = 0;
    pSaveFileStruct->savefile_p1_map = 0;
    pSaveFileStruct->savefile_p2_map = 0;
    pSaveFileStruct->savefile_p3_map = 0;
    pSaveFileStruct->savefile_p4_map = 0;
    pSaveFileStruct->savefile_p5_map = 0;
    pSaveFileStruct->savefile_p6_map = 0;
    pSaveFileStruct->savefile_p7_map = 0;
    pSaveFileStruct->savefile_p8_map = 0;
    pSaveFileStruct->savefile_p9_map = 0;
    pSaveFileStruct->savefile_p10_map = 0;
    
    savefile_is_new = false;
}

SaveFile::~SaveFile()
{
    // Delete File Info Struct
    if (pSaveFileStruct != NULL)
    {
        delete pSaveFileStruct;
        pSaveFileStruct = NULL;
    }
}

void SaveFile::savefile_load(bool is_debug)
{
    if (is_debug == true)
    {
        /* Debug Save File */
        
        pSaveFileStruct->savefile_health = 99;
        pSaveFileStruct->savefile_healthtanks = 0;
        pSaveFileStruct->savefile_healthtanks_available = 0;
        pSaveFileStruct->savefile_missles_total = 30;
        pSaveFileStruct->savefile_missles_available = 30;
        
        pSaveFileStruct->savefile_room = 1;
        pSaveFileStruct->savefile_phase = 1;
        pSaveFileStruct->savefile_x = 625;
        pSaveFileStruct->savefile_y = 27;
        pSaveFileStruct->savefile_metroids = 39;
        pSaveFileStruct->savefile_metroids_killed_phase = 0;
        
        pSaveFileStruct->savefile_activebeam = BEAM_STANDARD;
        pSaveFileStruct->savefile_icebeam = false;
        pSaveFileStruct->savefile_plasmabeam = false;
        pSaveFileStruct->savefile_wavebeam = false;
        pSaveFileStruct->savefile_spazerbeam = false;
        
        pSaveFileStruct->savefile_highjump = false;
        pSaveFileStruct->savefile_spacejump = false;
        pSaveFileStruct->savefile_screwattack = false;
        
        pSaveFileStruct->savefile_bombs = false;
        pSaveFileStruct->savefile_springball = false;
        pSaveFileStruct->savefile_spiderball = false;
        
        pSaveFileStruct->savefile_barriersuit = false;
        
        pSaveFileStruct->savefile_collected_healthtanks = 0;
        pSaveFileStruct->savefile_collected_missletanks = 0;
        
        pSaveFileStruct->savefile_phase_stages = 0;
        
        pSaveFileStruct->savefile_p0_map = 0;
        pSaveFileStruct->savefile_p1_map = 0;
        pSaveFileStruct->savefile_p2_map = 0;
        pSaveFileStruct->savefile_p3_map = 0;
        pSaveFileStruct->savefile_p4_map = 0;
        pSaveFileStruct->savefile_p5_map = 0;
        pSaveFileStruct->savefile_p6_map = 0;
        pSaveFileStruct->savefile_p7_map = 0;
        pSaveFileStruct->savefile_p8_map = 0;
        pSaveFileStruct->savefile_p9_map = 0;
        pSaveFileStruct->savefile_p10_map = 0;
    }
    else
    {
        /* SaveFile on Disk */

        // Open Save File
        std::ifstream savefile;
        savefile.open(FILE_SAVE, std::ios::in | std::ios::binary);
        
        // Read Save File
        if (savefile.is_open())
        {
            // File Pointer Position
            fileop_seekg_resetpos();
            
            // Read Magic & Version
            pSaveFileStruct->savefile_magic = fileop_read_u32(savefile);
            fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_magic));

            pSaveFileStruct->savefile_version = fileop_read_u32(savefile);
            fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_version));
            
            // Create New File if Magic or Version aren't matching
            if (pSaveFileStruct->savefile_magic != MAGIC_DECIMAL_SAVE || pSaveFileStruct->savefile_version != VERSION_DECIMAL_SAVE)
            {
                // Close Save File
                savefile.close();
                
                // Create New Save
                savefile_new();
            }
            else
            {
                // Get Player Health
                pSaveFileStruct->savefile_health = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_health));
                
                pSaveFileStruct->savefile_healthtanks = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_healthtanks));
                
                pSaveFileStruct->savefile_healthtanks_available = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_healthtanks_available));

                // Get Player Missles
                
                pSaveFileStruct->savefile_missles_total = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_missles_total));
                
                pSaveFileStruct->savefile_missles_available = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_missles_available));
                
                // Get Player Active Beam
                
                pSaveFileStruct->savefile_activebeam = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_activebeam));

                // Get Player Room Position
                
                pSaveFileStruct->savefile_room = fileop_read_u16(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_room));
                
                pSaveFileStruct->savefile_x = fileop_read_s32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_x));
                
                pSaveFileStruct->savefile_y = fileop_read_s32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_y));
                
                // Get Player Story Info
                
                pSaveFileStruct->savefile_phase = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_phase));
                
                pSaveFileStruct->savefile_metroids = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_metroids));
                
                pSaveFileStruct->savefile_metroids_killed_phase = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_metroids_killed_phase));
                
                // Get Player Beams
                
                pSaveFileStruct->savefile_icebeam = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_icebeam));
                
                pSaveFileStruct->savefile_wavebeam = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_wavebeam));
                
                pSaveFileStruct->savefile_spazerbeam = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_spazerbeam));
                
                pSaveFileStruct->savefile_plasmabeam = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_plasmabeam));
                
                // Get Player Jump
                
                pSaveFileStruct->savefile_highjump = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_highjump));
                
                pSaveFileStruct->savefile_spacejump = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_spacejump));
                
                pSaveFileStruct->savefile_screwattack = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_screwattack));
                
                // Get Player Morphball
                
                pSaveFileStruct->savefile_bombs = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_bombs));
                
                pSaveFileStruct->savefile_springball = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_springball));
                
                pSaveFileStruct->savefile_spiderball = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_spiderball));
                
                // Get Player Suit
                
                pSaveFileStruct->savefile_barriersuit = fileop_read_u8(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_barriersuit));
                
                // Get Player Collected Items
                
                pSaveFileStruct->savefile_collected_healthtanks = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_collected_healthtanks));
                
                pSaveFileStruct->savefile_collected_missletanks = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_collected_missletanks));
                
                // Get Player Map Phase
                
                pSaveFileStruct->savefile_phase_stages = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_phase_stages));
                
                // Get Player Minimap Explored
                
                pSaveFileStruct->savefile_p0_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p0_map));
                
                pSaveFileStruct->savefile_p1_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p1_map));
                
                pSaveFileStruct->savefile_p2_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p2_map));
                
                pSaveFileStruct->savefile_p3_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p3_map));
                
                pSaveFileStruct->savefile_p4_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p4_map));
                
                pSaveFileStruct->savefile_p5_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p5_map));
                
                pSaveFileStruct->savefile_p6_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p6_map));
                
                pSaveFileStruct->savefile_p7_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p7_map));
                
                pSaveFileStruct->savefile_p8_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p8_map));
                
                pSaveFileStruct->savefile_p9_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p9_map));
                
                pSaveFileStruct->savefile_p10_map = fileop_read_u32(savefile);
                fileop_seekg_file(savefile, sizeof(pSaveFileStruct->savefile_p10_map));
                
                // Close Save File
                savefile.close();
                
            }
            
            // File Pointer Reset
            fileop_seekg_resetpos();
        }
        else
        {
             // Slot isn't loadable, create new Save
            savefile_new();
            
            // Flag as new save
            savefile_is_new = true;
        }
    }
}

void SaveFile::savefile_new()
{
    /* New Save */
    
    // Reset all Values
    pSaveFileStruct->savefile_health = 99;
    pSaveFileStruct->savefile_healthtanks = 0;
    pSaveFileStruct->savefile_healthtanks_available = 0;
    pSaveFileStruct->savefile_missles_total = 30;
    pSaveFileStruct->savefile_missles_available = 30;
    
    pSaveFileStruct->savefile_room = 1;
    pSaveFileStruct->savefile_phase = 1;
    pSaveFileStruct->savefile_x = 625;
    pSaveFileStruct->savefile_y = 27;
    pSaveFileStruct->savefile_metroids = 39;
    pSaveFileStruct->savefile_metroids_killed_phase = 0;
    
    pSaveFileStruct->savefile_activebeam = BEAM_STANDARD;
    pSaveFileStruct->savefile_icebeam = false;
    pSaveFileStruct->savefile_plasmabeam = false;
    pSaveFileStruct->savefile_wavebeam = false;
    pSaveFileStruct->savefile_spazerbeam = false;
    
    pSaveFileStruct->savefile_highjump = false;
    pSaveFileStruct->savefile_spacejump = false;
    pSaveFileStruct->savefile_screwattack = false;
    
    pSaveFileStruct->savefile_bombs = false;
    pSaveFileStruct->savefile_springball = false;
    pSaveFileStruct->savefile_spiderball = false;
    
    pSaveFileStruct->savefile_barriersuit = false;
    
    pSaveFileStruct->savefile_collected_healthtanks = 0;
    pSaveFileStruct->savefile_collected_missletanks = 0;
    
    pSaveFileStruct->savefile_phase_stages = 0;
    
    pSaveFileStruct->savefile_p0_map = 0;
    pSaveFileStruct->savefile_p1_map = 0;
    pSaveFileStruct->savefile_p2_map = 0;
    pSaveFileStruct->savefile_p3_map = 0;
    pSaveFileStruct->savefile_p4_map = 0;
    pSaveFileStruct->savefile_p5_map = 0;
    pSaveFileStruct->savefile_p6_map = 0;
    pSaveFileStruct->savefile_p7_map = 0;
    pSaveFileStruct->savefile_p8_map = 0;
    pSaveFileStruct->savefile_p9_map = 0;
    pSaveFileStruct->savefile_p10_map = 0;
    
    // Save new File
    savefile_save();
}

void SaveFile::savefile_save()
{
    /* Save Slot */
    
    // Read Save File
    std::ofstream savefile;
    savefile.open(FILE_SAVE, std::ios::out | std::ios::binary);
    //savefile << std::flush;
    
    // Read Save File
    if (savefile.is_open())
    {
        // Read Magic & Version
        fileop_write_u32(savefile, MAGIC_DECIMAL_SAVE);
        fileop_write_u32(savefile, VERSION_DECIMAL_SAVE);
        
        // Write Player Health
        fileop_write_u8(savefile, pSaveFileStruct->savefile_health);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_healthtanks);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_healthtanks_available);
        
        // Write Player Missles
        fileop_write_u8(savefile, pSaveFileStruct->savefile_missles_total);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_missles_available);
        
        // Write Player Active Beam
        fileop_write_u8(savefile, pSaveFileStruct->savefile_activebeam);
        
        // Write Player Map Position
        fileop_write_u16(savefile, pSaveFileStruct->savefile_room);
        fileop_write_s32(savefile, pSaveFileStruct->savefile_x);
        fileop_write_s32(savefile, pSaveFileStruct->savefile_y);
        
        // Write Player Story Progression
        fileop_write_u8(savefile, pSaveFileStruct->savefile_phase);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_metroids);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_metroids_killed_phase);
        
        // Write Player Beams
        fileop_write_u8(savefile, pSaveFileStruct->savefile_icebeam);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_wavebeam);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_spazerbeam);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_plasmabeam);
        
        // Write Player Jump
        fileop_write_u8(savefile, pSaveFileStruct->savefile_highjump);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_spacejump);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_screwattack);
        
        // Write Player Morphball
        fileop_write_u8(savefile, pSaveFileStruct->savefile_bombs);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_springball);
        fileop_write_u8(savefile, pSaveFileStruct->savefile_spiderball);
        
        // Write Player Suit
        fileop_write_u8(savefile, pSaveFileStruct->savefile_barriersuit);
        
        // Write Player Collected Items
        fileop_write_u32(savefile, pSaveFileStruct->savefile_collected_healthtanks);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_collected_missletanks);
        
        // Write Player Stage Phase
        
        fileop_write_u32(savefile, pSaveFileStruct->savefile_phase_stages);

        // Write Player Explored Minimap
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p0_map);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p1_map);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p2_map);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p3_map);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p4_map);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p5_map);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p6_map);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p7_map);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p8_map);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p9_map);
        fileop_write_u32(savefile, pSaveFileStruct->savefile_p10_map);
        
        // Close Save File
        savefile.close();
    }
}

void SaveFile::savefile_set_minimap_flag(u32 flag)
{
    switch (pSaveFileStruct->savefile_phase_stages)
    {
        case 0:
            pSaveFileStruct->savefile_p0_map = pSaveFileStruct->savefile_p0_map + flag;
            break;
        case 1:
            pSaveFileStruct->savefile_p1_map = pSaveFileStruct->savefile_p1_map + flag;
            break;
        case 2:
            pSaveFileStruct->savefile_p2_map = pSaveFileStruct->savefile_p2_map + flag;
            break;
        case 3:
            pSaveFileStruct->savefile_p3_map = pSaveFileStruct->savefile_p3_map + flag;
            break;
        case 4:
            pSaveFileStruct->savefile_p4_map = pSaveFileStruct->savefile_p4_map + flag;
            break;
        case 5:
            pSaveFileStruct->savefile_p5_map = pSaveFileStruct->savefile_p5_map + flag;
            break;
        case 6:
            pSaveFileStruct->savefile_p6_map = pSaveFileStruct->savefile_p6_map + flag;
            break;
        case 7:
            pSaveFileStruct->savefile_p7_map = pSaveFileStruct->savefile_p7_map + flag;
            break;
        case 8:
            pSaveFileStruct->savefile_p8_map = pSaveFileStruct->savefile_p8_map + flag;
            break;
        case 9:
            pSaveFileStruct->savefile_p9_map = pSaveFileStruct->savefile_p9_map + flag;
            break;
        case 10:
            pSaveFileStruct->savefile_p10_map = pSaveFileStruct->savefile_p10_map + flag;
            break;
            
        default:
            break;
    }
}

u32 SaveFile::savefile_get_minimap_flag()
{
    switch (pSaveFileStruct->savefile_phase_stages)
    {
        case 0:
            return pSaveFileStruct->savefile_p0_map;
            break;
        case 1:
            return pSaveFileStruct->savefile_p1_map;
            break;
        case 2:
            return pSaveFileStruct->savefile_p2_map;
            break;
        case 3:
            return pSaveFileStruct->savefile_p3_map;
            break;
        case 4:
            return pSaveFileStruct->savefile_p4_map;
            break;
        case 5:
            return pSaveFileStruct->savefile_p5_map;
            break;
        case 6:
            return pSaveFileStruct->savefile_p6_map;
            break;
        case 7:
            return pSaveFileStruct->savefile_p7_map;
            break;
        case 8:
            return pSaveFileStruct->savefile_p8_map;
            break;
        case 9:
            return pSaveFileStruct->savefile_p9_map;
            break;
        case 10:
            return pSaveFileStruct->savefile_p10_map;
            break;
            
        default:
            return 0;
            break;
    }
}
