//
//  sound.cpp
//  3ds_metroid
//
//  Created by jbr373 on 26/01/16.
//

#include <stdlib.h>
#include <fstream>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <unistd.h>

#include "sound.h"
#include "filepaths.h"

#define MAX_CHANNELS_DSP        4

#define MAX_LOAD_AT_START       15 // Player Sounds are loaded at start, enemy sounds at stage loading

Sound::Sound()
{
    sound_enabled = true;

    fp = NULL;
    data = NULL;
    
    for (u8 i = 0; i < MAX_SOUND_FX; ++i)
    {
        pSound_FX[i] = NULL;
    }
}

Sound::~Sound()
{
}

/*************************************************************************************************************************/
/*                                                     Load Sound                                                        */
/*************************************************************************************************************************/

void Sound::sound_load_all()
{
    // Stop if sound is disabled
    if (sound_enabled == false)
    {
        return;
    }
    
    // Load all pre-defined Sounds
    for (u8 i = 0; i <= MAX_LOAD_AT_START; ++i)
    {
        // Load our pre-defined Sound File depending on ID
        switch (i)
        {
            case SOUND_PLAYER_RUN:
                sound_load(SND_PLR_RUN, i);
                break;
            case SOUND_PLAYER_SHOOT:
                sound_load(SND_PLR_SHOOT, i);
                break;
            case SOUND_PLAYER_CHARGE_START:
                sound_load(SND_PLR_CHARGE_START, i);
                break;
            case SOUND_PLAYER_CHARGE_ONGOING:
                sound_load(SND_PLR_CHARGE_ONGOING, i);
                break;
            case SOUND_PLAYER_CHARGE_SHOOT:
                sound_load(SND_PLR_CHARGE_SHOOT, i);
                break;
            case SOUND_PLAYER_JUMP_SPIN:
                sound_load(SND_PLR_JUMP_SPIN, i);
                break;
            case SOUND_PLAYER_MORPHBALL:
                sound_load(SND_PLR_MORPHBALL, i);
                break;
            case SOUND_PLAYER_STANDINGMOVE:
                sound_load(SND_PLR_STANDINGMOVE, i);
                break;
            case SOUND_PLAYER_DAMAGE:
                sound_load(SND_PLR_DAMAGE, i);
                break;
            case SOUND_PLAYER_EXPLOSION:
                sound_load(SND_PLR_EXPLOSION, i);
                break;
            case SOUND_PLAYER_UPGRADE_JINGLE:
                sound_load(SND_PLR_UPGRADE_JINGLE, i);
                break;
            case SOUND_PLAYER_RESTORE_JINGLE:
                sound_load(SND_PLR_RESTORE_JINGLE, i);
                break;
            case SOUND_PLAYER_LEDGE:
                sound_load(SND_PLR_LEDGE_GRAD, i);
                break;
            case SOUND_PLAYER_HEALTH_LOW:
                sound_load(SND_PLR_HEALTH_LOW, i);
                break;
            case SOUND_PLAYER_DEATH:
                sound_load(SND_PLR_DEATH, i);
                break;
            case SOUND_PLAYER_WADE:
                sound_load(SND_PLR_WADE, i);
                break;
                
            default:
                return;
                break;
        }
    }
}

void Sound::sound_load(const char* path, u8 sound_id)
{
    // Stop Loading
    if (pSound_FX[sound_id] != NULL || sound_id >= MAX_SOUND_FX)
    {
        return;
    }
    
    /* Load Sound Data to Memory */
    
    // Create Sound Struct
    pSound_FX[sound_id] = new SoundData;
    fp = NULL;
    pSound_FX[sound_id]->data = NULL;
    
    // Assign Sound Pointer
    fp = fopen(path, "rb");
    
    if(!fp)
    {
        return;
    }
    
    // Read Signature of File
    char signature[4];
    
    fread(signature, 1, 4, fp);
    
    if (signature[0] != 'R' && signature[1] != 'I' && signature[2] != 'F' && signature[3] != 'F')
    {
        fclose(fp);
        return;
    }
    
    // Read Soundfile Header
    fseek(fp,0,SEEK_END);
    pSound_FX[sound_id]->dataSize = ftell(fp);
    fseek(fp, 22, SEEK_SET);
    fread(&pSound_FX[sound_id]->channels, 2, 1, fp);
    fseek(fp, 24, SEEK_SET);
    fread(&pSound_FX[sound_id]->sampleRate, 4, 1, fp);
    fseek(fp, 34, SEEK_SET);
    fread(&pSound_FX[sound_id]->bitsPerSample, 2, 1, fp);
    
    // Check if our Data is valid
    if (pSound_FX[sound_id]->dataSize == 0 ||
        (pSound_FX[sound_id]->channels != 1 && pSound_FX[sound_id]->channels != 2) ||
        (pSound_FX[sound_id]->bitsPerSample != 8 && pSound_FX[sound_id]->bitsPerSample != 16))
    {
        fclose(fp);
        return;
    }
    
    // Allocating and reading samples
    pSound_FX[sound_id]->data = (u8*)linearAlloc(pSound_FX[sound_id]->dataSize);
    
    // Do we have any data?
    if(!pSound_FX[sound_id]->data)
    {
        fclose(fp);
        return;
    }
    
    // Read for Buffer
    fseek(fp, 44, SEEK_SET);
    fread(pSound_FX[sound_id]->data, 1, pSound_FX[sound_id]->dataSize, fp);
    fclose(fp);
    
    // Get Output for each Sample Rate
    if (pSound_FX[sound_id]->bitsPerSample == 8)
    {
        pSound_FX[sound_id]->ndspFormat = (pSound_FX[sound_id]->channels == 1) ?
        NDSP_FORMAT_MONO_PCM8 :
        NDSP_FORMAT_STEREO_PCM8;
    }
    else
    {
        pSound_FX[sound_id]->ndspFormat = (pSound_FX[sound_id]->channels == 1) ?
        NDSP_FORMAT_MONO_PCM16 :
        NDSP_FORMAT_STEREO_PCM16;
    }
    
    // Flag Sound File as loaded
    pSound_FX[sound_id]->is_loaded = true;
}

/*************************************************************************************************************************/
/*                                                     Play Sound                                                        */
/*************************************************************************************************************************/

void Sound::sound_play_bgm(std::string path, bool loop, bool force)
{
    /* Check Sound Settings */
    
    // Check if we're allowing Sound
    if (sound_enabled == false)
    {
        return;
    }
    
    /* Get Sound Channel */
    
    sound_channel = sound_get_channel(SOUNDTYPE_BGM);
    
    /* Sound Check */
    
    // Check for valid Sound Channel and Sound playing!
    if (sound_channel != 0)
    {
        // Check for Sound playback
        if (ndspChnIsPlaying(sound_channel) == true)
        {
            if (force == true)
            {
                // Force Sound Stop
                sound_stop(SOUNDTYPE_BGM);
            }
            else
            {
                // Sound playing, no force overwrite, return
                return;
            }
        }
    }
    else
    {
        // Invalid Channel, return
        return;
    }
    
    /* Sound File */
    
    // Assign Sound Pointer
    fp = fopen(path.c_str(), "rb");
    
    if(!fp)
    {
        return;
    }
    
    // Read Signature of File
    char signature[4];
    
    fread(signature, 1, 4, fp);
    
    if (signature[0] != 'R' && signature[1] != 'I' && signature[2] != 'F' && signature[3] != 'F')
    {
        fclose(fp);
        return;
    }
    
    // Read Soundfile Header
    fseek(fp,0,SEEK_END);
    dataSize = ftell(fp);
    fseek(fp, 22, SEEK_SET);
    fread(&channels, 2, 1, fp);
    fseek(fp, 24, SEEK_SET);
    fread(&sampleRate, 4, 1, fp);
    fseek(fp, 34, SEEK_SET);
    fread(&bitsPerSample, 2, 1, fp);
    
    // Check if our Data is valid
    if (dataSize == 0 || (channels != 1 && channels != 2) || (bitsPerSample != 8 && bitsPerSample != 16))
    {
        fclose(fp);
        return;
    }
    
    // Allocating and reading samples
    data = (u8*)linearAlloc(dataSize);
    
    // Do we have any data?
    if(!data)
    {
        fclose(fp);
        return;
    }
    
    // Read for Buffer
    fseek(fp, 44, SEEK_SET);
    fread(data, 1, dataSize, fp);
    fclose(fp);
    
    // Get Output for each Sample Rate
    if (bitsPerSample == 8)
    {
        ndspFormat = (channels == 1) ?
        NDSP_FORMAT_MONO_PCM8 :
        NDSP_FORMAT_STEREO_PCM8;
    }
    else
    {
        ndspFormat = (channels == 1) ?
        NDSP_FORMAT_MONO_PCM16 :
        NDSP_FORMAT_STEREO_PCM16;
    }
    
    /* Ready DSP */
    
    ndspChnWaveBufClear(sound_channel);
    
    // Reset DSP and set new values
    ndspChnReset(sound_channel);
    ndspChnSetInterp(sound_channel, NDSP_INTERP_NONE);
    ndspChnSetRate(sound_channel, (float)sampleRate);
    ndspChnSetFormat(sound_channel, ndspFormat);
    
    /* Sound Buffer */
    
    // Create Wave buffer
    memset(&waveBuf_bgm, 0, sizeof(ndspWaveBuf));
    waveBuf_bgm.data_vaddr = (const void*)data;
    waveBuf_bgm.nsamples = dataSize / (bitsPerSample >> 3);
    waveBuf_bgm.looping = loop; // Loop enabled
    waveBuf_bgm.status = NDSP_WBUF_FREE;
    
    // Flush Data
    DSP_FlushDataCache(data, dataSize);
    
    // Add Wave Buffer to Queue
    ndspChnWaveBufAdd(sound_channel, &waveBuf_bgm);
}

void Sound::sound_play_fx(SoundIDs sound_id, SoundTypes sound_type, bool loop, bool force)
{
    /* Check Sound Settings */
    
    // Check if we're allowing Sound
    if (sound_enabled == false)
    {
        return;
    }
    
    /* Get Sound Channel */
    
    pSound_FX[sound_id]->sound_channel = sound_get_channel(sound_type);
    
    /* Sound Check */
    
    // Check for valid Sound Channel and Sound playing!
    if (pSound_FX[sound_id]->sound_channel != 0)
    {
        // Check for Sound playback
        if (ndspChnIsPlaying(pSound_FX[sound_id]->sound_channel) == true)
        {
            if (force == true)
            {
                // Force Sound Stop
                sound_stop(sound_type);
            }
            else
            {
                // Sound playing, no force overwrite, return
                return;
            }
        }
    }
    else
    {
        // Invalid Channel, return
        return;
    }
    
    /* Sound File */

    /* Ready DSP */
    
    // Reset DSP and set new values
    
    ndspChnReset(pSound_FX[sound_id]->sound_channel);
    ndspChnSetInterp(pSound_FX[sound_id]->sound_channel, NDSP_INTERP_NONE);
    ndspChnSetRate(pSound_FX[sound_id]->sound_channel, (float)pSound_FX[sound_id]->sampleRate);
    ndspChnSetFormat(pSound_FX[sound_id]->sound_channel, pSound_FX[sound_id]->ndspFormat);
    
    /* Sound Buffer */
    
    // Create Wave buffer
    switch (sound_type)
    {
        case SOUNDTYPE_PLAYER_SHOOTING:
            memset(&waveBuf_plr_shoot, 0, sizeof(ndspWaveBuf));
            waveBuf_plr_shoot.data_vaddr = (const void*)pSound_FX[sound_id]->data;
            waveBuf_plr_shoot.nsamples = pSound_FX[sound_id]->dataSize / (pSound_FX[sound_id]->bitsPerSample >> 3);
            waveBuf_plr_shoot.looping = loop; // Loop enabled
            waveBuf_plr_shoot.status = NDSP_WBUF_FREE;
            break;
        case SOUNDTYPE_PLAYER_MOVING:
            memset(&waveBuf_plr_move, 0, sizeof(ndspWaveBuf));
            waveBuf_plr_move.data_vaddr = (const void*)pSound_FX[sound_id]->data;
            waveBuf_plr_move.nsamples = pSound_FX[sound_id]->dataSize / (pSound_FX[sound_id]->bitsPerSample >> 3);
            waveBuf_plr_move.looping = loop; // Loop enabled
            waveBuf_plr_move.status = NDSP_WBUF_FREE;
            break;
        case SOUNDTYPE_ENEMY:
            memset(&waveBuf_enemy, 0, sizeof(ndspWaveBuf));
            waveBuf_enemy.data_vaddr = (const void*)pSound_FX[sound_id]->data;
            waveBuf_enemy.nsamples = pSound_FX[sound_id]->dataSize / (pSound_FX[sound_id]->bitsPerSample >> 3);
            waveBuf_enemy.looping = loop; // Loop enabled
            waveBuf_enemy.status = NDSP_WBUF_FREE;
            break;
            
        default:
            memset(&waveBuf_plr_shoot, 0, sizeof(ndspWaveBuf));
            waveBuf_plr_shoot.data_vaddr = (const void*)pSound_FX[sound_id]->data;
            waveBuf_plr_shoot.nsamples = pSound_FX[sound_id]->dataSize / (pSound_FX[sound_id]->bitsPerSample >> 3);
            waveBuf_plr_shoot.looping = loop; // Loop enabled
            waveBuf_plr_shoot.status = NDSP_WBUF_FREE;
            break;
    }
    
    // Flush Data
    DSP_FlushDataCache(pSound_FX[sound_id]->data, pSound_FX[sound_id]->dataSize);
    
    // Add Wave Buffer to Queue
    switch (sound_type)
    {
        case SOUNDTYPE_PLAYER_SHOOTING:
            ndspChnWaveBufAdd(pSound_FX[sound_id]->sound_channel, &waveBuf_plr_shoot);
            break;
        case SOUNDTYPE_PLAYER_MOVING:
            ndspChnWaveBufAdd(pSound_FX[sound_id]->sound_channel, &waveBuf_plr_move);
            break;
        case SOUNDTYPE_ENEMY:
            ndspChnWaveBufAdd(pSound_FX[sound_id]->sound_channel, &waveBuf_enemy);
            break;
            
        default:
            ndspChnWaveBufAdd(pSound_FX[sound_id]->sound_channel, &waveBuf_plr_shoot);
            break;
    }
}

/*************************************************************************************************************************/
/*                                                      Stop Audio                                                       */
/*************************************************************************************************************************/

void Sound::sound_stop(SoundTypes sound_type)
{
    /* Check Sound Settings */
    
    // Check if we're allowing Sound
    if (sound_enabled == false)
    {
        return;
    }
    
    /* Stop Sound from Channel */
    
    int channel = sound_get_channel(sound_type);
    
    if (channel == 0)
        return;
    
    if (ndspChnIsPlaying(channel) == true)
    {
        ndspChnWaveBufClear(channel);
    }
}

void Sound::sound_stop_all()
{
    /* Check Sound Settings */
    
    // Check if we're allowing Sound
    if (sound_enabled == false)
    {
        return;
    }
    
    /* Stop all Sounds */
    
    // Do for all channels
    for (u8 i = 0; i <= MAX_CHANNELS_DSP; ++i)
    {
        // Stop each Channel
        if (ndspChnIsPlaying(i) == true)
        {
            ndspChnWaveBufClear(i);
        }
    }
}

/*************************************************************************************************************************/
/*                                                   Get Sound Info                                                      */
/*************************************************************************************************************************/

int Sound::sound_get_channel(SoundTypes sound_type)
{
    // Get Channel for Sound Type
    switch (sound_type)
    {
        case SOUNDTYPE_BGM:
            return 1;
            break;
        case SOUNDTYPE_PLAYER_SHOOTING:
            return 2;
            break;
        case SOUNDTYPE_PLAYER_MOVING:
            return 3;
            break;
        case SOUNDTYPE_ENEMY:
            return 4;
            break;
            
        default:
            return 0; // Error
            break;
    }
}

bool Sound::sound_is_playing(SoundTypes sound_type)
{
    /* Check Sound Settings */
    
    // Check if we're allowing Sound
    if (sound_enabled == false)
    {
        return false;
    }
    
    /* Get Channel Info */
    
    // Get Channel
    int channel = sound_get_channel(sound_type);
    
    // Return Result
    return ndspChnIsPlaying(channel);
}
