//
//  sound.h
//  3ds_metroid
//
//  Created by jbr373 on 26/01/16.
//

#ifndef sound_h
#define sound_h

#include <3ds.h>
#include <stdio.h>

#define MAX_SOUND_FX        21

enum SoundIDs
{
    SOUND_PLAYER_RUN = 0,
    SOUND_PLAYER_SHOOT = 1,
    SOUND_PLAYER_CHARGE_START = 2,
    SOUND_PLAYER_CHARGE_ONGOING = 3,
    SOUND_PLAYER_CHARGE_SHOOT = 4,
    SOUND_PLAYER_JUMP_SPIN = 5,
    SOUND_PLAYER_MORPHBALL = 6,
    SOUND_PLAYER_STANDINGMOVE = 7,
    SOUND_PLAYER_DAMAGE = 8,
    SOUND_PLAYER_EXPLOSION = 9,
    SOUND_PLAYER_UPGRADE_JINGLE = 10,
    SOUND_PLAYER_RESTORE_JINGLE = 11,
    SOUND_PLAYER_LEDGE = 12,
    SOUND_PLAYER_HEALTH_LOW = 13,
    SOUND_PLAYER_DEATH = 14,
    SOUND_PLAYER_WADE = 15,
    
    SOUND_ENEMY_CRY_WALKING_A = 16,
    SOUND_ENEMY_CRY_WALKING_B = 17,
    SOUND_ENEMY_CRY_FLYING_A = 18,
    SOUND_ENEMY_CRY_FLYING_B = 19,
    SOUND_ENEMY_CRY_BOSS_A = 20,
};

enum SoundTypes
{
    SOUNDTYPE_BGM = 1,
    SOUNDTYPE_PLAYER_SHOOTING = 2,
    SOUNDTYPE_PLAYER_MOVING = 3,
    SOUNDTYPE_ENEMY = 4,
};

struct SoundData
{
    // Load Flag
    bool is_loaded;
    
    // DSP Channel
    int sound_channel;

    // Sound Info
    u16 channels;
    u16 bitsPerSample;
    u16 ndspFormat;
    u32 sampleRate;
    
    // Sound Data
    u32 dataSize;
    u8* data;
};

class Sound
{
public:
    Sound();
    virtual ~Sound();
    
    void sound_load_all();
    void sound_load(const char* path, u8 sound_id);
    
    void sound_play_bgm(std::string path, bool loop, bool force);
    void sound_play_fx(SoundIDs sound_id, SoundTypes sound_type, bool loop, bool force);
    
    void sound_stop(SoundTypes sound_type);
    void sound_stop_all();
    
    // Setters
    void sound_set_enabled(bool enabled) { sound_enabled = enabled; }
    
    // Getters
    bool sound_is_playing(SoundTypes sound_type);
    bool sound_is_enabled() { return sound_enabled; };
    
private:
    int sound_get_channel(SoundTypes sound_type);
    
    bool sound_enabled;
    
    int sound_channel;
    u32 sampleRate;
    u32 dataSize;
    u16 channels;
    u16 bitsPerSample;
    u16 ndspFormat;
    ndspWaveBuf waveBuf_bgm;
    ndspWaveBuf waveBuf_plr_move;
    ndspWaveBuf waveBuf_plr_shoot;
    ndspWaveBuf waveBuf_enemy;
    u8* data;
    FILE* fp;
    
    SoundData* pSound_FX[MAX_SOUND_FX];
};

#endif /* sound_h */
