//
//  sprite_object.cpp
//  3ds_metroid
//
//  Created by jbr373 on 17/02/16.
//

#include <sfil.h>
#include <string.h>

#include "sprite_actor.h"
#include "filepaths.h"

#define MAX_OBJECT_SPRITE_FRAMES                    7

ActorSprite::ActorSprite()
{
    // Set Values
    actorsprite_filepath = "NONE";
    
    actorsprite_last_sprite_line = 0;
    actorsprite_last_min_frame = 0;
    actorsprite_last_max_frame = 0;
    actorsprite_max_sprite_line = 0;
    
    actorsprite_increment_frame = true;
    
    pActorSprite = NULL;
}

ActorSprite::~ActorSprite()
{
    // Free Texture
    actorsprite_delete();
}

void ActorSprite::actorsprite_delete()
{
    if (pActorSprite != NULL)
    {
        if (strcmp(actorsprite_filepath.c_str(), LINE_DO_NOT_USE) != 0)
        {
            if (pActorSprite->spritesheet_texture != NULL)
            {
                sf2d_free_texture(pActorSprite->spritesheet_texture);
            }
        
            delete pActorSprite;
            pActorSprite = NULL;
        }
    }
}

void ActorSprite::actorsprite_load(std::string filepath, u8 sprite_lines, s32 sprite_w_full, s32 sprite_h_full, u8 sprite_framespeed)
{
    // Load Spritesheet
    if (strcmp(filepath.c_str(), LINE_DO_NOT_USE) != 0)
    {
        // Create Struct
        pActorSprite = new SpriteSheet();
        
        // Texture Info for Render
        pActorSprite->spritesheet_texture = sfil_load_PNG_file(filepath.c_str(), SF2D_PLACE_RAM);
        pActorSprite->spritesheet_sprite_line = 0;
        pActorSprite->spritesheet_startframe = 0;
        pActorSprite->spritesheet_endframe = MAX_OBJECT_SPRITE_FRAMES;
        pActorSprite->spritesheet_frame = 0;
        pActorSprite->spritesheet_h_single = sprite_h_full / (sprite_lines + 1); // 1 Sprite Line (Idle) = 0, +1 fixes this
        pActorSprite->spritesheet_w_single = sprite_w_full / (MAX_OBJECT_SPRITE_FRAMES + 1); // 8 Frames total, 0-7 Counter, 7+1 = 8
        pActorSprite->spritesheet_frame_speed_current = 0;
        pActorSprite->spritesheet_frame_speed = sprite_framespeed;
        pActorSprite->spritesheet_w_full = sprite_w_full;
        pActorSprite->spritesheet_h_full = sprite_h_full;
        actorsprite_max_sprite_line = sprite_lines;
        
        if (pActorSprite->spritesheet_texture == NULL)
        {
            filepath = LINE_DO_NOT_USE;
        }
        else
        {
            actorsprite_filepath = filepath;
        }
    }
}

void ActorSprite::actorsprite_set_frame(u8 frame)
{
    if (pActorSprite != NULL || frame <= MAX_OBJECT_SPRITE_FRAMES)
        pActorSprite->spritesheet_frame = frame;
}

SpriteSheet* ActorSprite::actorsprite_get_sprite(u8 sprite_line, u8 frame_min, u8 frame_max, u8 frame_current)
{
    // Check for a wrong Sprite Line
    if (sprite_line > actorsprite_max_sprite_line)
    {
        return pActorSprite;
    }

    // Write last used Info
    actorsprite_last_sprite_line = sprite_line;
    actorsprite_last_min_frame = frame_min;
    actorsprite_last_max_frame = frame_max;

    // Set Frames
    pActorSprite->spritesheet_frame = frame_current;
    pActorSprite->spritesheet_startframe = frame_min;
    pActorSprite->spritesheet_endframe = frame_max;
    
    // Set Spritesheet Info
    pActorSprite->spritesheet_sprite_line = sprite_line;
    
    // Return Sprite
    return pActorSprite;
}