//
//  sprite_player.cpp
//  3ds_metroid
//
//  Created by jbr373 on 22/01/16.
//

#include <sfil.h>
#include <stdlib.h>

#include "sprite_player.h"
#include "filepaths.h"
#include "actor_facing.h"

#include "player_spritesheet_running.h"
#include "player_spritesheet_standing.h"

#define PLAYER_SPRITEFRAME_W                    100
#define PLAYER_SPRITESHEET_W                    800
#define PLAYER_SPRITEFRAME_H                    90
#define PLAYER_SPRITESHEET_H_STANDING           630
#define PLAYER_SPRITESHEET_H_RUNNING            900

#define PLAYER_FRAME_SPEED                      1

#define STANDING_FRAME_AIM_RIGHT                0
#define STANDING_FRAME_AIM_RIGHT_DOWN           1
#define STANDING_FRAME_AIM_RIGHT_UP             2
#define STANDING_FRAME_AIM_UP_RIGHT             3
#define STANDING_FRAME_AIM_LEFT                 4
#define STANDING_FRAME_AIM_LEFT_DOWN            5
#define STANDING_FRAME_AIM_LEFT_UP              6
#define STANDING_FRAME_AIM_UP_LEFT              7

#define RUNNING_FRAME_RUNAIM_FULL               7

#define RUNNING_FRAME_JUMP_RIGHT_START          0
#define RUNNING_FRAME_JUMP_RIGHT_END            3
#define RUNNING_FRAME_JUMP_LEFT_START           4
#define RUNNING_FRAME_JUMP_LEFT_END             7

#define STANDING_FRAME_CROUCHED_RIGHT           0
#define STANDING_FRAME_CROUCHED_RIGHT_UP        1
#define STANDING_FRAME_CROUCHED_UP_RIGHT        2
#define STANDING_FRAME_CROUCHED_RIGHT_DOWN      3
#define STANDING_FRAME_CROUCHED_LEFT            4
#define STANDING_FRAME_CROUCHED_LEFT_UP         5
#define STANDING_FRAME_CROUCHED_UP_LEFT         6
#define STANDING_FRAME_CROUCHED_LEFT_DOWN       7

#define RUNNING_FRAME_JUMP90_RIGHT              2
#define RUNNING_FRAME_JUMP90_UP_RIGHT           1
#define RUNNING_FRAME_JUMP90_DOWN_RIGHT         0
#define RUNNING_FRAME_JUMP90_LEFT               5
#define RUNNING_FRAME_JUMP90_UP_LEFT            4
#define RUNNING_FRAME_JUMP90_DOWN_LEFT          3

#define STANDING_FRAME_HANGING_DOWN_RIGHT       0
#define STANDING_FRAME_HANGING_RIGHT            1
#define STANDING_FRAME_HANGING_UP_RIGHT         2
#define STANDING_FRAME_HANGING_RIGHT_CLIMB      3
#define STANDING_FRAME_HANGING_LEFT_CLIMB       4
#define STANDING_FRAME_HANGING_UP_LEFT          5
#define STANDING_FRAME_HANGING_LEFT             6
#define STANDING_FRAME_HANGING_DOWN_LEFT        7

#define STANDING_FRAME_DAMAGE_RIGHT_START       0
#define STANDING_FRAME_DAMAGE_RIGHT_END         3
#define STANDING_FRAME_DAMAGE_LEFT_START        4
#define STANDING_FRAME_DAMAGE_LEFT_END          7

#define STANDING_FRAME_CHANGE_RIGHT_FULL        0
#define STANDING_FRAME_CHANGE_LEFT_FULL         5

#define STANDING_FRAME_FRONT_START              6
#define STANDING_FRAME_FRONT_END                6

#define DAMAGE_FRAME_RIGHT_START                0
#define DAMAGE_FRAME_RIGHT_END                  3
#define DAMAGE_FRAME_LEFT_START                 4
#define DAMAGE_FRAME_LEFT_END                   7

#define STANDING_FRAME_CLIMBING_LEFT            7
#define STANDING_FRAME_CLIMBING_RIGHT           3

#define STANDING_FRAME_HANGING_AIM_DOWN_RIGHT   0
#define STANDING_FRAME_HANGING_AIM_LEFT_RIGHT   1
#define STANDING_FRAME_HANGING_AIM_UP_RIGHT     2
#define STANDING_FRAME_HANGING_AIM_DOWN_LEFT    4
#define STANDING_FRAME_HANGING_AIM_UP_LEFT      5
#define STANDING_FRAME_HANGING_AIM_RIGHT_LEFT   6

enum PlayerStates
{
    PLAYERSTATE_IDLE = 0,
    PLAYERSTATE_JUMPING = 1,
    PLAYERSTATE_RUNNING = 2,
    PLAYERSTATE_FALLING = 3
};

enum PlayerAnim
{
    PLAYERANIM_STANDING = 0,
    PLAYERANIM_CROUCHED = 1,
    PLAYERANIM_JUMP90 = 2,
    PLAYERANIM_JUMPSPIN = 3,
    PLAYERANIM_JUMPSCREW = 4,
    PLAYERANIM_MORPHBALL = 5,
    PLAYERANIM_SPIDERBALL = 6,
    PLAYERANIM_RUN = 7,
    PLAYERANIM_HANGING = 8,
    PLAYERANIM_CLIMBING = 9, // Happens only after hanging
    PLAYERANIM_FALLING = 10,
    PLAYERANIM_DAMAGE = 11,
    PLAYERANIM_DEATH = 12,
    PLAYERANIM_FRONT = 13,
    PLAYERANIM_CHANGESTATE = 14,
};

enum PlayerFacing
{
    PLAYERFACING_RIGHT = 0,
    PLAYERFACING_RIGHTUP = 1,
    PLAYERFACING_RIGHTDOWN = 2,
    PLAYERFACING_LEFT = 3,
    PLAYERFACING_LEFTUP = 4,
    PLAYERFACING_LEFTDOWN = 5,
    PLAYERFACING_UP = 6,
    PLAYERFACING_DOWN = 7
};

enum PlayerSpritePosition
{
    RUNPOS_RIGHT = 0,
    RUNPOS_LEFT = 1,
    RUNPOS_RIGHT_AIM = 2,
    RUNPOS_LEFT_AIM = 3,
    RUNPOS_JUMP90 = 4,
    RUNPOS_JUMP = 5,
    RUNPOS_MORPHBALL = 6,
    RUNPOS_RIGHT_AIMUP = 7,
    RUNPOS_LEFT_AIMUP = 8,
    RUNPOS_SPIDERBALL = 9,
    
    STANDINGPOS_STANDING = 0,
    STANDINGPOS_CROUCHED = 1,
    STANDINGPOS_CHANGESTATE = 2,
    STANDINGPOS_FRONT = 2,
    STANDINGPOS_LEDGE = 3,
    STANDINGPOS_DAMAGE = 4,
    STANDINGPOS_RIGHT_DEATH = 5,
    STANDINGPOS_LEFT_DEATH = 6
};

PlayerSprite::PlayerSprite()
{
    // Set Anim Info
    player_lastanim = 0;
    player_laststate = 0;
    player_lastfacing = 0;
    player_lastshooting = false;
    
    // Set Spritesheets
    player_run = NULL;
    player_standing = NULL;
    player_lastsprite = NULL;
}

PlayerSprite::~PlayerSprite()
{
    // Clear Texture Data
    playersprite_clear_sprites();
    
    // Delete Running Spritesheet Struct
    if (player_run != NULL)
    {
        delete player_run;
        player_run = NULL;
    }
    
    // Delete Standing Spritesheet Struct
    if (player_standing != NULL)
    {
        delete player_standing;
        player_standing = NULL;
    }
    
    
    // Set last Sprite to NULL
    player_lastsprite = NULL;
}

void PlayerSprite::playersprite_clear_sprites()
{
    // Delete Running Texture
    if (player_run->spritesheet_texture != NULL)
    {
        sf2d_free_texture(player_run->spritesheet_texture);
        player_run->spritesheet_texture = NULL;
    }
    
    // Delete Standing Texture
    if (player_standing->spritesheet_texture != NULL)
    {
        sf2d_free_texture(player_standing->spritesheet_texture);
        player_standing->spritesheet_texture = NULL;
    }
}

const char* PlayerSprite::create_load_path(bool is_barrier_suit, std::string file)
{
    std::string base_path = FILEPATH_IMAGE;
    
    if (is_barrier_suit == true)
        base_path += FILEPATH_BARRIERSUIT;
    else
        base_path += FILEPATH_POWERSUIT;
    
    base_path += file;
    
    return base_path.c_str();
}

void PlayerSprite::playersprite_load_sprites(bool is_barrier_suit)
{
    /* Allocate Memory */
    player_run = new SpriteSheet();
    player_standing = new SpriteSheet();

    /* Load Texture Files */

    /* Set Standing Sprites Info */

    // Texture Info for Render
    player_standing->spritesheet_texture = sf2d_create_texture_mem_RGBA8(player_spritesheet_standing.pixel_data,
                                                                         player_spritesheet_standing.width,
                                                                         player_spritesheet_standing.height,
                                                                         TEXFMT_RGBA8,
                                                                         SF2D_PLACE_RAM);

    //player_standing->spritesheet_texture = sfil_load_PNG_file(create_load_path(is_barrier_suit, IMG_PLAYER_STANDING), SF2D_PLACE_RAM);

    player_standing->spritesheet_sprite_line = STANDINGPOS_STANDING;
    player_standing->spritesheet_startframe = STANDING_FRAME_AIM_RIGHT;
    player_standing->spritesheet_endframe = STANDING_FRAME_AIM_RIGHT;
    player_standing->spritesheet_frame = 0;
    player_standing->spritesheet_h_single = PLAYER_SPRITEFRAME_H;
    player_standing->spritesheet_w_single = PLAYER_SPRITEFRAME_W;
    player_standing->spritesheet_frame_speed_current = 0;
    player_standing->spritesheet_frame_speed = PLAYER_FRAME_SPEED;
    player_standing->spritesheet_w_full = PLAYER_SPRITESHEET_W;
    player_standing->spritesheet_h_full = PLAYER_SPRITESHEET_H_STANDING;
    
    /* Set Running Sprites Info */
  
    // Texture Info for Render

    player_run->spritesheet_texture = sf2d_create_texture_mem_RGBA8(player_spritesheet_running.pixel_data,
                                                                    player_spritesheet_running.width,
                                                                    player_spritesheet_running.height,
                                                                    TEXFMT_RGBA8,
                                                                    SF2D_PLACE_RAM);

    //player_run->spritesheet_texture = sfil_load_PNG_file(create_load_path(is_barrier_suit, IMG_PLAYER_RUNNING), SF2D_PLACE_RAM);
    
    player_run->spritesheet_sprite_line = RUNPOS_RIGHT;
    player_run->spritesheet_startframe = 0;
    player_run->spritesheet_endframe = RUNNING_FRAME_RUNAIM_FULL;
    player_run->spritesheet_frame = 0;
    player_run->spritesheet_h_single = PLAYER_SPRITEFRAME_H;
    player_run->spritesheet_w_single = PLAYER_SPRITEFRAME_W;
    player_run->spritesheet_frame_speed_current = 0;
    player_run->spritesheet_frame_speed = PLAYER_FRAME_SPEED;
    player_run->spritesheet_w_full = PLAYER_SPRITESHEET_W;
    player_run->spritesheet_h_full = PLAYER_SPRITESHEET_H_RUNNING;

    // Set Last Sprite
    player_lastsprite = player_standing;
}

SpriteSheet* PlayerSprite::playersprite_get_sprite(u8 player_anim, u8 player_facing, u8 player_state, bool player_shooting, bool player_is_in_water)
{
    // Check for NULL Sprite
    if (player_lastsprite == NULL)
    {
        return NULL;
    }
    
    // Alway Update Frame Speed for specific Animations
    if (player_anim == PLAYERANIM_DEATH)
    {
        // Set Sprite Speed for Animation
        player_run->spritesheet_frame_speed = PLAYER_FRAME_SPEED * 4;
        player_standing->spritesheet_frame_speed = PLAYER_FRAME_SPEED * 4;
        
        // Force Frame
        if (player_lastsprite->spritesheet_frame == RUNNING_FRAME_RUNAIM_FULL)
        {
            // Frame Counter
            player_standing->spritesheet_frame_speed = 254; // Some high value, so we're not setting frames in the render
        }
    }
    else if (player_is_in_water == true)
    {
        // Set Sprite Frame Speed Info
        player_run->spritesheet_frame_speed = PLAYER_FRAME_SPEED * 2;
        player_standing->spritesheet_frame_speed = PLAYER_FRAME_SPEED * 2;
    }
    else
    {
        // Reset Sprite Frame Speed Info
        player_run->spritesheet_frame_speed = PLAYER_FRAME_SPEED;
        player_standing->spritesheet_frame_speed = PLAYER_FRAME_SPEED;
    }
    
    // Don't calc new if we're still using the original Spritesheet
    if (player_anim == player_lastanim && player_facing == player_lastfacing)
    {
        if (player_shooting == false || player_lastshooting == true)
        {
            return player_lastsprite;
        }
    }
    
    // Save New "Last" Values
    player_lastfacing = player_facing;
    player_lastanim = player_anim;
    
    u8 player_last_frame = player_lastsprite->spritesheet_frame, player_frame_correction_six = 0;
    
    if (player_laststate == PLAYERSTATE_FALLING)
    {
        ++player_frame_correction_six;
    }
    
    // Reset Last Shooting
    if (player_anim == PLAYERANIM_RUN && player_shooting == true)
    {
        player_lastshooting = true;
    }
    else
    {
        player_lastshooting = false;
    }
    
    // Reset Frame Count
    player_run->spritesheet_frame = 0;
    player_standing->spritesheet_frame = 0;
    
    if (player_anim == PLAYERANIM_STANDING)
    {
        /* Standing / Aiming */
        
        // get the Right Frame
        switch (player_facing)
        {
            case PLAYERFACING_RIGHT:
                player_standing->spritesheet_startframe = STANDING_FRAME_AIM_RIGHT;
                player_standing->spritesheet_endframe = STANDING_FRAME_AIM_RIGHT;
                player_standing->spritesheet_frame = STANDING_FRAME_AIM_RIGHT;
                break;
            case PLAYERFACING_RIGHTDOWN:
                player_standing->spritesheet_startframe = STANDING_FRAME_AIM_RIGHT_DOWN;
                player_standing->spritesheet_endframe = STANDING_FRAME_AIM_RIGHT_DOWN;
                player_standing->spritesheet_frame = STANDING_FRAME_AIM_RIGHT_DOWN;
                break;
            case PLAYERFACING_RIGHTUP:
                player_standing->spritesheet_startframe = STANDING_FRAME_AIM_RIGHT_UP;
                player_standing->spritesheet_endframe = STANDING_FRAME_AIM_RIGHT_UP;
                player_standing->spritesheet_frame = STANDING_FRAME_AIM_RIGHT_UP;
                break;
            case PLAYERFACING_LEFT:
                player_standing->spritesheet_startframe = STANDING_FRAME_AIM_LEFT;
                player_standing->spritesheet_endframe = STANDING_FRAME_AIM_LEFT;
                player_standing->spritesheet_frame = STANDING_FRAME_AIM_LEFT;
                break;
            case PLAYERFACING_LEFTDOWN:
                player_standing->spritesheet_startframe = STANDING_FRAME_AIM_LEFT_DOWN;
                player_standing->spritesheet_endframe = STANDING_FRAME_AIM_LEFT_DOWN;
                player_standing->spritesheet_frame = STANDING_FRAME_AIM_LEFT_DOWN;
                break;
            case PLAYERFACING_LEFTUP:
                player_standing->spritesheet_startframe = STANDING_FRAME_AIM_LEFT_UP;
                player_standing->spritesheet_endframe = STANDING_FRAME_AIM_LEFT_UP;
                player_standing->spritesheet_frame = STANDING_FRAME_AIM_LEFT_UP;
                break;
            case PLAYERFACING_UP:
                if (player_standing->spritesheet_startframe < 4)
                {
                    player_standing->spritesheet_startframe = STANDING_FRAME_AIM_UP_RIGHT;
                    player_standing->spritesheet_endframe = STANDING_FRAME_AIM_UP_RIGHT;
                    player_standing->spritesheet_frame = STANDING_FRAME_AIM_UP_RIGHT;
                }
                else
                {
                    player_standing->spritesheet_startframe = STANDING_FRAME_AIM_UP_LEFT;
                    player_standing->spritesheet_endframe = STANDING_FRAME_AIM_UP_LEFT;
                    player_standing->spritesheet_frame = STANDING_FRAME_AIM_UP_LEFT;
                }
                break;
                
            default:
                player_standing->spritesheet_startframe = STANDING_FRAME_AIM_RIGHT;
                player_standing->spritesheet_endframe = STANDING_FRAME_AIM_RIGHT;
                player_standing->spritesheet_frame = STANDING_FRAME_AIM_RIGHT;
                break;
        }
        
        // Set Texture Line
        player_standing->spritesheet_sprite_line = STANDINGPOS_STANDING;
        
        // Return Texture
        player_lastsprite = player_standing;
        return player_standing;
    }
    else if (player_anim == PLAYERANIM_RUN)
    {
        /* Running */
        
        // Get Right Texture Line depending on facing
        switch (player_facing)
        {
            case FACING_RIGHT:
                if (player_shooting == true)
                    player_run->spritesheet_sprite_line = RUNPOS_RIGHT_AIM;
                else
                    player_run->spritesheet_sprite_line = RUNPOS_RIGHT;
                break;
            case FACING_LEFT:
                if (player_shooting == true)
                    player_run->spritesheet_sprite_line = RUNPOS_LEFT_AIM;
                else
                    player_run->spritesheet_sprite_line = RUNPOS_LEFT;
                break;
            case FACING_RIGHTUP:
                if (player_shooting == true)
                    player_run->spritesheet_sprite_line = RUNPOS_RIGHT_AIMUP;
                else
                    player_run->spritesheet_sprite_line = RUNPOS_RIGHT;
                break;
            case FACING_LEFTUP:
                if (player_shooting == true)
                    player_run->spritesheet_sprite_line = RUNPOS_LEFT_AIMUP;
                else
                    player_run->spritesheet_sprite_line = RUNPOS_LEFT;
                break;
                
            default:
                player_run->spritesheet_sprite_line = RUNPOS_RIGHT;
                break;
        }
        
        // Set Texture Frames
        player_run->spritesheet_startframe = 0;
        player_run->spritesheet_endframe = RUNNING_FRAME_RUNAIM_FULL;
        
        // Return Texture
        player_lastsprite = player_run;
        return player_run;
    }
    else if (player_anim == PLAYERANIM_CROUCHED)
    {
        /* Standing / Aiming */
        
        // get the Right Frame
        switch (player_facing)
        {
            case PLAYERFACING_RIGHT:
                player_standing->spritesheet_startframe = STANDING_FRAME_CROUCHED_RIGHT;
                player_standing->spritesheet_endframe = STANDING_FRAME_CROUCHED_RIGHT;
                player_standing->spritesheet_frame = STANDING_FRAME_CROUCHED_RIGHT;
                break;
            case PLAYERFACING_RIGHTDOWN:
                player_standing->spritesheet_startframe = STANDING_FRAME_CROUCHED_RIGHT_DOWN;
                player_standing->spritesheet_endframe = STANDING_FRAME_CROUCHED_RIGHT_DOWN;
                player_standing->spritesheet_frame = STANDING_FRAME_CROUCHED_RIGHT_DOWN;
                break;
            case PLAYERFACING_RIGHTUP:
                player_standing->spritesheet_startframe = STANDING_FRAME_CROUCHED_RIGHT_UP;
                player_standing->spritesheet_endframe = STANDING_FRAME_CROUCHED_RIGHT_UP;
                player_standing->spritesheet_frame = STANDING_FRAME_CROUCHED_RIGHT_UP;
                break;
            case PLAYERFACING_LEFT:
                player_standing->spritesheet_startframe = STANDING_FRAME_CROUCHED_LEFT;
                player_standing->spritesheet_endframe = STANDING_FRAME_CROUCHED_LEFT;
                player_standing->spritesheet_frame = STANDING_FRAME_CROUCHED_LEFT;
                break;
            case PLAYERFACING_LEFTDOWN:
                player_standing->spritesheet_startframe = STANDING_FRAME_CROUCHED_LEFT_DOWN;
                player_standing->spritesheet_endframe = STANDING_FRAME_CROUCHED_LEFT_DOWN;
                player_standing->spritesheet_frame = STANDING_FRAME_CROUCHED_LEFT_DOWN;
                break;
            case PLAYERFACING_LEFTUP:
                player_standing->spritesheet_startframe = STANDING_FRAME_CROUCHED_LEFT_UP;
                player_standing->spritesheet_endframe = STANDING_FRAME_CROUCHED_LEFT_UP;
                player_standing->spritesheet_frame = STANDING_FRAME_CROUCHED_LEFT_UP;
                break;
            case PLAYERFACING_UP:
                if (player_standing->spritesheet_startframe < 4)
                {
                    player_standing->spritesheet_startframe = STANDING_FRAME_CROUCHED_UP_RIGHT;
                    player_standing->spritesheet_endframe = STANDING_FRAME_CROUCHED_UP_RIGHT;
                    player_standing->spritesheet_frame = STANDING_FRAME_CROUCHED_UP_RIGHT;
                }
                else
                {
                    player_standing->spritesheet_startframe = STANDING_FRAME_CROUCHED_UP_LEFT;
                    player_standing->spritesheet_endframe = STANDING_FRAME_CROUCHED_UP_LEFT;
                    player_standing->spritesheet_frame = STANDING_FRAME_CROUCHED_UP_LEFT;
                }
                break;
                
            default:
                player_standing->spritesheet_startframe = STANDING_FRAME_CROUCHED_RIGHT;
                player_standing->spritesheet_endframe = STANDING_FRAME_CROUCHED_RIGHT;
                player_standing->spritesheet_frame = STANDING_FRAME_CROUCHED_RIGHT;
                break;
        }
        
        // Set Texture Line
        player_standing->spritesheet_sprite_line = STANDINGPOS_CROUCHED;
        
        // Return Texture
        player_lastsprite = player_standing;
        return player_standing;
    }
    else if (player_anim == PLAYERANIM_JUMPSPIN || player_anim == PLAYERANIM_JUMPSCREW)
    {
        /* Jumping Left or Right */
        if (player_facing < FACING_LEFT)
        {
            // Right
            player_run->spritesheet_startframe = RUNNING_FRAME_JUMP_RIGHT_START;
            player_run->spritesheet_endframe = RUNNING_FRAME_JUMP_RIGHT_END;
            player_run->spritesheet_frame = RUNNING_FRAME_JUMP_RIGHT_START;
        }
        else
        {
            // Left
            player_run->spritesheet_startframe = RUNNING_FRAME_JUMP_LEFT_START;
            player_run->spritesheet_endframe = RUNNING_FRAME_JUMP_LEFT_END;
            player_run->spritesheet_frame = RUNNING_FRAME_JUMP_LEFT_START;
        }
        
        // Set Frame Data
        player_run->spritesheet_sprite_line = RUNPOS_JUMP;
        
        // Return Texture
        player_lastsprite = player_run;
        return player_run;
    }
    else if (player_anim == PLAYERANIM_DAMAGE)
    {
        /* Jumping Left or Right */
        if (player_facing < FACING_LEFT)
        {
            // Right
            player_standing->spritesheet_startframe = DAMAGE_FRAME_RIGHT_START;
            player_standing->spritesheet_endframe = DAMAGE_FRAME_RIGHT_END;
            player_standing->spritesheet_frame = DAMAGE_FRAME_RIGHT_START;
        }
        else
        {
            // Left
            player_standing->spritesheet_startframe = DAMAGE_FRAME_LEFT_START;
            player_standing->spritesheet_endframe = DAMAGE_FRAME_LEFT_END;
            player_standing->spritesheet_frame = DAMAGE_FRAME_LEFT_START;
        }
        
        // Set Frame Data
        player_standing->spritesheet_sprite_line = STANDINGPOS_DAMAGE;
        
        // Return Texture
        player_lastsprite = player_standing;
        return player_standing;
    }
    else if (player_anim == PLAYERANIM_MORPHBALL || player_anim == PLAYERANIM_SPIDERBALL)
    {
        /* Morphball */
        
        // Set Frame Data
        player_run->spritesheet_startframe = 0;
        player_run->spritesheet_endframe = RUNNING_FRAME_RUNAIM_FULL;
        
        if (player_anim == PLAYERANIM_SPIDERBALL)
            player_run->spritesheet_sprite_line = RUNPOS_SPIDERBALL;
        else
            player_run->spritesheet_sprite_line = RUNPOS_MORPHBALL;
        
        // Return Texture
        player_lastsprite = player_run;
        return player_lastsprite;
    }
    else if (player_anim == PLAYERANIM_FALLING || player_anim == PLAYERANIM_JUMP90)
    {
        switch (player_facing)
        {
            case FACING_RIGHT:
                player_run->spritesheet_startframe = RUNNING_FRAME_JUMP90_RIGHT;
                player_run->spritesheet_endframe = RUNNING_FRAME_JUMP90_RIGHT;
                player_run->spritesheet_frame = RUNNING_FRAME_JUMP90_RIGHT;
                break;
            case FACING_LEFT:
                player_run->spritesheet_startframe = RUNNING_FRAME_JUMP90_LEFT;
                player_run->spritesheet_endframe = RUNNING_FRAME_JUMP90_LEFT;
                player_run->spritesheet_frame = RUNNING_FRAME_JUMP90_LEFT;
                break;
            case FACING_RIGHTUP:
                player_run->spritesheet_startframe = RUNNING_FRAME_JUMP90_UP_RIGHT;
                player_run->spritesheet_endframe = RUNNING_FRAME_JUMP90_UP_RIGHT;
                player_run->spritesheet_frame = RUNNING_FRAME_JUMP90_UP_RIGHT;
                break;
            case FACING_RIGHTDOWN:
                player_run->spritesheet_startframe = RUNNING_FRAME_JUMP90_DOWN_RIGHT;
                player_run->spritesheet_endframe = RUNNING_FRAME_JUMP90_DOWN_RIGHT;
                player_run->spritesheet_frame = RUNNING_FRAME_JUMP90_DOWN_RIGHT;
                break;
            case FACING_LEFTUP:
                player_run->spritesheet_startframe = RUNNING_FRAME_JUMP90_UP_LEFT;
                player_run->spritesheet_endframe = RUNNING_FRAME_JUMP90_UP_LEFT;
                player_run->spritesheet_frame = RUNNING_FRAME_JUMP90_UP_LEFT;
                break;
            case FACING_LEFTDOWN:
                player_run->spritesheet_startframe = RUNNING_FRAME_JUMP90_DOWN_LEFT;
                player_run->spritesheet_endframe = RUNNING_FRAME_JUMP90_DOWN_LEFT;
                player_run->spritesheet_frame = RUNNING_FRAME_JUMP90_DOWN_LEFT;
                break;
                
            default:
                player_run->spritesheet_startframe = RUNNING_FRAME_JUMP90_RIGHT;
                player_run->spritesheet_endframe = RUNNING_FRAME_JUMP90_RIGHT;
                player_run->spritesheet_frame = RUNNING_FRAME_JUMP90_RIGHT;
                break;
        }
        
        // Set Texture Line
        player_run->spritesheet_sprite_line = RUNPOS_JUMP90;
        
        // Return Texture
        player_lastsprite = player_run;
        return player_run;
    }
    else if (player_anim == PLAYERANIM_CHANGESTATE)
    {
        // get the Right Frame
        if (player_facing < FACING_LEFT)
        {
            player_standing->spritesheet_startframe = STANDING_FRAME_CHANGE_RIGHT_FULL;
            player_standing->spritesheet_endframe = STANDING_FRAME_CHANGE_RIGHT_FULL;
            player_standing->spritesheet_frame = STANDING_FRAME_CHANGE_RIGHT_FULL;
        }
        else
        {
            player_standing->spritesheet_startframe = STANDING_FRAME_CHANGE_LEFT_FULL;
            player_standing->spritesheet_endframe = STANDING_FRAME_CHANGE_LEFT_FULL;
            player_standing->spritesheet_frame = STANDING_FRAME_CHANGE_LEFT_FULL;
        }
        
        // Set Texture Line
        player_standing->spritesheet_sprite_line = STANDINGPOS_CHANGESTATE;
        
        // Return Texture
        player_lastsprite = player_standing;
        return player_standing;
    }
    else if (player_anim == PLAYERANIM_FRONT)
    {
        // Set Frame Info
        player_standing->spritesheet_startframe = STANDING_FRAME_FRONT_START;
        player_standing->spritesheet_endframe = STANDING_FRAME_FRONT_END;
        player_standing->spritesheet_frame = STANDING_FRAME_FRONT_START;
        
        // Set Texture Line
        player_standing->spritesheet_sprite_line = STANDINGPOS_CHANGESTATE;
        
        // Return Texture
        player_lastsprite = player_standing;
        return player_standing;
    }
    else if (player_anim == PLAYERANIM_HANGING)
    {
        if (player_last_frame < 4)
        {
            if (player_facing == FACING_UP)
            {
                player_standing->spritesheet_startframe = STANDING_FRAME_HANGING_AIM_UP_RIGHT;
                player_standing->spritesheet_endframe = STANDING_FRAME_HANGING_AIM_UP_RIGHT;
                player_standing->spritesheet_frame = STANDING_FRAME_HANGING_AIM_UP_RIGHT;
            }
            else if (player_facing == FACING_LEFT || player_facing == FACING_LEFTUP || player_facing == FACING_LEFTDOWN)
            {
                player_standing->spritesheet_startframe = STANDING_FRAME_HANGING_AIM_LEFT_RIGHT;
                player_standing->spritesheet_endframe = STANDING_FRAME_HANGING_AIM_LEFT_RIGHT;
                player_standing->spritesheet_frame = STANDING_FRAME_HANGING_AIM_LEFT_RIGHT;
            }
            else
            {
                player_standing->spritesheet_startframe = STANDING_FRAME_HANGING_AIM_DOWN_RIGHT;
                player_standing->spritesheet_endframe = STANDING_FRAME_HANGING_AIM_DOWN_RIGHT;
                player_standing->spritesheet_frame = STANDING_FRAME_HANGING_AIM_DOWN_RIGHT;
            }
        }
        else
        {
            if (player_facing == FACING_UP)
            {
                player_standing->spritesheet_startframe = STANDING_FRAME_HANGING_AIM_UP_LEFT;
                player_standing->spritesheet_endframe = STANDING_FRAME_HANGING_AIM_UP_LEFT;
                player_standing->spritesheet_frame = STANDING_FRAME_HANGING_AIM_UP_LEFT;
            }
            else if (player_facing == FACING_RIGHT || player_facing == FACING_RIGHTUP || player_facing == FACING_RIGHTDOWN)
            {
                player_standing->spritesheet_startframe = STANDING_FRAME_HANGING_AIM_RIGHT_LEFT;
                player_standing->spritesheet_endframe = STANDING_FRAME_HANGING_AIM_RIGHT_LEFT;
                player_standing->spritesheet_frame = STANDING_FRAME_HANGING_AIM_RIGHT_LEFT;
            }
            else
            {
                player_standing->spritesheet_startframe = STANDING_FRAME_HANGING_AIM_DOWN_LEFT;
                player_standing->spritesheet_endframe = STANDING_FRAME_HANGING_AIM_DOWN_LEFT;
                player_standing->spritesheet_frame = STANDING_FRAME_HANGING_AIM_DOWN_LEFT;
            }
        }
        
        // Set Texture Line
        player_standing->spritesheet_sprite_line = STANDINGPOS_LEDGE;
        
        // Return Texture
        player_lastsprite = player_standing;
        return player_standing;
    }
    else if (player_anim == PLAYERANIM_CLIMBING)
    {
        // get the Right Frame
        if (player_facing < FACING_LEFT)
        {
            player_standing->spritesheet_startframe = STANDING_FRAME_CLIMBING_RIGHT;
            player_standing->spritesheet_endframe = STANDING_FRAME_CLIMBING_RIGHT;
            player_standing->spritesheet_frame = STANDING_FRAME_CLIMBING_RIGHT;
        }
        else
        {
            player_standing->spritesheet_startframe = STANDING_FRAME_CLIMBING_LEFT;
            player_standing->spritesheet_endframe = STANDING_FRAME_CLIMBING_LEFT;
            player_standing->spritesheet_frame = STANDING_FRAME_CLIMBING_LEFT;
        }
        
        // Set Texture Line
        player_standing->spritesheet_sprite_line = STANDINGPOS_LEDGE;
        
        // Return Texture
        player_lastsprite = player_standing;
        return player_standing;
    }
    else if (player_anim == PLAYERANIM_DEATH)
    {
        // Get Right Facing Sprites
        if (player_facing < FACING_LEFT)
        {
            // Set Texture Line
            player_standing->spritesheet_sprite_line = STANDINGPOS_RIGHT_DEATH;
            
            // Set Frame Info
            player_standing->spritesheet_startframe = 0;
            player_standing->spritesheet_endframe = RUNNING_FRAME_RUNAIM_FULL;
            player_standing->spritesheet_frame = 0;
        }
        else
        {
            // Set Texture Line
            player_standing->spritesheet_sprite_line = STANDINGPOS_LEFT_DEATH;
            
            // Set Frame Info
            player_standing->spritesheet_startframe = 0;
            player_standing->spritesheet_endframe = RUNNING_FRAME_RUNAIM_FULL;
            player_standing->spritesheet_frame = 0;
        }
        
        // Return Texture
        player_lastsprite = player_standing;
        return player_standing;
    }
    else
        return NULL;
}

u8 PlayerSprite::playersprite_get_last_frame()
{
    if (player_lastsprite != NULL)
    {
        return player_lastsprite->spritesheet_frame;
    }
    else
    {
        return 0;
    }
}

u8 PlayerSprite::playersprite_get_last_anim()
{
    if (player_lastsprite != NULL)
    {
        return player_lastanim;
    }
    else
    {
        return 0;
    }
}

u8 PlayerSprite::playersprite_get_last_state()
{
    if (player_lastsprite != NULL)
    {
        return player_laststate;
    }
    else
    {
        return 0;
    }
}

u8 PlayerSprite::playersprite_get_last_facing()
{
    if (player_lastsprite != NULL)
    {
        return player_lastfacing;
    }
    else
    {
        return 0;
    }
}
